/*******************************************************************************
* FILE NAME: ibufferd.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the memory-debug implementations of operators new and   *
*   delete for IBuffer (see ibuffer.hpp).                                      *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifdef __DEBUG_ALLOC__

#include <ibuffer.hpp>

// This file provides the __DEBUG_ALLOC__ implementations of the
// operator new and delete for IBuffer.  The non-memory-debug version
// of the same operators is provided in ibuffern.inl/nbuffern.cpp.
// This file (ibufferd.cpp) exists so that an application can build
// with /Tm+, with or without I_NO_INLINES, and link dynamically to the
// class library DLL, without encountering an unresolved external--even
// if the class library code is built with /Tm-.
//
// This file must be compiled with /Tm+.  This compiler option is forced
// via the profile used to generate the makefile, which in turn is used
// to compile this file.

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IBUFFERD_CPP_
  #include <ipagetun.h>
#endif

void* IBuffer::operator new ( size_t,
                              const char* filename,
                              size_t linenum,
                              unsigned realSize )
{
  void* p = ::operator new( checkAddition( sizeof( IBuffer ), realSize ),
                            filename, linenum );
  return p;
}

void IBuffer::operator delete ( void* p,
                                const char* filename,
                                size_t linenum )
{
  ::operator delete( (char*)p, filename, linenum );
}

#else  // ! __DEBUG_ALLOC__
#ifdef __OS2__
   #error Error: ibufferd.cpp must be compiled with /Tm+.
   #error Check the makefile (or its profile) for a missing /Tm+ option.
#endif
#endif
