/*******************************************************************************
* FILE NAME: icontext.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*     IContext                                                                 *                                                                              *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*   Portions or this component are based on work originating from Taligent.    *
*   (C) Copyright Taligent, Inc. 1996 - All rights reserved.                   *
*                                                                              *
*******************************************************************************/
#include <icontext.hpp>
#include <stdlib.h>
#include <icontlst.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICONTEXT_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| IContext::Context                                                            |
|                                                                              |
| Constructor.                                                                 |
------------------------------------------------------------------------------*/
IContext::IContext()
: fNumberToObject(new IContextNumberSet),
  fObjectToNumber(new IContextElementSet),
  fCount(0)
{
}


/*------------------------------------------------------------------------------
| IContext::~Context                                                           |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IContext::~IContext()
{
    if (fNumberToObject)
    {
        delete (IContextNumberSet*)fNumberToObject;
        delete (IContextElementSet*)fObjectToNumber;
    }
}

/*------------------------------------------------------------------------------
| IContext::find                                                               |
|                                                                              |
| Find an object of the passed id.                                             |
------------------------------------------------------------------------------*/
const void*
IContext::find(ObjectNumber objectID)
{
    if (objectID < 1 || objectID > fCount)   //
        return 0;

    if (objectID <= kFastObjectsArraySize)
        return  fFastObjects[objectID-1];
    else
    {
        IContextNumberSet::Cursor cur(*(IContextNumberSet*)fNumberToObject);
        if ((IContextNumberSet*)fNumberToObject->locateElementWithKey(objectID, cur))
            return cur.element().object();
        else
            return 0;
    }
}

/*------------------------------------------------------------------------------
| IContext::reset                                                              |
|                                                                              |
| Remove the objects from the collection.                                      |
------------------------------------------------------------------------------*/
IContext&
IContext::reset()
{
    if (fNumberToObject)
    {
        ((IContextNumberSet*)fNumberToObject)->removeAll();
        ((IContextElementSet*)fObjectToNumber)->removeAll();
        // Only remove the elements, do not delete the collections
    }
    fCount = 0;
    return *this;
}

/*------------------------------------------------------------------------------
| IContext::count                                                              |
|                                                                              |
| Return the count of objects in the collection.                               |
------------------------------------------------------------------------------*/
unsigned long
IContext::count()
{
    return fCount;
}

/*------------------------------------------------------------------------------
| IContext::add                                                                |
|                                                                              |
| Add an object to the collection.  Return the object count and fill the       |
| passed Boolean with true if the object already exists, else set it to false. |
------------------------------------------------------------------------------*/
IContext::ObjectNumber
IContext::add(const void* object, IBoolean& newentry)
{
    if (fCount <= kFastObjectsArraySize)
    {
        for (ObjectNumber i = 0; i < fCount; i++) {
            if (fFastObjects[i] == object)
            {
                if (newentry == false)
                    return i+1;
                else
                {
                    fFastObjects[i] = 0;
                    break;
                }
            }
        }

        if (fCount < kFastObjectsArraySize)
        {
            fFastObjects[fCount] = (void*) object;
            newentry = true;
            return (ObjectNumber) ++fCount;
        }
        else if (! (IContextNumberSet*)fNumberToObject)
        {
            fNumberToObject = new IContextNumberSet;
            fObjectToNumber = new IContextElementSet;

            for (ObjectNumber j = 0; j < kFastObjectsArraySize ; j++) {
                (IContextNumberSet*)fNumberToObject->add(INumberToObjectElement(j+1, fFastObjects[j]));
                (IContextElementSet*)fObjectToNumber->add(IObjectToNumberElement(fFastObjects[j], j+1));
            }
        }
    }

    IContextElementSet::Cursor curNum(*(IContextElementSet*)fObjectToNumber);
    if ((IContextElementSet*)fObjectToNumber->locateElementWithKey((void*)object, curNum))
    {
        ObjectNumber num = curNum.element().number();
        if (newentry == false)
            return num;
        else
        {
            ((IContextElementSet*)fObjectToNumber)->removeAt(curNum);
            ((IContextNumberSet*)fNumberToObject)->removeElementWithKey(num);
        }
    }

    newentry = true;
    fCount++;
    (IContextNumberSet*)fNumberToObject->add(INumberToObjectElement(fCount, object));
    (IContextElementSet*)fObjectToNumber->add(IObjectToNumberElement(object, fCount));

    return fCount;
}


