/*******************************************************************************
* FILE NAME: idbcsb5.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*  This file contains the implementation of the IDBCSBuffer accessing          *
*  function "subString".                                                       *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#include <istring.hpp>

#include <idbcsbuf.hpp>



#if defined (__SOM_ENABLED__ ) && defined (__RRBC_LIB__)

#include <istrtest.hpp>

#endif    // SOM_ENABLED & RRBC_LIB


// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDBCSB5_CPP_
  #include <ipagetun.h>
#endif

#include "inlsutil.hpp"

/*------------------------------------------------------------------------------
| IDBCSBuffer::subString                                                       |
|                                                                              |
| Allocate a new buffer by calling the inherited version of this function.     |
|                                                                              |
| Then, if the substring split a DBCS character at either end, convert that    |
| character to the pad character.                                              |
------------------------------------------------------------------------------*/
IBuffer *IDBCSBuffer :: subString ( unsigned startPos,
                                    unsigned len,
                                    char     padCharacter ) const
{
#ifdef IC_NLS
  if (IString::isInternationalized()) {
    IBuffer *result;
    char *pStr = new char[len];

    result = newBuffer(getSubString(contents(), length(), startPos,
                                    padCharacter, pStr, len), len);
    delete[] pStr;

    return result;
  } else {
#endif

    IBuffer *result;

    if ( startPos == 0 )
       startPos = 1;

    //-------------------------------------------------------------------------
    // Calculate how much can come from receiver.
    //-------------------------------------------------------------------------
    unsigned int fromReceiver = ( startPos <= this->length() )
                                  ? this->length() - startPos + 1
                                  : 0;

    //-------------------------------------------------------------------------
    // Default length to rest of string.
    //-------------------------------------------------------------------------
    if ( len < fromReceiver )
       fromReceiver = len;

    if (startPos > this->length())
        startPos = 1;
    //-------------------------------------------------------------------------
    // Initialize what we can from the receiver and leave
    // room for the remainder.
    //-------------------------------------------------------------------------
    result = this->newBuffer( this->contents() + startPos - 1, fromReceiver,
                              0, len - fromReceiver,
                              0, 0,
                              padCharacter );

    //-------------------------------------------------------------------------
    // If the byte at startPos is the second byte of a DBCS character,
    // then that byte should be replaced with a padCharacter.
    // If the byte at (startPos + len - 1) is the first byte of a DBCS
    // character, then that byte should be replaced with a padCharacter.
    //-------------------------------------------------------------------------
    for ( unsigned i = 0; startPos + i <= this->length (); i++ ) {
      if ( this->charType( startPos + i ) > IStringEnum::mbcs1 )
        result->contents()[i] = padCharacter;
      else
        break;
    }

    if ( startPos + fromReceiver <= this->length () )
    {
      IStringEnum::CharType typeOfFirstAfterSubstring =
        this->charType ( startPos + len );
      if ( typeOfFirstAfterSubstring > IStringEnum::mbcs1 )
        for ( unsigned i = 1; i < typeOfFirstAfterSubstring && i <= len; i++)
          result->contents()[len - i] = padCharacter;
    }

    return result;
#ifdef IC_NLS
  }
#endif
}

#ifdef IC_NLS
IBuffer *IDBCSBuffer :: subString ( unsigned startPos ) const {
  IBuffer *result;

  if (startPos > length()) {
    // create null buffer
    result = newBuffer(0, 0);
  } else {
    // Allocate the number of remaining bytes times 2 to
    // ensure that we have enough room to store the resulting string.
    char *pStr = new char[(length()-startPos+1) * 2];

    unsigned len = getRemainString(contents(), length(), startPos, ' ', pStr);
    result = newBuffer(pStr, len);

    delete[] pStr;
  }

    return result;
}
#endif
