#ifdef __MVS__
                   ??=ifdef __COMPILER_VER__
                     ??=pragma filetag("IBM-1047")
                   ??=endif
                   #pragma nomargins nosequence
                   #pragma checkout(suspend)
#endif


#ifndef _IEXCBASE_
#define _IEXCBASE_
/*******************************************************************************
* FILE NAME: iexcbase.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IExceptionLocation                                                       *
*     IException                                                               *
*     IAccessError                                                             *
*     IAssertionFailure                                                        *
*     IDeviceError                                                             *
*     IInvalidParameter                                                        *
*     IInvalidRequest                                                          *
*     IResourceExhausted                                                       *
*     IOutOfMemory                                                             *
*     IOutOfSystemResource                                                     *
*     IOutOfWindowResource                                                     *
*                                                                              *
*   This file also contains many of the macros used to implement the           *
*   library exception handling mechanism.  This includes the IASSERT, ITHROW,  *
*   IRETHROW, IEXCEPTION_LOCATION, IEXCLASSDECLARE, IEXCLASSIMPLEMENT, and     *
*   and INO_EXCEPTIONS_SUPPORT macros.                                         *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef IC_IMPORTB
  #if defined(__WINDOWS__) && defined(__IMPORTLIB__)
    #define IC_IMPORTB  _Import
  #else
    #define IC_IMPORTB
  #endif
#endif
#ifndef IC_IMPORTU
  #if defined(__WINDOWS__) && defined(__IMPORTLIB__)
    #define IC_IMPORTU  _Import
  #else
    #define IC_IMPORTU
  #endif
#endif
#ifndef IC_IMPORTD
  #if defined(__WINDOWS__) && defined(__IMPORTLIB__)
    #define IC_IMPORTD  _Import
  #else
    #define IC_IMPORTD
  #endif
#endif
#ifndef IC_IMPORTM
  #if defined(__WINDOWS__) && defined(__IMPORTLIB__)
    #define IC_IMPORTM  _Import
  #else
    #define IC_IMPORTM
  #endif
#endif

#include <imsgtext.hpp>

#pragma pack(4)

#if defined (__SOM_ENABLED__)
#if defined (__RRBC_LIB__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif

class IExcText;

class IExceptionLocation {
public:

/*------------------------------- Constructors -------------------------------*/
  IExceptionLocation ( const char*   fileName = 0,
                       const char*   functionName = 0,
                       unsigned long lineNumber = 0 );

/*-------------------------------- Attributes --------------------------------*/
const char
 *fileName     ( ) const,
 *functionName ( ) const;

unsigned long
  lineNumber ( ) const;

private:
/*--------------------------------- Private ----------------------------------*/
const char
 *pClFile,
 *pClFunction;
unsigned long
  ulClLineNo;


#if defined (__SOM_ENABLED__ ) && defined (__RRBC_LIB__)

#pragma SOMClassVersion  (*, 3, 0)
#pragma SOMReleaseOrder( \
/*   1 */       IExceptionLocation(const char*,const char*,unsigned long),\
/*   2 */       fileName() const,\
/*   3 */       functionName() const,\
/*   4 */       lineNumber() const)

#endif    // SOM_ENABLED & RRBC_LIB


}; // IExceptionLocation

class IException {
public:

enum Severity { unrecoverable, recoverable };

/*------------------------------- Constructors -------------------------------*/
  IException ( const char*       errorText,
               unsigned long     errorId = 0,
               Severity          severity = IException::unrecoverable );
  IException ( const IException& exception );

virtual
 ~IException ( );

class TraceFn {
public:
/*--------------------------------- Tracing ----------------------------------*/
virtual void
  write   ( const char* buffer ),
  logData ( IException& exception );

protected:
/*-------------------------- Protected Constructors --------------------------*/
  TraceFn ( );

/*--------------------------------- Tracing ----------------------------------*/
virtual void
  exceptionLogged ( );


#if defined (__SOM_ENABLED__ ) && defined (__RRBC_LIB__)

#pragma SOMClassVersion  (*, 3, 0)
#pragma SOMReleaseOrder( \
/*   1 */       write(const char*),\
/*   2 */       logData(IException&),\
/*   3 */       exceptionLogged())

#endif    // SOM_ENABLED & RRBC_LIB


}; // TraceFn

/*---------------------------- Error Code Groups -----------------------------*/
typedef const char* ErrorCodeGroup;

static ErrorCodeGroup const
  IC_IMPORTB baseLibrary,
  IC_IMPORTB CLibrary,
  IC_IMPORTB operatingSystem,
  IC_IMPORTB presentationSystem,
  IC_IMPORTB other;

IException&
  setErrorCodeGroup ( ErrorCodeGroup errorGroup );

ErrorCodeGroup
  errorCodeGroup ( ) const;

/*------------------------------ Exception Text ------------------------------*/
IException
 &appendText  ( const char*   errorText ),
 &setText     ( const char*   errorText );

const char
 *text ( unsigned long indexFromTop = 0 ) const;

unsigned long
  textCount     ( ) const;

/*---------------------------- Exception Severity ----------------------------*/
IException
 &setSeverity ( Severity      severity );

virtual int
 isRecoverable ( ) const;

/*----------------------------- Error Identifier -----------------------------*/
IException
 &setErrorId  ( unsigned long errorId );

unsigned long
  errorId       ( ) const;

/*---------------------------- Exception Location ----------------------------*/
virtual IException
 &addLocation      ( const IExceptionLocation& location );

unsigned long
  locationCount ( ) const;

const IExceptionLocation
 *locationAtIndex ( unsigned long locationIndex ) const;

/*------------------------------ Exception Type ------------------------------*/
virtual const char
 *name ( ) const;

/*---------------------------- Exception Logging -----------------------------*/
static IException::TraceFn
  *setTraceFunction ( IException::TraceFn& traceFunction );

virtual IException
 &logExceptionData ( );

/*------------------------- Application Termination --------------------------*/
virtual void
  terminate ( );

/*------------------------------ Throw Support -------------------------------*/
static void
  assertParameter ( const char*        exceptionText,
                    IExceptionLocation location );

private:
/*----------------------------- Hidden Functions -----------------------------*/
IException
 &operator= ( const IException& exc );

/*--------------------------------- Private ----------------------------------*/
friend class IException::TraceFn;

Severity
  exsevCl;
unsigned long
  ulClErrorId;
IExceptionLocation
  exlocClArray[5];
unsigned long
  ulexlocClCount,
  ulClTxtLvlCount;
IExcText
 *msgtxtClTop;
ErrorCodeGroup
  fErrorGroup;


#if defined (__SOM_ENABLED__ ) && defined (__RRBC_LIB__)

#pragma SOMClassVersion  (*, 3, 0)
#pragma SOMReleaseOrder( \
/*   1 */       IException(const char*,unsigned long,Severity),\
/*   2 */       baseLibrary,\
/*   3 */       CLibrary,\
/*   4 */       operatingSystem,\
/*   5 */       presentationSystem,\
/*   6 */       other,\
/*   7 */       setErrorCodeGroup(ErrorCodeGroup),\
/*   8 */       errorCodeGroup() const,\
/*   9 */       appendText(const char*),\
/*  10 */       setText(const char*),\
/*  11 */       text(unsigned long) const,\
/*  12 */       textCount() const,\
/*  13 */       setSeverity(Severity),\
/*  14 */       isRecoverable() const,\
/*  15 */       setErrorId(unsigned long),\
/*  16 */       errorId() const,\
/*  17 */       addLocation(const IExceptionLocation&),\
/*  18 */       locationCount() const,\
/*  19 */       locationAtIndex(unsigned long) const,\
/*  20 */       name() const,\
/*  21 */       setTraceFunction(TraceFn&),\
/*  22 */       logExceptionData(),\
/*  23 */       terminate(),\
/*  24 */       assertParameter(const char*,IExceptionLocation),\
/*  25 */       operator=(const IException&))

#endif    // SOM_ENABLED & RRBC_LIB

}; // IException

#if !defined( INO_EXCEPTIONS_SUPPORT )
  #define INO_EXCEPTIONS_SUPPORT 0
#endif


#if defined ( INO_FUNCTION_NAMES )
  #define IEXCEPTION_LOCATION() \
                IExceptionLocation(__FILE__, 0, __LINE__)
#elif defined ( __FUNCTION__ )
  #define IEXCEPTION_LOCATION()  \
                IExceptionLocation(__FILE__, __FUNCTION__, __LINE__)
#else
  #define IEXCEPTION_LOCATION()   \
                IExceptionLocation(__FILE__, 0, __LINE__)
#endif

#if (INO_EXCEPTIONS_SUPPORT)
  #define ITHROW(exc)\
     exc.addLocation(IEXCEPTION_LOCATION()),\
     exc.logExceptionData(),\
     exc.terminate()
#else
  #define ITHROW(exc)\
     exc.addLocation(IEXCEPTION_LOCATION()),\
     exc.logExceptionData(),\
     throw(exc)
#endif

#define IRETHROW(exc)\
     exc.addLocation(IEXCEPTION_LOCATION()),\
     exc.logExceptionData(),\
     throw

#if defined(IC_DEVELOP)
  #define IASSERT(test)\
      if(!(test))\
      {\
         IException::assertParameter(   \
   "The following expression must be true, but evaluated to false: " #test,\
         IEXCEPTION_LOCATION());\
      }
#else
  #define IASSERT(test)
#endif

#define IEXCLASSDECLARE(child,parent) class child : public parent {\
public:\
  child(const char* a, unsigned long b = 0,\
        Severity c = IException::unrecoverable);\
  virtual const char* name() const;\
  virtual ~child();\
  child(const child &);\
private:\
  child &operator = ( const child & );\
}

#define IEXCLASSIMPLEMENT(child,parent)\
  child :: child(const char* a, unsigned long b,\
                 Severity c)\
    : parent(a, b, c)\
        { }\
  const char* child :: name() const\
    {\
     return ( # child);\
    }\
  child :: ~child() {;}\
  child :: child(const child & a) : parent(a) {;}

IEXCLASSDECLARE(IAccessError,IException);
IEXCLASSDECLARE(IAssertionFailure,IException);
IEXCLASSDECLARE(IDeviceError,IException);
IEXCLASSDECLARE(IInvalidParameter,IException);
IEXCLASSDECLARE(IInvalidRequest,IException);
IEXCLASSDECLARE(IResourceExhausted,IException);
  IEXCLASSDECLARE(IOutOfMemory,IResourceExhausted);
  IEXCLASSDECLARE(IOutOfSystemResource,IResourceExhausted);
  IEXCLASSDECLARE(IOutOfWindowResource,IResourceExhausted);


#if defined (__SOM_ENABLED__)
#pragma SOMAsDefault (pop)
#endif

/*-------------------------- Pragma Library Support --------------------------*/
#ifndef __NO_DEFAULT_LIBS__
  #ifdef __OS2__
    #ifdef __IMPORTLIB__
       #pragma library("CPPOOC3I.LIB")
    #else
       #pragma library("CPPOOC3.LIB")
    #endif
  #endif
  #ifdef __WINDOWS__
    #ifdef __IMPORTLIB__
       #pragma library("CPPWOB3I.LIB")
       #pragma library("CPPWOU3I.LIB")
       #pragma library("CPPWOD3I.LIB")
       #pragma library("CPPWOM3I.LIB")
       #pragma library("CPPWOF3I.LIB")
    #else
       #pragma library("CPPWOC3.LIB")
    #endif
  #endif
#endif

#pragma pack()

#endif /* _IEXCBASE_ */

#ifdef __MVS__
                   #pragma checkout(resume)
#endif

