/**********************************************************************
*                                                                     *
*  IBM(R) VisualAge(TM) for C++ for Windows(R), Version 3.5           *
*                                                                     *
*  PID: 5622-880                                                      *
*  - Licensed Material - Program-Property of IBM                      *
*  (C) Copyright IBM Corp. 1991, 1995 - All Right Reserved.           *
*                                                                     *
*  US Government Users Restricted Rights - Use, duplication or        *
*  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.  *
*                                                                     *
*  VisualAge, and IBM are trademarks or registered trademarks of      *
*  International Business Machines Corporation.                       *
*  Windows is a registered trademark of Microsoft Corporation.        *
*                                                                     *
**********************************************************************/

#include <iiabag.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)


#pragma SOMAsDefault (off)

class IPointer {
  IACollectionImpl* ivImpl;
public:
  IPointer (IACollectionImpl* impl)
  : ivImpl (impl) { ivImpl->Initialize (); }
 ~IPointer () { ivImpl->Delete (); }
  IACollectionImpl& operator* () { return *ivImpl; }
};

#pragma SOMAsDefault (pop)


// ---
// IABagImpl
// ---

// public members

IABagImpl::
~IABagImpl ()
{
}

void
IABagImpl::
AddAntivalence (IABagImpl const& collection1,
                IABagImpl const& collection2)
{
}

void
IABagImpl::
AddDifference (IABagImpl const& collection1,
               IABagImpl const& collection2)
{
  if (this == &collection1) {
    if (this != &collection2) {
      Union (*this);
      ICursorImpl& cursor2 = CursorOf (collection2);
      IBoolean hasNext;
      for (hasNext = collection2.SetToFirst (cursor2);
           hasNext;
           hasNext = collection2.SetToNextDifferentElement (cursor2)) {
        void const* element = collection2.ElementAt (cursor2);
        INumber n1 = NumberOfOccurrences (element) / 2,
                n2 = collection2.NumberOfOccurrences (element),
                n  = (n1 < n2 ? n1 : n2);
        while (n--) {
          Remove (element);
        }
      }
    }
  }
  else {
    ICursorImpl& cursor1 = CursorOf (collection1);
    IBoolean hasNext;
    for (hasNext = collection1.SetToFirst (cursor1);
         hasNext;
         hasNext = collection1.SetToNextDifferentElement (cursor1)) {
      void const* element = collection1.ElementAt (cursor1);
      INumber n1 = collection1.NumberOfOccurrences (element),
              n2 = collection2.NumberOfOccurrences (element),
              n  = (n1 < n2 ? 0 : n1 - n2);
      while (n--) {
        Add (element, CursorOf (*this));
      }
    }
  }
}

void
IABagImpl::
AddIntersection (IABagImpl const& collection1,
                 IABagImpl const& collection2)
{
  if (this == &collection1) {
    if (this == &collection2)
      Union (*this);
    else
      AddIntersection (collection2, collection1);
  }
  else {
    ICursorImpl& cursor2 = CursorOf (collection2);
    IBoolean hasNext;
    for (hasNext = collection2.SetToFirst (cursor2);
         hasNext;
         hasNext = collection2.SetToNextDifferentElement (cursor2)) {
      void const* element = collection2.ElementAt (cursor2);
      INumber n1 = collection1.NumberOfOccurrences (element),
              n2 = collection2.NumberOfOccurrences (element),
              n  = (n1 < n2 ? n1 : n2);
      for ( ; n != 0; n--)
        Add (element, CursorOf (*this));
    }
  }
}

void
IABagImpl::
AddUnion (IABagImpl const& collection1,
          IABagImpl const& collection2)
{
  if (this == &collection1) {
    if (this == &collection2) {
      IPointer bag (Clone ());
      AddAllFrom ((IABagImpl&)*bag);
      AddAllFrom ((IABagImpl&)*bag);
    }
    else {
      Union (collection1);
      Union (collection2);
    }
  }
  else {
    Union (collection2);
    Union (collection1);
  }
}

void
IABagImpl::
Antivalence (IABagImpl const& collection)
{
}

void
IABagImpl::
Difference (IABagImpl const& collection)
{
  if (this != &collection) {
    ICursorImpl& cursor = CursorOf (collection);
    for (collection.SetToFirst (cursor);
         cursor.IsValid ();
         collection.SetToNext (cursor)) {
      Remove (collection.ElementAt (cursor));
    }
  }
  else
    RemoveAll ();
}

void
IABagImpl::
Intersection (IABagImpl const& collection)
{ if (this != &collection) {
    IACollectionImpl const* eq = &collection;
    RemoveAll (IACollectionImpl::Predicate::IsNotContained, (void*)eq);
  }
}

void
IABagImpl::
Union (IABagImpl const& collection)
{ if (this != &collection) {
    AddAllFrom (collection);
  }
  else {
    IPointer bag (Clone ());
    AddAllFrom ((IABagImpl&)*bag);
  }
}

IBoolean
IABagImpl::
operator!= (IABagImpl const& collection) const
{ return ! operator== (collection);
}

// protected members

IABagImpl::
IABagImpl (Ops& ops)
: IACollectionImpl (ops)
{
}

#pragma info (restore)
#pragma pack ()
