/**********************************************************************
*                                                                     *
*  IBM(R) VisualAge(TM) for C++ for Windows(R), Version 3.5           *
*                                                                     *
*  PID: 5622-880                                                      *
*  - Licensed Material - Program-Property of IBM                      *
*  (C) Copyright IBM Corp. 1991, 1995 - All Right Reserved.           *
*                                                                     *
*  US Government Users Restricted Rights - Use, duplication or        *
*  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.  *
*                                                                     *
*  VisualAge, and IBM are trademarks or registered trademarks of      *
*  International Business Machines Corporation.                       *
*  Windows is a registered trademark of Microsoft Corporation.        *
*                                                                     *
**********************************************************************/

#include <iiacllct.h>
#include <iiexc.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

#if defined (__SOM_ENABLED__) && defined (__RRBC_LIB__)
#pragma SOMDefine (ICursorImpl)
#endif

// -----------
// ICursorImpl
// -----------

// public members

IBoolean
ICursorImpl::
IsMngCursor () const
{ return false;
}

// ----------------
// IACollectionImpl
// ----------------

ICollectionExceptionText
IACollectionImpl::cvExceptionText (IC_OUT_OF_COLLECTION_MEMORY);

// public members

IACollectionImpl::
~IACollectionImpl ()
{ delete ivDummyCursor;
}

void
IACollectionImpl::
AddAllFrom (IACollectionImpl const& collection)
{ ICHECK_Identical (ClassName (), "AddAllFrom")

  LockBeforeWriting ();
  ((IACollectionImpl&)collection).LockBeforeReading ();
  for (collection.SetToFirst (CursorOf (collection));
       CursorOf (collection).IsValid ();
       collection.SetToNext (CursorOf (collection))) {
    Add (collection.ElementAt (CursorOf (collection)),
         CursorOf (*this));
  }
  ((IACollectionImpl&)collection).UnlockAfterReading ();
  UnlockAfterWriting ();
}

void
IACollectionImpl::
AddAsFirst (void const*, ICursorImpl&)
{ // just a dummy
}

void
IACollectionImpl::
AddAsLast (void const*, ICursorImpl&)
{ // just a dummy
}

void
IACollectionImpl::
AddAsNext (void const*, ICursorImpl&)
{ // just a dummy
}

void
IACollectionImpl::
AddAtPosition (IPosition, void const*, ICursorImpl&)
{ // just a dummy
}

void
IACollectionImpl::
AddAsPrevious (void const*, ICursorImpl&)
{ // just a dummy
}

IBoolean
IACollectionImpl::
AddOrReplaceElementWithKey (void const*, ICursorImpl&)
{ return false; // just a dummy
}

IBoolean
IACollectionImpl::
AllElementsDo (IApplFunc applFunc, void* addArg)
{ for (SetToFirst (CursorOf (*this));
       CursorOf (*this).IsValid ();
       SetToNext (CursorOf (*this))) {
    if (! (*applFunc) (ElementAt (CursorOf (*this)), addArg))
      return false;
  }
  return true;
}

void*
IACollectionImpl::
CheckPointer (void* ptr)
{ if (ptr == 0) {
    throw IOutOfCollectionMemoryImpl (cvExceptionText);
  }
  return ptr;
}

IBoolean
IACollectionImpl::
ContainsAllFrom (IACollectionImpl const& collection) const
{ IBoolean result = true;
  ICursorImpl& cursor = CursorOf (collection);

  for (collection.SetToFirst (cursor);
       cursor.IsValid ();
       collection.SetToNext (cursor)) {
    if (! Contains (collection.ElementAt (cursor))) {
      result = false;
      break;
    }
  }
  return result;
};

IBoolean
IACollectionImpl::
ContainsAllKeysFrom (IACollectionImpl const& collection) const
{ ICursorImpl& cursor = CursorOf (collection);
  IBoolean result = true;
  for (collection.SetToFirst (cursor);
       cursor.IsValid ();
       collection.SetToNext (cursor)) {
    if (! ContainsElementWithKey
         (Key (collection.ElementAt (cursor)))) {
      result = false;
      break;
    }
  }
  return result;
}

IBoolean
IACollectionImpl::
ContainsElementWithKey (void const*) const
{ return false; // just a dummy
}

void
IACollectionImpl::
Copy (IACollectionImpl const& collection)
{ if (this != &collection) {
    RemoveAll ();
    AddAllFrom (collection);
  }
}

void
IACollectionImpl::
Delete ()
{ LockBeforeWriting ();
  if (--ivReferenceCount == 0) {
    delete ivOps; // quasimodo suicide
    return;       // ..thus, return as soon as possible
  }
  UnlockAfterWriting ();
}

void*
IACollectionImpl::
ElementAtPosition (IPosition) const
{ return 0; // just a dummy
}

void*
IACollectionImpl::
ElementWithKey (void const*) const
{ return 0; // just a dummy
}

void*
IACollectionImpl::
First () const
{ SetToFirst (CursorOf (*this));
  return ElementAt (CursorOf (*this));
}

void
IACollectionImpl::
Initialize ()
{ LockBeforeWriting ();
  if (ivReferenceCount++ == 0)
    ivDummyCursor = CreateCursor ();
  UnlockAfterWriting ();
}

IBoolean
IACollectionImpl::
IsEmpty () const
{ return (NumberOfElements () == 0);
}

IBoolean
IACollectionImpl::
IsFirstAt (ICursorImpl const& cursor) const
{ SetToFirst (CursorOf (*this));
  return (cursor == CursorOf (*this));
}

IBoolean
IACollectionImpl::
IsLastAt (ICursorImpl const& cursor) const
{ SetToLast (CursorOf (*this));
  return (cursor == CursorOf (*this));
}

void*
IACollectionImpl::
Key (void const*) const
{ return 0; // just a dummy
}

void*
IACollectionImpl::
Last () const
{ SetToLast (CursorOf (*this));
  return ElementAt (CursorOf (*this));
}

IBoolean
IACollectionImpl::
Locate (void const*, ICursorImpl&) const
{ return false; // just a dummy
}

IBoolean
IACollectionImpl::
LocateNext (void const*, ICursorImpl&) const
{ return false; // just a dummy
}

IBoolean
IACollectionImpl::
LocateElementWithKey (void const*, ICursorImpl&) const
{ return false; // just a dummy
}

IBoolean
IACollectionImpl::
LocateNextElementWithKey (void const*, ICursorImpl&) const
{ return false; // just a dummy
}

IBoolean
IACollectionImpl::
LocateOrAdd (void const*, ICursorImpl&)
{ return false; // just a dummy
}

IBoolean
IACollectionImpl::
LocateOrAddElementWithKey (void const*, ICursorImpl&)
{ return false; // just a dummy
}

void
IACollectionImpl::
LockBeforeReading (long)
{
}

void
IACollectionImpl::
LockBeforeWriting (long)
{
}

INumber
IACollectionImpl::
MaxNumberOfElements () const
{ ICHECK_NotBounded (ClassName (), "MaxNumberOfElements")
  return 0;
}

INumber
IACollectionImpl::
NumberOfDifferentKeys () const
{ return 0; // just a dummy
}

INumber
IACollectionImpl::
NumberOfElementsWithKey (void const*) const
{ return 0; // just a dummy
}

INumber
IACollectionImpl::
NumberOfOccurrences (void const*) const
{ return 0; // just a dummy
}

IPosition
IACollectionImpl::
PositionAt (ICursorImpl const&) const
{ return 0; // just a dummy
}

IBoolean
IACollectionImpl::
Remove (void const*)
{ return false; // just a dummy
}

INumber
IACollectionImpl::
RemoveAllElementsWithKey (void const*)
{ return 0; // just a dummy
}

INumber
IACollectionImpl::
RemoveAllOccurrences (void const*)
{ return 0; // just a dummy
}

void
IACollectionImpl::
RemoveAtPosition (IPosition)
{ // just a dummy
}

IBoolean
IACollectionImpl::
RemoveElementWithKey (void const*)
{ return false; // just a dummy
}

void
IACollectionImpl::
RemoveFirst ()
{ SetToFirst (CursorOf (*this));
  RemoveAt (CursorOf (*this));
}

void
IACollectionImpl::
RemoveLast ()
{ SetToLast (CursorOf (*this));
  RemoveAt (CursorOf (*this));
}

void
IACollectionImpl::
ReplaceAtPosition (IPosition position, void const* element)
{ SetToPosition (position, CursorOf (*this));
  ReplaceAt (CursorOf (*this), element);
}

IBoolean
IACollectionImpl::
ReplaceElementWithKey (void const*, ICursorImpl&)
{ return false; // just a dummy
}

void
IACollectionImpl::
Reverse ()
{ // just a dummy
}

IBoolean
IACollectionImpl::
SetToLast (ICursorImpl&) const
{ return false; // just a dummy
}

IBoolean
IACollectionImpl::
SetToNextWithDifferentKey (ICursorImpl&) const
{ return false; // just a dummy
}

void
IACollectionImpl::
SetToPosition (IPosition, ICursorImpl&) const
{ // just a dummy
}

IBoolean
IACollectionImpl::
SetToPrevious (ICursorImpl&) const
{ return false; // just a dummy
}

void
IACollectionImpl::
Sort (ICompFunc, void*)
{ // just a dummy
}

void
IACollectionImpl::
UnlockAfterReading ()
{
}

void
IACollectionImpl::
UnlockAfterWriting ()
{
}

// protected members

IACollectionImpl::
IACollectionImpl (Ops& ops)
: ivOps (&ops), ivReferenceCount (0), ivDummyCursor (0)
{
}

// ---------------------------
// IACollectionImpl::Predicate
// ---------------------------

// public members

IBoolean
IACollectionImpl::Predicate::
IsContained (void const* element, void* addArg)
{ return ((IACollectionImpl*)addArg)->Contains (element);
}

IBoolean
IACollectionImpl::Predicate::
IsNotContained (void const* element, void* addArg)
{ return ! ((IACollectionImpl*)addArg)->Contains (element);
}

// ---------------
// IACollectionOps
// ---------------

// public members

IACollectionOps::
~IACollectionOps ()
{
}

#pragma pack ()
#pragma info (restore)
