/**********************************************************************
*                                                                     *
*  IBM(R) VisualAge(TM) for C++ for Windows(R), Version 3.5           *
*                                                                     *
*  PID: 5622-880                                                      *
*  - Licensed Material - Program-Property of IBM                      *
*  (C) Copyright IBM Corp. 1991, 1995 - All Right Reserved.           *
*                                                                     *
*  US Government Users Restricted Rights - Use, duplication or        *
*  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.  *
*                                                                     *
*  VisualAge, and IBM are trademarks or registered trademarks of      *
*  International Business Machines Corporation.                       *
*  Windows is a registered trademark of Microsoft Corporation.        *
*                                                                     *
**********************************************************************/

#include <iiamap.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)


// ---
// IAMapImpl
// ---

// public members

IAMapImpl::
~IAMapImpl ()
{
}

void
IAMapImpl::
AddAntivalence (IAMapImpl const& collection1,
                IAMapImpl const& collection2)
{
}

void
IAMapImpl::
AddDifference (IAMapImpl const& collection1,
               IAMapImpl const& collection2)
{
  if (this == &collection1)
    return;
  else if (this == &collection2)
    Union (collection1);
  else {
    ICursorImpl& cursor1 = CursorOf (collection1);
    IBoolean hasNext;
    for (hasNext = collection1.SetToFirst (cursor1);
         hasNext;
         hasNext = collection1.SetToNext (cursor1)) {
      void const* element = collection1.ElementAt (cursor1);
      if (! collection2.Contains (element))
        Add (collection1.ElementAt (cursor1), CursorOf (*this));
    }
  }
}

void
IAMapImpl::
AddIntersection (IAMapImpl const& collection1,
                 IAMapImpl const& collection2)
{
  if (this != &collection1 && this != &collection2) {
    ICursorImpl& cursor1 = CursorOf (collection1);
    IBoolean hasNext;
    for (hasNext = collection1.SetToFirst (cursor1);
         hasNext;
         hasNext = collection1.SetToNext (cursor1)) {
      void const* element = collection1.ElementAt (cursor1);
      if (collection2.Contains (element)) {
        Add (element, CursorOf (*this));
      }
    }
  }
}

void
IAMapImpl::
AddUnion (IAMapImpl const& collection1,
          IAMapImpl const& collection2)
{
  Union (collection1);
  Union (collection2);
}

void
IAMapImpl::
Antivalence (IAMapImpl const& collection)
{
}

void
IAMapImpl::
Difference (IAMapImpl const& collection)
{
  if (this != &collection) {
    ICursorImpl& cursor = CursorOf (collection);
    for (collection.SetToFirst (cursor);
         cursor.IsValid ();
         collection.SetToNext (cursor)) {
      Remove (collection.ElementAt (cursor));
    }
  }
  else
    RemoveAll ();
}

void
IAMapImpl::
Intersection (IAMapImpl const& collection)
{ if (this != &collection) {
    IACollectionImpl const* eq = &collection;
    RemoveAll (IACollectionImpl::Predicate::IsNotContained, (void*)eq);
  }
}

void
IAMapImpl::
Union (IAMapImpl const& collection)
{ if (this != &collection) {
    AddAllFrom (collection);
  }
}

IBoolean
IAMapImpl::
operator!= (IAMapImpl const& collection) const
{ return ! operator== (collection);
}

// protected members

IAMapImpl::
IAMapImpl (Ops& ops)
: IACollectionImpl (ops)
{
}

IBoolean
IAMapImpl::
LocateNext (void const*, ICursorImpl& cursor) const
{ cursor.Invalidate ();
  return false;
}

IBoolean
IAMapImpl::
LocateNextElementWithKey (void const*, ICursorImpl& cursor) const
{ cursor.Invalidate ();
  return false;
}

INumber
IAMapImpl::
NumberOfDifferentKeys () const
{ return NumberOfElements ();
}

INumber
IAMapImpl::
NumberOfElementsWithKey (void const* key) const
{ if (ContainsElementWithKey (key))
    return 1;
  else
    return 0;
}

INumber
IAMapImpl::
NumberOfOccurrences (void const* element) const
{ if (Contains (element))
    return 1;
  else
    return 0;
}

INumber
IAMapImpl::
RemoveAllOccurrences (void const* element)
{ if (Remove (element))
    return 1;
  else
    return 0;
}

INumber
IAMapImpl::
RemoveAllElementsWithKey (void const* key)
{ if (RemoveElementWithKey (key))
    return 1;
  else
    return 0;
}

IBoolean
IAMapImpl::
SetToNextWithDifferentKey (ICursorImpl& cursor) const
{ return SetToNext (cursor);
}

#pragma info (restore)
#pragma pack ()
