/**********************************************************************
*                                                                     *
*  IBM(R) VisualAge(TM) for C++ for Windows(R), Version 3.5           *
*                                                                     *
*  PID: 5622-880                                                      *
*  - Licensed Material - Program-Property of IBM                      *
*  (C) Copyright IBM Corp. 1991, 1995 - All Right Reserved.           *
*                                                                     *
*  US Government Users Restricted Rights - Use, duplication or        *
*  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.  *
*                                                                     *
*  VisualAge, and IBM are trademarks or registered trademarks of      *
*  International Business Machines Corporation.                       *
*  Windows is a registered trademark of Microsoft Corporation.        *
*                                                                     *
**********************************************************************/

#include <iiexc.h>

#if defined (__LIB_BUILD__)
inline Direction avlDirection (long comp)
{ IASSERT (comp != 0);
  return comp < 0 ? kAvlRight : kAvlLeft;
}

#endif
// -------------------------------
// IAKeySortedSetAsAvlTreeNodeImpl
// -------------------------------

// public members

inline
IAKeySortedSetAsAvlTreeNodeImpl::
IAKeySortedSetAsAvlTreeNodeImpl ()
: ivLeft (0), ivRight (0), ivParent (0), ivDeeperSubtree (kAvlNone)
{
}

#if defined (__LIB_BUILD__)
inline IAKeySortedSetAsAvlTreeNodeImpl*&
IAKeySortedSetAsAvlTreeNodeImpl::
Link (Direction d)
{ IASSERT (d != kAvlNone);
  return d == kAvlLeft ? ivLeft : ivRight;
}

inline void
IAKeySortedSetAsAvlTreeNodeImpl::
Parent (IAKeySortedSetAsAvlTreeNodeImpl*& parent, Direction& d)
{ parent = ivParent;
  if (parent != 0)
    d = (parent->ivLeft == this ? kAvlLeft : kAvlRight);
}

inline void
IAKeySortedSetAsAvlTreeNodeImpl::
ConnectChild (Direction d, IAKeySortedSetAsAvlTreeNodeImpl* node)
{ IASSERT (node != 0);
  Link (d) = node;
  node->ivParent = this;
}

inline void
IAKeySortedSetAsAvlTreeNodeImpl::
ConnectChild0 (Direction d, IAKeySortedSetAsAvlTreeNodeImpl* node)
{ Link (d) = node;
  if (node != 0)
    node->ivParent = this;
}
#endif

// ------------------------------
// IKeySortedSetAsAvlTreeNodeImpl
// ------------------------------

// public members

template <class Element>
inline
IKeySortedSetAsAvlTreeNodeImpl <Element>::
IKeySortedSetAsAvlTreeNodeImpl (Element const& element)
: IAKeySortedSetAsAvlTreeNodeImpl (), ivElement (element)
{
}

template <class Element>
inline
IKeySortedSetAsAvlTreeNodeImpl <Element>::
~IKeySortedSetAsAvlTreeNodeImpl ()
{
}

// --------------------------------
// IKeySortedSetAsAvlTreeCursorImpl
// --------------------------------

// public members

inline void
IKeySortedSetAsAvlTreeCursorImpl::
Invalidate ()
{ ivNode = 0;
}

inline IBoolean
IKeySortedSetAsAvlTreeCursorImpl::
IsInBetween () const
{ return false;
}

inline IBoolean
IKeySortedSetAsAvlTreeCursorImpl::
IsValid () const
{ return (ivNode != 0);
}

inline void
IKeySortedSetAsAvlTreeCursorImpl::
operator= (IKeySortedSetAsAvlTreeCursorImpl const& cursor)
{ Inherited::operator= (cursor);
  ivNode = cursor.ivNode;
}

inline IBoolean
IKeySortedSetAsAvlTreeCursorImpl::
operator== (ICursorImpl const& cursor) const
{ return (Inherited::operator== (cursor) &&
          ivNode == ((Self const&)cursor).ivNode);
}


// --------------------------
// IKeySortedSetAsAvlTreeImpl
// --------------------------

// public members

#if defined (__LIB_BUILD__)

#if ! defined (__2nd_LEVEL__)
#define inline
#endif

inline void*
IKeySortedSetAsAvlTreeImpl::
Any () const
{ return ElementOf (ivRoot);
}

inline IBoolean
IKeySortedSetAsAvlTreeImpl::
CheckReplacement (ICursorImpl const& cursor, void const* element) const
{ return (CompareKeys (Self::ElementAt (cursor),
                       element, kElementElement) == 0);
}

inline char const*
IKeySortedSetAsAvlTreeImpl::
ClassName () const
{ return cvClassName;
}

inline IBoolean
IKeySortedSetAsAvlTreeImpl::
ContainsElementWithKey (void const* key) const
{ return Self::LocateElementWithKey (key, CursorOf (*this));
}

inline void*
IKeySortedSetAsAvlTreeImpl::
ElementAt (ICursorImpl const& cursor) const
{ return ElementOf (NodeOf (cursor));
}

inline IBoolean
IKeySortedSetAsAvlTreeImpl::
IsBounded () const
{ return false;
}

inline IBoolean
IKeySortedSetAsAvlTreeImpl::
IsEmpty () const
{ return (ivNumberOfElements == 0);
}

inline IBoolean
IKeySortedSetAsAvlTreeImpl::
IsFirstAt (ICursorImpl const& cursor) const
{ Self::SetToFirst (CursorOf (*this));
  return (NodeOf (cursor) == NodeOf (CursorOf (*this)));
}

inline IBoolean
IKeySortedSetAsAvlTreeImpl::
IsFull () const
{ return false;
}

inline IBoolean
IKeySortedSetAsAvlTreeImpl::
IsLastAt (ICursorImpl const& cursor) const
{ Self::SetToLast (CursorOf (*this));
  return (NodeOf (cursor) == NodeOf (CursorOf (*this)));
}

inline void*
IKeySortedSetAsAvlTreeImpl::
Key (void const* key) const
{ return OpsOf (*this).Key (key);
}

inline IBoolean
IKeySortedSetAsAvlTreeImpl::
LocateOrAddElementWithKey (void const* element, ICursorImpl& cursor)
{ return ! Self::Add (element, cursor);
}

inline INumber
IKeySortedSetAsAvlTreeImpl::
NumberOfElements () const
{ return ivNumberOfElements;
}

inline void
IKeySortedSetAsAvlTreeImpl::
ReplaceAt (ICursorImpl const& cursor, void const* element)
{ Assign (Self::ElementAt (cursor), element);
}

#if ! defined (__2nd_LEVEL__)
#undef inline
#endif

// protected members

inline void
IKeySortedSetAsAvlTreeImpl::
Assign (void* element1, void const* element2) const
{ OpsOf (*this).Assign (element1, element2);
}

inline long
IKeySortedSetAsAvlTreeImpl::
CompareKeys (void const* arg1, void const* arg2, IArgType argType) const
{ return OpsOf (*this).CompareKeys (arg1, arg2, argType);
}

inline void*
IKeySortedSetAsAvlTreeImpl::
CreateNode (void const* element) const
{ return OpsOf (*this).CreateNode (element);
}

inline void
IKeySortedSetAsAvlTreeImpl::
DeleteNode (void* node) const
{ OpsOf (*this).DeleteNode (node);
}

inline void*
IKeySortedSetAsAvlTreeImpl::
StreamIn (TStream& fromWhere) const
{ return OpsOf (*this).StreamIn (fromWhere);
}

inline void
IKeySortedSetAsAvlTreeImpl::
StreamOut (void const* element, TStream& toWhere) const
{ OpsOf (*this).StreamOut (element, toWhere);
}

// private members

inline void*
IKeySortedSetAsAvlTreeImpl::
ElementOf (Node const* node) const
{ typedef IKeySortedSetAsAvlTreeNodeImpl <long> TypedNode;
  return (OpsOf (*this).ivContainsDTSObjects ?
    (void*) ((TypedNode const*)node)->ivElement :
    (void*)&((TypedNode const*)node)->ivElement);
}

inline IBoolean
IKeySortedSetAsAvlTreeImpl::
IsInBetween (ICursorImpl const& cursor)
{
  return false;
}

inline IKeySortedSetAsAvlTreeImpl::Node*&
IKeySortedSetAsAvlTreeImpl::
NextNodeOf (ICursorImpl const& cursor)
{
  return ((Cursor&)cursor).ivNode; // just a dummy
}

inline IKeySortedSetAsAvlTreeImpl::Node*&
IKeySortedSetAsAvlTreeImpl::
NodeOf (ICursorImpl const& cursor)
{ return ((Cursor&)cursor).ivNode;
}

inline IKeySortedSetAsAvlTreeImpl::Ops&
IKeySortedSetAsAvlTreeImpl::
OpsOf (IKeySortedSetAsAvlTreeImpl const& collection)
{ return (Ops&) Inherited::OpsOf (collection);
}

inline IKeySortedSetAsAvlTreeImpl::Node*&
IKeySortedSetAsAvlTreeImpl::
PreviousNodeOf (ICursorImpl const& cursor)
{
  return ((Cursor&)cursor).ivNode; // just a dummy
}

#endif

// --------------------------
// IAKeySortedSetAsAvlTreeOps
// --------------------------

// public members

inline
IAKeySortedSetAsAvlTreeOps::
~IAKeySortedSetAsAvlTreeOps ()
{
}

// protected members

inline
IAKeySortedSetAsAvlTreeOps::
IAKeySortedSetAsAvlTreeOps (IBoolean containsDTSObjects)
: ivContainsDTSObjects (containsDTSObjects)
{
}
