/**********************************************************************
*                                                                     *
*  IBM(R) VisualAge(TM) for C++ for Windows(R), Version 3.5           *
*                                                                     *
*  PID: 5622-880                                                      *
*  - Licensed Material - Program-Property of IBM                      *
*  (C) Copyright IBM Corp. 1991, 1995 - All Right Reserved.           *
*                                                                     *
*  US Government Users Restricted Rights - Use, duplication or        *
*  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.  *
*                                                                     *
*  VisualAge, and IBM are trademarks or registered trademarks of      *
*  International Business Machines Corporation.                       *
*  Windows is a registered trademark of Microsoft Corporation.        *
*                                                                     *
**********************************************************************/

#include <iiexc.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

// ------------------------------------
// IKeySortedSetOnSequenceMngCursorImpl
// ------------------------------------

// public members

template <class Base>
IKeySortedSetOnSequenceMngCursorImpl <Base>::
IKeySortedSetOnSequenceMngCursorImpl
  (IKeySortedSetOnSequenceImpl <Base> const& collection)
: Inherited (IKeySortedSetOnSequenceImpl <Base>::BaseOf (collection))
{ ivCollection = &collection;
}

template <class Base>
IKeySortedSetOnSequenceMngCursorImpl <Base>::
~IKeySortedSetOnSequenceMngCursorImpl ()
{
}

// ---------------------------
// IKeySortedSetOnSequenceImpl
// ---------------------------

// public members

template <class Base>
IKeySortedSetOnSequenceImpl <Base>::
IKeySortedSetOnSequenceImpl (Ops& ops, INumber)
: IAKeySortedSetImpl ((Inherited::Ops&)ops), ivBase (ops)
{ BaseOf (*this).IACollectionImpl::Initialize ();
}

template <class Base>
IKeySortedSetOnSequenceImpl <Base>::
IKeySortedSetOnSequenceImpl
  (Ops& ops, IKeySortedSetOnSequenceImpl <Base> const& collection)
: IAKeySortedSetImpl ((Inherited::Ops&)ops), ivBase (ops)
{ BaseOf (*this).IACollectionImpl::Initialize ();
}

template <class Base>
IKeySortedSetOnSequenceImpl <Base>::
~IKeySortedSetOnSequenceImpl ()
{
}

template <class Base>
IBoolean
IKeySortedSetOnSequenceImpl <Base>::
Add (void const* element, ICursorImpl& cursor)
{ IBoolean isContained = LocateKey (Key (element), cursor);

  if (! isContained) {
    if (cursor.IsValid ())
      BaseOf (*this).Base::AddAsPrevious (element, cursor);
    else
      BaseOf (*this).Base::AddAsLast (element, cursor);
  }

  return ! isContained;
}

template <class Base>
IBoolean
IKeySortedSetOnSequenceImpl <Base>::
AddOrReplaceElementWithKey (void const* element, ICursorImpl& cursor)
{ IBoolean isContained = LocateKey (Key (element), cursor);

  if (isContained)
    BaseOf (*this).Base::ReplaceAt (cursor, element);
  else
    Self::Add (element, cursor);

  return ! isContained;
}

template <class Base>
IACollectionImpl*
IKeySortedSetOnSequenceImpl <Base>::
Clone () const
{ return (IACollectionImpl*) CheckPointer (OpsOf (*this).Clone ());
}

template <class Base>
IBoolean
IKeySortedSetOnSequenceImpl <Base>::
ContainsAllKeysFrom (IACollectionImpl const& collection) const
{ if (Self::ClassName () == collection.ClassName ()) {
    if (this != &collection) {
      Self const& kss = (Self const&)collection;
      ICursorImpl& thisCursor = CursorOf (*this),
                 & thatCursor = CursorOf (collection);

      Self::SetToFirst (thisCursor);
      for (kss.Self::SetToFirst (thatCursor);
           thatCursor.IsValid ();
           kss.Self::SetToNext (thatCursor)) {
        IBoolean found = false;
        while (! found && thisCursor.IsValid ()) {
          long comp = CompareKeys (Self::ElementAt (thisCursor),
                                   kss.Self::ElementAt (thatCursor),
                                   kElementElement);
          if (comp < 0)
            Self::SetToNext (thisCursor);
          else if (comp == 0)
            found = true;
          else
            return false;
        }
        if (! found)
          return false;
      }
    }
    return true;
  }
  else
    return Inherited::ContainsAllKeysFrom (collection);
}

template <class Base>
void
IKeySortedSetOnSequenceImpl <Base>::
Copy (IACollectionImpl const& collection)
{ if (Self::ClassName () == collection.ClassName ())
    operator= ((Self const&)collection);
  else
    Inherited::Copy (collection);
}

template <class Base>
IKeySortedSetOnSequenceImpl <Base>::Cursor*
IKeySortedSetOnSequenceImpl <Base>::
CreateCursor () const
{ return (Cursor*) CheckPointer (new Cursor (*this));
}

template <class Base>
IKeySortedSetOnSequenceImpl <Base>::MngCursor*
IKeySortedSetOnSequenceImpl <Base>::
CreateMngCursor () const
{ return (MngCursor*) CheckPointer (new MngCursor (*this));
}

template <class Base>
void*
IKeySortedSetOnSequenceImpl <Base>::
ElementWithKey (void const* key) const
{ Cursor cursor (*this);
  ICHECK_ContainsKey (ClassName (), "ElementWithKey")
  return Self::ElementAt (cursor);
}

template <class Base>
IBoolean
IKeySortedSetOnSequenceImpl <Base>::
LocateElementWithKey (void const* key, ICursorImpl& cursor) const
{ IBoolean isContained = LocateKey (key, cursor);

  if (! isContained)
    cursor.Invalidate ();

  return isContained;
}

template <class Base>
IBoolean
IKeySortedSetOnSequenceImpl <Base>::
LocateNextElementWithKey (void const* key, ICursorImpl& cursor) const
{ SetToNext (cursor);
  IBoolean isContained = LocateNextKey (key, cursor);

  if (! isContained)
    cursor.Invalidate ();

  return isContained;
}

template <class Base>
IBoolean
IKeySortedSetOnSequenceImpl <Base>::
LocateOrAddElementWithKey (void const* element, ICursorImpl& cursor)
{ IBoolean isContained = LocateKey (Key (element), cursor);

  if (! isContained)
    Self::Add (element, cursor);

  return isContained;
}

template <class Base>
IBoolean
IKeySortedSetOnSequenceImpl <Base>::
RemoveElementWithKey (void const* key)
{ IBoolean isContained = LocateKey (key, CursorOf (*this));

  if (isContained)
    Self::RemoveAt (CursorOf (*this));

  return isContained;
}

template <class Base>
IBoolean
IKeySortedSetOnSequenceImpl <Base>::
ReplaceElementWithKey (void const* element, ICursorImpl& cursor)
{ IBoolean isContained = LocateKey (Key (element), cursor);

  if (isContained)
    BaseOf (*this).Base::ReplaceAt (cursor, element);
  else
    cursor.Invalidate ();

  return isContained;
}

// private members

template <class Base>
IBoolean
IKeySortedSetOnSequenceImpl <Base>::
LocateKey (void const* key, ICursorImpl& cursor) const
{ Self::SetToFirst (cursor);
  return LocateNextKey (key, cursor);
}

template <class Base>
IBoolean
IKeySortedSetOnSequenceImpl <Base>::
LocateNextKey (void const* key, ICursorImpl& cursor) const
{ for (; cursor.IsValid (); Self::SetToNext (cursor)) {
    long comp =
      CompareKeys (Self::ElementAt (cursor), key, kElementKey);
    if (comp >= 0)
      return comp == 0;
  }
  return false;
}

#pragma info (restore)
#pragma pack ()
