/**********************************************************************
*                                                                     *
*  IBM(R) VisualAge(TM) for C++ for Windows(R), Version 3.5           *
*                                                                     *
*  PID: 5622-880                                                      *
*  - Licensed Material - Program-Property of IBM                      *
*  (C) Copyright IBM Corp. 1991, 1995 - All Right Reserved.           *
*                                                                     *
*  US Government Users Restricted Rights - Use, duplication or        *
*  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.  *
*                                                                     *
*  VisualAge, and IBM are trademarks or registered trademarks of      *
*  International Business Machines Corporation.                       *
*  Windows is a registered trademark of Microsoft Corporation.        *
*                                                                     *
**********************************************************************/

// ------------------------
// ISequenceAsTableNodeImpl
// ------------------------

// public member

template <class Element>
inline
ISequenceAsTableNodeImpl <Element>::
ISequenceAsTableNodeImpl (Element const& element)
: ivElement (element)
{
}

template <class Element>
inline
ISequenceAsTableNodeImpl <Element>::
~ISequenceAsTableNodeImpl ()
{
}

// --------------------------
// ISequenceAsTableCursorImpl
// --------------------------

// public members

inline void
ISequenceAsTableCursorImpl::
Invalidate ()
{ ivIndex = 0;
}

inline IBoolean
ISequenceAsTableCursorImpl::
IsInBetween () const
{ return false;
}

inline IBoolean
ISequenceAsTableCursorImpl::
IsValid () const
{ return (ivIndex != 0);
}

inline void
ISequenceAsTableCursorImpl::
operator= (ISequenceAsTableCursorImpl const& cursor)
{ Inherited::operator= (cursor);
  ivIndex = cursor.ivIndex;
}

inline IBoolean
ISequenceAsTableCursorImpl::
operator== (ICursorImpl const& cursor) const
{ return (Inherited::operator== (cursor) &&
          ivIndex == ((Self const&)cursor).ivIndex);
}

// --------------------
// ISequenceAsTableImpl
// --------------------

// public members

#if defined (__LIB_BUILD__)

#if ! defined (__2nd_LEVEL__)
#define inline
#endif

inline IBoolean
ISequenceAsTableImpl::
Add (void const* element, ICursorImpl& cursor)
{ Self::AddAsLast (element, (Cursor&)cursor);
  return true;
}

inline void*
ISequenceAsTableImpl::
Any () const
{ return ElementOf (ivTable.NodeAtIndex (ivTable.ivFirst));
}

inline char const*
ISequenceAsTableImpl::
ClassName () const
{ return cvClassName;
}

inline void*
ISequenceAsTableImpl::
ElementAt (ICursorImpl const& cursor) const
{ return ElementOf (ivTable.NodeAtIndex (IndexOf (cursor)));
}

inline void*
ISequenceAsTableImpl::
ElementAtPosition (IPosition position) const
{ Self::SetToPosition (position, CursorOf (*this));
  return Self::ElementAt (CursorOf (*this));
}

inline void*
ISequenceAsTableImpl::
First () const
{ return ElementOf (ivTable.NodeAtIndex (ivTable.ivFirst));
}

inline IBoolean
ISequenceAsTableImpl::
IsBounded () const
{ return false;
}

inline IBoolean
ISequenceAsTableImpl::
IsEmpty () const
{ return (ivTable.ivNumberOfElements == 0);
}

inline IBoolean
ISequenceAsTableImpl::
IsFirstAt (ICursorImpl const& cursor) const
{ return (IndexOf (cursor) == ivTable.ivFirst);
}

inline IBoolean
ISequenceAsTableImpl::
IsFull () const
{ return false;
}

inline IBoolean
ISequenceAsTableImpl::
IsLastAt (ICursorImpl const& cursor) const
{ return (IndexOf (cursor) == ivTable.ivLast);
}

inline void*
ISequenceAsTableImpl::
Last () const
{ return ElementOf (ivTable.NodeAtIndex (ivTable.ivLast));
}

inline INumber
ISequenceAsTableImpl::
NumberOfElements () const
{ return ivTable.ivNumberOfElements;
}

inline IPosition
ISequenceAsTableImpl::
PositionAt (ICursorImpl const& cursor) const
{ return ivTable.Pos (IndexOf (cursor));
}

inline void
ISequenceAsTableImpl::
RemoveAt (ICursorImpl& cursor)
{ RemoveAtIndex (IndexOf (cursor));
  if (! IsInBetween (cursor))
    cursor.Invalidate ();
}

inline void
ISequenceAsTableImpl::
RemoveFirst ()
{ RemoveAtIndex (ivTable.ivFirst);
}

inline void
ISequenceAsTableImpl::
RemoveLast ()
{ RemoveAtIndex (ivTable.ivLast);
}

inline void
ISequenceAsTableImpl::
ReplaceAt (ICursorImpl const& cursor, void const* element)
{ Assign (Self::ElementAt (cursor), element);
}

inline void
ISequenceAsTableImpl::
ReplaceAtPosition (IPosition position, void const* element)
{ Self::SetToPosition (position, CursorOf (*this));
  Self::ReplaceAt (CursorOf (*this), element);
}

inline IBoolean
ISequenceAsTableImpl::
SetToFirst (ICursorImpl& cursor) const
{ return Self::IsEmpty () ? (IndexOf (cursor) = 0) != 0 :
                            (IndexOf (cursor) = ivTable.ivFirst) != 0;
}

inline IBoolean
ISequenceAsTableImpl::
SetToLast (ICursorImpl& cursor) const
{ return Self::IsEmpty () ? (IndexOf (cursor) = 0) != 0 :
                            (IndexOf (cursor) = ivTable.ivLast) != 0;
}

inline IBoolean
ISequenceAsTableImpl::
SetToNext (ICursorImpl& cursor) const
{
    return Self::IsLastAt (cursor) ?
      (IndexOf (cursor) = 0) != 0 :
      ivTable.IncMod (IndexOf (cursor)) != 0;
}

inline void
ISequenceAsTableImpl::
SetToPosition (IPosition position, ICursorImpl& cursor) const
{ IndexOf (cursor) = ivTable.ivFirst;
  ivTable.AddMod (IndexOf (cursor), position - 1);
}

inline IBoolean
ISequenceAsTableImpl::
SetToPrevious (ICursorImpl& cursor) const
{
    return Self::IsFirstAt (cursor) ?
      (IndexOf (cursor) = 0) != 0 :
      ivTable.DecMod (IndexOf (cursor)) != 0;
}

inline void
ISequenceAsTableImpl::
Sort (ICompFunc compFunc, void* addArg)
{ if (Self::NumberOfElements () > 1)
    Quicksort (compFunc, addArg, ivTable.ivFirst, ivTable.ivLast);
}

#if ! defined (__2nd_LEVEL__)
#undef inline
#endif

// protected members

inline void
ISequenceAsTableImpl::
Assign (void* element1, void const* element2) const
{ OpsOf (*this).Assign (element1, element2);
}

inline void
ISequenceAsTableImpl::
BlockLeft (INumber target, INumber count)
{ OpsOf (*this).BlockLeft (target, count);
}

inline void
ISequenceAsTableImpl::
BlockRight (INumber target, INumber count)
{ OpsOf (*this).BlockRight (target, count);
}

inline void
ISequenceAsTableImpl::
CopyFrom (ITableImpl const& from)
{ OpsOf (*this).CopyFrom (from);
}

inline void
ISequenceAsTableImpl::
ConstructNode (void* node, void const* element) const
{ OpsOf (*this).ConstructNode (node, element);
}

inline void*
ISequenceAsTableImpl::
CreateNodes (INumber n, size_t& size)
{ return OpsOf (*this).CreateNodes (n, size);
}

inline void
ISequenceAsTableImpl::
DeleteNodes (void* nodes)
{ OpsOf (*this).DeleteNodes (nodes);
}

inline void
ISequenceAsTableImpl::
DestroyNodes (void* node, INumber count) const
{ OpsOf (*this).DestroyNodes (node, count);
}

inline void*
ISequenceAsTableImpl::
StreamIn (TStream& fromWhere) const
{ return OpsOf (*this).StreamIn (fromWhere);
}

inline void
ISequenceAsTableImpl::
StreamOut (void const* element, TStream& toWhere) const
{ OpsOf (*this).StreamOut (element, toWhere);
}

inline void
ISequenceAsTableImpl::
Swap (INumber index1, INumber index2) const
{ OpsOf (*this).Swap (index1, index2);
}

// private members

inline void*
ISequenceAsTableImpl::
ElementOf (void const* node) const
{ return (OpsOf (*this).ivContainsDTSObjects ?
    (void*)*(void const**)node :
    (void*) (void const**)node);
};

inline INumber&
ISequenceAsTableImpl::
IndexOf (ICursorImpl const& cursor)
{ return ((Cursor&)cursor).ivIndex;
}

inline IBoolean
ISequenceAsTableImpl::
IsInBetween (ICursorImpl const& cursor)
{
  return false;
}

inline INumber&
ISequenceAsTableImpl::
NextIndexOf (ICursorImpl const& cursor)
{
  return ((Cursor&)cursor).ivIndex; // just a dummy
}

inline ISequenceAsTableImpl::Ops&
ISequenceAsTableImpl::
OpsOf (ISequenceAsTableImpl const& collection)
{ return (Ops&) IACollectionImpl::OpsOf (collection);
}

inline INumber&
ISequenceAsTableImpl::
PreviousIndexOf (ICursorImpl const& cursor)
{
  return ((Cursor&)cursor).ivIndex; // just a dummy
}

#endif

// --------------------
// IASequenceAsTableOps
// --------------------

// public members

inline
IASequenceAsTableOps::
~IASequenceAsTableOps ()
{
}

// protected members

inline
IASequenceAsTableOps::
IASequenceAsTableOps (IBoolean containsDTSObjects)
: ivContainsDTSObjects (containsDTSObjects)
{
}

inline ITableImpl&
IASequenceAsTableOps::
TableOf (ISequenceAsTableImpl const& collection)
{ return ((ISequenceAsTableImpl&)collection).ivTable;
}

inline void
IASequenceAsTableOps::
UpdateMngCursorsForAssign (ISequenceAsTableImpl const& collection,
                           INumber target, INumber source) const
{ collection.UpdateMngCursorsForAssign (target, source);
}

inline void
IASequenceAsTableOps::
UpdateMngCursorsForCopyFrom (ISequenceAsTableImpl const& collection,
                             INumber fromIndex, INumber toIndex) const
{ collection.UpdateMngCursorsForCopyFrom (fromIndex, toIndex);
}

inline void
IASequenceAsTableOps::
UpdateMngCursorsForPredRemove (ISequenceAsTableImpl const& collection,
                               INumber removed, INumber previous) const
{ collection.UpdateMngCursorsForPredRemove (removed, previous);
}

inline void
IASequenceAsTableOps::
UpdateMngCursorsForRemoveOf (ISequenceAsTableImpl const& collection,
                             INumber index) const
{ collection.UpdateMngCursorsForRemoveOf (index);
}

inline void
IASequenceAsTableOps::
UpdateMngCursorsForShiftLeftOf (ISequenceAsTableImpl const& collection,
                                INumber index) const
{ collection.UpdateMngCursorsForShiftLeftOf (index);
}

inline void
IASequenceAsTableOps::
UpdateMngCursorsForShiftRightOf (ISequenceAsTableImpl const& collection,
                                 INumber index) const
{ collection.UpdateMngCursorsForShiftRightOf (index);
}

inline void
IASequenceAsTableOps::
UpdateMngCursorsForSwap (ISequenceAsTableImpl const& collection,
                         INumber index1, INumber index2) const
{ collection.UpdateMngCursorsForSwap (index1, index2);
}
