/*******************************************************************************
* FILE NAME: itime.cpp                                                         *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in itime.hpp.                                                              *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <itime.hpp>

extern "C"
  {
  #include <time.h>
  #include <stdio.h>
  #include <string.h>
#ifndef IC_MOTIF
  #define INCL_WINSTDCNR
  #include <iwindefs.h>
#endif
  }


#include <istream.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ITIME_CPP_
  #include <ipagetun.h>
#endif


#define self (*this)

/*------------------------------------------------------------------------------
| operator <<                                                                  |
|                                                                              |
|  Output the receiver asString() to the output stream.                        |
------------------------------------------------------------------------------*/
ostream& operator << ( ostream     &aStream,
                       const ITime &aTime )
   {
   return aStream << aTime.asString();
   }
/*------------------------------------------------------------------------------
| ITime::ITime                                                                 |
|                                                                              |
| Construct from a container ICnrTime stucture using its hours/minutes/seconds.|
------------------------------------------------------------------------------*/
ITime :: ITime ( const ICnrTime &cnrTime )
  {
  long secs = cnrTime.hours;
  secs *= 60;
  secs += cnrTime.minutes;
  secs *= 60;
  secs += cnrTime.seconds;
  this->initialize( secs );
  }

/*------------------------------------------------------------------------------
| ITime::asString                                                              |
|                                                                              |
| Use strftime to get ascii representation as hh:mm:ss.                        |
------------------------------------------------------------------------------*/
IString ITime :: asString ( const char *fmt ) const
  {
  IString result( 0, ( ( fmt ) ? strlen( fmt ) : 0 ) + 6 );
  struct tm dummy = { 0 };
  dummy.tm_hour = self.hours();
  dummy.tm_min  = self.minutes();
  dummy.tm_sec  = self.seconds();
  mktime( &dummy );
  while ( !strftime( (char*)result, result.length(), fmt, &dummy ) )
    result += result;
  return IString( (char*)result );
  }

/*------------------------------------------------------------------------------
| ITime::asICnrTime                                                            |
|                                                                              |
| Construct a container CTIME result structure from the receiver's             |
| hours/minutes/seconds.                                                       |
------------------------------------------------------------------------------*/
ICnrTime ITime :: asICnrTime ( ) const
  {
  ICnrTime
    result = { (unsigned char)self.hours(),
               (unsigned char)self.minutes(),
               (unsigned char)self.seconds() };
  return result;
  }

#if ( IC_OBSOLETE <= IC_OBSOLETE_2 )
#ifdef IC_PM
/*------------------------------------------------------------------------------
| ITime::asCTIME                                                               |
|                                                                              |
| Construct a container CTIME result structure from the receiver's             |
| hours/minutes/seconds.                                                       |
------------------------------------------------------------------------------*/
_CTIME ITime :: asCTIME ( ) const
  {
  CTIME
    result = { (unsigned char)self.hours(),
               (unsigned char)self.minutes(),
               (unsigned char)self.seconds() };
  return result;
  }

/*------------------------------------------------------------------------------
| ITime::ITime                                                                 |
|                                                                              |
| Construct from a container CTIME stucture using its hours/minutes/seconds.   |
------------------------------------------------------------------------------*/
ITime :: ITime ( const _CTIME &cTime )
  {
  long secs = cTime.hours;
  secs *= 60;
  secs += cTime.minutes;
  secs *= 60;
  secs += cTime.seconds;
  this->initialize( secs );
  }
#endif
#endif
