/*******************************************************************************
* FILE NAME: iddeinfo.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iddeinfo.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147481424 )

extern "C"
{
   #define INCL_WINDDE
   #define INCL_WINATOM
   #define INCL_DOSMEMMGR
   #include <iwindefs.h>
   #include <string.h>
   #ifdef IC_WIN
      #include <dde.h>            // for Win32 DDE definition
   #endif
}

#include <iddeinfo.hpp>
#include <iddeevt.hpp>
#include <itrace.hpp>
#include <iexcept.hpp>
#include <iqueue.h>
#ifdef IC_WIN
   #include <iddecomm.hpp>
   #include <ikeyset.h>
#endif

#ifdef IC_WIN
class IDDEAtomElement {
   IString       sAtomString ;
   unsigned long ulAtomId ;
   public:
      IDDEAtomElement(IString string, unsigned long atom) :
         sAtomString(string), ulAtomId(atom) {}
      ~IDDEAtomElement() {}

      IString
         const& atomString() const  { return sAtomString ; }
      unsigned long
         const& atomId()     const  { return ulAtomId ; }
}; // IDDEAtomElement

class IDDEAtomsSet : public IKeySet<IDDEAtomElement, unsigned long> {
   public:
      IDDEAtomsSet  () :
         IKeySet<IDDEAtomElement, unsigned long>(100)
         {}
      ~IDDEAtomsSet () {}
} ;
#endif

inline unsigned long const& key(IDDEAtomElement const& atomElement) {
   return atomElement.atomId() ;
}

class IDDEAckQueue : public IQueue<IDDEServerAcknowledgeEvent*> {
public:
   IDDEAckQueue ( );
  ~IDDEAckQueue ( );
}; // IDDEAckQueue

/*------------------------------------------------------------------------------
| IDdeleteElemAQ                                                               |
------------------------------------------------------------------------------*/
static Boolean
  IDdeleteElemAQ ( IDDEServerAcknowledgeEvent* const& ackEvent,
                   void*                      anything )
{
   delete (IDDEServerAcknowledgeEvent*)ackEvent;
   return true;
}

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDDEINFO_CPP_
  #include <ipagetun.h>
#endif

#ifdef IC_WIN
#pragma data_seg(ICLStaticConst)
/*------------------------------------------------------------------------------
| Static type definitions.                                                     |
|                                                                              |
| These values are defined using the values used for the PM toolkit DDE        |
| values.                                                                      |
------------------------------------------------------------------------------*/
const IDDEInfo::Status IDDEInfo::acknowledge           ( 0x0001 ),  //DDE_FACK
                       IDDEInfo::busy                  ( 0x0002 ),  //DDE_FBUSY
                       IDDEInfo::noData                ( 0x0004 ),  //DDE_FNODATA
                       IDDEInfo::acknowledgeRequested  ( 0x0008 ),  //DDE_FACKREQ
                       IDDEInfo::responseFromReq       ( 0x0010 ),  //DDE_FRESPONSE
                       IDDEInfo::notProcessed          ( 0x0020 ),  //DDE_NOTPROCESSED
                       IDDEInfo::applicationSpecific   ( 0xFF00 ),  //DDE_FAPPSTATUS
                       // newly added for Windows version, not from OS/2 PM definition
                       IDDEInfo::releaseData           ( 0x0040 );

#pragma data_seg()

/*------------------------------------------------------------------------------
| IDDEInfo__loWord                                                             |
|                                                                              |
| Extract the low word of the packed DDE lParam.                               |
------------------------------------------------------------------------------*/
UINT IDDEInfo__loWord ( UINT msgId, LPARAM lParam )
{
   IMODTRACE_DEVELOP("IDDEInfo::loWord");
   //ITRACE_ALL(IString("Message Id is ") + IString(msgId));
   //ITRACE_ALL(IString("Parameter is ") + IString(lParam));
   IASSERT(msgId!=0);
   IASSERT(lParam!=0);

   UINT uLo,
        uHi;
   if ( UnpackDDElParam( msgId, lParam, &uLo, &uHi ) )
   {
      return uLo;
   }

   return (UINT)NULL;
}

/*------------------------------------------------------------------------------
| IDDEInfo__hiWord                                                             |
|                                                                              |
| Extract the high word of the packed DDE lParam.                              |
------------------------------------------------------------------------------*/
UINT IDDEInfo__hiWord ( UINT msgId, LPARAM lParam )
{
   IMODTRACE_DEVELOP("IDDEInfo::hiWord");
   //ITRACE_ALL(IString("Message Id is ") + IString(msgId));
   //ITRACE_ALL(IString("Parameter is ") + IString(lParam));
   IASSERT(msgId!=0);
   IASSERT(lParam!=0);

   UINT uLo,
        uHi;
   if ( UnpackDDElParam( msgId, lParam, &uLo, &uHi ) )
   {
      return uHi;
   }

   return (UINT)NULL;
}
#endif  /* IC_WIN */

/*------------------------------------------------------------------------------
| IDDEInfo__stringFromAtom                                                     |
|                                                                              |
| Query the system atom table for the string name of an atom.                  |
------------------------------------------------------------------------------*/
IString IDDEInfo__stringFromAtom ( unsigned long atom )
{
   IMODTRACE_DEVELOP("IDDEInfo::stringFromAtom");
   IASSERT(atom != 0);
   char szBuf[256];
   memset(szBuf, 0, 256) ;

#ifdef IC_PM
   unsigned long rc = WinQueryAtomName(WinQuerySystemAtomTable(),
                                       (ATOM)atom,
                                       (PSZ)szBuf, 256);
   ITRACE_ALL(IString("Atom string is ") + IString(szBuf));
   if (!rc)
      ITHROWGUIERROR2("WinQueryAtomName",IBaseErrorInfo::invalidParameter,
          IException::recoverable);
#endif  /* IC_PM */
#ifdef IC_WIN

   //ITRACE_ALL(IString("Atom string is ") + IString(szBuf));
   UINT uLen(0) ;

   try {
      IDDEAtomElement anAtom((IDDEStatics::setOfAtoms())->elementWithKey(atom)) ;
      IString sTemp(anAtom.atomString()) ;
      memcpy(szBuf, (char *) sTemp, sTemp.length())  ;
      uLen   = 1 ;
   }
   catch (INotContainsKeyException exc) {
      uLen = GlobalGetAtomName( (ATOM)atom, szBuf, 256 );
      if (uLen > 0)
         (IDDEStatics::setOfAtoms())->add(IDDEAtomElement(IString(szBuf),
                                                          atom) ) ;
   }
   catch(...) {
      // ignore
   }

   if ( uLen == 0 )
      ITHROWGUIERROR2("GlobalGetAtomName",IBaseErrorInfo::invalidParameter,
                      IException::recoverable);

#endif  /* IC_WIN */

   return szBuf;
}

/*------------------------------------------------------------------------------
| IDDEInfo__atomFromString                                                     |
|                                                                              |
| Query the system atom table for the atom from a string name.                 |
------------------------------------------------------------------------------*/
unsigned long IDDEInfo__atomFromString ( const char* atomAsString,
                                         Boolean*    atomAdded )
{
   IMODTRACE_DEVELOP("IDDEInfo::atomFromString");
   IASSERT(atomAsString!=0);

#ifdef IC_PM
   unsigned long ulAtom = WinFindAtom(WinQuerySystemAtomTable(),
                                      (PSZ)atomAsString);
#endif
#ifdef IC_WIN
   unsigned long ulAtom = (unsigned long)GlobalFindAtom( (LPCTSTR)atomAsString );
#endif

   if (!ulAtom)
   {
      *atomAdded = true;

#ifdef IC_PM
      ulAtom = WinAddAtom(WinQuerySystemAtomTable(),
                          (PSZ)atomAsString);
      ITRACE_ALL(IString("Atom is ") + IString(ulAtom));
      if (!ulAtom)
         ITHROWGUIERROR2("WinAddAtom",IBaseErrorInfo::invalidParameter,
            IException::recoverable);
#endif   /* IC_PM */
#ifdef IC_WIN
      ulAtom = (unsigned long)GlobalAddAtom( (LPCTSTR)atomAsString );
      if (!ulAtom)
         ITHROWGUIERROR2("GlobalAddAtom",IBaseErrorInfo::invalidParameter,
                         IException::recoverable);
#endif   /* IC_WIN */

   }
   else
      *atomAdded = false;

#ifdef ICWIN
   (IDDEStatics::setOfAtoms())->add(IDDEAtomElement(IString(atomAsString),
                                                    ulAtom) ) ;
#endif

   return ulAtom;
}

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IDDEInfo__formatFromString                                                   |
|                                                                              |
| Obtain the data format Id from a format string name.                         |
------------------------------------------------------------------------------*/
UINT IDDEInfo__formatFromString ( const char* formatAsString )
{
   IMODTRACE_DEVELOP("IDDEInfo::formatFromString");
   //ITRACE_DEVELOP("Format string is " + IString(formatAsString));
   IASSERT(formatAsString!=0);

   IString strFormat( formatAsString );

   if ( strFormat == IDDE::textFormat )
      return CF_TEXT;
   if ( strFormat == IDDE::bitmapFormat )
      return (UINT)CF_BITMAP;
   if ( strFormat == IDDE::metafilePictureFormat )
      return (UINT)CF_METAFILEPICT;
   if ( strFormat == IDDE::sylkFormat )
      return (UINT)CF_SYLK;
   if ( strFormat == IDDE::difFormat )
      return (UINT)CF_DIF;
   if ( strFormat == IDDE::tiffFormat )
      return (UINT)CF_TIFF;
   if ( strFormat == IDDE::oemTextFormat )
      return (UINT)CF_OEMTEXT;
   if ( strFormat == IDDE::dibFormat )
      return (UINT)CF_DIB;
   if ( strFormat == IDDE::paletteFormat )
      return (UINT)CF_PALETTE;
   if ( strFormat == IDDE::metafileFormat )
      return (UINT)CF_ENHMETAFILE;
   if ( strFormat == IDDE::displayTextFormat )
      return (UINT)CF_DSPTEXT;
   if ( strFormat == IDDE::displayBitmapFormat )
      return (UINT)CF_DSPBITMAP;
   if ( strFormat == IDDE::displayMetafileFormat )
      return (UINT)CF_DSPENHMETAFILE;
   if ( strFormat == IDDE::displayMetafilePictureFormat )
      return (UINT)CF_DSPMETAFILEPICT;

   if (0 == strcmp(formatAsString, "CF_PENDATA"))
      return (UINT)CF_PENDATA ;

   if (0 == strcmp(formatAsString, "CF_RIFF"))
      return (UINT)CF_RIFF    ;

   if (0 == strcmp(formatAsString, "CF_WAVE"))
      return (UINT)CF_WAVE    ;

   if (0 == strcmp(formatAsString, "CF_UNICODETEXT"))
      return (UINT)CF_UNICODETEXT  ;

   if (0 == strcmp(formatAsString, "CF_GDIOBJFIRST"))
      return (UINT)CF_GDIOBJFIRST  ;

   if (0 == strcmp(formatAsString, "CF_GDIOBJLAST"))
      return (UINT)CF_GDIOBJLAST   ;

   if (0 == strcmp(formatAsString, "CF_OWNERDISPLAY"))
      return (UINT)CF_OWNERDISPLAY ;

   if (0 == strcmp(formatAsString, "CF_PRIVATEFIRST"))
      return (UINT)CF_PRIVATEFIRST ;

   if (0 == strcmp(formatAsString, "CF_PRIVATELAST"))
      return (UINT)CF_PRIVATELAST  ;

   if (0 == strcmp(formatAsString, "CF_HDROP"))
      return (UINT)CF_HDROP        ;

   if (0 == strcmp(formatAsString, "CF_LOCALE"))
      return (UINT)CF_LOCALE       ;

   // cannot identify format type
   return RegisterClipboardFormat(formatAsString) ;
}

/*------------------------------------------------------------------------------
| IDDEInfo__freeAtom                                                           |
------------------------------------------------------------------------------*/
void IDDEInfo__freeAtom ( ATOM aAtom )
{
   IMODTRACE_DEVELOP("IDDEInfo::freeAtom");
   ATOM aAtomRet = 0;
   //ITRACE_DEVELOP("::freeAtom " + IString(aAtom)) ;
   aAtomRet = GlobalDeleteAtom( aAtom );
   if ( aAtomRet )
      ITHROWSYSTEMERROR( GetLastError(),
                         "GlobalDeleteAtom",IBaseErrorInfo::accessError,
                         IException::recoverable);
}
#endif  /* IC_WIN */

/*------------------------------------------------------------------------------
| IDDEInfo__freeMemory                                                         |
------------------------------------------------------------------------------*/
#ifdef IC_PM
void IDDEInfo__freeMemory ( _DDESTRUCT* ddeInfo )
{
   IMODTRACE_DEVELOP("IDDEInfo ::freeMemory");
   unsigned long rc = 0;
   rc = DosFreeMem((PVOID)ddeInfo);
   if (rc)
      ITHROWSYSTEMERROR(rc,"DosFreeMem",IBaseErrorInfo::accessError,
        IException::recoverable);
}
#endif
#ifdef IC_WIN
void IDDEInfo__freeMemory ( HGLOBAL hDdeStruct )
{
   IMODTRACE_DEVELOP("IDDEInfo::freeMemory");
   // ITRACE_DEVELOP("freeing location " + IString((UINT) hDdeStruct)) ;
   HGLOBAL hMem = 0;
   hMem = GlobalFree( hDdeStruct );
   if ( hMem != NULL )
      ITHROWSYSTEMERROR( GetLastError(),
                         "GlobalFree",IBaseErrorInfo::accessError,
                         IException::recoverable);
}
#endif

/*------------------------------------------------------------------------------
| IDDEInfo__buildDDEStruct                                                     |
|                                                                              |
| Build a DDE structure in shared memory, making the object giveable.          |
------------------------------------------------------------------------------*/
#ifdef IC_PM
_DDESTRUCT* IDDEInfo__buildDDEStruct ( const char* itemName,
                                       const char* dataFormat,
                                       unsigned short status,
                                       const void* xferData,
                                       unsigned long dataLength,
                                       Boolean* atomAdded )
{
   IMODTRACE_DEVELOP("IDDEInfo::buildDDEStruct");
   //ITRACE_ALL(IString("Item is ") + IString(itemName));
   //ITRACE_ALL(IString("Format is ") + IString(dataFormat));
   //ITRACE_ALL(IString("Status is ") + IString(status));
   //ITRACE_ALL(IString("Data is ") + IString(xferData, dataLength));
   PDDESTRUCT pddes;
   void* pbuf;
   unsigned long rc = 0;
   unsigned long ulItemLen = 1;
   if (itemName != 0)
      ulItemLen = strlen(itemName) + 1;
//  Allocate the shared memory
   rc = DosAllocSharedMem(&pbuf,
                          0,
                          sizeof(DDESTRUCT) + ulItemLen + dataLength,
                          PAG_COMMIT | PAG_READ | PAG_WRITE | OBJ_GIVEABLE);
   if (rc)
      ITHROWSYSTEMERROR(rc,"DosAllocSharedMem",IBaseErrorInfo::outOfMemory,
         IException::recoverable);
// Fill in the new DDE structure
   pddes = (PDDESTRUCT)pbuf;
   pddes->cbData = dataLength;
   pddes->fsStatus = status;
   if (dataFormat != 0 && *dataFormat != '\0')
      pddes->usFormat = IDDEInfo__atomFromString(dataFormat, atomAdded);
   else
      pddes->usFormat = 0;
   pddes->offszItemName = sizeof(DDESTRUCT);
   pddes->offabData = sizeof(DDESTRUCT) + ulItemLen;
// copy the item name
   if (itemName !=0)
      strcpy(DDES_PSZITEMNAME(pddes), itemName);
   else
      strcpy(DDES_PSZITEMNAME(pddes), "");
// copy the data
   if (xferData !=0 && dataLength != 0)
      memcpy(DDES_PABDATA(pddes), xferData, dataLength);
   return pddes;
}
#endif  /* IC_PM */
#ifdef IC_WIN
UINT IDDEInfo__buildDDEAck ( unsigned short status )
{
   IMODTRACE_DEVELOP("IDDEInfo::buildDDEAck") ;
   UINT buffer(0) ;
   DDEACK* lpDdeAck = (DDEACK *) &buffer ;

   // Fill in the struct data
   ( status & IDDEInfo::busy )        ? (lpDdeAck->fBusy = true)
                                      : (lpDdeAck->fBusy = false);
   ( status & IDDEInfo::acknowledge ) ? (lpDdeAck->fAck = true)
                                      : (lpDdeAck->fAck = false);

   // set the application specific data
   if ( (status & IDDEInfo::applicationSpecific) == 0 )
      lpDdeAck->bAppReturnCode = 0;
   else
   lpDdeAck->bAppReturnCode = (unsigned short)( status >> 8 );

   return buffer ;
} /* IDDEInfo__buildDDEAck */

HGLOBAL IDDEInfo__buildDDEStruct ( int            ddeStructType,
                                   const char    *dataFormat,
                                   unsigned short status,
                                   const void    *xferData,
                                   unsigned long  dataLength,
                                   Boolean       *atomAdded )
{
   IMODTRACE_DEVELOP("IDDEInfo::buildDDEStruct");

   HGLOBAL hDdeShareMem = 0;

   switch ( ddeStructType )
   {
      // create DDEDATA and DDEPOKE struct
      case IDDEData:
         {
         //  Allocate the shared memory
         int iDDEStructSize(sizeof(DDEDATA) + dataLength) ;
         hDdeShareMem = GlobalAlloc( GHND | GMEM_DDESHARE,
                                      iDDEStructSize );
         if ( hDdeShareMem == NULL )
            ITHROWSYSTEMERROR(GetLastError(),"GlobalAlloc",
                              IBaseErrorInfo::outOfMemory, IException::recoverable);

         DDEDATA *lpDdeData = (DDEDATA *)GlobalLock( hDdeShareMem );
         if ( lpDdeData == NULL )
            ITHROWSYSTEMERROR(GetLastError(),"GlobalLock",
                              IBaseErrorInfo::accessError, IException::recoverable);

         // Fill in the struct data
             // only DDEDATA fResponse means FACKREQ
          memset(lpDdeData, 0, iDDEStructSize) ;
         ( status & IDDEInfo::acknowledgeRequested ) ? lpDdeData->fAckReq = true
                                                     : lpDdeData->fAckReq = false ;

         ( status & IDDEInfo::releaseData )          ? lpDdeData->fRelease = true
                                                     : lpDdeData->fRelease = false;

         ( status & IDDEInfo::responseFromReq )      ? lpDdeData->fResponse = true
                                                     : lpDdeData->fResponse = false ;

         // set the data format
         if (dataFormat != 0 && *dataFormat != '\0')
            lpDdeData->cfFormat = IDDEInfo__formatFromString( dataFormat );
         else
            lpDdeData->cfFormat = 0;

         // copy the data
         if (xferData !=0 && dataLength != 0) {
            //  lstrcpy( (LPSTR)lpDdeData->Value, (LPCSTR)xferData );
             memcpy(&(lpDdeData->Value),  xferData, dataLength) ;
         }

         GlobalUnlock( hDdeShareMem );
         }
         break;

      case IDDEPoke:
         {
         //  Allocate the shared memory
         int iDDEStructSize(sizeof(DDEPOKE) + dataLength) ;
         hDdeShareMem = GlobalAlloc( GHND | GMEM_DDESHARE,
                                      iDDEStructSize );
         if ( hDdeShareMem == NULL )
            ITHROWSYSTEMERROR(GetLastError(),"GlobalAlloc",
                              IBaseErrorInfo::outOfMemory, IException::recoverable);

         DDEPOKE *lpDdeData = (DDEPOKE *)GlobalLock( hDdeShareMem );
         if ( lpDdeData == NULL )
            ITHROWSYSTEMERROR(GetLastError(),"GlobalLock",
                           IBaseErrorInfo::accessError, IException::recoverable);

         // Fill in the struct data
          memset(lpDdeData, 0, iDDEStructSize) ;
         ( status & IDDEInfo::releaseData ) ? (lpDdeData->fRelease = true)
                                            : (lpDdeData->fRelease = false);

         // set the data format
         if (dataFormat != 0 && *dataFormat != '\0')
            lpDdeData->cfFormat = IDDEInfo__formatFromString( dataFormat );
         else
            lpDdeData->cfFormat = 0;

         // copy the data
         if (xferData !=0 && dataLength != 0) {
            //  lstrcpy( (LPSTR)lpDdeData->Value, (LPCSTR)xferData );
             memcpy(&(lpDdeData->Value),  xferData, dataLength) ;
         }

         GlobalUnlock( hDdeShareMem );
         }
         break;

      // create DDEADVISE struct
      case IDDEAdvise:
         {
         // Allocate the shared memory
         hDdeShareMem = GlobalAlloc( GHND | GMEM_DDESHARE,
                                     sizeof( DDEADVISE )   );
         if ( hDdeShareMem == NULL )
            ITHROWSYSTEMERROR(GetLastError(),"GlobalAlloc",
                              IBaseErrorInfo::outOfMemory, IException::recoverable);

         DDEADVISE  *lpDdeAdvise = (DDEADVISE *)GlobalLock( hDdeShareMem );
         if ( lpDdeAdvise == NULL )
            ITHROWSYSTEMERROR(GetLastError(),"GlobalLock",
                              IBaseErrorInfo::accessError, IException::recoverable);

         // Fill in the struct data
         ( status & IDDEInfo::acknowledgeRequested ) ? (lpDdeAdvise->fAckReq = true)
                                                     : (lpDdeAdvise->fAckReq = false);
         ( status & IDDEInfo::noData )               ? (lpDdeAdvise->fDeferUpd = true)
                                                     : (lpDdeAdvise->fDeferUpd = false);

         // set the data format
         if (dataFormat != 0 && *dataFormat != '\0')
            lpDdeAdvise->cfFormat = IDDEInfo__formatFromString( dataFormat );
         else
            lpDdeAdvise->cfFormat = 0;

         GlobalUnlock( hDdeShareMem );
         }
         break;

      // create DDEACK struct
      case IDDEAck:
         {
         // Allocate the shared memory
         hDdeShareMem = GlobalAlloc( GHND | GMEM_DDESHARE,
                                     sizeof( DDEACK )   );
         if ( hDdeShareMem == NULL )
            ITHROWSYSTEMERROR(GetLastError(),"GlobalAlloc",
                              IBaseErrorInfo::outOfMemory, IException::recoverable);

//       DDEACK FAR *lpDdeAck = (DDEACK FAR *)GlobalLock( hDdeShareMem );
         DDEACK *lpDdeAck = (DDEACK *)GlobalLock( hDdeShareMem );
         if ( lpDdeAck == NULL )
            ITHROWSYSTEMERROR(GetLastError(),"GlobalLock",
                              IBaseErrorInfo::accessError, IException::recoverable);

         // Fill in the struct data
         ( status & IDDEInfo::busy )        ? (lpDdeAck->fBusy = true)
                                            : (lpDdeAck->fBusy = false);
         ( status & IDDEInfo::acknowledge ) ? (lpDdeAck->fAck = true)
                                            : (lpDdeAck->fAck = false);

         // set the application specific data
         if ( (status & IDDEInfo::applicationSpecific) == 0 )
            lpDdeAck->bAppReturnCode = 0;
         else
            lpDdeAck->bAppReturnCode = (unsigned short)( status >> 8 );

         GlobalUnlock( hDdeShareMem );
         }
         break;

      // create share memory for commands
      case IDDECommands:
         {
         // Allocate the shared memory
         int iDDEStructSize = dataLength + 1 ;
         hDdeShareMem = GlobalAlloc( GHND | GMEM_DDESHARE,
                                     iDDEStructSize);
         if ( hDdeShareMem == NULL )
            ITHROWSYSTEMERROR( GetLastError(), "GlobalAlloc",
                               IBaseErrorInfo::outOfMemory, IException::recoverable);

         char  *lpDdeCmd = (char *)GlobalLock( hDdeShareMem );
         if ( lpDdeCmd == NULL )
            ITHROWSYSTEMERROR( GetLastError(), "GlobalLock",
                               IBaseErrorInfo::accessError, IException::recoverable);

         // Fill in the commands
         memset(lpDdeCmd, 0, iDDEStructSize) ;
         memcpy( lpDdeCmd, xferData, dataLength );

         GlobalUnlock( hDdeShareMem );
         }
         break;
   }

   return hDdeShareMem;
}
#endif  /* IC_WIN */

/*------------------------------------------------------------------------------
| IDDEFormat::IDDEFormat                                                       |
------------------------------------------------------------------------------*/
IDDEFormat :: IDDEFormat ( unsigned long ddeFormat )
    : ulClFormat(ddeFormat), pNextCl(0)
{
   ;
}

/*------------------------------------------------------------------------------
| IDDEFormat::~IDDEFormat                                                      |
|                                                                              |
| Destructor for IDDEFormat. Remove any formats added from the system atom     |
| table.                                                                       |
------------------------------------------------------------------------------*/
IDDEFormat :: ~IDDEFormat ( )
{
#ifdef IC_PM
   if (pNextCl)
      delete pNextCl;
   ATOM atom = WinDeleteAtom(WinQuerySystemAtomTable(), ulClFormat);
#endif
}

/*------------------------------------------------------------------------------
| IDDEFormatSet::IDDEFormatSet                                                 |
------------------------------------------------------------------------------*/
IDDEFormatSet :: IDDEFormatSet ( )
      : pFirstCl(0)
{
   ;
}

/*------------------------------------------------------------------------------
| IDDEFormatSet::~IDDEFormatSet                                                |
------------------------------------------------------------------------------*/
IDDEFormatSet :: ~IDDEFormatSet ( )
{
   if (pFirstCl)
      delete pFirstCl;
}

/*------------------------------------------------------------------------------
| IDDEFormatSet::add                                                           |
------------------------------------------------------------------------------*/
void IDDEFormatSet :: add ( unsigned long ddeFormat )
{
   if (!pFirstCl)
      pFirstCl = new IDDEFormat(ddeFormat);
   else {
      IDDEFormat *pNext = pFirstCl,
                 *pCurrent;
      while ( pNext ) {
         pCurrent = pNext;
         pNext = pNext->pNextCl;
      }
      pCurrent->pNextCl = new IDDEFormat( ddeFormat );
   }
}

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IDDEItemAtom::IDDEItemAtom                                                   |
------------------------------------------------------------------------------*/
IDDEItemAtom :: IDDEItemAtom ( unsigned long ddeItem )
                 : ulClItemAtom(ddeItem), pNextCl(0)
{
   ;
}

/*------------------------------------------------------------------------------
| IDDEItemAtom::~IDDEItemAtom                                                  |
|                                                                              |
| Destructor for IDDEItemAtom.                                                 |
------------------------------------------------------------------------------*/
IDDEItemAtom :: ~IDDEItemAtom ( )
{
   IMODTRACE_DEVELOP("IDDEItemAtom::~IDDEItemAtom") ;
   if (pNextCl)
      delete pNextCl;
   ATOM atom = GlobalDeleteAtom( ulClItemAtom );
}

/*------------------------------------------------------------------------------
| IDDEItemAtomSet::IDDEItemAtomSet                                             |
------------------------------------------------------------------------------*/
IDDEItemAtomSet :: IDDEItemAtomSet ( )
      : pFirstCl(0)
{
   ;
}

/*------------------------------------------------------------------------------
| IDDEItemAtomSet::~IDDEItemAtomSet                                            |
------------------------------------------------------------------------------*/
IDDEItemAtomSet :: ~IDDEItemAtomSet ( )
{
   if (pFirstCl)
      delete pFirstCl;
}

/*------------------------------------------------------------------------------
| IDDEItemAtomSet::add                                                         |
------------------------------------------------------------------------------*/
void IDDEItemAtomSet :: add ( unsigned long ddeItem )
{
   if (!pFirstCl)
      pFirstCl = new IDDEItemAtom(ddeItem);
   else {
      IDDEItemAtom *pNext = pFirstCl,
                 *pCurrent;
      while ( pNext ) {
         pCurrent = pNext;
         pNext = pNext->pNextCl;
      }
      pCurrent->pNextCl = new IDDEItemAtom( ddeItem );
   }
}
#endif  /* IC_WIN */

/*------------------------------------------------------------------------------
| IDDEAckQueue::IDDEAckQueue                                                   |
------------------------------------------------------------------------------*/
IDDEAckQueue :: IDDEAckQueue ( )
      : IQueue<IDDEServerAcknowledgeEvent*>(3)
{
   ;
}

/*------------------------------------------------------------------------------
| IDDEAckQueue::~IDDEAckQueue                                                  |
------------------------------------------------------------------------------*/
IDDEAckQueue :: ~IDDEAckQueue ( )
{
   ;
}

/*------------------------------------------------------------------------------
| IDDEServerConversation::~IDDEServerConversation                              |
------------------------------------------------------------------------------*/
IDDEServerConversation :: ~IDDEServerConversation ( )
{
   if (pAckQCl)
   {
      pAckQCl->allElementsDo(&IDdeleteElemAQ);
      pAckQCl->removeAll();
      delete pAckQCl;
   }
}

/*------------------------------------------------------------------------------
| IDDEServerConversation::eventCount                                           |
------------------------------------------------------------------------------*/
unsigned long IDDEServerConversation :: eventCount ( ) const
{
   if (pAckQCl)
      return (pAckQCl->numberOfElements());
   return 0;
}

/*------------------------------------------------------------------------------
| IDDEServerConversation::isAckOutstanding                                     |
------------------------------------------------------------------------------*/
Boolean IDDEServerConversation :: isAckOutstanding ( IString item,
                                                     IString format ) const
{
   IMODTRACE_DEVELOP("IDDEServerConversation::isAckOutstanding");
   Boolean bFound = false;
   if (pAckQCl)
   {
      //ITRACE_DEVELOP("Number of elements in Q is: " +
      //    IString(pAckQCl->numberOfElements()));
      IDDEServerAcknowledgeEvent* pEvt;
      IDDEAckQueue::Cursor myCurs(*pAckQCl);
      IString inItem = IString::upperCase(item);
      IString inFormat = IString::upperCase(format);
      IString qItem;
      IString qFormat;
      forCursor(myCurs)
      {
         pEvt = pAckQCl->elementAt(myCurs);
         qItem = IString::upperCase(pEvt->item());
         qFormat = IString::upperCase(pEvt->format());
         if (inItem == qItem && inFormat == qFormat)
            bFound = true;
      }
   }
   //else
   //   ITRACE_DEVELOP("Number of elements in Q is: 0");
   return bFound;
}

/*------------------------------------------------------------------------------
| IDDEServerConversation::removeEvent                                          |
------------------------------------------------------------------------------*/
IDDEServerAcknowledgeEvent* IDDEServerConversation :: removeEvent ( )
{
   IMODTRACE_DEVELOP("IDDEServerConversation::removeEvent");
   if (pAckQCl)
   {
      const IDDEServerAcknowledgeEvent* pEvt;
      pEvt = pAckQCl->firstElement();
      pAckQCl->removeFirst();
      Boolean bEmpty = pAckQCl->isEmpty();
      if (bEmpty)
      {
         //ITRACE_DEVELOP("Number of elements in Q is: 0");
         delete pAckQCl;
         pAckQCl = 0;
      }
      //else
      //   ITRACE_DEVELOP("Number of elements in Q is: " +
      //      IString(pAckQCl->numberOfElements()));
      return (IDDEServerAcknowledgeEvent*)pEvt;
   }
   return 0;
}

/*------------------------------------------------------------------------------
| IDDEServerConversation::addEvent                                             |
------------------------------------------------------------------------------*/
void IDDEServerConversation :: addEvent ( IDDEServerAcknowledgeEvent* pEvt)
{
   IMODTRACE_DEVELOP("IDDEServerConversation::addEvent");
   if (pAckQCl == 0)
      pAckQCl = new IDDEAckQueue;
   pAckQCl->add(pEvt);
   //ITRACE_DEVELOP("Number of elements in Q is: " +
   //   IString(pAckQCl->numberOfElements()));
}


/*------------------------------------------------------------------------------
| IDDEStatics::semaphor                                                        |
------------------------------------------------------------------------------*/
IPrivateResource& IDDEStatics :: semaphor ( )
{
   if (pPriResCl == 0)
   {
      pPriResCl = new IPrivateResource();
   }
   return *pPriResCl;
}

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IDDEStatics::setOfAtoms                                                      |
------------------------------------------------------------------------------*/
IDDEAtomsSet* IDDEStatics :: setOfAtoms ( )
{
   if (pSetOfAtoms == 0)
      pSetOfAtoms = new IDDEAtomsSet() ;
   return pSetOfAtoms;
}
#endif

/*------------------------------------------------------------------------------
| IDDEStatics::~IDDEStatics                                                    |
------------------------------------------------------------------------------*/
IDDEStatics :: ~IDDEStatics ( )
{
   if (pPriResCl)
      delete pPriResCl;
#ifdef IC_WIN
   if (pSetOfAtoms)
      delete pSetOfAtoms ;
#endif
}
