/* NOSHIP */
/*****************************************************************************
 *
 * Class: CAswAsync
 *
 * (C) Copyright Lotus Development Coporation 1994, 1996. All Rights Reserved.
 *
 * Description:
 *    Aswan Asynchronous method class
 *
 *****************************************************************************/

#ifndef ASASYNC_H
#define ASASYNC_H
#include <aspack.h>
#define WM_ASWAN_USER            WM_USER + 1000
#define WM_ASWAN_ONVIEWCHANGE    WM_ASWAN_USER + 1
#define WM_ASWAN_ONDATACHANGE    WM_ASWAN_USER + 2
#define WM_ASWAN_ONRENAME        WM_ASWAN_USER + 3
#define WM_ASWAN_ONSAVE          WM_ASWAN_USER + 4
class ASWPTR CAswAsyncData : public CAswMemoryBase
{
public:
   inline CAswAsyncData( PASWDATAMANAGER  pDataMgr,
                         LPFORMATETC      pFormatEtc,
                         LPSTGMEDIUM       )
      : m_pDataMgr( pDataMgr )
      , m_pMedium( NULL )
   {
      ASWGLOBALASSERT( pDataMgr );
      ASWGLOBALASSERT( pFormatEtc );
      m_fmtetc = *pFormatEtc;
   }
#ifndef __DEBUG_ALLOC__   
   inline PASWVOID  operator new( size_t cb )
#else
   inline PASWVOID  operator new( size_t cb, const char* file, size_t line )
#endif
   {
      return ::OleStdMalloc( cb );
   }
#ifndef __DEBUG_ALLOC__   
   inline void  operator delete( PASWVOID p )
#else
   inline void  operator delete( PASWVOID p, const char*, size_t )
#endif
   {
      if ( p )
         ::OleStdFree( p );
   }
   void SendOnDataChange( void );
private:
   PASWDATAMANAGER  m_pDataMgr;
   FORMATETC        m_fmtetc;
   LPSTGMEDIUM      m_pMedium;
};
class ASWPTR CAswAsyncRename : public CAswMemoryBase
{
public:
   inline CAswAsyncRename( PASWDATAMANAGER  pDataMgr,
                           LPMONIKER        pMoniker )
      : m_pDataMgr( pDataMgr )
      , m_pMoniker( pMoniker )
   {
      ASWGLOBALASSERT( pDataMgr );
      ASWGLOBALASSERT( ISRPTROKS( pMoniker ) );
      if (m_pMoniker)
         m_pMoniker->AddRef();
   }
   inline ~CAswAsyncRename( void )
   {
      if (m_pMoniker)
         m_pMoniker->Release();
   }
#ifndef __DEBUG_ALLOC__
   inline PASWVOID operator new( size_t cb )
#else   
   inline PASWVOID operator new( size_t cb, const char*, size_t )
#endif
   {
      return ::OleStdMalloc( cb );
   }
#ifndef __DEBUG_ALLOC__
   inline void operator delete( PASWVOID p )
#else
   inline void operator delete( PASWVOID p, const char*, size_t )
#endif
   {
      if ( p )
         ::OleStdFree( p );
   }
   void SendOnRename( void );
private:
   PASWDATAMANAGER   m_pDataMgr;
   LPMONIKER         m_pMoniker;
};
typedef CAswAsyncData   ASWPTR * PASWASYNCDATA;
typedef CAswAsyncRename ASWPTR * PASWASYNCRENAME;
class CAswAsyncObject : public CAswDebugBase
{
public:
   CAswAsyncObject( PASWCONTROL pAswControl );
   virtual ~CAswAsyncObject( void );
   CAswResult Setup( PASWASYNC        pAswAsync,
                     UINT             uMsgType, 
                     PASWDATAMANAGER  pDataManager, 
                     DWORD            dwAspect,
                     LONG             index ); 
   CAswResult Shutdown( BOOL fEmergency = FALSE ); 
   inline BOOL IsMatchingDataMgr( PASWDATAMANAGER pDataManager )
   {
      return ( pDataManager == GetDataMgr() );
   }
   inline BOOL IsMatchingOnViewChange( PASWDATAMANAGER  pDataManager,
                                       DWORD            dwAspect,
                                       LONG             index )
   {
      return (   (pDataManager          == GetDataMgr())
              && (WM_ASWAN_ONVIEWCHANGE == GetMsgType())
              && (dwAspect              == GetAspect())
              && (index                 == GetIndex()) );
   }
   void SendOnViewChange( void );
   inline BOOL IsMatchingOnSave( PASWDATAMANAGER  pDataManager )
   {
      return (   (pDataManager    == GetDataMgr())
              && (WM_ASWAN_ONSAVE == GetMsgType()) );
   }
   void SendOnSave( void );
public:
   CAswListEl        m_leAsyncObjectListElement;
private:
   inline void EstablishConstructorState( void )
   {
      m_pAswAsync    = NULL;
      m_pDataManager = NULL;
      m_uMsgType     = 0;
      m_dwAspect     = 0;
      m_index        = -1;
   }
   inline PASWDATAMANAGER GetDataMgr( void )
   {
      return m_pDataManager;
   }
   inline LONG GetIndex( void )
   {
      return m_index;
   }
   inline DWORD GetAspect( void )
   {
      return m_dwAspect;
   }
   inline UINT GetMsgType( void )
   {
      return m_uMsgType;
   }
   PASWDATAMANAGER   m_pDataManager;
   PASWASYNC         m_pAswAsync;
   UINT              m_uMsgType;
   DWORD             m_dwAspect;
   LONG              m_index;
};
MAKELISTHEADCLASS( CAswAsyncObject );
class CAswAsync : public CAswCtlBase
{
   friend LRESULT ASWCALLBACK CALLBACK AswanAsyncWndProc( HWND,   UINT, 
                                                          WPARAM, LONG );
public:
   CAswAsync( PASWCONTROL pAswControl );
   ~CAswAsync( void );
   CAswResult Setup( HINSTANCE hInst, HINSTANCE hInstPrev = NULL );
   CAswResult Shutdown( BOOL fEmergency = FALSE );
   BOOL PostOnViewChange( PASWDATAMANAGER pDataMgr,
                          DWORD dwAspect,
                          LONG index);
   BOOL PostOnDataChange( PASWDATAMANAGER  pDataMgr,
                          LPFORMATETC      pFormatEtc,
                          LPSTGMEDIUM      pMedium);
   BOOL PostOnRename( PASWDATAMANAGER pDataMgr,
                      LPMONIKER       pmk);
   BOOL PostOnSave( PASWDATAMANAGER pDataMgr );
   void KillPendingNotifications( PASWDATAMANAGER  pDataMgr );
   inline void AddAsyncObjectToList( PASWASYNCOBJECT pAsyncObject )
   {
      m_lhListOfAsyncObjects.Insert( pAsyncObject ); 
   }   
   inline void RemoveAsyncObjectFromList( PASWASYNCOBJECT pAsyncObject )
   { 
      m_lhListOfAsyncObjects.Remove( pAsyncObject ); 
   }   
   inline CAswAsyncObjectListHead * GetListHead(void)
   {
      return &m_lhListOfAsyncObjects;
   }
   PASWASYNCOBJECT FindLiveObject( PASWASYNCOBJECT pMsgObject );
   static PASWASYNCOBJECT FindLiveObject( HWND hWnd, LPARAM lParam );
private:
   HWND                          m_hAsyncWnd;         
   CAswAsyncObjectListHead       m_lhListOfAsyncObjects;
   CAswAsync( const CAswAsync & SourceObject );
   void operator= ( const CAswAsync & SourceObject );
};
#include <aspop.h>
#endif 
