/* NOSHIP */
/*****************************************************************************
 *
 * asctl.h
 *
 * (C) Copyright Lotus Development Coporation 1994, 1996. All Rights Reserved.
 *
 * LIST OF CLASSES IN THIS FILE
 *      CAswControl
 *
 * Description:
 *  This file declares the CAswControl class.  For each Aswan
 *  application there is exactly one instance of CAswControl, which
 *  the application must allocate before creating any other Aswan classes.
 *
 *  Most instances of other Aswan classes inherit from CAswCtlBase,
 *  which gives them access to CAswControl.  CAswControl itself inherits
 *  from CAswBase, which in turn inherits from CAswCtlBase.  This allows
 *  macros in CAswControl code to find the CAswControl instance in the
 *  same manner that all other Aswan code does.
 *
 *****************************************************************************/

#ifndef ASCTL_H
#define ASCTL_H
#include "aswassrt.h"
#include "stddef.h"
#include <aspack.h>
const int MAX_INDENT_LEVEL = 13;
const int DEFAULT_INDENT = 3;
const int NUM_DEFAULT_VERBS = 7;
#define OLEINITIALIZED ASWPROPUSER_1
#define ISOLEINITIALIZED() GETDEFAULTSTDPROP(OLEINITIALIZED)
#define SETOLEINITIALIZED_TRUE() SETDEFAULTSTDPROPTRUE(OLEINITIALIZED)
#define SETOLEINITIALIZED_FALSE() SETDEFAULTSTDPROPFALSE(OLEINITIALIZED)
#define SETOLEINITIALIZED(boolval) SETDEFAULTSTDPROPFALSE(OLEINITIALIZED,boolval)
#define SHAREINSTALLED ASWPROPUSER_2
#define ISSHAREINSTALLED() GETDEFAULTSTDPROP(SHAREINSTALLED)
#define SETSHAREINSTALLED_TRUE() SETDEFAULTSTDPROPTRUE(SHAREINSTALLED)
#define SETSHAREINSTALLED_FALSE() SETDEFAULTSTDPROPFALSE(SHAREINSTALLED)
#define SETSHAREINSTALLED(boolval) SETDEFAULTSTDPROP(SHAREINSTALLED,boolval)
#define DEBUGOUTPUTOK ASWPROPUSER_3
#define ISDEBUGOUTPUTOK() GETDEFAULTSTDPROP(DEBUGOUTPUTOK)
#define SETDEBUGOUTPUTOK_FALSE() SETDEFAULTSTDPROPFALSE(DEBUGOUTPUTOK)
#define SETDEBUGOUTPUTOK_TRUE() SETDEFAULTSTDPROPTRUE(DEBUGOUTPUTOK)
#define SETDEBUGOUTPUTOK(boolval) SETDEFAULTSTDPROP(DEBUGOUTPUTOK,boolval)
typedef struct tagASWSTRINGINFO
{
   UINT      uiStringID;
   HINSTANCE hInstance;
}
   ASWSTRINGINFO;
typedef ASWSTRINGINFO ASWPTR * PASWSTRINGINFO;
 struct CAswPackMember
 {
 public:
   inline void Set( size_t Size, size_t Offset )
   {
      m_Size = Size;
      m_Offset = Offset;
   }
   inline BOOL operator== ( const CAswPackMember & other ) const
   {
      return (   ( other.m_Size == m_Size )
              && ( other.m_Offset == m_Offset ) );
   }
private:
   size_t m_Size;
   size_t m_Offset;
 };
 class CAswPackCheck
 {
 public:
   inline CAswPackCheck( void )
   {
      m_Pack[0].Set( sizeof(ASWSTATSTG), offsetof(ASWSTATSTG,cbSize) );
   }
   inline operator== ( const CAswPackCheck & other ) const
   {
      return ( other.m_Pack[ 0 ] == m_Pack[ 0 ] );
   }
private:
   CAswPackMember m_Pack[ 1 ];
 };
typedef CAswPackCheck ASWPTR * PASWPACKCHECK;
class ASWPUBCLASS CAswControl : public CAswBase
{
public:
   ASWPUBMETHOD CAswControl(HINSTANCE hinstApp=NULL,HINSTANCE hinstPrev = NULL);
   ASWPUBMETHOD ~CAswControl();
   CAswResult ASWPUBMETHOD Setup( LPCSTR          lpszLanguageID,
                                  BOOL            bShareIsKnownToBeInstalled = FALSE,
                                  HINSTANCE       hInstResourceDLL = NULL,
                                  const CAswPackCheck & rClientPackCheck = CAswPackCheck() );
   virtual CAswResult ASWPUBMETHOD Shutdown(BOOL fEmergency = FALSE);
   virtual CAswResult ASWPUBMETHOD 
   Initialize( LPASWOLESTR       pwszAppName,
               PASWMALLOC        pAswMalloc = NULL,
               ASWMSGPENDINGPROC pMsgPendingProc = NULL );
   virtual void ASWPUBMETHOD Uninitialize( void );
   void ASWPUBMETHOD GetOleVersion( LPWORD pwMajor, LPWORD pwMinor ) const;
#ifndef LTSOS2   
   inline LPASWCOLESTR GetApplicationName( void ) const
#else
   inline LPASWCOLESTR GetApplicationName( void )
#endif
   {
      return m_awAppName;
   }
   virtual CAswResult ASWPUBMETHOD InitOLE( void );
   virtual void ASWPUBMETHOD UninitOLE( void );
   virtual BOOL ASWPUBMETHOD CheckBuildVersion( LPWORD pWord ) const;
   BOOL ASWPUBMETHOD CheckShareInstalled( void ) const;
   inline BOOL IsShareInstalled( void )
   {
      return ( ISSHAREINSTALLED() );
   }
   BOOL ASWPUBMETHOD AddPendingMessage(UINT message);
   BOOL ASWPUBMETHOD RemovePendingMessage(UINT message);
   void ASWPUBMETHOD FreePendingList(void);
   inline void ClearPendingList(void)
   {
      m_cPendingMsg = 0;
   }
   inline HWND SetParentWindow(HWND hwnd)
   {
      m_hwndDlgParent = hwnd;
      return OleStdMsgFilter_SetParentWindow(m_pMsgFilter, hwnd);
   }
   inline HWND GetParentWindow(void)
   {
      return m_hwndDlgParent;
   }
   inline BOOL EnableBusyDialog(BOOL fEnable = TRUE)
   {
      return OleStdMsgFilter_EnableBusyDialog(m_pMsgFilter, fEnable);
   }
   inline BOOL EnableNotRespondingDialog(BOOL fEnable = TRUE)
   {
      return OleStdMsgFilter_EnableNotRespondingDialog(m_pMsgFilter, fEnable);
   }
   inline DWORD GetInComingCallStatus(void)
   {
      return OleStdMsgFilter_GetInComingCallStatus(m_pMsgFilter);
   }
   inline void SetInComingCallStatus(DWORD dwInComingCallStatus)
   {
      OleStdMsgFilter_SetInComingCallStatus(m_pMsgFilter, dwInComingCallStatus);
   }
   inline void RejectInComingCalls(void)
   {
      OleStdMsgFilter_SetInComingCallStatus(m_pMsgFilter, SERVERCALL_RETRYLATER);
   }
   inline void AllowInComingCalls(void)
   {
      OleStdMsgFilter_SetInComingCallStatus(m_pMsgFilter, SERVERCALL_ISHANDLED);
   }
   inline PASWOBJECTINSTANCE GetUIActiveObjectInstance(void)
   {
      return m_pActiveObjectInstance;
   }
   void ASWPUBMETHOD RecordCOMAPI( HRESULT hr,
                                   LPCSTR  lpszAPIName );
   void ASWPUBMETHOD RecordCOMMethod( HRESULT hr,
                                      LPCSTR  lpszMethodName,
                                      REFIID  riid );
   inline void ClearCOMError( void )
   {
      SetCOMError( NOERROR );
      SetErrorMethodName( FALSE );
      SetErrorAPIName( FALSE );
   }
   inline BOOL COMErrorIsSet( void ) const
   {
      return ( FAILED( m_hrCOMRESULT ) );
   }
   inline HRESULT GetCOMError( void ) const
   {
      return m_hrCOMRESULT;
   }
   inline BOOL IsCOMMethodError( void ) const
   {
      return (NULL != GetErrorMethodName());
   }
   inline BOOL IsCOMAPIError( void ) const
   {
      return (NULL != GetErrorAPIName());
   }
   inline REFIID GetErrorIID( void ) const
   {
      return *m_lpCOMIID;
   }
   inline LPCSTR GetErrorMethodName( void ) const
   {
      return m_lpszMethodName;
   }
   inline LPCSTR GetErrorAPIName( void ) const
   {
      return m_lpszAPIName;
   }
   virtual HCURSOR ASWPUBMETHOD DisplayWaitCursor( void ) const;
   virtual HCURSOR ASWPUBMETHOD DisplayNormalCursor( HCURSOR hOldCursor ) const;
   virtual int ASWPUBMETHOD AswMessageBox( LPCSTR lpszText,
                                        WORD   wFlags = MB_OK,
                                        HWND   hWndParent = NULL );
   virtual int ASWPUBMETHOD AswMessageBox( PASWSTRINGINFO pStringInfo,
                                        WORD           wFlags = MB_OK,
                                        HWND   hWndParent = NULL );
   inline void GetPendingMsgArray( LPUINT ASWPTR * ppuiMsg,
                                   UINT   ASWPTR * pcMsg )
   {
      *ppuiMsg = m_puiPendingMsg;
      *pcMsg = m_cPendingMsg;
   }
   inline LPMESSAGEFILTER GetMessageFilter(void) const
   {
      return m_pMsgFilter;
   }
   inline PASWASYNC GetAsync( void )
   {
      return ( &m_async );
   }
   inline void SetCurrentDragSource( PASWDRAGSOURCE pCurrentDragSource )
   {
      m_pCurrentDragSource = pCurrentDragSource;
   }
   inline void ClearCurrentDragSource( void )
   {
      m_pCurrentDragSource = NULL;
   }
   inline PASWDRAGSOURCE GetCurrentDragSource( void ) const
   {
      return m_pCurrentDragSource;
   }
   virtual PASWSHUTDOWN ASWPUBMETHOD GetShutdownObject( void );
   inline void SetCOMError( HRESULT hrNew )
   {
      m_hrCOMRESULT = hrNew;
   }
   inline void SetErrorIID( REFIID riid )
   {
      m_lpCOMIID = &riid;
   }
   inline void SetErrorMethodName( LPCSTR lpszName )
   {
      m_lpszMethodName = lpszName;
   }
   inline void SetErrorAPIName( LPCSTR lpszAPIName )
   {
      m_lpszAPIName = lpszAPIName;
   }
   inline void SetAppInstanceHandles( HINSTANCE hinstApp,
                                      HINSTANCE hinstPrev = NULL )
   {
      m_hinstApp = hinstApp;
      m_hinstPrev = hinstPrev;
   }
   inline HINSTANCE GetAppInstance( void ) const
   {
      return ( m_hinstApp );
   }
   inline HINSTANCE GetPrevInstance( void ) const
   {
      return ( m_hinstPrev );
   }
   inline void SetContainerListHead( PASWCONTAINER pHead )
   {
      m_pAswContainerListHead = pHead;
   }
   inline PASWCONTAINER GetContainerListHead( void ) const
   {
      return m_pAswContainerListHead;
   }
   inline void SetObjectClassListHead( PASWOBJECTCLASS pHead )
   {
      m_pAswObjectClassListHead = pHead;
   }
   inline PASWOBJECTCLASS GetObjectClassListHead( void ) const
   {
      return m_pAswObjectClassListHead;
   }
   inline void SetUIActiveObjectInstance(PASWOBJECTINSTANCE pOI)
   {
      ASWMETHOD("SetUIActiveObjectInstance", (CH_CLASS_CONTROL));
      ASWENTRYF("pOI=%s", (ASWSTR, ASWCLASSID(pOI)));
      m_pActiveObjectInstance = pOI;
      ASWEXIT();
   }
   CAswResult GetDefaultVerbTable(PASWVERBTABLE ASWPTR*  ppVerbTable,
                                  LPWORD                 pcVerbEntries);
private:
#if !defined (_WIN32) && defined(ASWANASSERT) && !defined (LTSOS2)
   CAswResult CheckMessageQueueSize( void );
#endif
   friend BOOL ASWCALLBACK CALLBACK AswMessagePendingProc( MSG FAR * pMsg,
                                                           DWORD     dwCookie );
   CAswControl( const CAswControl & SourceObject );
   void operator= ( const CAswControl & SourceObject );
private:
   DECLARE_ASW_DEFAULT_STDPROPERTIES_MEMBER();
   LPCSTR            m_lpszAPIName;
   HRESULT           m_hrCOMRESULT;
   const IID  FAR *  m_lpCOMIID;
   LPCSTR            m_lpszMethodName;
   HINSTANCE         m_hinstApp;
   HINSTANCE         m_hinstPrev;
   DWORD             m_dwBuildVersion;
   PASWMALLOC        m_pAswMalloc;
   PASWDRAGSOURCE    m_pCurrentDragSource;
   CAswOLEString     m_awAppName;
   LPMESSAGEFILTER   m_pMsgFilter;
   LPUINT            m_puiPendingMsg;
   UINT              m_cPendingMsg;
   UINT              m_cPendingMsgAlloc;
   HWND              m_hwndDlgParent;
   PASWOBJECTINSTANCE   m_pActiveObjectInstance;
   ASWVERBTABLEENTRY m_vtDefaultVerbTable[NUM_DEFAULT_VERBS];
   CAswOLEString     m_ostrVerbNames[NUM_DEFAULT_VERBS];
   CAswShutdown      m_shutdown;
   CAswAsync         m_async;
   PASWCONTAINER     m_pAswContainerListHead;
   PASWOBJECTCLASS   m_pAswObjectClassListHead;
#ifdef ASWANASSERT
public:
   inline ULONG GenerateInstanceID( void )
   {
       return m_ulConstructions++;    
   }
   virtual void ASWPUBMETHOD AddToShutdownList( PASWDEBUGBASE pDebugBase );
   virtual void ASWPUBMETHOD RemoveFromShutdownList( PASWDEBUGBASE pDebugBase );
#if defined(ASWANDEBUG)
   void ShowShutdownList( void );
#endif
#if defined(MAKEDLL) || defined(ASWDLL)
   virtual void ASWPUBMETHOD AddToLoseControlList( PASWDEBUGBASE pDebugBase );
   virtual void ASWPUBMETHOD RemoveFromLoseControlList( PASWDEBUGBASE pDebugBase );
#if defined(ASWANDEBUG)
   void ShowLoseControlList( void );
#endif
private:
   CAswDebugBaseList m_NeedsLoseControl;
public:
#endif   
   void ASWPUBMETHOD __cdecl Printf(LPCSTR szFormatString, ...);
   void ASWPUBMETHOD CauseAssert( void ) const;
   void ASWPUBMETHOD CauseGlobalAssert( void ) const;
   void ASWPUBMETHOD DisplayAssert( LPCSTR lpszString );
private:
   void AswanAssert( LPCSTR lpszString );
   CAswDebugBaseList m_NeedsShutdown; 
   ULONG             m_ulConstructions; 
#endif 
public:
   int ASWPUBMETHOD LoadResourceString( UINT  uiStringID,
                                        LPSTR lpszString,
                                        int   iStringSize );
private:
   void LoadResourceDLL( void );
   void UnloadResourceDLL( void );
   inline void MarkResourceDLLUnloaded( void )
   {
      m_hinstResourceDLL = NULL;
   }
   void SetMsgBoxTitle( void );
public:
   static void LoadResourceDLL( HINSTANCE ASWPTR * phInstance );
   inline HINSTANCE GetResourceDLLInstance( void )
   {
      LoadResourceDLL();
      return ( m_hinstResourceDLL );
   }
private:
   HINSTANCE         m_hinstResourceDLL;
   BOOL              m_bLoadedResourceDLL;
#ifdef ASWANASSERT
   CAswBoolRegValue  m_AssertRegKey;
#endif 
private:
   CAswANSIString m_MBTitle;
#ifdef ASWANDEBUG
public:
   inline char ASWPTR * GetDebugIDStringBuffer( void ) const
   {
     return (char ASWPTR *)m_DebugIDStringBuffer;
   }
   void ASWPUBMETHOD SetDebugOut( PASWDEBUGOUT pNewDevice );
   void ASWPUBMETHOD RemoveDebugOut( PASWDEBUGOUT pNewDevice );
   inline void SetMessageChannelMap( CAswChannelMap *pMessageChannelMap )
   {
      m_pMessageChannelMap = pMessageChannelMap;
   }
   inline void ResetBuffer( void )
   {
      if ( m_pOutputBuffer )
         m_pOutputBuffer->Null();
   }
   inline void InitDebugOut( void )         
   {
      m_iIndentBy = DEFAULT_INDENT;          
      m_iCurrIndentLevel = -1;               
      m_uicDebugDevices = 0;                 
      m_DebugIDStringBuffer[ 0 ]= '\0';
      m_pMessageChannelMap = NULL;
      ResetBuffer();                            
      SETDEBUGOUTPUTOK( m_FormatDebugRegKey.Get() );
   }
   inline LPSTR GetBuffer( void ) const
   {
      if ( m_pOutputBuffer )
         return ( m_pOutputBuffer->GetANSIPtr() );
      else
         return NULL;
   }
   inline int GetBufferCount( void ) const
   {
      if ( m_pOutputBuffer )
         return ( m_pOutputBuffer->GetCount() );
      else
         return 0;
   }
   inline void EndMessage( void )
   {
      if ( ISDEBUGOUTPUTOK() )
      {
         Write("\r\n");
         FlushBuffer();
      }
   }
   inline void Indent(int fIndent)              
   {                                            
      if ( fIndent )
      {
         m_iCurrIndentLevel++;
      }
      else
      {
         m_iCurrIndentLevel--;
      }
   }
   inline void SetIndentBy(int iIndentBy)
   {
      m_iIndentBy = iIndentBy;
   }
   inline void Identify( LPCSTR lpszClassName,
                         long   lInstanceID,
                         LPCSTR lpszMethodName )
   {
      if ( ISDEBUGOUTPUTOK() )
      {
         WriteIndent();
         Printf( "%Fs(%ld)::%Fs ", lpszClassName, lInstanceID, lpszMethodName );
      }
   }
   void ASWPUBMETHOD WriteIndent( void );
   void ASWPUBMETHOD Write( LPCSTR szNullTerminatedString );
   void ASWPUBMETHOD FlushBuffer( void );
   DWORD GetTime(void);
private:
   int               m_iIndentBy;
   int               m_iCurrIndentLevel;
   CAswChannelMap *  m_pMessageChannelMap;
   PASWDEBUGOUT      m_pDebugDevice[ MAX_DEBUGDEVICES ];
   UINT              m_uicDebugDevices;
   char              m_DebugIDStringBuffer[ MAX_IDSTRING ];
   CAswBoolRegValue  m_FormatDebugRegKey;
   CAswBoolRegValue  m_ShowTickCount;
   static CAswWatch NEAR cm_AswWatch;
   CAswBoolRegValue  m_ShowTaskID;
   CAswANSIString   * m_pOutputBuffer;
#endif 
};
#ifdef ASWANASSERT
inline void CAswDebugBase::SetInstanceTime( void )
{
#if defined(ASWANDEBUG)
   if ( HasControl() )
      m_dwInstanceTime = GetControl()->GetTime();
   else
      m_dwInstanceTime = 0;
#endif
}
inline DWORD CAswDebugBase::GetInstanceTime( void )
{
#if defined(ASWANDEBUG)
   return m_dwInstanceTime;
#else
   return 0;
#endif
}
#endif   
#include <aspop.h>
#endif 
