/* NOSHIP */
/*****************************************************************************
 *
 * Class: CAswObjectClass
 *
 * (C) Copyright Lotus Development Coporation 1994, 1996. All Rights Reserved.
 *
 * Description:
 *    The Aswan Object Class is used by the application to tell OLE about
 *    a type of object, designated by its class id, that is supported by
 *    this server. The object class supports the OLE IClassFactory interface,
 *    its registration and revocation, and coordination of objects created
 *    with that interface.
 *
 *    The application creates one object class for each type of object it
 *    supports.
 *
 *    The applciation must provide an implementation of the CreateInstance
 *    method, which is used to create instance-specific data structures.
 *    The details of this method are listed in the comments of the
 *    CreateInstance API description.
 *
 *****************************************************************************/

#ifndef ASOBJCLS_H
#define ASOBJCLS_H
#include <aspack.h>
class ASWPUBCLASS CAswObjectClass : public CAswCOMBase
{
public:
   ASWPUBMETHOD CAswObjectClass(PASWCONTROL pControl,
		LPUNKNOWN pUnknownOuter = NULL);
   ASWPUBMETHOD ~CAswObjectClass(void); 
   CAswResult ASWPUBMETHOD Setup(REFCLSID rclsid,
                                 DWORD    dwClsContext,
                                 DWORD    dwFlags,
                                 BOOL     fRegister = TRUE);
   virtual CAswResult ASWPUBMETHOD Shutdown(BOOL fEmergency = FALSE);
   virtual CAswResult
   CreateInstance(LPUNKNOWN                 pUnkOuter,
                  PASWCOMBASE ASWPTR *      ppCOMBase) PURE;
   inline void AddObjectInstanceToList(PASWOBJECTINSTANCE pInstance)
   {
      m_lhObjectInstances.Insert(pInstance);
   }
   inline void RemoveObjectInstanceFromList(PASWOBJECTINSTANCE pInstance)
   {
      m_lhObjectInstances.Remove(pInstance);
   }
   inline REFCLSID GetClassID(void) const
   {
      return m_clsid;
   }
   inline void SetNextObjectClass(PASWOBJECTCLASS pAswObjectClass)
   {
      m_pAswObjectClassNext = pAswObjectClass;
   }
   inline PASWOBJECTCLASS GetNextObjectClass(void)
   {
      return m_pAswObjectClassNext;
   }
   virtual PASWSHUTDOWN ASWPUBMETHOD GetShutdownObject(void);
   virtual PASWBASE ASWPUBMETHOD GetParentObject(void);
   inline LPCLASSFACTORY GetClassFactory(void)
   {
      return &m_ClassFactory;
   }
private:
   ASWAN_DECLARATIONS(1);
   CAswClassFactory  m_ClassFactory;      
   DWORD             m_dwRegisterCookie;  
	PASWOBJECTCLASS	m_pAswObjectClassNext;
   CAswObjectInstanceListHead m_lhObjectInstances;
   CLSID             m_clsid;
   CAswShutdown      m_shutdown;
};
#include <aspop.h>
#endif 
