/* NOSHIP */
/*****************************************************************************
 *
 * assrvspc.h
 *
 * (C) Copyright Lotus Development Coporation 1994, 1996. All Rights Reserved.
 *
 * LIST OF CLASSES IN THIS FILE
 *    CAswServSpace
 *
 * Description:
 *    In-place server space negotiation management class declaration.
 *
 *****************************************************************************/

#ifndef ASSRVSPC_H
#define ASSRVSPC_H
#include <aspack.h>
class ASWPUBCLASS CAswServSpace : public CAswCtlBase
{
   friend CAswObjectInstance;             
   friend CAswOleInPlaceActiveObjectImpl; 
public:
   ASWPUBMETHOD CAswServSpace( PASWCONTROL pControl );
   ASWPUBMETHOD ~CAswServSpace( void ); 
   CAswResult ASWPUBMETHOD Setup( BOOL fIsFrameSpace ); 
   CAswResult ASWPUBMETHOD Shutdown( BOOL fEmergency = FALSE ); 
   inline BOOL IsSetup(void)
   {
      return(m_fSrvSpaceSetup);
   }
   inline BOOL IsFrameSpace(void)
   {
      return( m_fIsFrameSpace );
   }
   inline CAswResult GetBorder(LPRECT lprectBorder)
   {
      ASWMETHOD("GetBorder",(CH_CLASS_SERVSPACE, CH_SERVER_SPACE));
      ASWENTRY();
      HRESULT hr = GetIPInterface()->GetBorder(lprectBorder);
      ASWEXITR(CAswResult(hr, ASD_UNCHANGED));
   }
   inline CAswResult RequestBorderSpace(LPCBORDERWIDTHS lpborderwidths)
   {
      ASWMETHOD("RequestBorderSpace",(CH_CLASS_SERVSPACE, CH_SERVER_SPACE));
      ASWENTRY();
      HRESULT hr = GetIPInterface()->RequestBorderSpace(lpborderwidths); 
      ASWEXITR( CAswResult( hr, ASD_UNCHANGED) );
   }
   inline CAswResult SetBorderSpace(LPCBORDERWIDTHS lpborderwidths)
   {
      ASWMETHOD("SetBorderSpace",(CH_CLASS_SERVSPACE, CH_SERVER_SPACE));
      ASWENTRY();
      HRESULT hr = GetIPInterface()->SetBorderSpace(lpborderwidths);
      ASWEXITR(CAswResult(hr, ASD_UNCHANGED));
   }
   inline CAswResult GetWindow(HWND FAR* phwnd)
   {
      ASWMETHOD("GetWindow",(CH_CLASS_SERVSPACE, CH_SERVER_SPACE));
      ASWENTRY();
      HRESULT hr = GetIPInterface()->GetWindow(phwnd);
      ASWEXITR(CAswResult(hr, ASD_UNCHANGED));
   }
   virtual CAswResult AddTools( LPRECT prectBorder, BOOL fIsMDIContainer ) PURE;
   virtual void RemoveTools( void ) PURE;
   virtual CAswResult ASWPUBMETHOD ResizeBorder( LPCRECT prcBorder );
   inline PASWOBJECTINSTANCE GetObjectInstance(void) const
   {
      return m_pObjInstance;
   }
private:
   BOOL               m_fIsFrameSpace;
   BOOL               m_fSrvSpaceSetup;
   PASWOBJECTINSTANCE m_pObjInstance;
   inline LPOLEINPLACEUIWINDOW GetIPInterface(void)
   {
      ASWMETHOD("GetIPInterface",(CH_CLASS_SERVSPACE, CH_SERVER_SPACE))
      ASWENTRYF("IsFrameSpace=%s", (ASWSTR, ASWSHOWTF(IsFrameSpace())));
      LPOLEINPLACEUIWINDOW pIP = (IsFrameSpace())
         ? GetObjectInstance()->GetIPFrame()
         : GetObjectInstance()->GetIPUIWindow();
      ASWASSERT(pIP);
      ASWEXITF(pIP, "pIP=%p", (ASWSTR, pIP));
   }
   inline void SetObjectInstance(PASWOBJECTINSTANCE pObjInstance)
   {
      m_pObjInstance = pObjInstance;
   }
   CAswResult DoAddTools(void);                    
   CAswResult DoResizeBorder(LPCRECT prcBorder);   
   void       DoRemoveTools(void);                 
};
#include <aspop.h>
#endif      
