/* NOSHIP */
/*****************************************************************************
 *
 * Source Module: asstring.h
 *
 * (C) Copyright Lotus Development Coporation 1994, 1996. All Rights Reserved.
 *
 * Purpose:       Dynamic string allocation classes -
 *                   CAswStringBase
 *                   CAswANSIString
 *                   CAswOLEString
 *
 * Usage:         Used to store strings that are needed by the Toolkit.
 *                These include strings that are passed over OLE 2 interface
 *                and strings that are used in debugging.
 *
 *****************************************************************************/

#ifndef ASSTRING_H
#define ASSTRING_H
#include <aspack.h>
#if defined(ASWANASSERT) && defined(MAKEDLL)
REFASWEXTRNCODESEGSTRING( szSBAllocFailed, ASszSBAllocFailed );
REFASWEXTRNCODESEGSTRING( szSBLostIMalloc, ASszSBLostIMalloc );
#endif
#if defined(ASWANASSERT)
#define SIGN_STRINGS 0
#else
#define SIGN_STRINGS 0
#endif
#if SIGN_STRINGS
#if defined(MAKEDLL)
REFASWEXTRNCODESEGSTRING( szBadBack,  ASszBadBack );
REFASWEXTRNCODESEGSTRING( szBadFront, ASszBadFront );
REFASWEXTRNCODESEGSTRING( szBadPSig,  ASszBadPSig );
REFASWEXTRNCODESEGSTRING( szTooBig,   ASszTooBig );
#endif
const int STRING_SIGNATURE_SIZE = (sizeof(DWORD));
const DWORD STRING_SIGNATURE = 0xabaddadaL;
#define PREFIX_ALLOC()\
ASWSIMPLEASSERT(iSizeBytes<=INT_MAX-(STRING_SIGNATURE_SIZE*2),szTooBig);\
iSizeBytes+=(STRING_SIGNATURE_SIZE*2)
#define ADJUST_FOR_SIGNATURE(p) p=(PASWVOID)(((LPBYTE)p)-(STRING_SIGNATURE_SIZE));
#define PREFIX_REALLOC()\
PREFIX_ALLOC();\
ADJUST_FOR_SIGNATURE(m_lpszAlloc)
#define POSTFIX_ALLOC()\
ASWSIMPLEASSERT(m_lpszAlloc,szSBAllocFailed);\
iSizeBytes-=(STRING_SIGNATURE_SIZE*2);\
if (m_lpszAlloc)\
{\
 *((DWORD ASWPTR *)m_lpszAlloc)=STRING_SIGNATURE;\
 m_lpszAlloc=(PASWVOID)(((LPBYTE)m_lpszAlloc)+(STRING_SIGNATURE_SIZE));\
 *((DWORD ASWPTR *)(((LPBYTE)m_lpszAlloc)+iSizeBytes))=STRING_SIGNATURE;\
}
#define CHECK_POINTER(pCheck)\
ASWSIMPLEASSERT(ISRPTROKLEN(pCheck,PrivateGetSize()+(STRING_SIGNATURE_SIZE*2)),szSBAllocFailed);\
DWORD ASWPTR *pSignature=(DWORD ASWPTR *)(((LPBYTE)pCheck)-STRING_SIGNATURE_SIZE);\
ASWSIMPLEASSERT(STRING_SIGNATURE==*pSignature,szBadFront);\
ASWSIMPLEASSERT(STRING_SIGNATURE==*((DWORD ASWPTR *)(((LPBYTE)pCheck)+PrivateGetSize())),szBadBack);
#define CHECK_INTERNAL_STATE() CheckInternalState()
#define FREE_CHECK(pToFree)\
CHECK_POINTER(pToFree);\
ADJUST_FOR_SIGNATURE(pToFree)
#else
#define PREFIX_ALLOC()
#define PREFIX_REALLOC()
#define POSTFIX_ALLOC() ASWSIMPLEASSERT(m_lpszAlloc,szSBAllocFailed)
#define ADJUST_FOR_SIGNATURE(p)
#define CHECK_POINTER(pCheck)
#define CHECK_INTERNAL_STATE()
#define FREE_CHECK(pToFree)
#endif
class ASWPUBCLASS CAswStringBase : public CAswMemoryBase
{
public:
   ASWPUBMETHOD CAswStringBase( int iSizeBytes = 0 );
   ASWPUBMETHOD ~CAswStringBase( void );
   LPVOID ASWPUBMETHOD AllocBytes( int iSizeBytes );
   BOOL  ASWPUBMETHOD Free( void );
   inline void InitAllocator( void )
   {
      CHECK_INTERNAL_STATE();
      if ( NULL == m_lpMalloc )
      {
         if ( NOERROR != CoGetMalloc( MEMCTX_TASK, &m_lpMalloc ) )
         {
            m_lpMalloc = NULL;
         }
      }
   }
   inline LPVOID GetPtr( void ) const
   {
      CHECK_INTERNAL_STATE();
      return ( m_lpszAlloc );
   }
   inline int GetSize( void ) const
   {
      CHECK_INTERNAL_STATE();
      return ( PrivateGetSize() );
   }
#if SIGN_STRINGS
   void ASWPUBMETHOD CheckInternalState( void ) const;
#endif
#if defined(ASWANDEBUG)
   static BOOL ASWPUBFUNC IsBalanced( void );
   static void ShowLiveList( void );
   static PASWTASKINFO GetTaskInfoPtr( DWORD dwIndex );
   static void ShowHelp( HWND hwndDlg );
#endif 
private:
   inline void InitPtr( void )
   {
      m_lpszAlloc = NULL;
      m_iAllocSize = 0;
   }
   inline int PrivateGetSize( void ) const
   {
      return ( m_iAllocSize > 0 ? m_iAllocSize : - m_iAllocSize );
   }
   void ReallocPtr( int iSizeBytes );
   void ReallocFromPtrNew( int iSizeBytes );
private:
   LPMALLOC m_lpMalloc;
   LPVOID   m_lpszAlloc;
   int      m_iAllocSize;
#ifdef ASWANASSERT   
   static CAswPerTaskInfo NEAR cm_PerTaskInfo;
#if SIGN_STRINGS
   static CAswBoolRegValue NEAR cm_CheckInternalState;
#endif
   void ASWPUBMETHOD RemoveItemFromList( void );
#if defined(ASWANDEBUG)
   void ASWPUBMETHOD AddItemToList( int    iSizeBytes, 
                                    LPCSTR lpsz__FILE__, 
                                    int    i__LINE__ );
private:
   char           m_szDebugInfo[ 256 ];
   PASWSTRINGBASE m_pAswStringNext;
public:
   LPCSTR GetDebugInfo( void ) const
   {
      return ( m_szDebugInfo );
   }
   inline void SetNextString(PASWSTRINGBASE pAswString)
   {
      m_pAswStringNext = pAswString;
   }
   inline PASWSTRINGBASE GetNextString(void)
   {
      return m_pAswStringNext;
   }
#else  
   void ASWPUBMETHOD AddItemToList( int    , 
                                    LPCSTR , 
                                    int     );
#endif 
#endif 
};
class ASWPUBCLASS CAswANSIString : public CAswStringBase
{
public:
   inline CAswANSIString( int iSizeChars = 0 )
      : CAswStringBase( iSizeChars )
   {  }
   inline CAswANSIString( LPCSTR psz )
      : CAswStringBase( 0 )
   {
      Copy( psz );
   }
   inline CAswANSIString( const CAswANSIString & a_str )
      : CAswStringBase( 0 )
   {
      Copy( a_str.GetANSIPtr() );
   }
#if defined(_WIN32) && !defined(LTSOS2)
   inline CAswANSIString( LPASWCOLESTR posz )
      : CAswStringBase( 0 )
   {
      TranslateW2A( posz );
   }
   inline CAswANSIString& operator = ( LPASWCOLESTR posz )
   {
      TranslateW2A( posz );
      return *this;
   }
#endif
   inline operator LPCSTR()
   {
      return GetANSIPtr();
   }
   inline CAswANSIString & operator = ( const CAswANSIString & a_str )
   {
      Copy( a_str.GetANSIPtr() );
      return *this;
   }
   inline CAswANSIString & operator = ( LPCSTR psz )
   {
      Copy( psz );
      return *this;
   }
   LPVOID ASWPUBMETHOD AllocCharacters( int iSizeChars );
   inline LPSTR GetANSIPtr( void ) const
   {
      return ( (LPSTR) GetPtr() );
   }
   inline int GetCount( void ) const
   {
      if ( GetANSIPtr() )
         return ( ::lstrlen( GetANSIPtr() ) );
      else
         return ( 0 );
   }
   inline void Null( void )
   {
      LPSTR lpsz = GetANSIPtr();
      if ( lpsz )
         *lpsz = '\0';
   }
   LPSTR ASWPUBMETHOD Copy( LPCSTR lpszString );
   LPSTR ASWPUBMETHOD Append( LPCSTR lpszString );
   LPSTR ASWPUBMETHOD AppendChar( char ch, int iNum );
   LPSTR ASWPUBMETHOD TranslateW2A(LPASWCOLESTR lpOleString);
   int ASWPUBMETHOD LoadString( HINSTANCE hInst, UINT uiStringID );
private:
};
class ASWPUBCLASS CAswOLEString : public CAswStringBase
{
public:
   inline CAswOLEString( int iSizeChars = 0 )
      : CAswStringBase( iSizeChars * sizeof( ASWOLECHAR ) )
   {  }
   inline CAswOLEString( LPASWCOLESTR posz )
      : CAswStringBase( 0 )
   {
      Copy( posz );
   }
   inline CAswOLEString( const CAswOLEString& o_str )
      : CAswStringBase( 0 )
   {
      Copy( o_str.GetOLEPtr() );
   }
#if defined(_WIN32) && !defined(LTSOS2)
   inline CAswOLEString( LPCSTR psz )
      : CAswStringBase( 0 )
   {
      TranslateA2W( psz );
   }
   inline CAswOLEString & operator = ( LPCSTR psz )
   {
      TranslateA2W( psz );
      return *this;
   }
#endif
   inline operator LPASWCOLESTR() const
   {
      return GetOLEPtr();
   }
   inline CAswOLEString & operator = ( const CAswOLEString & o_str )
   {
      Copy( o_str.GetOLEPtr() );
      return *this;
   }
   inline CAswOLEString & operator = ( LPASWCOLESTR posz )
   {
      Copy( posz );
      return *this;
   }
   LPVOID ASWPUBMETHOD AllocCharacters( int iSizeChars );
   inline LPASWOLESTR GetOLEPtr( void ) const
   {
      return ( (LPASWOLESTR) GetPtr() );
   }
   inline int GetCount( void ) const
   {
      if ( GetOLEPtr() )
         return ( ::ASWLSTRLENW( GetOLEPtr() ) );
      else
         return ( 0 );
   }
   inline void Null( void )
   {
      LPASWOLESTR lpolestr = GetOLEPtr();
      if ( lpolestr )
         *lpolestr = (ASWOLECHAR) 0;
   }
   LPASWOLESTR ASWPUBMETHOD Copy( LPASWCOLESTR lpolesString );
   LPASWOLESTR ASWPUBMETHOD Append( LPASWCOLESTR lpolesString );
   LPASWOLESTR ASWPUBMETHOD AppendOLEChar( ASWOLECHAR ch, int iNum );
   LPASWOLESTR ASWPUBMETHOD TranslateA2W(LPCSTR lpszString);
   int ASWPUBMETHOD LoadString( HINSTANCE hInst, UINT uiStringID );
private:
};
class ASWPUBCLASS CAswOPAQUEString : public CAswStringBase
{
public:
   inline CAswOPAQUEString( int iSizeBytes = 0 )
      : CAswStringBase( iSizeBytes )
   {  }
   inline CAswOPAQUEString( PVOID psz, int dwSizeBytes )
      : CAswStringBase( 0 )
   {
      Copy( psz, dwSizeBytes );
   }
   inline CAswOPAQUEString( const CAswOPAQUEString & a_str, 
                            int dwSizeBytes )
      : CAswStringBase( 0 )
   {
      Copy( a_str.GetOPAQUEPtr(), dwSizeBytes );
   }
   inline CAswOPAQUEString & operator = ( const CAswOPAQUEString & a_str )
   {
      Copy( a_str.GetOPAQUEPtr(), a_str.GetSize() );
      return *this;
   }
   inline PVOID GetOPAQUEPtr( void ) const
   {
      return ( (PVOID) GetPtr() );
   }
   inline void Null( void )
   {
      PVOID psz = GetOPAQUEPtr();
      if ( psz )
         *(LPSTR)psz = '\0';
   }
   inline PVOID  Copy( PVOID pString,
                       int   dwSizeBytes )
   {
      AllocBytes( dwSizeBytes );
      PVOID psz = GetOPAQUEPtr();
      if ( psz )
         ::memcpy( psz, pString, dwSizeBytes );
      return ( psz );
   }
private:
};
class ASWPUBCLASS CAswTCHARString : public CAswMemoryBase
{
public:
   inline CAswTCHARString( int iSizeChars = 0 )
      : m_pTCHAR( NULL )
      , m_iSizeCharsAllocated( 0 )
   {
      AllocCharacters( iSizeChars );
   }
   inline CAswTCHARString( LPCTSTR psz )
      : m_pTCHAR( NULL )
      , m_iSizeCharsAllocated( 0 )
   {
      Copy( psz );
   }
   inline CAswTCHARString( const CAswTCHARString & a_str )
      : m_pTCHAR( NULL )
      , m_iSizeCharsAllocated( 0 )
   {
      Copy( a_str.GetTCHARPtr() );
   }
   ~CAswTCHARString( void )
   {
      Free();
   }
   inline operator LPCTSTR()
   {
      return GetTCHARPtr();
   }
   inline CAswTCHARString & operator = ( const CAswTCHARString & a_str )
   {
      Copy( a_str.GetTCHARPtr() );
      return *this;
   }
   inline CAswTCHARString & operator = ( LPCSTR psz )
   {
      Copy( psz );
      return *this;
   }
   inline LPTSTR GetTCHARPtr( void ) const
   {
      return ( m_pTCHAR );
   }
   inline int GetSize( void ) const
   {
      return ( sizeof(TCHAR) * m_iSizeCharsAllocated );
   }
   inline int GetCount( void ) const
   {
      if ( GetTCHARPtr() )
         return ( ::lstrlen( GetTCHARPtr() ) );
      else
         return ( 0 );
   }
   inline void Null( void )
   {
      LPTSTR lpsz = GetTCHARPtr();
      if ( lpsz )
         *lpsz = '\0';
   }
   LPTSTR ASWPUBMETHOD Copy( LPCTSTR lpszString );
   LPTSTR ASWPUBMETHOD Append( LPCTSTR lpszString );
   LPTSTR ASWPUBMETHOD AppendChar( TCHAR ch, int iNum );
   LPTSTR ASWPUBMETHOD TranslateA2T( LPCSTR lpANSIString );
   int ASWPUBMETHOD LoadString( HINSTANCE hInst, UINT uiStringID );
private:
   LPTSTR AllocCharacters( int iSizeChars );
   inline void Free( void )
   {
      if ( m_pTCHAR )
         delete [] m_pTCHAR;
      m_pTCHAR = NULL;
      m_iSizeCharsAllocated = 0;
   }
   LPTSTR m_pTCHAR;
   int    m_iSizeCharsAllocated;
};
#include <aspop.h>
#endif 
