/* NOSHIP */
/*****************************************************************************
 *
 * Classes: CAswUILinkBase
 *          CAswUILinkContainer
 *
 * (C) Copyright Lotus Development Coporation 1994, 1996. All Rights Reserved.
 *
 * Description:
 *    This Header file defines three classes which provide extensible support
 *    for the Manage Links dialog box.  
 *
 *    CAswUILinkBase, which is a virtual base class which provides all the 
 *                    methods required for callout by the Manage Links dialog 
 *                    box.  Clients override this class and implement the 
 *                    methods to provide their own links to show up in Manage 
 *                    Links's list.
 *
 *    CAswUILinkContainer, which contains a list of CAswUILinkBase derived 
 *                         classes, such as itself or classes derived from
 *                         CAswUILink.  It is used to collect items to be shown 
 *                         by the Manage Links dialog box.
 *
 *                            ----CAswUILinkBase----
 *
 *****************************************************************************/

#ifndef ASUIMNGLK_H
#define ASUIMNGLK_H
#if !defined ( _WIN32 )
#include <toolhelp.h>      
#endif
#include <aspack.h>
#define ASW_MAX_BASKET_TEXT 13   
class ASWPUBCLASS CAswUILinkBase : public CAswCOMBase
{
public:
   ASWPUBMETHOD CAswUILinkBase(PASWCONTROL lpControl,
                               PMAPSTATE   pMapState,
                               LPUNKNOWN   lpUnknownOuter = NULL);
   ASWPUBMETHOD CAswUILinkBase(PASWCONTROL lpControl,
                               LPUNKNOWN   lpUnknownOuter = NULL);
   virtual ASWPUBMETHOD ~CAswUILinkBase(void);
   virtual CAswResult SetLinkUpdateOptions (DWORD dwUpdateOpt) PURE;
   virtual CAswResult GetLinkUpdateOptions (LPDWORD lpdwUpdateOpt) PURE;
   virtual CAswResult SetLinkSource (LPASWCOLESTR  lpszDisplayName,
                                     ULONG         lenFileName,
                                     LPULONG       pchEaten,
                                     BOOL          fValidateSource) PURE;
   virtual CAswResult OpenLinkSource (void) PURE;
   virtual CAswResult UpdateLink (BOOL fErrorMessage,
                                  BOOL fErrorAction) PURE;
   virtual CAswResult CancelLink (void) PURE;
   virtual CAswResult GetLinkSourceEx (LPASWOLESTR ASWPTR * lplpszLinkName,
                                       LPASWOLESTR ASWPTR * lplpszSourceName,
                                       LPULONG lplenFileName,
                                       LPASWOLESTR ASWPTR * lplpszFullLinkType,
                                       LPASWOLESTR ASWPTR * lplpszShortLinkType,
                                       BOOL ASWPTR *  lpbSourceAvailable,
                                       BOOL ASWPTR *  lpbIsSelected,
                                       BOOL ASWPTR *  lpbIsBasket,
                                       LPDWORD lpdwSupportedMask) PURE;
protected:
   ASWAN_DECLARATIONS( 2 );
   CAswListEl              m_leUILinkListElement;
private:
};
MAKELISTHEADCLASS(CAswUILinkBase);
class ASWPUBCLASS CAswUILinkContainer : public CAswUILinkBase
{
public:
   ASWPUBMETHOD CAswUILinkContainer(PASWCONTROL lpControl,
                                    LPUNKNOWN   lpUnknownOuter = NULL);
   virtual ASWPUBMETHOD ~CAswUILinkContainer(void);
   CAswResult ASWPUBMETHOD Setup(void);
   virtual CAswResult ASWPUBMETHOD Shutdown(BOOL fEmergency = FALSE);
   virtual CAswResult ASWPUBMETHOD SetLinkUpdateOptions (DWORD dwUpdateOpt);
   virtual CAswResult ASWPUBMETHOD GetLinkUpdateOptions (LPDWORD lpdwUpdateOpt);
   virtual CAswResult ASWPUBMETHOD SetLinkSource (LPASWCOLESTR lpszDisplayName,
                                                  ULONG       lenFileName,
                                                  LPULONG     pchEaten,
                                                  BOOL        fValidateSource);
   virtual CAswResult ASWPUBMETHOD GetLinkSourceEx (
                                         LPASWOLESTR ASWPTR * lplpszLinkName,
                                         LPASWOLESTR ASWPTR * lplpszSourceName,
                                         LPULONG lplenFileName,
                                         LPASWOLESTR ASWPTR * lplpszFullLinkType,
                                         LPASWOLESTR ASWPTR * lplpszShortLinkType,
                                         BOOL ASWPTR *  lpbSourceAvailable,
                                         BOOL ASWPTR *  lpbIsSelected,
                                         BOOL ASWPTR *  lpbIsBasket,
                                         LPDWORD lpdwSupportedMask);
   virtual CAswResult ASWPUBMETHOD OpenLinkSource (void);
   virtual CAswResult ASWPUBMETHOD UpdateLink (BOOL fErrorMessage,
                                               BOOL fErrorAction);
   virtual CAswResult ASWPUBMETHOD CancelLink (void);
   virtual CAswResult ASWPUBMETHOD GetBasketInfo (LPASWOLESTR ASWPTR* lplpszBasketText,
                                                  BOOL ASWPTR *  lpbUpdateAsGroup);
   inline void Insert(PASWUILINKBASE pUILinkBase)
   {
      m_lhListOfUILinks.Insert (pUILinkBase);
   }
   inline void Remove(PASWUILINKBASE pUILinkBase)
   {
      m_lhListOfUILinks.Remove (pUILinkBase);
   }
   inline PASWUILINKBASE GetFirst()
   {
      return (m_lhListOfUILinks.GetFirst ());
   }
   inline PASWUILINKBASE GetNext()
   {
      return (m_lhListOfUILinks.GetNext ());
   }
   virtual PASWUILINKCONTAINER ASWPUBMETHOD GetCurrentBasket (void);
   virtual void ASWPUBMETHOD SetCurrentBasket (PASWUILINKCONTAINER pauilcCurrent);
   virtual CAswResult ASWPUBMETHOD DoManageLinksDlg (
                                    HWND           hWndOwner,  
                                    BOOL           fConfirmBreakLink = FALSE,   
                                    BOOL           fShowHelp = TRUE,
                                    BOOL           fDisableUpdateNow = FALSE,   
                                    BOOL           fDisableOpenSource = FALSE,  
                                    BOOL           fDisableChangeSource = FALSE,
                                    BOOL           fDisableCancelLink = FALSE,  
                                    LPASWCOLESTR   lpszCaption = NULL,
                                    LPFNOLEUIHOOK  lpfnHook = NULL,
                                    LPARAM         lCustData = 0L,
                                    HINSTANCE      hInstance = NULL,
                                    LPASWCOLESTR   lpszTemplate = NULL,
                                    HRSRC          hResource = NULL);
   BOOL ASWPUBMETHOD UpdateLinks (HWND hWndParent, LPASWCOLESTR lpszDialogTitle, 
                                  UINT uiNumLinks);
   inline void SetUpdateAsGroup (BOOL fUpdateAsGroup)
   {
      m_fUpdateAsGroup = fUpdateAsGroup;
   }
   inline BOOL GetUpdateAsGroup (void)
   {
      return m_fUpdateAsGroup;
   }
   inline void SetText (LPASWCOLESTR pszBasketText)
   {
      m_aostrBasketText = pszBasketText;
   }
   inline LPASWOLESTR GetText (void)
   {
      return m_aostrBasketText.GetOLEPtr();
   }
   inline void SetLinkInfo (LPASWCOLESTR lpszLinkName,
                            LPASWCOLESTR lpszSourceName,
                            ULONG lenFileName,
                            LPASWCOLESTR lpszFullLinkType,
                            LPASWCOLESTR lpszShortLinkType)
   {
      m_aostrLinkName = lpszLinkName;
      m_aostrSourceName = lpszSourceName;
      m_lenFileName = lenFileName;
      m_aostrFullLinkType = lpszFullLinkType;
      m_aostrShortLinkType = lpszShortLinkType;
   }
private:
   CAswUILinkBaseListHead  m_lhListOfUILinks;
   PASWUILINKCONTAINER  m_pauilcCurrent;
   CAswIOleUILinkContainerImpl   m_CAswIOleUILinkContainerImpl;
   CAswIOleUILtsLinkContainerImpl   m_CAswIOleUILtsLinkContainerImpl;
   BOOL  m_fIsSetup;
   BOOL  m_fUpdateAsGroup;
   CAswOLEString  m_aostrBasketText;
   CAswOLEString  m_aostrLinkName;
   CAswOLEString  m_aostrSourceName;
   ULONG       m_lenFileName;
   CAswOLEString  m_aostrFullLinkType;
   CAswOLEString  m_aostrShortLinkType;
};
#include <aspop.h>
#endif
