/* NOSHIP */
/*******************************************************************************
* FILE NAME: iecmpimp.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IEmbeddedComponentImp                                                    *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _IECMPIMP_
#define _IECMPIMP_

#include <irect.hpp>
#include <imetatyp.hpp>
#include <icompnen.hpp>

// classes defined in this module
class IEmbeddedComponentImp;
class IEmbeddedComponent;
class IAswAspect;
class IAswPresSite;

/*******************************************************************************
*                                                                              *
*                           IAswAspect                                         *
*                                                                              *
*******************************************************************************/

class IAswAspect : public CAswAspect
{
public:
    //--------------------------------------------------
    // Construction/Destruction
    //--------------------------------------------------
                                IAswAspect();
                                ~IAswAspect();

    //--------------------------------------------------
    // Base Class overrides
    //--------------------------------------------------
    // Notification that an aspect's presentation view has changed. 
    // NOTE: would not be needed if Aswan were working properly
    virtual BOOL ASWPUBMETHOD OnViewChange( void );

    // Notification that the object changed its extents for presentations of
    // this aspect. The toolkit user will typically override this method.
    virtual void ASWPUBMETHOD OnExtentChange( SIZEL sizelNewExtentsHiMetric ); 

    
};


/*******************************************************************************
*                                                                              *
*                           IAswPresSite                                       *
*                                                                              *
*******************************************************************************/

class IAswPresSite : public CAswInPlacePresSite
{
public:
    //--------------------------------------------------
    // Construction/Destruction
    //--------------------------------------------------
                            IAswPresSite( IEmbeddedComponentImp& Parent );
                            ~IAswPresSite();

    //--------------------------------------------------
    // Inlines
    //--------------------------------------------------
    Boolean                 isInPlace() { return fbInPlace; }
    void                    setIsInPlace( Boolean bInPlace ) { fbInPlace = bInPlace; }
    
private:
    IEmbeddedComponentImp&  fParent;
    Boolean                 fbInPlace;

public:
    //--------------------------------------------------
    // base class implementations and overrides
    //--------------------------------------------------
    virtual CAswResult      GetPosition( HWND ASWPTR* phWnd, LPRECT lprcPixelPos );

   // Notification that the inplace presentation changed its extents, and is
   // thus asking for more or less position to edit in place. 
   virtual void ASWPUBMETHOD InPlaceObjectRequestsNewPosition( LPCRECT lprcPosClientPixelsIn,
                                                               LPRECT  lprcPosClientPixelsOut ); 


};


/*******************************************************************************
*                                                                              *
*                           IEmbeddedComponentImp                              *
*                                                                              *
*******************************************************************************/

class IEmbeddedComponentImp : public CAswContainedObject
{
public:
    //--------------------------------------------------
    // Construction / Destruction
    //--------------------------------------------------
    
                            IEmbeddedComponentImp( IEmbeddedComponent*,
                                                   IComponent::EAspect );
//                          IEmbeddedComponentImp( const IEmbeddedComponentImp& );
    virtual                 ~IEmbeddedComponentImp();
        
    //--------------------------------------------------
    // public interface
    //--------------------------------------------------
    void                    initialize( Boolean, Boolean );
    void                    uninitialize();
    BOOL                    isInitialized() const { return fInitialized; }
    IRectangle              area();
    void                    setArea( IRectangle );

    //--------------------------------------------------
    // Inlines
    //--------------------------------------------------
    IAswAspect&             aspect()        { return *fpAspect; }
    IAswAspect&             contentAspect() { return fContentAspect; }
    IAswAspect&             iconAspect()    { return fIconAspect; }

    void                    switchAspect( IAswAspect& );

    CAswObjRendDev&         rendDev()       { return fRendDev; }
    IAswPresSite&           presSite()      { return fPresSite; }
    
    virtual void            ObjectIsEmpty( void );

private:
    IEmbeddedComponentImp&  operator=( const IEmbeddedComponentImp& );
    enum Version { kOriginalVersion };

    IEmbeddedComponent*     fpEmbeddedComp;
    IAswAspect*             fpAspect;
    IAswAspect              fContentAspect;
    IAswAspect              fIconAspect;
    CAswObjRendDev          fRendDev;
    IAswPresSite            fPresSite;
    BOOL                    fInitialized;

};


#endif

