/*******************************************************************************
* FILE NAME: immdev.cpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immdev.hpp.                                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
extern "C"
  {
  #include <string.h>
  #include <math.h>
  #define INCL_MCIOS2
  #define INCL_WINATOM
  #define INCL_WINMESSAGEMGR
  #define IC_MULTIMEDIA
  #include <immdefs.h>
  }

#include <immdev.hpp>
#include <ithread.hpp>
#include <immexcpt.hpp>
#include <itrace.hpp>
#include <immevt.hpp>
#include <immdvhdr.hpp>
#include <immowin.hpp>
#include <imultsta.hpp>
#include <inotifev.hpp>

#ifdef IC_WIN
#include <iqueue.h>
typedef IQueue<unsigned short> ListOfUnacknowledgedCommands ;
#endif


// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMMDEV_CPP_
  #include <ipagetun.h>
#endif

// Initialize static objects
#pragma data_seg(ICLStaticConst)
const unsigned long
  IMMDevice::cdxa       = 17,
  IMMDevice::audioCD    = 3,
  IMMDevice::dat        = 4,
  IMMDevice::audioTape  = 5,
  IMMDevice::waveAudio  = 7,
  IMMDevice::sequencer  = 8,
  IMMDevice::ampMixer   = 9,
  IMMDevice::videoTape  = 1,
  IMMDevice::digitalVideo = 12,
  IMMDevice::videoDisc  = 2,
  IMMDevice::overlay    = 10,
  IMMDevice::animation  = 11,
  IMMDevice::speaker    = 13,
  IMMDevice::headphone  = 14,
  IMMDevice::microphone = 15,
  IMMDevice::monitor    = 16,
  IMMDevice::other      = 6,
  IMMDevice::allDevices = 0xFFFF;
#pragma data_seg()

class IMMDeviceData : public IBase
{
public:
  IMMDeviceData(IString anAlias, IString curDev)
         : id(0),
           userParm(0),
           alias(anAlias),
           errorId(0),
           aDeviceWindow(0),
           deviceHandler(0),
           closeOnDestroy(true),
           currentDevice(curDev),
           sawFirstPassDevice(false),
           openStatus(IMMDevice::openNotAttempted)
#ifdef IC_WIN
           , aSystemMixer(0)
           , baseDevice(curDev)
           , myTimeFormatTouched(false)
#endif
     { }

IMMMessageWindow
  *aDeviceWindow;
IMMDeviceHandler
  *deviceHandler;
Boolean
  closeOnDestroy,
  sawFirstPassDevice;
unsigned short
  userParm;
unsigned long
  id,
  errorId;
IString
  alias,
  currentDevice;
IMMDevice::OpenStatus
  openStatus;
static unsigned long
  aliasCount;
#ifdef IC_WIN
unsigned long
  aSystemMixer;
IString
  baseDevice;                   // physical device (ie WAVEAUDIO)
IMMTime::Format
  myTimeFormat;
Boolean
  myTimeFormatTouched;
ListOfUnacknowledgedCommands
  unAckCommandsList ;
#endif

};

#pragma data_seg(ICLNonConst)
unsigned long IMMDeviceData::aliasCount = 0;
#pragma data_seg()

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IMMDevice::firstUnacknowledgedCommand                                        |
|                                                                              |
| Returns 1st element in queue of commands that have not finished              |
------------------------------------------------------------------------------*/
unsigned short IMMDevice::firstUnacknowledgedCommand()
{
   IMODTRACE_DEVELOP("IMMDevice::firstUnacknowledgedCommand") ;
   unsigned short usMCIMsgId(0) ;

   if (! (fDeviceData->unAckCommandsList).isEmpty())
      (fDeviceData->unAckCommandsList).dequeue(usMCIMsgId) ;

   return usMCIMsgId ;
} // firstUnacknowledgedCommand
#endif

/*------------------------------------------------------------------------------
| IMMDevice::acquire                                                           |
| Requests assignment of (access to) the device.  If queued is true, the       |
| request is queued and executed as soon as the device resources are available.|
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::acquire(ShareMode acquire,
                              Boolean queued,
                              CallType call)
{
  IMODTRACE_DEVELOP("IMMDevice::acquire");


#ifdef IC_PM
  IASSERTSTATE(isOpen());

  MCI_GENERIC_PARMS parms;

  parms.hwndCallback = HWND(fDeviceData->aDeviceWindow->handle());
  sendCommand(MCI_ACQUIREDEVICE,
              acquire | call | (queued ? MCI_ACQUIRE_QUEUE : 0),
              &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::supportsVolumeAdjustment                                          |
| Returns true if the device supports software control of its volume.          |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::supportsVolumeAdjustment(CallType call) const
{
  IMODTRACE_DEVELOP("IMMDevice::supportsVolumeAdjustment");
#ifdef IC_PM

  Boolean rc =  itemCapability(MCI_GETDEVCAPS_CAN_SETVOLUME, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
#endif
#ifdef IC_WIN
  return true;
#endif
}

/*------------------------------------------------------------------------------
| IMMDevice::supportsRecordInsertion                                           |
| Returns true if the device supports insertion of data while recording.       |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::supportsRecordInsertion(CallType call) const
{
  IMODTRACE_DEVELOP("IMMDevice::supportsRecordInsertion");
#ifdef IC_PM

  Boolean rc =  itemCapability(MCI_GETDEVCAPS_CAN_RECORD_INSERT, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
#endif
#ifdef IC_WIN
  return false;
#endif
}

/*------------------------------------------------------------------------------
| IMMDevice::supportsDisableEject                                                   |
| Returns true if the device can disable the manual                            |
| ejection of media.                                                           |
------------------------------------------------------------------------------*/
Boolean IMMDevice::supportsDisableEject(CallType call) const
{
  IMODTRACE_DEVELOP("IMMDevice::supportsDisableEject");

#ifdef IC_PM
  Boolean rc = itemCapability(MCI_GETDEVCAPS_CAN_LOCKEJECT, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
#endif
#ifdef IC_WIN
  return false;
#endif
}
/*------------------------------------------------------------------------------
| IMMDevice::supportsStreaming                                                 |
| Returns true if the device can continuously transfer digital data to or from |
| another device, as determined by the device's connections.                   |
------------------------------------------------------------------------------*/
Boolean IMMDevice::supportsStreaming(CallType call) const
{
  IMODTRACE_DEVELOP("IMMDevice::supportsStreaming");

#ifdef IC_PM
  Boolean rc = itemCapability(MCI_GETDEVCAPS_CAN_STREAM, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
#endif
#ifdef IC_WIN
  return false;
#endif
}

/*------------------------------------------------------------------------------
| IMMDevice::supportsDigitalTransfer                                                |
| Returns true if the device can internally process digital data, for example  |
| a digital-to-analog converter on a CD player.                                |
------------------------------------------------------------------------------*/
Boolean IMMDevice::supportsDigitalTransfer(CallType call) const
{
  IMODTRACE_DEVELOP("IMMDevice::supportsDigitalTransfer");

#ifdef IC_PM
  Boolean rc =  itemCapability(MCI_GETDEVCAPS_CAN_PROCESS_INTERNAL, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
#endif
#ifdef IC_WIN
  return false;
#endif
}
/*------------------------------------------------------------------------------
| IMMDevice::supportsCommand                                                   |
| Returns true if the device supports the passed in command.                   |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::supportsCommand(IMMNotifyEvent::Command command,
                                  CallType call) const
{
  IMODTRACE_DEVELOP("IMMDevice::supportsCommand");

#ifdef IC_PM
  IASSERTSTATE(isOpen());

  MCI_GETDEVCAPS_PARMS parms;

  memset(&parms, 0, sizeof(MCI_GETDEVCAPS_PARMS));
  parms.hwndCallback = HWND(fDeviceData->aDeviceWindow->handle());
  parms.usMessage    = (unsigned short)command;
  ((IMMDevice*)this)->sendCommand(MCI_GETDEVCAPS,
              MCI_GETDEVCAPS_MESSAGE | call,
              &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return parms.ulReturn;
#endif
#ifdef IC_WIN
  return false;
#endif
}

/*------------------------------------------------------------------------------
| IMMDevice::supportsEject                                                          |
| Returns true if the device can eject media.                                  |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::supportsEject(CallType call) const
{
  IMODTRACE_DEVELOP("IMMDevice::supportsEject");

  Boolean rc =  itemCapability(MCI_GETDEVCAPS_CAN_EJECT, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}


/*------------------------------------------------------------------------------
| IMMDevice::supportsPlay                                                           |
| Returns true if the device can play.                                         |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::supportsPlay(CallType call) const
{
  IMODTRACE_DEVELOP("IMMDevice::supportsPlay");

  Boolean rc =  itemCapability(MCI_GETDEVCAPS_CAN_PLAY, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDevice::supportsRecord                                                         |
| Returns true if the device can record.                                       |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::supportsRecord(CallType call) const
{
  IMODTRACE_DEVELOP("IMMDevice::supportsRecord");

#ifdef IC_WIN
  // For open() we need to know if a device can support recording.
  // Since the device is not opened yet, we cannot use itemcapability(),
  // so we use the string interface to query the base device.

  // To make it more interesting, we first check if the device is
  // a CD player to avoid a bug in the Win95 MCI (cannot query the
  // generic device's caps if an instance is already open - share violation).
  if ( "CDAUDIO" == fDeviceData->baseDevice )
     return false;

  if ( !isOpen() )
  {
     // cast away the const to call sendCommand (which is not const)
     IMMDevice * const nonConstThis = (IMMDevice * const) this;

     IString str("capability ");
     str += fDeviceData->baseDevice;
     str += " can record";
     IString resultString = nonConstThis->sendCommand(str);
     if (lastError())
        ITHROWMMERROR(lastError(),"sendCommand");
     if (resultString[0] == 't')
        return true;
     else
        return false;
  }
#endif

  Boolean rc =  itemCapability(MCI_GETDEVCAPS_CAN_RECORD, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDevice::supportsSave                                                           |
| Returns true if the device can save data to some type of media (for example  |
| a file).                                                                     |
------------------------------------------------------------------------------*/
Boolean IMMDevice::supportsSave(CallType call) const
{
  IMODTRACE_DEVELOP("IMMDevice::supportsSave");

  Boolean rc =  itemCapability(MCI_GETDEVCAPS_CAN_SAVE, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDevice::supportsAudio                                                     |
| Returns true if the device has support for audio playback.                   |
------------------------------------------------------------------------------*/
Boolean IMMDevice::supportsAudio(CallType call) const
{
  IMODTRACE_DEVELOP("IMMDevice::supportsAudio");

  Boolean rc =  itemCapability(MCI_GETDEVCAPS_HAS_AUDIO, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDevice::supportsVideo                                                     |
| Returns true if the device has support for video playback.                   |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::supportsVideo(CallType call) const
{
  IMODTRACE_DEVELOP("IMMDevice::supportsVideo");

  Boolean rc =  itemCapability(MCI_GETDEVCAPS_HAS_VIDEO, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDevice::itemCapability                                                    |
| Gets the device's capabilities.                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMDevice::itemCapability(unsigned long item,
                                        CallType call) const
{
  IMODTRACE_DEVELOP("IMMDevice::itemCapability");

  IASSERTSTATE(isOpen());

  MCI_GETDEVCAPS_PARMS parms;
  CallType waitOrNotify(call) ;

  memset(&parms, 0, sizeof(parms));
#ifdef IC_PM
  parms.hwndCallback = HWND(fDeviceData->aDeviceWindow->handle());
  parms.ulItem       = item;
#endif
#ifdef IC_WIN
  parms.dwCallback = fDeviceData->aDeviceWindow->handle().asUnsigned();
  parms.dwItem     = item;
  waitOrNotify     = IMMDevice::wait ;  // for Win32, simplify our life a bit
                                        // so we don't have to handle too many
                                        // MM_MCINOTIFYs with nulls in mp2
#endif
  ((IMMDevice*)this)->sendCommand(MCI_GETDEVCAPS,
              MCI_GETDEVCAPS_ITEM | waitOrNotify,
              &parms);
#ifdef IC_PM
  return parms.ulReturn;
#endif
#ifdef IC_WIN
  return parms.dwReturn;
#endif
}


/*------------------------------------------------------------------------------
| IMMDevice::requiresFiles                                                     |
| Returns true if the device requires the use of files.                        |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::requiresFiles(CallType call) const
{
  IMODTRACE_DEVELOP("IMMDevice::requiresFiles");

#ifdef IC_WIN
  // For open() we need to know if a device is compound or not.
  // Since the device is not opened yet, we cannot use itemcapability(),
  // so we use the string interface to query the base device.

  // To make it more interesting, we first check if the device is
  // a CD player to avoid a bug in the Win95 MCI (cannot query the
  // generic device's caps if an instance is already open - share violation).
  if ( "CDAUDIO" == fDeviceData->baseDevice )
     return false;

  if ( !isOpen() )
  {
     // cast away the const to call sendCommand (which is not const)
     IMMDevice * const nonConstThis = (IMMDevice * const) this;

     IString str("capability ");
     str += fDeviceData->baseDevice;
     str += " uses files";
     IString resultString = nonConstThis->sendCommand(str);
     if (lastError())
        ITHROWMMERROR(lastError(),"sendCommand");
     if (resultString[0] == 't')
        return true;
     else
        return false;
  }
#endif

  Boolean rc =  itemCapability(MCI_GETDEVCAPS_USES_FILES, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDevice::close                                                             |
| Closes the device.                                                           |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::close(CallType call)
{
  IMODTRACE_DEVELOP("IMMDevice::close");
  IASSERTSTATE(isOpen());

  MCI_GENERIC_PARMS parms;

  ITRACE_DEVELOP("IMMDevice::close, DeviceID = " + IString(fDeviceData->id)
                 + ",name=" + IString(fDeviceData->currentDevice));

#ifdef IC_PM
  parms.hwndCallback = HWND(fDeviceData->aDeviceWindow->handle());
#endif
#ifdef IC_WIN
  parms.dwCallback = fDeviceData->aDeviceWindow->handle().asUnsigned();
#endif
  sendCommand(MCI_CLOSE, call, &parms);
  if (lastError() && call == IMMDevice::wait)
     ITHROWMMERROR(lastError(),"sendCommand");
  releaseAlias(fDeviceData->alias);
  fDeviceData->id = 0;
  fDeviceData->currentDevice = "";
  fDeviceData->sawFirstPassDevice = false;
  fDeviceData->openStatus = openNotAttempted;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::deviceType                                                        |
| Returns the type of the device.                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMDevice::deviceType(CallType call) const
{
  IMODTRACE_DEVELOP("IMMDevice::deviceType");

  unsigned long rc = itemCapability(MCI_GETDEVCAPS_DEVICE_TYPE, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}


/*------------------------------------------------------------------------------
| IMMDevice::mode                                                              |
| Returns the current mode of the device.                                      |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice::Mode IMMDevice::mode(CallType call) const
{
  IMODTRACE_DEVELOP("IMMDevice::mode");

#ifdef IC_WIN
  // Win32 compound devices do not recognize the mode query
  // when opened without a device element, unless they are opened
  // in "new record" mode. Since some compound devices
  // do not support record, there are valid cases where this
  // will happen (OS/2 supports mode for ANY device). We intercept
  // these cases, and emulate the mode query.
  // This logic essentially translates to:
  //   If the device is not open, OR
  //   If the device is a compound device and it is open,
  //   but it's device element is not loaded...
  //
  // Note that if this evaluates to be false, execution falls through
  // to the normal processing. At that point, either a device is open and
  // loaded OR not open. In the case that it is not open, the IASSERT
  // in itemStatus will detect and except. In the case that it is open
  // and loaded, it is a fully functional device that supports mode.
  if ( !isOpen() ||
       ( isOpen()
         && fDeviceData->currentDevice == fDeviceData->baseDevice // file not loaded
         && requiresFiles()                       // compound device
         && !supportsRecord() )                   // not recordable
     )
  {
     ITRACE_DEVELOP("IMMDevice::mode, emulating MCI_STATUS_MODE (notReady).");
     return IMMDevice::notReady;
  }
#endif

  IMMDevice::Mode rc;
  IMMMODE(rc, itemStatus(MCI_STATUS_MODE, 0, call));
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");

  ITRACE_DEVELOP("IMMDevice::mode, mode = [" + IString(rc) + "]");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDevice::open                                                              |
| Opens the file or device. First creates an alias for the device, then calls  |
| sendString to delegate most of the work to the media control interface.      |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::open(const IString &fileOrDevice,
                                       Boolean shareable,
                                       CallType call)
{
  IString temp;
#ifdef IC_WIN
  Boolean newRecordMode = False;
#endif

  IMODTRACE_DEVELOP("IMMDevice::open #1");
  ITRACE_DEVELOP("IMMDevice::open #1, fileOrDevice = "+fileOrDevice);
  ITRACE_DEVELOP("IMMDevice::open #1, shareable    = "+IString(shareable));
  ITRACE_DEVELOP("IMMDevice::open #1, call         = "+IString(call));

#ifdef IC_WIN
  // Newly constructed compound devices specifying openNow=true are
  // opened in "new record" mode if that device is able to record
  if ( fileOrDevice == fDeviceData->baseDevice  // CTOR openNow=true
       && requiresFiles()                       // compound device
       && supportsRecord() )                    // recordable
  {
     ITRACE_DEVELOP("IMMDevice::open #1, opening [" + fileOrDevice +
                    "] device for new recording");
     newRecordMode = True;
  }
#endif

  if (fileOrDevice.length() == 0)
  {
#ifdef IC_PM
     temp = deviceName();
#endif
#ifdef IC_WIN
     // In Win32, a null file name passed indicates that a device
     // should be opened for new recording.
     temp = fDeviceData->baseDevice;
     ITRACE_DEVELOP("IMMDevice::open #1, opening [" + temp +
                    "] device for new recording");
     newRecordMode = True;
#endif
  }
  else
  {
     temp=fileOrDevice;
  } /* endif */

  IString resultString(0,512);

  if (!isOpen() && isOpenStringValid(temp))
  {
     establishAlias();
     if (aliasName().length())
     {
        IString str("open ");

#ifdef IC_WIN
        // Open device in record mode if applicable
        if (newRecordMode)
           str += "new type ";
#endif

        if (temp.length() != 0)
        {
           str += temp;
           fDeviceData->currentDevice = temp;
        }
        else if (fDeviceData->currentDevice.length() !=0 )
                str += fDeviceData->currentDevice;

        if (shareable)
           str += " shareable";

        str += " alias " + aliasName() + (call == wait ? " wait" : " notify");

        resultString = sendCommand(str);

        fDeviceData->id = resultString.asUnsigned();

        ITRACE_DEVELOP("IMMDevice::open #1, device ID = "+IString(fDeviceData->id));

        if (call == nowait)
        {
           // If MCI_NOTIFY is specified and an error occurs during mciSendString
           // or mciSendCommand a notification is not sent. So we do it.
           if (lastError())
              fDeviceData->aDeviceWindow->handle().postEvent(MM_MCINOTIFY,
                IEventParameter1((unsigned short)(lastError()),userParameter()),
                IEventParameter2(0,MCI_OPEN));
        }
        else if (call == wait)
        {
           if (lastError())
              ITHROWMMERROR(lastError(),"sendCommand");
           else
           {
              // Send a message to this device's handler. This is necessary
              // to activate observer notification.
              fDeviceData->aDeviceWindow->handle().sendEvent(MM_MCINOTIFY,
                IEventParameter1(MCI_NOTIFY_SUCCESSFUL,userParameter()),
                IEventParameter2(0,MCI_OPEN));
           }
        } /* endif */
     }
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::openOnThread                                                      |
| Opens the file or device, creating a thread to do the actual opening.  This  |
| allows pre-emption, thus freeing up the windowing system.                    |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::openOnThread( const IString &fileOrDevice,
                                    Boolean shareable,
                                    CallType call)
{
  IMODTRACE_DEVELOP("IMMDevice::openOnThread #1");

#ifdef IC_WIN
  ITRACE_DEVELOP("IMMDevice::openOnThread #1, Win32 bypass--threads not supported in Win32 MCI");
  open(fileOrDevice, shareable, call);
#endif /* IC_WIN */

#ifdef IC_PM
  IString temp;
  QMSG qmsg;

  class OpenSyncFn : public IThreadFn
  {
  public:
    OpenSyncFn(IMMDevice* dev, const IString& fileOrDev, Boolean shareable)
             : device(dev),
               file(fileOrDev),
               share(shareable)
               {;}
  void
    run() { device->open(file, share, nowait); }

  private:
//    OpenSyncFn           ( const OpenSyncFn& fun );
//  OpenSyncFn
//   &operator=            ( const OpenSyncFn& fun );
  IMMDevice
    *device;
  const IString
    file;
  Boolean
    share;
  };

  // Create our thread function object then construct an IThread
  // with the function object and auto initializing PM
  if (fileOrDevice.length() == 0)
  {
     temp=deviceName();
  } else {
     temp=fileOrDevice;
  } /* endif */
  OpenSyncFn *openSync = new OpenSyncFn(this, temp, shareable);
  IThread thread(openSync, true);

  while (openFailed != openStatus())
  {
    if (IPEEKMSG(IThread::current().anchorBlock(),
                   &qmsg,
                   (HWND)0,
                   0,
                   0,
                   PM_REMOVE))
    {
       IDISPATCHMSG(IThread::current().anchorBlock(), &qmsg);
    }
  }
  if (openFailed == openStatus())
     ITHROWMMERROR(lastError(),"sendCommand");

#endif /* IC_PM */

  return *this;
}


/*------------------------------------------------------------------------------
| IMMDevice::product                                                           |
| Returns the description of the hardware associated with this device.         |
|                                                                              |
------------------------------------------------------------------------------*/
IString IMMDevice::description(CallType call) const
{
  IMODTRACE_DEVELOP("IMMDevice::product");

  IASSERTSTATE(isOpen());

  char           product[MAX_PRODINFO + 1];
  MCI_INFO_PARMS parms;
  IString        result;

  memset(&parms, 0, sizeof(MCI_INFO_PARMS));
  memset(product, 0, sizeof(product));
#ifdef IC_PM
  parms.hwndCallback = HWND(fDeviceData->aDeviceWindow->handle());
  parms.pszReturn = product;
  parms.ulRetSize = MAX_PRODINFO;
#endif
#ifdef IC_WIN
  parms.dwCallback = fDeviceData->aDeviceWindow->handle().asUnsigned();
  parms.lpstrReturn = product;
  parms.dwRetSize = MAX_PRODINFO;
#endif
  ((IMMDevice*)this)->sendCommand(MCI_INFO, MCI_INFO_PRODUCT | call,
                                  (void *)&parms, 0);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  result = product;
  return result;
}

/*------------------------------------------------------------------------------
| IMMDevice::isAcquired                                                        |
| Returns true if the device is ready.  A device must be acquired (in ready    |
| state) before it can be used.                                                |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::isAcquired(CallType call) const
{
  IMODTRACE_DEVELOP("IMMDevice::ready");

#ifdef IC_WIN
  // In Win32, a device never has to be acquired. Also, MCI_STATUS_READY
  // has a different meaning than in OS/2, so we intercept and emulate.
  // For this function, we don't care if a compound device has a file
  // loaded yet or not - as long as it is opened, it is considered acquired.
  // If it is not open, itemStatus will detect and except.
  if ( isOpen() )
  {
     ITRACE_DEVELOP("IMMDevice::mode, emulating MCI_STATUS_READY.");
     return true;
  }
#endif

  Boolean rc = itemStatus(MCI_STATUS_READY,0, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDevice::speedFormat                                                       |
| Returns the currently set speed format for the device.                       |
|                                                                              |
------------------------------------------------------------------------------*/
IMMSpeed::Format IMMDevice::speedFormat(CallType call) const
{
  IMODTRACE_DEVELOP("IMMDevice::speedFormat");

#ifdef IC_PM
  IMMSpeed::Format rc = (IMMSpeed::Format)itemStatus(MCI_STATUS_SPEED_FORMAT,0, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
#endif
#ifdef IC_WIN
  return IMMSpeed::framesPerSecond;     // only speed Win32 MCI knows about
#endif
}

/*------------------------------------------------------------------------------
| IMMDevice::setSpeedFormat                                                    |
| Sets the speed format for the device.                                        |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::setSpeedFormat(IMMSpeed::Format format,
                                     CallType call)
{
  IMODTRACE_DEVELOP("IMMDevice::setSpeedFormat");

  IASSERTSTATE(isOpen());

  MCI_SET_PARMS parms;

  memset(&parms, 0, sizeof(parms));
#ifdef IC_PM
  parms.hwndCallback  = HWND(fDeviceData->aDeviceWindow->handle());
  parms.ulSpeedFormat = (unsigned long)format;
  sendCommand(MCI_SET,
              MCI_SET_SPEED_FORMAT | call,
              &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
#endif
#ifdef IC_WIN
// nothing for now ... cannot set speed format in Win32 MCI
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| immdevice::settimeformat                                                     |
| Sets the time format for the device.                                         |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::setTimeFormat(IMMTime::Format format,
                                    CallType call)
{
  IMODTRACE_DEVELOP("IMMDevice::setTimeFormat");
  ITRACE_DEVELOP(IString("IMMDevice::setTimeFormat, format = ")+IString(format));

  MCI_SET_PARMS parms;

  memset(&parms, 0, sizeof(parms));

#ifdef IC_PM
  IASSERTSTATE(isOpen());
  parms.hwndCallback = HWND(fDeviceData->aDeviceWindow->handle());
  parms.ulTimeFormat = (unsigned long)format;
  sendCommand(MCI_SET, MCI_SET_TIME_FORMAT | call, &parms);
#endif
#ifdef IC_WIN
  // This logic essentially translates to:
  //   If the device is open, and it is a fully functional device (compound
  //   devices either have a file loaded or support record)...
  if ( isOpen() &&
       !(fDeviceData->currentDevice == fDeviceData->baseDevice // file not loaded
         && requiresFiles()                       // compound device
         && !supportsRecord()) )                  // is recordable
  {
     parms.dwCallback = fDeviceData->aDeviceWindow->handle().asUnsigned();
     IMMSETTIME(parms.dwTimeFormat, (DWORD)format);
     ITRACE_DEVELOP("IMMDevice::setTimeFormat, dwTimeFormat = "+IString(parms.dwTimeFormat));
     sendCommand(MCI_SET, MCI_SET_TIME_FORMAT | call, &parms);
  }

  // Save setting for possible future reload (whatever state the device is in).
  ITRACE_DEVELOP("IMMDevice::setTimeFormat, saving for MCI_LOAD emulation.");
  fDeviceData->myTimeFormat = format;
  fDeviceData->myTimeFormatTouched = true;
#endif

  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::timeFormat                                                        |
| Returns the currently set time format for the device.                        |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTime::Format IMMDevice::timeFormat(CallType call) const
{
  IMODTRACE_DEVELOP("IMMDevice::timeFormat");

#ifdef IC_WIN
  // Win32 compound devices do not recognize the time format query
  // when opened without a device element, unless they are opened
  // in "new record" mode. Since some compound devices
  // do not support record, there are valid cases where this
  // will happen (OS/2 supports this query for ANY device). We intercept
  // these cases, and emulate the query.
  // This logic essentially translates to:
  //   If the device is not open, OR
  //   If the device is a compound device and it is open,
  //   but it's device element is not loaded...
  if ( !isOpen() ||
       ( isOpen()
         && fDeviceData->currentDevice == fDeviceData->baseDevice // file not loaded
         && requiresFiles()                       // compound device
         && !supportsRecord() )                   // not recordable
     )
  {
     ITRACE_DEVELOP("IMMDevice::timeFormat, emulating MCI_STATUS_TIME_FORMAT.");
     if (fDeviceData->myTimeFormatTouched)
        return fDeviceData->myTimeFormat;
     else
        return IMMTime::milliseconds;    // Default setting
  }
#endif

  IMMTime::Format rc;
  IMMTIME(rc,itemStatus(MCI_STATUS_TIME_FORMAT,0, call));
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}


/*------------------------------------------------------------------------------
| IMMDevice::setVolume                                                         |
| Sets the volume for the device.                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::setVolume(unsigned long volume,
                                IMMDevice::AudioChannel channel,
                                const IMMMillisecondTime &over,
                                CallType call)
{
  IMODTRACE_DEVELOP("IMMDevice::setVolume");


#ifdef IC_PM
  IASSERTSTATE(isOpen());

  MCI_SET_PARMS parms;
  Boolean       includeOver = false;

  memset(&parms, 0, sizeof(MCI_SET_PARMS));
  includeOver = over.isValid();
  parms.hwndCallback = HWND(fDeviceData->aDeviceWindow->handle());
  parms.ulAudio      = (unsigned long)channel;
  parms.ulLevel      = (unsigned long)volume;
  parms.ulOver       = (includeOver ? (unsigned long)over : 0);
  sendCommand(MCI_SET, MCI_SET_AUDIO | MCI_SET_VOLUME |
              (includeOver ?  MCI_OVER : 0) | call, &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
#endif
#ifdef IC_WIN
  // Note that MCI_OVER is not supported by Win32
  unsigned long       ulDetails[2];
  Boolean             uniform = false;  // Assume a specific channel is requested

  if (channel == all)
  {
     uniform = true;       // 1 value returned
  }
  else
  {
     // Get current values of both channels, so we can rembember the one that won't change
     if (!mixerControlValues(
                 MIXERLINE_COMPONENTTYPE_DST_SPEAKERS,
                 MIXERCONTROL_CONTROLTYPE_VOLUME,
                 ulDetails, uniform))
        ITHROWMMERROR(lastError(),"mixerControlValues");
  }

  if (channel == right)
     ulDetails[1] = volume;  // Right channel is in second position
  else
     ulDetails[0] = volume;  // Left channel or uniform value in first position

  if (!setMixerControlValues(
              MIXERLINE_COMPONENTTYPE_DST_SPEAKERS,
              MIXERCONTROL_CONTROLTYPE_VOLUME,
              ulDetails, uniform))
     ITHROWMMERROR(lastError(),"setMixerControlValues");
#endif
  return *this;
}


/*------------------------------------------------------------------------------
| IMMDevice::enableAudio                                                       |
| Turns on or turns off the audio for the device.                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::enableAudio( Boolean enable,
                                   IMMDevice::AudioChannel channel,
                                   const IMMMillisecondTime &over,
                                   CallType call)
{
  IMODTRACE_DEVELOP("IMMDevice::enableAudio");

#ifdef IC_PM
  IASSERTSTATE(isOpen());

  MCI_SET_PARMS parms;
  Boolean       includeOver;

  memset(&parms, 0, sizeof(MCI_SET_PARMS));

  includeOver = over.isValid();
  parms.ulOver       = (includeOver ? (unsigned long)over : 0);
  parms.hwndCallback = HWND(fDeviceData->aDeviceWindow->handle());
  parms.ulAudio      = channel;
  sendCommand(MCI_SET, MCI_SET_AUDIO |
              (enable ? MCI_SET_ON : MCI_SET_OFF) |
              (includeOver ?  MCI_OVER : 0) | call, &parms);
#endif
#ifdef IC_WIN
  // Note: Win32 MCI does not support fadeOver

  // Win 32 MCI_SET barfing on these calls - replace with mixer mute for now
  //parms.dwCallback = (fDeviceData->aDeviceWindow->handle()).asUnsigned() ;
  //IMMSETCHANNEL(parms.dwAudio, channel) ;
  //sendCommand(MCI_SET, MCI_SET_AUDIO |
  //            (enable ? MCI_SET_ON : MCI_SET_OFF) | call,
  //            &parms);

  unsigned short      usDetails[2];

  if (enable)
     usDetails[0] = 0; // for mute, 0=off
  else
     usDetails[0] = 1; // for mute, 1=on

  if (!setMixerControlValues(
              MIXERLINE_COMPONENTTYPE_DST_SPEAKERS,       // master mute
              MIXERCONTROL_CONTROLTYPE_MUTE,
              usDetails, true))    // multiple items = 0 for mute
     ITRACE_DEVELOP("IMMDevice::enableAudio - setMixerControlValues error");
#endif
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::disableAudio                                                      |
| Turns off the audio for the device.                                          |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::disableAudio(IMMDevice::AudioChannel channel,
                                         const IMMMillisecondTime &over,
                                         CallType call)
{
  IMODTRACE_DEVELOP("IMMDevice::disableAudio");

  enableAudio(false,channel,over,call);
  return *this;
}


/*------------------------------------------------------------------------------
| IMMDevice::sendCommand                                                       |
| Sends a command string to the device.  Ultimately calls the mciSendString    |
| function.                                                                    |
------------------------------------------------------------------------------*/
IString   IMMDevice::sendCommand(const IString &command)
{
  IMODTRACE_DEVELOP("IMMDevice::sendCommand(string #1)");
  return sendCommand(command, userParameter());
}

/*------------------------------------------------------------------------------
| IMMDevice::sendCommand                                                       |
| Sends a command string to the device.  Ultimately calls the mciSendString    |
| function.                                                                    |
------------------------------------------------------------------------------*/
IString  IMMDevice::sendCommand(const IString &command,
                                unsigned short userParms)
{
  IMODTRACE_DEVELOP("IMMDevice::sendCommand(string #2)");

  unsigned long rc;
  IString achResult(0,513);

  rc = IMultimediaStatics::sendString(command,
                     achResult,
                     (unsigned short)(achResult.length()),
                     HWND(fDeviceData->aDeviceWindow->handle())
                     ,userParms);
  rc &= 0xFFFF;
  if (rc)
  {
#ifdef IC_PM
     if (rc != MCIERR_INSTANCE_INACTIVE)
     {
        setLastError(rc);
     } else {
        acquire();
        return sendCommand(command,userParms);
     } /* endif */
#endif
#ifdef IC_WIN
     setLastError(rc);
#endif
  } /* endif */
  else
     setLastError(0);

  return achResult;
}

/*------------------------------------------------------------------------------
| IMMDevice::sendCommand                                                       |
| Sends a command to the device.  Ultimately calls the mciSendCommand          |
| function.                                                                    |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::sendCommand(unsigned short message,
                            unsigned long param1,
                            void *param2)
{
  IMODTRACE_DEVELOP("IMMDevice::sendCommand(command #1)");

  sendCommand(message, param1, param2, userParameter());
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::sendCommand                                                       |
| Sends a command to the device.  Ultimately calls the mciSendCommand          |
| function.                                                                    |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::sendCommand(unsigned short message,
                            unsigned long param1,
                            void *param2,
                            unsigned short userParms)
{
  IMODTRACE_DEVELOP("IMMDevice::sendCommand(command #2)");

  unsigned long rc;
  rc = IMultimediaStatics::sendCommand((unsigned short)deviceId(),
                      message,
                      param1,
                      param2
                      ,userParms);

  rc &= 0xFFFF;
  if (rc)
  {
#ifdef IC_PM
     if (rc == MCIERR_INSTANCE_INACTIVE)
     {
        acquire();
        sendCommand( message, param1, param2);
     } else {
        setLastError(rc);
     } /* endif */
#endif
#ifdef IC_WIN
        setLastError(rc);
#endif
  } else {
     setLastError(0);
     if (param1 & MCI_WAIT)
     {
        IMMNotifyEvent notifyEvent(IEvent(fDeviceData->aDeviceWindow->handle(),
                                          MM_MCINOTIFY,
                                          IEventParameter1(0,userParameter()),
                                          IEventParameter2(0,message)));
        notifyObservers(INotificationEvent(commandNotifyId,*this,true,&notifyEvent));
     } /* endif */
#ifdef IC_WIN
     else
        // doing the command in nowait mode. Save the command
        // because we'll need it when we process the MM_MCINOTIFY message
        (fDeviceData->unAckCommandsList).enqueue(message) ;
#endif
  }
  return *this;
}


/*------------------------------------------------------------------------------
| IMMDevice::setUserParameter                                                  |
| Sets the user parameter.  This parameter is automatically passed in          |
| most of the notification messages.                                           |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::setUserParameter(unsigned short newUserParm)
{
  IMODTRACE_DEVELOP("IMMDevice::setUserParameter");
  fDeviceData->userParm = newUserParm;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::userParameter                                                     |
| Returns the user parameter.                                                  |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned short IMMDevice::userParameter() const
{
  IMODTRACE_DEVELOP("IMMDevice::userParameter");
  return fDeviceData->userParm;
}

/*------------------------------------------------------------------------------
| IMMDevice::lastError                                                         |
| Returns the return code of the last low-level function call.                 |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMDevice::lastError() const
{
  IMODTRACE_DEVELOP("IMMDevice::lastError");
  return fDeviceData->errorId;
}

/*------------------------------------------------------------------------------
| IMMDevice::setLastError                                                      |
| Sets the return code of the last low-level function call.                    |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::setLastError(unsigned long errorId)
{
  IMODTRACE_DEVELOP("IMMDevice::setLastError");
  fDeviceData->errorId=errorId;
  ITRACE_DEVELOP("IMMDevice::setLastError, errorId = "
                 + IString(errorId));
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::release                                                           |
| Releases exclusive use of the device resources.                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::release(CallType call)
{
  IMODTRACE_DEVELOP("IMMDevice::release");

#ifdef IC_PM
  IASSERTSTATE(isOpen());

  MCI_GENERIC_PARMS parms;

  parms.hwndCallback = HWND(fDeviceData->aDeviceWindow->handle());
  sendCommand(MCI_RELEASEDEVICE, call, &parms);
#endif
  return *this;
}


/*------------------------------------------------------------------------------
| IMMDevice::uniqueAlias                                                       |
| Uses the atom table to create a unique alias.                                |
|                                                                              |
------------------------------------------------------------------------------*/
IString IMMDevice::uniqueAlias()
{
  IMODTRACE_DEVELOP("IMMDevice::uniqueAlias");

  IString  base("IMMDevice");
  Boolean  foundUnique = false;
  IString  atom;
  while (!foundUnique)
  {
    fDeviceData->aliasCount++;
    atom = base + IString(fDeviceData->aliasCount);
    if (!IFINDSYSATOM(atom))
    {
       IADDSYSATOM(atom);
       foundUnique = true;
       break;
    }
  }
  return atom;
}

/*------------------------------------------------------------------------------
| IMMDevice::volume                                                            |
| Returns the currently set volume for the device.                             |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMDevice::volume(AudioChannel channel ,CallType call) const
{
  IMODTRACE_DEVELOP("IMMDevice::volume");

#ifdef IC_PM
  unsigned long temp = itemStatus(MCI_STATUS_VOLUME, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  if (channel == right)
     return temp>>16;
  else
     return (temp & 0xFFFF);
#endif
#ifdef IC_WIN
  unsigned long       ulDetails[2];
  Boolean             uniform = false;  // Assume a specific channel is requested

  if (channel == all)
     uniform = true;       // 1 value returned

  // cast away the const to call mixerControlValues (which is not const)
  IMMDevice * const nonConstThis = (IMMDevice * const) this ;

  if (!nonConstThis->mixerControlValues(
              MIXERLINE_COMPONENTTYPE_DST_SPEAKERS,
              MIXERCONTROL_CONTROLTYPE_VOLUME,
              ulDetails, uniform))
     ITHROWMMERROR(lastError(),"mixerControlValues");

  if (channel == right)
     return ( ulDetails[1] );  // Right channel is in second position
  else
     return ( ulDetails[0] );  // Left channel or uniform value in first position
#endif
}

/*------------------------------------------------------------------------------
| IMMDevice::releaseAlias                                                      |
| Frees the atom associated with the passed in alias.                          |
|                                                                              |
------------------------------------------------------------------------------*/
void IMMDevice::releaseAlias(const IString &aliasRel)
{
  IMODTRACE_DEVELOP("IMMDevice::releaseAlias");

  ATOM     atom;

  atom = IFINDSYSATOM(aliasRel);
  if (atom)
     IDELETESYSATOM(atom);
}

/*------------------------------------------------------------------------------
| IMMDevice::establishAlias                                                    |
| Assigns a unique alias to the device.                                        |
|                                                                              |
------------------------------------------------------------------------------*/
void IMMDevice::establishAlias()
{
  IMODTRACE_DEVELOP("IMMDevice::establishAlias");
  fDeviceData->alias = uniqueAlias();
}

/*------------------------------------------------------------------------------
| IMMDevice::deletePendingEvents                                               |
| Removes all of the events of the type passed in from the queue.              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::deletePendingEvents(EventType evtType)
{
  IMODTRACE_DEVELOP("IMMDevice::deletePendingEvents");

  QMSG qmsg;

  while (IPEEKMSG(IThread::current().anchorBlock(),
                    &qmsg,
                    fDeviceData->aDeviceWindow->handle(),
                    (evtType == allEvents) ? notifyEvent : evtType,
                    (evtType == allEvents) ? synchEvent  : evtType,
                    PM_REMOVE));
  return *this;
}



/*------------------------------------------------------------------------------
| IMMDevice::itemStatus                                                       |
| Returns the status of the item passed.                                       |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMDevice::itemStatus(unsigned long item,
                                      unsigned long value,
                                      CallType call) const
{
  IMODTRACE_DEVELOP("IMMDevice::itemStatus");

  IASSERTSTATE(isOpen());

  MCI_STATUS_PARMS parms;
  CallType waitOrNotify(call) ;

  memset(&parms, 0, sizeof(MCI_STATUS_PARMS));
#ifdef IC_PM
  parms.ulItem       = item;
  parms.ulValue      = value;
  parms.hwndCallback = HWND(fDeviceData->aDeviceWindow->handle());
#endif
#ifdef IC_WIN
  parms.dwItem       = item;
  parms.dwTrack      = value;
  parms.dwCallback   = fDeviceData->aDeviceWindow->handle().asUnsigned();
  waitOrNotify       = IMMDevice::wait ;   // see the comments in itemCapabilities
#endif
  ((IMMDevice*)this)->sendCommand(MCI_STATUS, MCI_STATUS_ITEM | waitOrNotify,
                                  (void *)&parms, 0);
#ifdef IC_PM
  return parms.ulReturn;
#endif
#ifdef IC_WIN
  return parms.dwReturn;
#endif
}


/*------------------------------------------------------------------------------
| IMMDevice::isOpenStringValid                                                 |
| Returns true if the string passed in is valid for this device.               |
| Derived classes should override this function for more specific handling.    |
------------------------------------------------------------------------------*/
Boolean IMMDevice::isOpenStringValid(const IString &deviceName) const
{
  IMODTRACE_DEVELOP("IMMDevice::isOpenStringValid");
  // By default, the string looks good. Let the open occur.
  return true;
}

/*------------------------------------------------------------------------------
| IMMDevice::IMMDevice                                                         |
| This constructor creates a new device object and sets up its event handling  |
| protocol.                                                                    |
------------------------------------------------------------------------------*/
IMMDevice::IMMDevice(const IString& fileOrDevice )
         : fDeviceData(new IMMDeviceData(IString(),fileOrDevice))
{
  IMODTRACE_DEVELOP("IMMDevice::ctor #1");
  ITRACE_DEVELOP("IMMDevice::ctor #1 - device name: [" + fileOrDevice + "]");

  fDeviceData->aDeviceWindow = new IMMMessageWindow(*this);
  fDeviceData->deviceHandler = new IMMDeviceHandler();
  fDeviceData->deviceHandler->handleEventsFor(this);
}

/*------------------------------------------------------------------------------
| IMMDevice::IMMDevice                                                         |
| This constructor wrappers an existing device, and gives a derived class the  |
| ability to specify a desired alias (instead of ICLUI generating it).         |
------------------------------------------------------------------------------*/
IMMDevice::IMMDevice(unsigned long newDeviceId, const IString &newAlias)
         : fDeviceData(new IMMDeviceData(newAlias,IString()))
{
  IMODTRACE_DEVELOP("IMMDevice::ctor #2");
  ITRACE_DEVELOP("IMMDevice::ctor #2 - device ID: " + IString(newDeviceId));
  ITRACE_DEVELOP("IMMDevice::ctor #2 - newAlias : [" + newAlias + "]");

  fDeviceData->id          = newDeviceId;
  fDeviceData->openStatus = openSuccessful;
  fDeviceData->sawFirstPassDevice = true;
  fDeviceData->closeOnDestroy = true;
  fDeviceData->aDeviceWindow = new IMMMessageWindow(*this);
  fDeviceData->deviceHandler = new IMMDeviceHandler();
  fDeviceData->deviceHandler->handleEventsFor(this);
}

/*------------------------------------------------------------------------------
| IMMDevice::~IMMDevice                                                        |
| This destructor performs final housekeeping.                                 |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice::~IMMDevice()
{
  IMODTRACE_DEVELOP("IMMDevice::dtor");

  if (fDeviceData)
  {
     if (isOpen() && fDeviceData->closeOnDestroy)
        close(nowait);

     if (fDeviceData->deviceHandler)
     {
        fDeviceData->deviceHandler->stopHandlingEventsFor(this);
        delete fDeviceData->deviceHandler;
     }

     if (fDeviceData->aDeviceWindow)
        delete fDeviceData->aDeviceWindow;

#ifdef IC_WIN
     if ( fDeviceData->aSystemMixer )      // Has the system mixer been opened?
     {
        ITRACE_DEVELOP("IMMDevice::dtor - closing system mixer handle: "
                       + IString(fDeviceData->aSystemMixer));
        MMRESULT rc = IMultimediaStatics::mixerClose( (HMIXER)fDeviceData->aSystemMixer );
        if (MMSYSERR_NOERROR != rc)
           ITHROWMMERROR(rc,"mixerClose");
     }
#endif

     delete fDeviceData;

  } /* endif */
}


/*------------------------------------------------------------------------------
| IMMDevice::open                                                              |
| Opens the device. First creates an alias for the device, then calls          |
| sendString to delegate most of the work to the media control interface.      |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::open(unsigned long instance,
                           Boolean shareable,
                           CallType call)
{
  IMODTRACE_DEVELOP("IMMDevice::open #2");

  IString devName = deviceName();

  if (instance)
  {
     devName += IString(instance).rightJustify(2, '0');
  }
  open(devName, shareable, call);
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::openOnThread                                                      |
| Opens the device, creating a thread to do the actual opening.  This          |
| allows pre-emption, thus freeing up the windowing system.                    |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::openOnThread(unsigned long instance,
                                   Boolean shareable)
{
  IMODTRACE_DEVELOP("IMMDevice::openOnThread #2");

  IString devName = deviceName();

  if (instance)
  {
     devName += IString(instance).rightJustify(2, '0');
  }
  openOnThread(devName, shareable);
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::isCloseOnDestroy                                                  |
| Returns true if the device closes itself when the device object is deleted.  |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean   IMMDevice::isCloseOnDestroy() const
{
  IMODTRACE_DEVELOP("IMMDevice::isCloseOnDestroy");
  return fDeviceData->closeOnDestroy;
}

/*------------------------------------------------------------------------------
| IMMDevice::setCloseOnDestroy                                                 |
| Closes the device when this object is destroyed.  This is the default.       |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::setCloseOnDestroy(Boolean close)
{
  IMODTRACE_DEVELOP("IMMDevice::setCloseOnDestroy");
  fDeviceData->closeOnDestroy = close;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::deviceWindow                                                      |
| Returns the window object that the device uses for communication with the    |
| operating system.                                                            |
------------------------------------------------------------------------------*/
IMMMessageWindow& IMMDevice::deviceWindow() const
{
  IMODTRACE_DEVELOP("IMMDevice::deviceWindow");
  return *(fDeviceData->aDeviceWindow);
}

/*------------------------------------------------------------------------------
| IMMDevice::deviceId                                                          |
| Returns the ID of the current device.  This value is                         |
| used by the system to identify the device.                                   |
------------------------------------------------------------------------------*/
unsigned long IMMDevice::deviceId() const
{
  IMODTRACE_DEVELOP("IMMDevice::deviceId");
  return fDeviceData->id;
}

/*------------------------------------------------------------------------------
| IMMDevice::aliasName                                                         |
| Returns the alias associated with the device                                 |
|                                                                              |
------------------------------------------------------------------------------*/
IString IMMDevice::aliasName() const
{
  IMODTRACE_DEVELOP("IMMDevice::aliasName");
  return fDeviceData->alias;
}

/*------------------------------------------------------------------------------
| IMMDevice::deviceName                                                        |
| Returns the name of the device. This could be a file name or a device name.  |
|                                                                              |
------------------------------------------------------------------------------*/
IString IMMDevice::deviceName() const
{
  IMODTRACE_DEVELOP("IMMDevice::deviceName");
  return fDeviceData->currentDevice;
}

/*------------------------------------------------------------------------------
| IMMDevice::wasPassDeviceRequested                                            |
| Returns true if a passDevice event has already been processed.               |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::wasPassDeviceRequested() const
{
  IMODTRACE_DEVELOP("IMMDevice::wasPassDeviceRequested");
  return fDeviceData->sawFirstPassDevice;
}

/*------------------------------------------------------------------------------
| IMMDevice::openStatus                                                        |
| Returns the result of the open attempt.                                      |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice::OpenStatus IMMDevice::openStatus() const
{
  IMODTRACE_DEVELOP("IMMDevice::openStatus");
  return fDeviceData->openStatus;
}

/*------------------------------------------------------------------------------
| IMMDevice::setOpenStatus                                                     |
| Sets the open device flag.                                                   |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::setOpenStatus(OpenStatus success)
{
  IMODTRACE_DEVELOP("IMMDevice::setOpenStatus");
  fDeviceData->openStatus = success;
  ITRACE_DEVELOP("IMMDevice::setOpenStatus, success = "
                 + IString(success));
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::setPassDeviceRequested                                            |
| Sets the pass device flag.  This indicates that the                          |
| device object has received use of the actual hardware device.                |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::setPassDeviceRequested(Boolean sawPassDevice)
{
  IMODTRACE_DEVELOP("IMMDevice::setPassDeviceRequested");
  fDeviceData->sawFirstPassDevice = sawPassDevice;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::isOpen                                                            |
| Returns true if the device is opened.                                        |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::isOpen() const
{
  IMODTRACE_DEVELOP("IMMDevice::isOpen");
#ifdef IC_PM
  return ((fDeviceData->id
           && openStatus() == IMMDevice::openSuccessful
           && wasPassDeviceRequested()) ? true : false);
#endif
#ifdef IC_WIN
  // Device ID of 0 is OK (system mixer device)
  // openStatus and PassDeviceRequested are enough to show a device is open
  return ((openStatus() == IMMDevice::openSuccessful
           && wasPassDeviceRequested()) ? true : false);
#endif
}

/*------------------------------------------------------------------------------
| Returns the Id of the device that is connected to the passed in              |
| connector type of this device.                                               |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMDevice::connectedDeviceId(ConnectorType type,
                                    CallType call) const
{
  IMODTRACE_DEVELOP("IMMDevice::connectedDeviceId");

#ifdef IC_PM
  IASSERTSTATE(isOpen());

  MCI_CONNECTION_PARMS parms;

  memset(&parms, 0, sizeof(MCI_CONNECTION_PARMS));
  parms.ulConnectorType = type;
  parms.hwndCallback = HWND(fDeviceData->aDeviceWindow->handle());
  ((IMMDevice*)this)->sendCommand(MCI_CONNECTION,
                        MCI_CONNECTOR_TYPE |
                        MCI_QUERY_CONNECTION | call, (void *)&parms, 0);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return parms.usToDeviceID;
#endif
#ifdef IC_WIN
  return 0;
#endif
}

/*------------------------------------------------------------------------------
| IMMDevice::enableConnector                                                   |
| Turns on or turns off the connector specified by the passed in connector     |
| type.                                                                        |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::enableConnector(ConnectorType type,
                                      Boolean enable,CallType call)
{
  IMODTRACE_DEVELOP("IMMDevice::enableConnector");

#ifdef IC_PM
  IASSERTSTATE(isOpen());

  MCI_CONNECTOR_PARMS parms;

  memset(&parms, 0, sizeof(MCI_CONNECTOR_PARMS));
  parms.ulConnectorType = type;
  parms.hwndCallback = HWND(fDeviceData->aDeviceWindow->handle());
  sendCommand(MCI_CONNECTOR, (enable ? MCI_ENABLE_CONNECTOR : MCI_DISABLE_CONNECTOR) |
              MCI_CONNECTOR_TYPE | call, (void *)&parms, 0);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
#endif

#ifdef IC_WIN
  unsigned short      usDetails[2];

  // connectors cannot be disabled in Win32 - if they exist, they are
  // always enabled.
  if (!enable)
     return *this;

  switch (type)
  {
     case lineIn:
        usDetails[0] = 1;   // First control in array refers to line-in
        usDetails[1] = 0;   // Second control refers to microphone
        break;

     case microphones:
        usDetails[0] = 0;   // First control in array refers to line-in
        usDetails[1] = 1;   // Second control refers to microphone
        break;

     default:
        return *this;       // Any other connector type is N/A in Win32
  }

  // Work with the Wave Input component of the Win32 MCI system mixer
  if (!setMixerControlValues(
              MIXERLINE_COMPONENTTYPE_DST_WAVEIN,
              MIXERCONTROL_CONTROLTYPE_MUX,
              usDetails, true,2))    // multiple items = 2 for this mux
     ITRACE_DEVELOP("IMMDevice::enableConnector - setMixerControlValues error");
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::disableConnector                                                  |
| Turns off the connector specified by the passed in connector                 |
| type.                                                                        |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::disableConnector(ConnectorType type, CallType call)
{
   IMODTRACE_DEVELOP("IMMDevice::disableConnector");
   return enableConnector(type,false,call);
}


/*------------------------------------------------------------------------------
| IMMDevice::isConnectionSupported                                             |
| Returns true if the connector is valid for this device.                      |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean    IMMDevice::isConnectionSupported(ConnectorType type,CallType call) const
{
  IMODTRACE_DEVELOP("IMMDevice::isConnectionSupported");

#ifdef IC_PM
  MCI_CONNECTORINFO_PARMS parms;

  memset(&parms, 0, sizeof(MCI_CONNECTORINFO_PARMS));
  parms.ulConnectorType = type;
  parms.ulDeviceTypeID  = deviceType();
  parms.hwndCallback = HWND(fDeviceData->aDeviceWindow->handle());
  ((IMMDevice*)this)->sendCommand(MCI_CONNECTORINFO, MCI_ENUMERATE_CONNECTORS |
                                  MCI_CONNECTOR_TYPE | call,
                                  (void *)&parms, 0);
  if (lastError())
  {
     if (lastError() == MCIERR_UNSUPPORTED_CONN_TYPE)
        return false;
     else
        ITHROWMMERROR(lastError(),"sendCommand");
  }
#endif
  return false;
}
/*------------------------------------------------------------------------------
| IMMDevice::isConnectorEnabled                                                |
| Returns true if the connector is enabled for this device.                    |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean    IMMDevice::isConnectorEnabled(ConnectorType type,CallType call) const
{
  IMODTRACE_DEVELOP("IMMDevice::isConnectorEnabled");
#ifdef IC_PM

  MCI_CONNECTOR_PARMS parms;

  memset(&parms, 0, sizeof(MCI_CONNECTOR_PARMS));
  parms.ulConnectorType = type;
  parms.hwndCallback = HWND(fDeviceData->aDeviceWindow->handle());
  ((IMMDevice*)this)->sendCommand(MCI_CONNECTOR,
                                  MCI_QUERY_CONNECTOR_STATUS |
                                  MCI_CONNECTOR_TYPE | call,
                                  (void *)&parms, 0);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  if (parms.ulReturn)
     return true;
  else
     return false;
#endif
#ifdef IC_WIN
  unsigned short      usDetails[2];

  // cast away the const to call mixerControlValues (which is not const)
  IMMDevice * const nonConstThis = (IMMDevice * const) this ;

  if (!nonConstThis->mixerControlValues(
              MIXERLINE_COMPONENTTYPE_DST_WAVEIN,
              MIXERCONTROL_CONTROLTYPE_MUX,
              usDetails, true,2))    // multiple items = 2 for this mux
  {
     ITRACE_DEVELOP("IMMDevice::isConnectorEnabled - setMixerControlValues error");
     return false;
  }

  switch (type)
  {
     case lineIn:
        return usDetails[0]; // First control in array refers to line-in
        break;

     case microphones:
        return usDetails[1]; // Second control refers to microphone
        break;

     default:
        return false;       // Any other connector type is N/A in Win32
  }
#endif
return false;       // catch all
}

#ifdef IC_PM
/*------------------------------------------------------------------------------
| IMMDevice::prerollTime                                                       |
| Returns the amount of preroll time.                                          |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTime  IMMDevice::prerollTime(CallType call) const
{
  IMODTRACE_DEVELOP("IMMDevice::prerollTime");

  unsigned long temp = 0;
  temp = itemCapability(MCI_GETDEVCAPS_PREROLL_TIME, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  if (temp)
     return IMMTime(temp);
  else
     return IMMTime();
}

/*------------------------------------------------------------------------------
| IMMDevice::prerollType                                                       |
| Returns the preroll characteristics of the device.                           |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice::PrerollType IMMDevice::prerollType(CallType call) const
{
  IMODTRACE_DEVELOP("IMMDevice::prerollType");

  IMMDevice::PrerollType rc = (IMMDevice::PrerollType)(itemCapability(MCI_GETDEVCAPS_PREROLL_TYPE, call));
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}
#endif

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IMMDevice::systemMixerHandle                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMDevice::systemMixerHandle() const
{
  IMODTRACE_DEVELOP("IMMDevice::systemMixerHandle");

  if (fDeviceData->aSystemMixer)
  {
     ITRACE_DEVELOP("IMMDevice::systemMixerHandle - returning existing handle: "
                     + IString(fDeviceData->aSystemMixer));
     return fDeviceData->aSystemMixer;
  }

  // Open the system mixer for this device, callback hwnd not needed
  ITRACE_DEVELOP("IMMDevice::systemMixerHandle - opening system mixer for this device");
  unsigned long uMxId = 0;  // System Mixer device's ID=0
  unsigned long hMixer;
  MMRESULT rc;
  rc = IMultimediaStatics::mixerOpen((LPHMIXER)&hMixer, uMxId, 0, 0L, 0);
  if (MMSYSERR_NOERROR != rc)
      ITHROWMMERROR(rc,"mixerOpen");

  fDeviceData->aSystemMixer = hMixer;

  ITRACE_DEVELOP("IMMDevice::systemMixerHandle - new handle: "
                  + IString(fDeviceData->aSystemMixer));

  return fDeviceData->aSystemMixer;
}

/*------------------------------------------------------------------------------
| IMMDevice::setSystemMixerHandle                                              |
| Attaches a handle for the system mixer to this device.                       |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::setSystemMixerHandle(unsigned long hmx)
{
  IMODTRACE_DEVELOP("IMMDevice::setSystemMixerHandle");
  fDeviceData->aSystemMixer = hmx;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::mixerControlValues                                                |
| Obtains values for the mixer control specified.                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::mixerControlValues (
                           unsigned long dwComponentType,
                           unsigned long dwControlType,
                           unsigned long *ulDetails,
                           Boolean uniform,
                           unsigned long multipleItems,
                           unsigned long hmx)
{
  IMODTRACE_DEVELOP("IMMDevice::mixerControlValues #1");

  MIXERLINE           mxl;
  MMRESULT            rc;
  unsigned long       hMixer = hmx;

  if (!hMixer)      // Does caller want to use the system mixer handle?
     hMixer = systemMixerHandle();

  // Prep MIXERLINE struct
  mxl.cbStruct      = sizeof(mxl);
  mxl.dwComponentType = dwComponentType;

  // Get line information for component type specified (if it exists)
  rc = IMultimediaStatics::mixerGetLineInfo((HMIXEROBJ)hMixer, &mxl, MIXER_GETLINEINFOF_COMPONENTTYPE);
  if (MMSYSERR_NOERROR != rc)
  {
     ITRACE_DEVELOP("IMMDevice::mixerControlValues - mixerGetLineInfo() failed, rc="
                     +IString(rc));
     setLastError(rc);
     return false;
  }
  ITRACE_DEVELOP("IMMDevice::mixerControlValues - line ID #"
                  +IString(mxl.dwLineID));

  // Quick sanity check
  if (mxl.cControls == 0)
  {
     ITRACE_DEVELOP("IMMDevice::mixerControlValues - line does not contain controls");
     // Don't abort - just log and catch error from next mixer call
     // return false;
  }

  MIXERCONTROL        mxctrl;
  MIXERLINECONTROLS   mxlc;

  // Stuff MIXERLINECONTROL struct
  mxlc.cbStruct       = sizeof(mxlc);
  mxlc.dwLineID       = mxl.dwLineID;
  mxlc.dwControlType  = dwControlType;    // Passed in by user
  mxlc.cControls      = 1;
  mxlc.cbmxctrl       = sizeof(MIXERCONTROL);
  mxlc.pamxctrl       = &mxctrl;

  // Get control information for control type specified (if it exists)
  rc = IMultimediaStatics::mixerGetLineControls((HMIXEROBJ)hMixer, &mxlc, MIXER_GETLINECONTROLSF_ONEBYTYPE);
  if (MMSYSERR_NOERROR != rc)
  {
     ITRACE_DEVELOP("IMMDevice::mixerControlValues - mixerGetLineControls() failed, rc="
                     +IString(rc));
     setLastError(rc);
     return false;
  }
  ITRACE_DEVELOP("IMMDevice::mixerControlValues - control ID #"
                  +IString(mxctrl.dwControlID));

  MIXERCONTROLDETAILS mxcd;

  mxcd.cbStruct       = sizeof(mxcd);
  mxcd.dwControlID    = mxctrl.dwControlID;
  if (uniform)
     mxcd.cChannels   = 1;  // Treat L&R channels uniformly
  else
     mxcd.cChannels   = 2;  // Treat L&R channels separately
  mxcd.cMultipleItems = multipleItems;  // This should be 0 for non-multiple controls
  mxcd.cbDetails      = sizeof(*ulDetails);
  mxcd.paDetails      = ulDetails;

  // Get current state of control(s)
  // If user specifed uniform=true, one control value represents both channels
  // Otherwise, two control values, one for each channel, is returned
  rc = IMultimediaStatics::mixerGetControlDetails(
          (HMIXEROBJ)hMixer, &mxcd, MIXER_GETCONTROLDETAILSF_VALUE);
  if (MMSYSERR_NOERROR != rc)
  {
     ITRACE_DEVELOP("IMMDevice::mixerControlValues - mixerGetControlDetails() failed, rc="
                     +IString(rc));
     setLastError(rc);
     return false;
  }

  // Normalize control value to a value between 0 and 100 (convert to percentage)
  unsigned long realValue1 = *ulDetails;
  ulDetails[0] = ceil( (*ulDetails)*100.0 / 65535.0 );

  if (uniform)
  {
     ITRACE_DEVELOP("IMMDevice::mixerControlValues - uniform volume value: "
                     +IString(realValue1)+", "
                     +IString(ulDetails[0])+" percent of maximum)" );
  }
  else
  {
     // Normalize control value to a value between 0 and 100 (convert to percentage)
     unsigned long realValue2 = *(ulDetails+1);
     ulDetails[1] = ceil( ( *(ulDetails+1) )*100.0 / 65535.0 );

     ITRACE_DEVELOP("IMMDevice::mixerControlValues - left volume value: "
                     +IString(realValue1)+", "
                     +IString(ulDetails[0])+" percent of maximum)" );
     ITRACE_DEVELOP("IMMDevice::mixerControlValues - right volume value: "
                     +IString(realValue2)+", "
                     +IString(ulDetails[1])+" percent of maximum)" );
  }

  return true;
}

/*------------------------------------------------------------------------------
| IMMDevice::mixerControlValues                                                |
| Obtains values for the mixer control specified.                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::mixerControlValues (
                           unsigned long dwComponentType,
                           unsigned long dwControlType,
                           unsigned short *usDetails,
                           Boolean uniform,
                           unsigned long multipleItems,
                           unsigned long hmx)
{
  IMODTRACE_DEVELOP("IMMDevice::mixerControlValues #2");

  MIXERLINE           mxl;
  MMRESULT            rc;
  unsigned long       hMixer = hmx;

  if (!hMixer)      // Does caller want to use the system mixer handle?
     hMixer = systemMixerHandle();

  // Prep MIXERLINE struct
  mxl.cbStruct      = sizeof(mxl);
  mxl.dwComponentType = dwComponentType;

  // Get line information for component type specified (if it exists)
  rc = IMultimediaStatics::mixerGetLineInfo((HMIXEROBJ)hMixer, &mxl, MIXER_GETLINEINFOF_COMPONENTTYPE);
  if (MMSYSERR_NOERROR != rc)
  {
     ITRACE_DEVELOP("IMMDevice::mixerControlValues - mixerGetLineInfo() failed, rc="
                     +IString(rc));
     setLastError(rc);
     return false;
  }
  ITRACE_DEVELOP("IMMDevice::mixerControlValues - line ID #"
                  +IString(mxl.dwLineID));

  // Quick sanity check
  if (mxl.cControls == 0)
  {
     ITRACE_DEVELOP("IMMDevice::mixerControlValues - line does not contain controls");
     // Don't abort - just log and catch error from next mixer call
     // return false;
  }

  MIXERCONTROL        mxctrl;
  MIXERLINECONTROLS   mxlc;

  // Stuff MIXERLINECONTROL struct
  mxlc.cbStruct       = sizeof(mxlc);
  mxlc.dwLineID       = mxl.dwLineID;
  mxlc.dwControlType  = dwControlType;    // Passed in by user
  mxlc.cControls      = 1;
  mxlc.cbmxctrl       = sizeof(MIXERCONTROL);
  mxlc.pamxctrl       = &mxctrl;

  // Get control information for control type specified (if it exists)
  rc = IMultimediaStatics::mixerGetLineControls((HMIXEROBJ)hMixer, &mxlc, MIXER_GETLINECONTROLSF_ONEBYTYPE);
  if (MMSYSERR_NOERROR != rc)
  {
     ITRACE_DEVELOP("IMMDevice::mixerControlValues - mixerGetLineControls() failed, rc="
                     +IString(rc));
     setLastError(rc);
     return false;
  }
  ITRACE_DEVELOP("IMMDevice::mixerControlValues - control ID #"
                  +IString(mxctrl.dwControlID));

  MIXERCONTROLDETAILS mxcd;
  MIXERCONTROLDETAILS_BOOLEAN  mxcd_bool[2];

  mxcd.cbStruct       = sizeof(mxcd);
  mxcd.dwControlID    = mxctrl.dwControlID;
  if (uniform)
     mxcd.cChannels   = 1;  // Treat L&R channels uniformly
  else
     mxcd.cChannels   = 2;  // Treat L&R channels separately
  mxcd.cMultipleItems = multipleItems;  // This should be 0 for non-multiple controls
  mxcd.cbDetails      = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
  mxcd.paDetails      = &mxcd_bool;

  // Get current state of control(s)
  // If user specifed uniform=true, one control value represents both channels
  // Otherwise, two control values, one for each channel, is returned
  rc = IMultimediaStatics::mixerGetControlDetails(
          (HMIXEROBJ)hMixer, &mxcd, MIXER_GETCONTROLDETAILSF_VALUE);
  if (MMSYSERR_NOERROR != rc)
  {
     ITRACE_DEVELOP("IMMDevice::mixerControlValues - mixerGetControlDetails() failed, rc="
                     +IString(rc));
     setLastError(rc);
     return false;
  }

  if (uniform)
  {
     ITRACE_DEVELOP("IMMDevice::mixerControlValues - item 1 uniform value: "
                     +IString(mxcd_bool[0].fValue));
     ITRACE_DEVELOP("IMMDevice::mixerControlValues - item 2 uniform value: "
                     +IString(mxcd_bool[1].fValue));
  }
  else
  {
     ITRACE_DEVELOP("IMMDevice::mixerControlValues - item 1 multi-channel value: "
                     +IString(mxcd_bool[0].fValue));
     ITRACE_DEVELOP("IMMDevice::mixerControlValues - item 2 multi-channel value: "
                     +IString(mxcd_bool[1].fValue));
  }

  usDetails[0] = (unsigned short)mxcd_bool[0].fValue;
  usDetails[1] = (unsigned short)mxcd_bool[1].fValue;

  return true;
}

/*------------------------------------------------------------------------------
| IMMDevice::setMixerControlValues                                             |
| Sets values for the mixer control specified.                                 |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::setMixerControlValues (
                           unsigned long dwComponentType,
                           unsigned long dwControlType,
                           unsigned long *ulDetails,
                           Boolean uniform,
                           unsigned long multipleItems,
                           unsigned long hmx)
{
  IMODTRACE_DEVELOP("IMMDevice::setMixerControlValues #1");

  MIXERLINE           mxl;
  MMRESULT            rc;
  unsigned long       hMixer = hmx;

  if (!hMixer)      // Does caller want to use the system mixer handle?
     hMixer = systemMixerHandle();

  // Prep MIXERLINE struct
  mxl.cbStruct      = sizeof(mxl);
  mxl.dwComponentType = dwComponentType;

  // Get line information for component type specified (if it exists)
  rc = IMultimediaStatics::mixerGetLineInfo((HMIXEROBJ)hMixer, &mxl, MIXER_GETLINEINFOF_COMPONENTTYPE);
  if (MMSYSERR_NOERROR != rc)
  {
     ITRACE_DEVELOP("IMMDevice::setMixerControlValues - mixerGetLineInfo() failed, rc="
                     +IString(rc));
     setLastError(rc);
     return false;
  }
  ITRACE_DEVELOP("IMMDevice::setMixerControlValues - line is ID #"
                  +IString(mxl.dwLineID));

  // Quick sanity check
  if (mxl.cControls == 0)
  {
     ITRACE_DEVELOP("IMMDevice::setMixerControlValues - line does not contain controls");
     // Don't abort - just log and catch error from next mixer call
     //return false;
  }

  MIXERCONTROL        mxctrl;
  MIXERLINECONTROLS   mxlc;

  // Stuff MIXERLINECONTROL struct
  mxlc.cbStruct       = sizeof(mxlc);
  mxlc.dwLineID       = mxl.dwLineID;
  mxlc.dwControlType  = dwControlType;    // Passed in by user
  mxlc.cControls      = 1;
  mxlc.cbmxctrl       = sizeof(MIXERCONTROL);
  mxlc.pamxctrl       = &mxctrl;

  // Get control information for control type specified (if it exists)
  rc = IMultimediaStatics::mixerGetLineControls((HMIXEROBJ)hMixer, &mxlc, MIXER_GETLINECONTROLSF_ONEBYTYPE);
  if (MMSYSERR_NOERROR != rc)
  {
     ITRACE_DEVELOP("IMMDevice::setMixerControlValues - mixerGetLineControls() failed, rc="
                     +IString(rc));
     setLastError(rc);
     return false;
  }
  ITRACE_DEVELOP("IMMDevice::setMixerControlValues - control ID #"
                  +IString(mxctrl.dwControlID));

  MIXERCONTROLDETAILS mxcd;
  MIXERCONTROLDETAILS_UNSIGNED  mxcd_unsign[2];

  // Convert values from a percentage to the control's range (0 --> 65535)
  mxcd_unsign[0].dwValue = floor(ulDetails[0] * 655.35);

  // Adjust for possible out of bounds condition (MMPM does this for us)
  if (mxcd_unsign[0].dwValue > 65535)
     mxcd_unsign[0].dwValue = 65535;

  if (uniform)
  {
     ITRACE_DEVELOP("IMMDevice::setMixerControlValues - setting uniform volume value: "
                     +IString(mxcd_unsign[0].dwValue)+", "
                     +IString(ulDetails[0])+" percent of maximum)" );
  }
  else
  {
     // Convert values from a percentage to the control's range (0 --> 65535)
     mxcd_unsign[1].dwValue = floor(ulDetails[1] * 655.35);

     // Adjust for possible out of bounds condition (MMPM does this for us)
     if (mxcd_unsign[1].dwValue > 65535)
        mxcd_unsign[1].dwValue = 65535;

     ITRACE_DEVELOP("IMMDevice::setMixerControlValues - setting left volume value: "
                     +IString(mxcd_unsign[0].dwValue)+", "
                     +IString(ulDetails[0])+" percent of maximum)" );
     ITRACE_DEVELOP("IMMDevice::setMixerControlValues - setting right volume value: "
                     +IString(mxcd_unsign[1].dwValue)+", "
                     +IString(ulDetails[1])+" percent of maximum)" );
  }

  mxcd.cbStruct       = sizeof(mxcd);
  mxcd.dwControlID    = mxctrl.dwControlID;
  if (uniform)
     mxcd.cChannels   = 1;  // Treat L&R channels uniformly
  else
     mxcd.cChannels   = 2;  // Treat L&R channels separately
  mxcd.cMultipleItems = multipleItems;  // This should be 0 for non-multiple controls
  mxcd.cbDetails      = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
  mxcd.paDetails      = &mxcd_unsign;

  // Set current state of control(s)
  // If user specifed uniform=true, one control value represents both channels
  // Otherwise, two control values, one for each channel, are used
  rc = IMultimediaStatics::mixerSetControlDetails(
          (HMIXEROBJ)hMixer, &mxcd, MIXER_SETCONTROLDETAILSF_VALUE);
  if (MMSYSERR_NOERROR != rc)
  {
     ITRACE_DEVELOP("IMMDevice::setMixerControlValues - mixerGetControlDetails() failed, rc="
                     +IString(rc));
     setLastError(rc);
     return false;
  }

  return true;
}

/*------------------------------------------------------------------------------
| IMMDevice::setMixerControlValues                                             |
| Sets values for the mixer control specified.                                 |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::setMixerControlValues (
                           unsigned long dwComponentType,
                           unsigned long dwControlType,
                           unsigned short *usDetails,
                           Boolean uniform,
                           unsigned long multipleItems,
                           unsigned long hmx)
{
  IMODTRACE_DEVELOP("IMMDevice::setMixerControlValues #2");

  MIXERLINE           mxl;
  MMRESULT            rc;
  unsigned long       hMixer = hmx;

  if (!hMixer)      // Does caller want to use the system mixer handle?
     hMixer = systemMixerHandle();

  // Prep MIXERLINE struct
  mxl.cbStruct      = sizeof(mxl);
  mxl.dwComponentType = dwComponentType;

  // Get line information for component type specified (if it exists)
  rc = IMultimediaStatics::mixerGetLineInfo((HMIXEROBJ)hMixer, &mxl, MIXER_GETLINEINFOF_COMPONENTTYPE);
  if (MMSYSERR_NOERROR != rc)
  {
     ITRACE_DEVELOP("IMMDevice::setMixerControlValues - mixerGetLineInfo() failed, rc="
                     +IString(rc));
     setLastError(rc);
     return false;
  }
  ITRACE_DEVELOP("IMMDevice::setMixerControlValues - line is ID #"
                  +IString(mxl.dwLineID));

  // Quick sanity check
  if (mxl.cControls == 0)
  {
     ITRACE_DEVELOP("IMMDevice::setMixerControlValues - line does not contain controls");
     // Don't abort - just log and catch error from next mixer call
     //return false;
  }

  MIXERCONTROL        mxctrl;
  MIXERLINECONTROLS   mxlc;

  // Stuff MIXERLINECONTROL struct
  mxlc.cbStruct       = sizeof(mxlc);
  mxlc.dwLineID       = mxl.dwLineID;
  mxlc.dwControlType  = dwControlType;    // Passed in by user
  mxlc.cControls      = 1;
  mxlc.cbmxctrl       = sizeof(MIXERCONTROL);
  mxlc.pamxctrl       = &mxctrl;

  // Get control information for control type specified (if it exists)
  rc = IMultimediaStatics::mixerGetLineControls((HMIXEROBJ)hMixer, &mxlc, MIXER_GETLINECONTROLSF_ONEBYTYPE);
  if (MMSYSERR_NOERROR != rc)
  {
     ITRACE_DEVELOP("IMMDevice::setMixerControlValues - mixerGetLineControls() failed, rc="
                     +IString(rc));
     setLastError(rc);
     return false;
  }
  ITRACE_DEVELOP("IMMDevice::setMixerControlValues - control ID #"
                  +IString(mxctrl.dwControlID));

  MIXERCONTROLDETAILS mxcd;
  MIXERCONTROLDETAILS_BOOLEAN  mxcd_bool[2];

  mxcd_bool[0].fValue = usDetails[0];
  mxcd_bool[1].fValue = usDetails[1];

  mxcd.cbStruct       = sizeof(mxcd);
  mxcd.dwControlID    = mxctrl.dwControlID;
  if (uniform)
     mxcd.cChannels   = 1;  // Treat L&R channels uniformly
  else
     mxcd.cChannels   = 2;  // Treat L&R channels separately
  mxcd.cMultipleItems = multipleItems;  // This should be 0 for non-multiple controls
  mxcd.cbDetails      = sizeof(MIXERCONTROLDETAILS_BOOLEAN);
  mxcd.paDetails      = &mxcd_bool;

  // Set current state of control(s)
  // If user specifed uniform=true, one control value represents both channels
  // Otherwise, two control values, one for each channel, are used
  rc = IMultimediaStatics::mixerSetControlDetails(
          (HMIXEROBJ)hMixer, &mxcd, MIXER_SETCONTROLDETAILSF_VALUE);
  if (MMSYSERR_NOERROR != rc)
  {
     ITRACE_DEVELOP("IMMDevice::setMixerControlValues - mixerGetControlDetails() failed, rc="
                     +IString(rc));
     setLastError(rc);
     return false;
  }

  return true;
}
#endif

/*------------------------------------------------------------------------------
| IMMDevice::isAudioEnabled                                                    |
| Returns true if the audio for the passed in channel is turned on.            |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::isAudioEnabled(AudioChannel channel,
                                  CallType call)
{
  IMODTRACE_DEVELOP("IMMDevice::isAudioEnabled");

#ifdef IC_PM
  // value contains the correct MMPM constant for the audio channel
  // assume default of all
  unsigned long value = MCI_STATUS_AUDIO_ALL;

  // if not default, stuff the appropriate constant
  if (channel == left)
     value = MCI_STATUS_AUDIO_LEFT;

  if (channel == right)
     value = MCI_STATUS_AUDIO_RIGHT;

  Boolean rc = itemStatus(MCI_STATUS_AUDIO, value, call);

  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");

  return rc;
#endif
#ifdef IC_WIN
  // Although Win32 has a MCI_SET_AUDIO, it does not have a corresponding
  // MCI_STATUS_AUDIO, so we have to revert to the mixer speaker line mute
  // control for testing enablement. Only one mute control is present
  // for both channels, so we cannot support channel=left/right.

  unsigned short  usDetails[2];

  if (!mixerControlValues(
              MIXERLINE_COMPONENTTYPE_DST_SPEAKERS,       // master mute
              MIXERCONTROL_CONTROLTYPE_MUTE,
              usDetails, true))    // multiple items = 0 for mute
     ITRACE_DEVELOP("IMMDevice::isAudioEnabled - mixerControlValues error");

  if (usDetails[0])
     return false;     // mute is on; speakers are not enabled
  else
     return true;

#endif
}

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IMMDevice::saveDeviceSettings                                                |
| Save class-specific settings for the current device, to allow MCI_LOAD       |
| emulation in Win32.                                                          |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::saveDeviceSettings()
{
  IMODTRACE_DEVELOP("IMMDevice::saveDeviceSettings");
  setTimeFormat( timeFormat() );
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::restoreDeviceSettings                                             |
| Restore class-specific settings for the current device, to allow MCI_LOAD    |
| emulation in Win32.                                                          |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::restoreDeviceSettings(Boolean newRecordMode)
{
  IMODTRACE_DEVELOP("IMMDevice::restoreDeviceSettings");
  // Note that we cannot use the getter in this case, since we want
  // to recover the setting of the device as it was previously.
  // On a reload, the device will already be open, and the getter
  // will query the actual state of the device, which would be its default.
  // If the setting had been changed, we would miss it. If the setting had
  // not been previously changed (ie touched), we will leave it as is.
  if (fDeviceData->myTimeFormatTouched)
     setTimeFormat(fDeviceData->myTimeFormat);
  return *this;
}

#endif
