/*******************************************************************************
* FILE NAME: immplyhd.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immplyhd.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C"
   {
   #define INCL_WINDIALOGS
   #define INCL_OS2MM
   #define INCL_SW
   #define INCL_SELSLIDE
   #define IC_MULTIMEDIA
   #include <immdefs.h>
   }

#include <immplyhd.hpp>
#include <iwindow.hpp>
#include <immexcpt.hpp>
#include <ianimbut.hpp>
#include <immplypn.hpp>
#include <immplayd.hpp>
#include <icconst.h>
#include <itrace.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMMPLYHD_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IMMPlayerPanelHandler::IMMPlayerPanelHandler                                 |
|                                                                              |
------------------------------------------------------------------------------*/
IMMPlayerPanelHandler::IMMPlayerPanelHandler()
{ IMODTRACE_DEVELOP("IMMPlayerPanelHandler::ctor"); }

/*------------------------------------------------------------------------------
| IMMPlayerPanelHandler::~IMMPlayerPanelHandler                                |
|                                                                              |
------------------------------------------------------------------------------*/
IMMPlayerPanelHandler::~IMMPlayerPanelHandler()
{ IMODTRACE_DEVELOP("IMMPlayerPanelHandler::dtor"); }

/*------------------------------------------------------------------------------
| IMMPlayerPanelHandler::command                                               |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMPlayerPanelHandler::command(ICommandEvent& event)
{
  IMODTRACE_DEVELOP("IMMPlayerPanelHandler::command");

  Boolean handled = false;
  IMMPlayerPanel* panel= (IMMPlayerPanel*) (event.window());

  switch (event.commandId())
  {
    case IC_FF:
         handled = fastForward(*panel);
         break;
    case IC_PAUSE:
         handled = pause(*panel);
         break;
    case IC_STOP:
         handled = stop(*panel);
         break;
    case IC_PLAY1:
         handled = play(*panel);
         break;
    case IC_REWIND:
         handled = rewind(*panel);
         break;
    case IC_STEPF:
         handled = stepForward(*panel);
         break;
    case IC_STEPB:
         handled = stepBackward(*panel);
         break;
  } /* endswitch */
  return handled;
}

/*------------------------------------------------------------------------------
| IMMPlayerPanelHandler::play                                                  |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMPlayerPanelHandler::play(const IMMPlayerPanel& panel)
{
  IMODTRACE_DEVELOP("IMMPlayerPanelHandler::play");

  IMMPlayableDevice* device = panel.playableDevice();
  IASSERTSTATE(device);
  if (panel.stopButton())
     panel.stopButton()->enable();
  if (panel.pauseButton())
  {
     panel.pauseButton()->enable();
     panel.pauseButton()->unlatch();
  }

  device->play();
  return true;
}

/*------------------------------------------------------------------------------
| IMMPlayerPanelHandler::stop                                                  |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMPlayerPanelHandler::stop(const IMMPlayerPanel& panel)
{
  IMODTRACE_DEVELOP("IMMPlayerPanelHandler::stop");

  IMMPlayableDevice* device = panel.playableDevice();
  IASSERTSTATE(device);
  if (panel.playButton())
  {
     panel.playButton()->unlatch();
  }
  if (panel.stopButton())
     panel.stopButton()->disable();
  if (panel.pauseButton())
  {
     panel.pauseButton()->unlatch();
     panel.pauseButton()->disable();
  }
  device->stop();
  device->seekToStart();
  return true;
}

/*------------------------------------------------------------------------------
| IMMPlayerPanelHandler::pause                                                 |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMPlayerPanelHandler::pause(const IMMPlayerPanel& panel)
{
  IMODTRACE_DEVELOP("IMMPlayerPanelHandler::pause");

  IMMPlayableDevice* device = panel.playableDevice();
  IASSERTSTATE(device);
#ifdef IC_PM
  if (IMMDevice::paused == device->mode())
#endif
#ifdef IC_WIN
  // Win32 CD does not support paused mode (MCI_PAUSE puts CD in stopped state),
  // so we check for pause button being latched.
  if (IMMDevice::paused == device->mode() || !panel.pauseButton()->isLatched())
#endif
  {
     if (panel.playButton())
        panel.playButton()->latch();

     panel.pauseButton()->unlatch();
     device->resume();
  }
  else
  {
     if (panel.playButton())
        panel.playButton()->unlatch();
     device->pause();
  }

  return true;
}

/*------------------------------------------------------------------------------
| IMMPlayerPanelHandler::fastForward                                           |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMPlayerPanelHandler::fastForward(const IMMPlayerPanel& panel)
{
  IMODTRACE_DEVELOP("IMMPlayerPanelHandler::fastForward");

  IMMPlayableDevice* device = panel.playableDevice();
  IASSERTSTATE(device);
  if (panel.playButton())
  {
     panel.playButton()->unlatch();
  }
  if (panel.stopButton())
     panel.stopButton()->disable();
  if (panel.pauseButton())
  {
     panel.pauseButton()->unlatch();
     panel.pauseButton()->disable();
  }
  device->seekToEnd();
  return true;
}

/*------------------------------------------------------------------------------
| IMMPlayerPanelHandler::rewind                                                |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMPlayerPanelHandler::rewind(const IMMPlayerPanel& panel)
{
  IMODTRACE_DEVELOP("IMMPlayerPanelHandler::rewind");

  IMMPlayableDevice* device = panel.playableDevice();
  IASSERTSTATE(device);
  if (panel.playButton())
  {
     panel.playButton()->unlatch();
  }
  if (panel.stopButton())
     panel.stopButton()->disable();
  if (panel.pauseButton())
  {
     panel.pauseButton()->unlatch();
     panel.pauseButton()->disable();
  }
  device->seekToStart();
  return true;
}

/*------------------------------------------------------------------------------
| IMMPlayerPanelHandler::stepForward                                           |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMPlayerPanelHandler::stepForward(const IMMPlayerPanel& panel)
{
  IMODTRACE_DEVELOP("IMMPlayerPanelHandler::stepForward");

  IMMPlayableDevice* device = panel.playableDevice();
  IASSERTSTATE(device);
  if (panel.playButton())
  {
     panel.playButton()->unlatch();
  }
  // JAD - MMPM's mmtime (Universal Chinatown Time) is not supported in Win32
  //       verify this works with revisions to time macros in IMMDEFS.H
  //       For Win32, IMMTime::mmTime maps to IMMTime::milliseconds
  IMMTime::Format format = device->timeFormat();
  device->setTimeFormat(IMMTime::mmTime);
  if (device->position() < device->length())
  {
     device->setTimeFormat(format);
     device->stepFrame();
  }
  else
     device->setTimeFormat(format);
  return true;
}

/*------------------------------------------------------------------------------
| IMMPlayerPanelHandler::stepBackward                                          |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMPlayerPanelHandler::stepBackward(const IMMPlayerPanel& panel)
{
  IMODTRACE_DEVELOP("IMMPlayerPanelHandler::stepBackward");

  IMMPlayableDevice* device = panel.playableDevice();
  IASSERTSTATE(device);
  if (panel.playButton())
  {
     panel.playButton()->unlatch();
  }
  // JAD - MMPM's mmtime (Universal Chinatown Time) is not supported in Win32
  //       verify this works with revisions to time macros in IMMDEFS.H
  //       For Win32, IMMTime::mmTime maps to IMMTime::milliseconds
  IMMTime::Format format = device->timeFormat();
  device->setTimeFormat(IMMTime::mmTime);
  if (device->position())
  {
     device->setTimeFormat(format);
     device->stepFrame(1,false);
  }
  else
     device->setTimeFormat(format);
  return true;
}

