/*******************************************************************************
* FILE NAME: immreme1.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immremnh.hpp and some in immremed.hpp.                                  *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

#define INCL_WINSYS

extern "C" {
   #include <immdefs.h>
}

#include <iwindow.hpp>
#include <immowin.hpp>
#include <immremed.hpp>
#include <immremnh.hpp>
#include <ievent.hpp>
#include <inotifev.hpp>
#include <iexcept.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMMREME1_CPP_
  #include <ipagetun.h>
#endif

// Class to clean up static objects
class IMMRemovableMediaNotifyStatics
{
public:
  ~IMMRemovableMediaNotifyStatics ()
  {
    delete removableMediaNotifyHandler;
    removableMediaNotifyHandler = 0;
  }

static IMMRemovableMediaNotifyHandler
  &notificationHandler ( );

static IMMRemovableMediaNotifyHandler
  *removableMediaNotifyHandler;
};

// Initialize static objects and notification strings.
#pragma data_seg(ICLStaticConst)
static IMMRemovableMediaNotifyStatics removableMediaStatics;
INotificationId const IMMRemovableMedia::mediaLoadedId = "IMMRemovableMedia::mediaLoaded";
#pragma data_seg()

#pragma data_seg(ICLNonConst)
IMMRemovableMediaNotifyHandler*
  IMMRemovableMediaNotifyStatics::removableMediaNotifyHandler = 0;
#pragma data_seg()

/*------------------------------------------------------------------------------------
| IMMRemovableMediaNotifyStatics::notificationHandler                                |
|                                                                                    |
| Static function to create and return a notification handler for IMMRemovableMedia. |
-------------------------------------------------------------------------------------*/
IMMRemovableMediaNotifyHandler& IMMRemovableMediaNotifyStatics::notificationHandler ()
{
  IMODTRACE_DEVELOP("IMMRemovableMediaNotifyStatics::notificationHandler") ;
  if (!(IMMRemovableMediaNotifyStatics::removableMediaNotifyHandler))
  {
    IMMRemovableMediaNotifyStatics::removableMediaNotifyHandler =
                                   new IMMRemovableMediaNotifyHandler();
  }
  return( *IMMRemovableMediaNotifyStatics::removableMediaNotifyHandler );
}


/*------------------------------------------------------------------------------
| IMMRemovableMedia::enableNotification                                        |
|                                                                              |
| Flags this part as being ready to accept notifications.  A Notifier, like    |
| IMMRemovableMedia, does not send notifications to observer objects until it  |
| is enabled.                                                                  |
|                                                                              |
| In this class, we override the INotifier function to allocate the correct    |
| notification handler. We use our msg window to manage the handler & boolean. |
------------------------------------------------------------------------------*/
IMMRemovableMedia& IMMRemovableMedia::enableNotification (Boolean enabled)
{
  IMODTRACE_DEVELOP("IMMRemovableMedia::enableNotification") ;
  if (!notificationHandler())
  {
    setNotificationHandler(
               &(IMMRemovableMediaNotifyStatics::notificationHandler()));
  }
  deviceWindow().enableNotification(enabled);    // Enable our proxy window, so our notification
                                                 // handler gets called (also enables IWindow notifIds e.g., destroyId)
  Inherited::enableNotification(enabled);        // Enable this device also, so notifyObservers works
  return *this;
}


/*------------------------------------------------------------------------------
| IMMRemovableMedia::notificationHandler                **aj**                 |
|                                                                              |
| Returns a pointer to the notification handler                                |
------------------------------------------------------------------------------*/
IMMRemovableMediaNotifyHandler* IMMRemovableMedia::notificationHandler    ( ) const
{
   IMODTRACE_DEVELOP("IMMRemovableMedia::notificationHandler") ;
  IMMRemovableMediaNotifyHandler* notifyHandler =
           (IMMRemovableMediaNotifyHandler*)(deviceWindow().notificationHandler());
  return notifyHandler;
}

/*------------------------------------------------------------------------------
| IMMRemovableMedia::setNotificationHandler           **aj**                   |
|                                                                              |
| Sets the notification handler to the passed value.                           |
------------------------------------------------------------------------------*/
IMMRemovableMedia& IMMRemovableMedia::setNotificationHandler (
                                     IMMRemovableMediaNotifyHandler* notifyHandler )
{
   IMODTRACE_DEVELOP("IMMRemovableMedia::setNotificationHandler") ;
  IASSERTSTATE(notifyHandler!=0);
  deviceWindow().setNotificationHandler(notifyHandler);
  return *this;
}


/*------------------------------------------------------------------------------
| IMMRemovableMediaNotifyHandler::IMMRemovableMediaNotifyHandler               |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
IMMRemovableMediaNotifyHandler::IMMRemovableMediaNotifyHandler()
{
   IMODTRACE_DEVELOP("IMMRemovableMediaNotifyHandler::IMMRemovableMediaNotificationHandler") ;
}

/*------------------------------------------------------------------------------
| IMMRemovableMediaNotifyHandler::~IMMRemovableMediaNotifyHandler              |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IMMRemovableMediaNotifyHandler::~IMMRemovableMediaNotifyHandler()
{
   IMODTRACE_DEVELOP("IMMRemovableMediaNotifyHandler::~IMMRemovableMediaNotificationHandler") ;
}

/*------------------------------------------------------------------------------
| IMMRemovableMediaNotifyHandler::dispatchHandlerEvent                         |
|                                                                              |
| This notifies IMMRemovableMedia Windows of the following events:             |
|                                                                              |
| mediaLoaded(Id) - either loaded (true) or unloaded (false)                   |
------------------------------------------------------------------------------*/

Boolean IMMRemovableMediaNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
  IMODTRACE_DEVELOP("IMMRemovableMediaNotifyHandler::dispatchHandlerEvent") ;
 IMMMessageWindow* thisWindow;
 IMMDevice* device;

  switch (anEvent.eventId())
  {
    case IC_UM_MEDIA_LOADED:
      thisWindow = (IMMMessageWindow*)(anEvent.window());
      device = &(thisWindow->device());
      device->notifyObservers(
                    INotificationEvent(IMMRemovableMedia::mediaLoadedId,
                                       *device, true, true));
      break;

    case IC_UM_MEDIA_UNLOADED:
      thisWindow = (IMMMessageWindow*)(anEvent.window());
      device = &(thisWindow->device());
      device->notifyObservers(
                    INotificationEvent(IMMRemovableMedia::mediaLoadedId,
                                       *device, true, false));
      break;

    default:
      Inherited::dispatchHandlerEvent(anEvent);
      break;
  }  /* end switch */

   return (Inherited::dispatchHandlerEvent(anEvent));
}

