/*******************************************************************************
* FILE NAME: immremed.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immremed.hpp                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
extern "C"
{
  #include <string.h>
  #define INCL_WINTIMER
  #define INCL_MCIOS2
  #define IC_MULTIMEDIA
  #include <immdefs.h>
}

#include <immremed.hpp>
#include <inotifev.hpp>
#include <itimer.hpp>
#include <immexcpt.hpp>
#include <immremhd.hpp>
#include <immowin.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMMREMED_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| IMMRemovableMediaData                                                        |
| This private class encapsulates interaction with IMMRemovableMediaHandler.   |
|                                                                              |
------------------------------------------------------------------------------*/
class IMMRemovableMediaData : public IBase
{
public:
  IMMRemovableMediaData()
         : deviceHandler(0),
           fMediaTimer(0),               //aj
           fDiscIsInDrive(false)         //aj
     { }

IMMRemovableMediaHandler
  *deviceHandler;
ITimer                                    //aj
  *fMediaTimer;
Boolean                                   //aj
  fDiscIsInDrive;
};


/*------------------------------------------------------------------------------
| IMMRemovableMedia::IMMRemovableMedia                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMRemovableMedia::IMMRemovableMedia(const IString &baseDeviceName,
                                       Boolean openNow,
                                       unsigned long instance,
                                       Boolean shareable)
   : IMMPlayableDevice(baseDeviceName, openNow, instance, shareable),
     fRemovableMediaData(new IMMRemovableMediaData())
{
  IMODTRACE_DEVELOP("IMMRemovableMedia::ctor #1");
  fRemovableMediaData->deviceHandler = new IMMRemovableMediaHandler();
  fRemovableMediaData->deviceHandler->handleEventsFor(this);
}

/*------------------------------------------------------------------------------
| IMMRemovableMedia::~IMMRemovableMedia                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMRemovableMedia::~IMMRemovableMedia()
{
   IMODTRACE_DEVELOP("IMMRemovableMedia::dtor");

   if (fRemovableMediaData)
   {
      if (fRemovableMediaData->deviceHandler)
      {
         fRemovableMediaData->deviceHandler->stopHandlingEventsFor(this);
         delete fRemovableMediaData->deviceHandler;
      }
      if (fRemovableMediaData->fMediaTimer)              //aj
      {
         fRemovableMediaData->fMediaTimer->stop();       //aj
         delete fRemovableMediaData->fMediaTimer;        //aj
      }
      delete fRemovableMediaData;
   }
}

/*------------------------------------------------------------------------------
| IMMRemovableMedia::closeDoor                                                 |
| Retracts the tray and closes the door, if possible.                          |
|                                                                              |
------------------------------------------------------------------------------*/
IMMRemovableMedia& IMMRemovableMedia::closeDoor(CallType call)
{
  IMODTRACE_DEVELOP("IMMRemovableMedia::closeDoor");
  return openDoor(false,call);
}

#ifdef IC_PM
/*------------------------------------------------------------------------------
| IMMRemovableMedia::lockDoor                                                  |
| Locks the media cover on the device, if any.  This disables manual ejection  |
| of the media from the device.                                                |
------------------------------------------------------------------------------*/
IMMRemovableMedia& IMMRemovableMedia::lockDoor(Boolean lock, CallType call)
{
  IMODTRACE_DEVELOP("IMMRemovableMedia::lockDoor");

  IASSERTSTATE(isOpen());

  MCI_SET_PARMS parms;
  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback = HWND(deviceWindow().handle());
  sendCommand(MCI_SET,(lock ? MCI_SET_DOOR_LOCK : MCI_SET_DOOR_UNLOCK) |
              call, &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMRemovableMedia::unlockDoor                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMRemovableMedia& IMMRemovableMedia::unlockDoor(CallType call)
{
  IMODTRACE_DEVELOP("IMMRemovableMedia::unLockDoor");
  return lockDoor(false,call);
}
#endif  // IC_PM

/*------------------------------------------------------------------------------
| IMMRemovableMedia::isMediaPresent                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMRemovableMedia::isMediaPresent(CallType call) const
{
  IMODTRACE_DEVELOP("IMMRemovableMedia::isMediaPresent");
  IASSERTSTATE(isOpen());

  Boolean rc = itemStatus(MCI_STATUS_MEDIA_PRESENT, 0, call);
  ITRACE_DEVELOP("IMMRemovableMedia - media present rc = " + IString(rc));

#ifdef IC_PM
  // OS2 actually tells us if the media is not valid as part of this call,
  // so we check for this condition.
  if (lastError()== MCIERR_INVALID_MEDIA_TYPE)
  {
     ITRACE_DEVELOP("IMMRemovableMedia - media present, but not valid type");
     return false;
  }
#endif  // IC_PM

  // If the first item status call failed, throw an exception now.
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");

#ifdef IC_PM
  return rc;
#endif

#ifdef IC_WIN
  // Win32 does not inform us of invalid media as part of first call;
  // we need to do additional work.
  // First of all, if we know media is not present we don't have to go further.
  if (!rc)
     return rc;

  // Make another call to status, this time querying the type of track 1.
  // The assumption is any type of media contains at least 1 track. For
  // media containing mixed tracks, the first track (track 1) is used
  // to determine validity.
  MCI_STATUS_PARMS parms;
  memset(&parms, 0, sizeof(MCI_STATUS_PARMS));
  parms.dwItem       = MCI_CDA_STATUS_TYPE_TRACK;
  parms.dwTrack      = 1;   // Any media type has at least one track
  // Calling a non-const from a const...
  ((IMMRemovableMedia*)this)->sendCommand(MCI_STATUS,
                                  MCI_WAIT | MCI_STATUS_ITEM | MCI_TRACK,
                                  (void *)&parms, 0);
  if (lastError())
     ITHROWMMERROR(lastError(),"MCI_CDA_STATUS_TYPE_TRACK - sendCommand");

  ITRACE_DEVELOP("IMMRemovableMedia - media type rc = " +
                  IString(parms.dwReturn));

  if (MCI_CDA_TRACK_AUDIO == parms.dwReturn)
  {
     ITRACE_DEVELOP("IMMRemovableMedia - valid audio media present");
     return true;
  }

  if (MCI_CDA_TRACK_OTHER == parms.dwReturn)
  {
     ITRACE_DEVELOP("IMMRemovableMedia - media present, but not valid type");
     return false;
  }

  // MCI_CDA_STATUS_TYPE_TRACK should only return one of the above,
  // but just in case we get a bogus value, return false.
  return false;
#endif  // IC_WIN
}

/*------------------------------------------------------------------------------
| IMMRemovableMedia::openDoor                                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMRemovableMedia& IMMRemovableMedia::openDoor(Boolean open, CallType call)
{
  IMODTRACE_DEVELOP("IMMRemovableMedia::openDoor");

  IASSERTSTATE(isOpen());

  MCI_SET_PARMS parms;
  memset(&parms, 0, sizeof(parms));
#ifdef IC_PM
  parms.hwndCallback = HWND(deviceWindow().handle());
#endif  // IC_PM
#ifdef IC_WIN
  parms.dwCallback = deviceWindow().handle().asUnsigned();
#endif  // IC_WIN
  sendCommand(MCI_SET,(open ? MCI_SET_DOOR_OPEN : MCI_SET_DOOR_CLOSED) |
              call, &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMRemovableMedia::IMMRemovableMedia                                         |
| This constructor wrappers an existing device, and gives a derived class the  |
| ability to specify a desired alias (instead of ICLUI generating it).         |
------------------------------------------------------------------------------*/
IMMRemovableMedia::IMMRemovableMedia(unsigned long newDeviceId,
                                       const IString &newAlias)
                  : IMMPlayableDevice(newDeviceId, newAlias),
                    fRemovableMediaData(new IMMRemovableMediaData())
{
  IMODTRACE_DEVELOP("IMMRemovableMedia::ctor #2");
  fRemovableMediaData->deviceHandler = new IMMRemovableMediaHandler();
  fRemovableMediaData->deviceHandler->handleEventsFor(this);
}

/*------------------------------------------------------------------------------
| IMMRemovableMedia::startCheckMediaTimer       *** aj ***                     |
| Starts a timer to periodically check the device and post a user event when   |
| a disc is inserted or removed.                                               |
------------------------------------------------------------------------------*/
IMMRemovableMedia& IMMRemovableMedia::startCheckMediaTimer()
{
  IMODTRACE_DEVELOP("IMMRemovableMedia::startCheckMediaTimer");

  if ( wasPassDeviceRequested() )
  {
     fRemovableMediaData->fDiscIsInDrive = isMediaPresent();
     if (!fRemovableMediaData->fMediaTimer)
        fRemovableMediaData->fMediaTimer = new ITimer();
     if (!fRemovableMediaData->fMediaTimer->isStarted())
        fRemovableMediaData->fMediaTimer->start(new ITimerMemberFn<IMMRemovableMedia>(*this,checkMedia));
  }

  return *this;
}

/*------------------------------------------------------------------------------
| IMMRemovableMedia::stopCheckMediaTimer        *** aj ***                     |
| Stops the timer started in startCheckMediaTimer().                           |
------------------------------------------------------------------------------*/
IMMRemovableMedia& IMMRemovableMedia::stopCheckMediaTimer()
{
  IMODTRACE_DEVELOP("IMMRemovableMedia::stopCheckMediaTimer");

   fRemovableMediaData->fMediaTimer->stop();
   return *this;
}

/*------------------------------------------------------------------------------
| IMMRemovableMedia::checkMedia                  ** aj **                      |
| Check to see if media is present; send an IOC user event when state changes. |
|                                                                              |
------------------------------------------------------------------------------*/
void IMMRemovableMedia::checkMedia(unsigned long timerId)
{
   IMODTRACE_DEVELOP("IMMRemovableMedia::checkMedia");

   if (isMediaPresent())
   {
      if ( !(fRemovableMediaData->fDiscIsInDrive) )
      {
         fRemovableMediaData->fDiscIsInDrive = true;
         // Send (not post) event, since a) we already endured overhead of WM_TIMER, b) we're not within
         //   a handler-dispatch so we can safely do a send.
         deviceWindow().sendEvent(IC_UM_MEDIA_LOADED);
      }
   }
   else
   {
      if (fRemovableMediaData->fDiscIsInDrive)
      {
         fRemovableMediaData->fDiscIsInDrive = false;
         deviceWindow().sendEvent(IC_UM_MEDIA_UNLOADED);
      }
   }
}

