/*******************************************************************************
* FILE NAME: imultsta.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in imultsta.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Note we depend on ireslib.cpp being first
#pragma priority( -2147482102 )
extern "C"
  {
  #define INCL_MCIOS2
  #define IC_MULTIMEDIA
  #include <immdefs.h>
  }

#ifdef IC_WIN
MCIERROR dumpRc(MCIERROR);
MCIERROR dumpRc(MCIERROR, char*);
#endif

#include <imultsta.hpp>

#ifndef _IRESLIB_
  #include <ireslib.hpp>
#endif

#ifndef _ITRACE_
   #include <itrace.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMULTSTA_CPP_
  #include <ipagetun.h>
#endif

#pragma data_seg(ICLNonConst)
  static IMultimediaStatics mmpmMgr;
#pragma data_seg()

IGetErrorString*     IMultimediaStatics::entryGetErrorString  = 0;
ISendString*         IMultimediaStatics::entrySendString      = 0;
ISendCommand*        IMultimediaStatics::entrySendCommand     = 0;
IRegisterSelectionSlider* IMultimediaStatics::entryRegisterSelectionSlider = 0;
IRegisterCircularSlider*  IMultimediaStatics::entryRegisterCircularSlider  = 0;
IRegisterGraphicButton*   IMultimediaStatics::entryRegisterGraphicButton   = 0;
IDynamicLinkLibrary* IMultimediaStatics::mmpmDynLinkLib       = 0;
IDynamicLinkLibrary* IMultimediaStatics::mmctlDynLinkLib      = 0;

#ifdef IC_WIN
IMixerGetNumDevs*    IMultimediaStatics::entryMixerGetNumDevs = 0;
IMixerOpen*          IMultimediaStatics::entryMixerOpen       = 0;
IMixerClose*         IMultimediaStatics::entryMixerClose      = 0;
IMixerGetLineInfo*   IMultimediaStatics::entryMixerGetLineInfo= 0;
IMixerGetLineControls*   IMultimediaStatics::entryMixerGetLineControls  = 0;
IMixerGetControlDetails* IMultimediaStatics::entryMixerGetControlDetails= 0;
IMixerSetControlDetails* IMultimediaStatics::entryMixerSetControlDetails= 0;
#endif

/*------------------------------------------------------------------------------
| IMultimediaStatics::~IMultimediaStatics                                      |
------------------------------------------------------------------------------*/
IMultimediaStatics ::  ~IMultimediaStatics ( )
{
  IMODTRACE_DEVELOP("IMMStatics::dtor");
  if (mmpmDynLinkLib)
     delete mmpmDynLinkLib;
  if (mmctlDynLinkLib)
     delete mmctlDynLinkLib;
}

/*------------------------------------------------------------------------------
| IMultimediaStatics::mmpmDLL                                                  |
------------------------------------------------------------------------------*/
IDynamicLinkLibrary& IMultimediaStatics :: mmpmDLL ( )
{
  IMODTRACE_DEVELOP("IMMStatics::mmpmDLL");
  if (!mmpmDynLinkLib)
#ifdef IC_PM
     mmpmDynLinkLib = new IDynamicLinkLibrary("mdm");
#endif
#ifdef IC_WIN
     mmpmDynLinkLib = new IDynamicLinkLibrary("winmm");
#endif
  return *mmpmDynLinkLib;
}

/*------------------------------------------------------------------------------
| IMultimediaStatics::mmctlDLL                                                 |
------------------------------------------------------------------------------*/
IDynamicLinkLibrary& IMultimediaStatics :: mmctlDLL ( )
{
  IMODTRACE_DEVELOP("IMMStatics::mctlDLL");
  if (!mmctlDynLinkLib)
     mmctlDynLinkLib = new IDynamicLinkLibrary("sw");
  return *mmctlDynLinkLib;
}

/*------------------------------------------------------------------------------
| IMultimediaStatics::registerGraphicButton                                    |
------------------------------------------------------------------------------*/
Boolean
  IMultimediaStatics :: registerGraphicButton()
{
  IMODTRACE_DEVELOP("IMMStatics::registerGraphicButton");
  if (!entryRegisterGraphicButton)
     entryRegisterGraphicButton =
        (IRegisterGraphicButton*)mmctlDLL().procAddress( "WinRegisterGraphicButton");
  return (*entryRegisterGraphicButton)();
}

/*------------------------------------------------------------------------------
| IMultimediaStatics::registerSelectionSlider                                  |
------------------------------------------------------------------------------*/
Boolean
  IMultimediaStatics :: registerSelectionSlider()
{
  IMODTRACE_DEVELOP("IMMStatics::registerSelectionSlider");
  if (!entryRegisterSelectionSlider)
     entryRegisterSelectionSlider =
        (IRegisterSelectionSlider*)mmctlDLL().procAddress( "WinRegisterSelectionSlider");
  return (*entryRegisterSelectionSlider)();
}

/*------------------------------------------------------------------------------
| IMultimediaStatics::registerCircularSlider                                   |
------------------------------------------------------------------------------*/
Boolean
  IMultimediaStatics :: registerCircularSlider()
{
  IMODTRACE_DEVELOP("IMMStatics::registerCircularSlider");
  if (!entryRegisterCircularSlider)
     entryRegisterCircularSlider =
        (IRegisterCircularSlider*)mmctlDLL().procAddress( "WinRegisterCircularSlider");
  return (*entryRegisterCircularSlider)();
}

/*------------------------------------------------------------------------------
| IMultimediaStatics::getErrorString                                           |
------------------------------------------------------------------------------*/
unsigned long
  IMultimediaStatics :: getErrorString( unsigned long error, IString &buffer,
                                        unsigned short bufferLength)
{
  IMODTRACE_DEVELOP("IMMStatics::getErrorString");
  ITRACE_DEVELOP("IMMStatics::getErrorString error = " + IString(error));

  unsigned long rc;

  if (!entryGetErrorString)
     entryGetErrorString =
        (IGetErrorString*)mmpmDLL().procAddress(
#ifdef IC_PM
                                                 "mciGetErrorString"
#endif
#ifdef IC_WIN
                                                 "mciGetErrorStringA"
#endif
                                               );

#ifdef IC_PM
  rc = (*entryGetErrorString)(error,(char*)buffer,bufferLength);
#endif

#ifdef IC_WIN
  rc = (*entryGetErrorString)(error,(char*)buffer,255);
#endif

  ITRACE_DEVELOP("IMMStatics::getErrorString string = [" + IString((char *)buffer) + "]");

  return(rc);
}

/*------------------------------------------------------------------------------
| IMultimediaStatics::sendString                                               |
------------------------------------------------------------------------------*/
unsigned long
  IMultimediaStatics :: sendString( const IString &command, IString &returnData,
                                    unsigned short returnLength,
                                    IWindowHandle window,
                                    unsigned short userParam)
{
  IMODTRACE_DEVELOP("IMMStatics::sendString");
  ITRACE_DEVELOP("IMMStatics::sendString, command = [" + command + "]");

  unsigned long rc;

  if (!entrySendString)
     entrySendString =
        (ISendString*)(mmpmDLL().procAddress(
#ifdef IC_PM
                                            "mciSendString"
#endif
#ifdef IC_WIN
                                            "mciSendStringA"
#endif
                                           ));
#ifdef IC_PM
  rc = (*entrySendString)((char*)command,(char*)returnData,
                          returnLength,window,userParam);
#endif

#ifdef IC_WIN
  rc = (*entrySendString)((char*)command,(char*)returnData,
                          255,window);
#endif

  ITRACE_DEVELOP("IMMStatics::sendString, rc         = " + IString(rc));
  ITRACE_DEVELOP("IMMStatics::sendString, returnData = " + IString((char *)returnData));
  return rc;
}

/*------------------------------------------------------------------------------
| IMultimediaStatics::sendCommand                                              |
------------------------------------------------------------------------------*/
unsigned long
  IMultimediaStatics :: sendCommand( unsigned short deviceId,
                                     unsigned short message,
                                     unsigned long param1, void* param2,
                                     unsigned short userParam)
{
  IMODTRACE_DEVELOP("IMMStatics::sendCommand");
  ITRACE_DEVELOP("IMMStatics::sendCommand, deviceId = " + IString(deviceId));
  ITRACE_DEVELOP("IMMStatics::sendCommand, message  = 0x" + IString::d2x(message));
  ITRACE_DEVELOP("IMMStatics::sendCommand, param1   = 0x" + IString::d2x(param1));

  unsigned long rc;

  if (!entrySendCommand)
        entrySendCommand =
          (ISendCommand*)mmpmDLL().procAddress(
#ifdef IC_PM
                                               "mciSendCommand"
#endif
#ifdef IC_WIN
                                               "mciSendCommandA"
#endif
                                              );
#ifdef IC_PM
  rc = (*entrySendCommand)(deviceId,message,param1,param2,userParam);
#endif
#ifdef IC_WIN
  rc = (*entrySendCommand)(deviceId,message,param1,(DWORD)param2);
#endif

  ITRACE_DEVELOP("IMMStatics::sendCommand, rc = " + IString(rc));
  return rc;

}

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IMultimediaStatics::mixerGetNumDevs                                          |
------------------------------------------------------------------------------*/
Boolean
  IMultimediaStatics :: mixerGetNumDevs()
{
  IMODTRACE_DEVELOP("IMMStatics::mixerGetNumDevs");

  unsigned long rc;

  if (!entryMixerGetNumDevs)
     entryMixerGetNumDevs =
        (IMixerGetNumDevs*)mmpmDLL().procAddress("mixerGetNumDevs");

  rc = (*entryMixerGetNumDevs)();

  ITRACE_DEVELOP("IMMStatics::mixerGetNumDevs rc = [" + IString(rc) + "]");

  return(rc);
}

/*------------------------------------------------------------------------------
| IMultimediaStatics::mixerOpen                                          |
------------------------------------------------------------------------------*/
MMRESULT
  IMultimediaStatics :: mixerOpen(LPHMIXER phmx,
                                  UINT uMxId,
                                  DWORD dwCallback,
                                  DWORD dwInstance,
                                  DWORD fdwOpen)
{
  IMODTRACE_DEVELOP("IMMStatics::mixerOpen");

  MMRESULT rc;

  if (!entryMixerOpen)
     entryMixerOpen =
        (IMixerOpen*)mmpmDLL().procAddress("mixerOpen");

  rc = (*entryMixerOpen)(phmx,uMxId,dwCallback,dwInstance,fdwOpen);

  ITRACE_DEVELOP("IMMStatics::mixerOpen rc = [" + IString(rc) + "]");

  return(rc);
}

/*------------------------------------------------------------------------------
| IMultimediaStatics::mixerClose                                          |
------------------------------------------------------------------------------*/
MMRESULT
  IMultimediaStatics :: mixerClose(HMIXER hmx)
{
  IMODTRACE_DEVELOP("IMMStatics::mixerClose");

  MMRESULT rc;

  if (!entryMixerClose)
     entryMixerClose =
        (IMixerClose*)mmpmDLL().procAddress("mixerClose");

  rc = (*entryMixerClose)(hmx);

  ITRACE_DEVELOP("IMMStatics::mixerClose rc = [" + IString(rc) + "]");

  return(rc);
}

/*------------------------------------------------------------------------------
| IMultimediaStatics::mixerGetLineInfo                                          |
------------------------------------------------------------------------------*/
MMRESULT
  IMultimediaStatics :: mixerGetLineInfo(HMIXEROBJ hmxobj,
                                         LPMIXERLINE pmxl,
                                         DWORD fdwInfo)
{
  IMODTRACE_DEVELOP("IMMStatics::mixerGetLineInfo");

  MMRESULT rc;

  if (!entryMixerGetLineInfo)
     entryMixerGetLineInfo =
        (IMixerGetLineInfo*)mmpmDLL().procAddress("mixerGetLineInfoA");

  rc = (*entryMixerGetLineInfo)(hmxobj,pmxl,fdwInfo);

  ITRACE_DEVELOP("IMMStatics::mixerGetLineInfo rc = [" + IString(rc) + "]");

  return(rc);
}

/*------------------------------------------------------------------------------
| IMultimediaStatics::mixerGetLineControls                                          |
------------------------------------------------------------------------------*/
MMRESULT
  IMultimediaStatics :: mixerGetLineControls(HMIXEROBJ hmxobj,
                                             LPMIXERLINECONTROLS pmxlc,
                                             DWORD fdwControls)
{
  IMODTRACE_DEVELOP("IMMStatics::mixerGetLineControls");

  MMRESULT rc;

  if (!entryMixerGetLineControls)
     entryMixerGetLineControls =
        (IMixerGetLineControls*)mmpmDLL().procAddress("mixerGetLineControlsA");

  rc = (*entryMixerGetLineControls)(hmxobj,pmxlc,fdwControls);

  ITRACE_DEVELOP("IMMStatics::mixerGetLineControls rc = [" + IString(rc) + "]");

  return(rc);
}

/*------------------------------------------------------------------------------
| IMultimediaStatics::mixerGetControlDetails                                          |
------------------------------------------------------------------------------*/
MMRESULT
  IMultimediaStatics :: mixerGetControlDetails(HMIXEROBJ hmxobj,
                                               LPMIXERCONTROLDETAILS pmxcd,
                                               DWORD fdwDetails)
{
  IMODTRACE_DEVELOP("IMMStatics::mixerGetControlDetails");

  MMRESULT rc;

  if (!entryMixerGetControlDetails)
     entryMixerGetControlDetails =
        (IMixerGetControlDetails*)mmpmDLL().procAddress("mixerGetControlDetailsA");

  rc = (*entryMixerGetControlDetails)(hmxobj,pmxcd,fdwDetails);

  ITRACE_DEVELOP("IMMStatics::mixerGetControlDetails rc = [" + IString(rc) + "]");

  return(rc);
}

/*------------------------------------------------------------------------------
| IMultimediaStatics::mixerSetControlDetails                                          |
------------------------------------------------------------------------------*/
MMRESULT
  IMultimediaStatics :: mixerSetControlDetails(HMIXEROBJ hmxobj,
                                               LPMIXERCONTROLDETAILS pmxcd,
                                               DWORD fdwDetails)
{
  IMODTRACE_DEVELOP("IMMStatics::mixerSetControlDetails");

  MMRESULT rc;

  if (!entryMixerSetControlDetails)
     entryMixerSetControlDetails =
        (IMixerSetControlDetails*)mmpmDLL().procAddress("mixerSetControlDetails");

  rc = (*entryMixerSetControlDetails)(hmxobj,pmxcd,fdwDetails);

  ITRACE_DEVELOP("IMMStatics::mixerSetControlDetails rc = [" + IString(rc) + "]");

  return(rc);
}

/*------------------------------------------------------------------------------
| dumpRc                                                                       |
------------------------------------------------------------------------------*/
MCIERROR dumpRc(MCIERROR mciRc)
{
  ITRACE_DEVELOP("IMMStatics::dumpRc, rc = " + IString(mciRc));
  return(mciRc);
}

MCIERROR dumpRc(MCIERROR mciRc,char* mciRetData)
{
  ITRACE_DEVELOP("IMMStatics::dumpRc, rc = " + IString(mciRc));
  ITRACE_DEVELOP("IMMStatics::sendString, returnData = " + IString((char *)mciRetData));
  return(mciRc);
}
#endif
