/* NOSHIP */

/*************************************************************************
**
** asuminfo.h
**
** Copyright (c) 1990,1995 Microsoft Corp.  All Rights Reserved
**               (derived from SUMINFO.H)
** Copyright (c) 1995,1996 Lotus Development Corporation. All Rights Reserved
** 
** Description:
**    This file contains file contains data structure defintions,
**    function prototypes, constants, etc. for OLE 2.0 Property Set
**    utilities used to manage the Summary Info property set.
**
*************************************************************************/

#ifndef _ASUMINFO_H_
#define _ASUMINFO_H_
#include <ole2.h>
#if defined(WIN32)
#include <oleauto.h>
#else
#include <storage.h>
#include <variant.h>
#include <compobj.h>
#endif
#include <aspack.h>  
#define OLEUI_WORDMAX 256     
typedef     union {
     short    iVal;             
     long     lVal;             
     float    fltVal;           
     double       dblVal;       
     DWORD bool;                
     SCODE    scodeVal;         
     DWORD    systimeVal;       
     unsigned char bstrVal[OLEUI_WORDMAX]; 
   } OLEUI_VTUNION;
typedef struct _OLEUI_FMTID
   {
   DWORD dword;
   WORD words[2];
   BYTE bytes[8];
   } OLEUI_FMTID;
typedef struct _OLEUI_PROPSETLIST
   {
   OLEUI_FMTID formatID;
   DWORD byteOffset;
   } OLEUI_PROPSETLIST;
typedef struct _OLEUI_PROPIDLIST
   {
   DWORD propertyID;
   DWORD byteOffset;
   } OLEUI_PROPIDLIST;
typedef struct _OLEUI_PROPVALUE
   {
   DWORD vtType;
   OLEUI_VTUNION vtValue;
   } OLEUI_PROPVALUE;
typedef struct _OLEUI_SECTION
   {
   DWORD cBytes;
   DWORD cProperties;
   OLEUI_PROPIDLIST rgPropId[1];  
   OLEUI_PROPVALUE rgPropValue[1];      
   } OLEUI_SECTION;
typedef struct _OLEUI_SUMMARYINFO
   {
   WORD byteOrder;
   WORD formatVersion;
   WORD getOSVersion;
   WORD osVersion;
   CLSID classId;  
   DWORD cSections;
   OLEUI_PROPSETLIST rgPropSet[1]; 
   OLEUI_SECTION rgSections[1];    
   } OLEUI_SUMMARYINFO;
#define osWinOnDos 0
#define osMac 1
#define osWinNT 2
#define PID_DICTIONARY 0X00000000
#define PID_CODEPAGE 0X00000001
#define PID_TITLE 0X00000002
#define PID_SUBJECT 0X00000003
#define PID_AUTHOR 0X00000004
#define PID_KEYWORDS 0X00000005
#define PID_COMMENTS 0X00000006
#define PID_TEMPLATE 0X00000007
#define PID_LASTAUTHOR 0X00000008
#define PID_REVNUMBER 0X00000009
#define PID_EDITTIME 0X0000000A
#define PID_LASTPRINTED 0X0000000B
#define PID_CREATE_DTM_RO 0X0000000C
#define PID_LASTSAVE_DTM 0X0000000D
#define PID_PAGECOUNT 0X0000000E
#define PID_WORDCOUNT 0X0000000F
#define PID_CHARCOUNT 0X00000010
#define PID_THUMBNAIL 0X00000011
#define PID_APPNAME 0X00000012
#define PID_SECURITY 0X00000013
#define cPID_STANDARD (PID_SECURITY+1-2)
#ifndef WIN32
#define MAXWORD 256         
#endif
typedef struct _OLEUI_STDZ
   {
   DWORD vtType;
   union {
   DWORD vtByteCount;
   ASWOLECHAR fill[4];  
   };
   ASWOLECHAR rgchars[MAXWORD];
   } OLEUI_STDZ;
#define OLEUI_VTCB fill[3]    
typedef struct _OLEUI_THUMB
   {
   DWORD vtType;
   DWORD cBytes;       
   DWORD selector;     
   DWORD clipFormat;
   LPASWOLESTR lpstzName;
   LPASWOLESTR lpByte;
   } OLEUI_THUMB;
#define VT_CF_BYTES 0
#define VT_CF_WIN (-1)
#define VT_CF_MAC (-2)
#define VT_CF_FMTID (-3)
#define VT_CF_NAME (-4)
#define VT_CF_EMPTY (-5)
#define VT_CF_OOM (-6)      
typedef OLEUI_THUMB FAR *LPOLEUI_THUMB;
typedef OLEUI_STDZ FAR *LPOLEUI_STDZ;
typedef struct _OLEUI_TIME
   {
   DWORD vtType;
   FILETIME time;
   } OLEUI_TIME;
typedef struct _OLEUI_INTS
   {
   DWORD vtType;
   DWORD value;
   } OLEUI_INTS;
#define OLEUI_MAXTIME (PID_LASTSAVE_DTM-PID_EDITTIME+1)
#define OLEUI_MAXINTS (PID_CHARCOUNT-PID_PAGECOUNT+1+1)
#define OLEUI_MAXSTDZ (PID_REVNUMBER-PID_TITLE+1+1)
typedef struct _OLEUI_STANDARDSECINMEM
   {
   DWORD cBytes;
   DWORD cProperties;
   OLEUI_PROPIDLIST rgPropId[cPID_STANDARD];  
   OLEUI_TIME rgTime[OLEUI_MAXTIME];
   OLEUI_INTS rgInts[OLEUI_MAXINTS];
   LPOLEUI_STDZ rglpsz[OLEUI_MAXSTDZ];
   OLEUI_THUMB thumb;
   } OLEUI_STANDARDSECINMEM;
#define OLEUI_OFFSET_NIL 0X00000000
#define AllSecurityFlagsEqNone 0
#define fSecurityPassworded 1
#define fSecurityRORecommended 2
#define fSecurityRO 4
#define fSecurityLockedForAnnotations 8
#define PropStreamNamePrefixByte '\005'
#define PropStreamName "\005SummaryInformation"
#define cbNewSummaryInfo(nSection) (sizeof(OLEUI_SUMMARYINFO)-sizeof(OLEUI_SECTION)+sizeof(OLEUI_PROPSETLIST)*((nSection)-1))
#define cbNewSection(nPropIds) (sizeof(OLEUI_SECTION)-sizeof(OLEUI_PROPVALUE)+sizeof(OLEUI_PROPIDLIST)*((nPropIds)-1))
#define FIntelOrder(prop) ((prop)->byteOrder==0xfffe)
#ifndef LTSOS2    
#define SetOs(prop, os) {(prop)->osVersion=os; (prop)->getOSVersion=LOWORD(GetVersion());}
#else             
#define SetOs(prop, os) {(prop)->osVersion=0; (prop)->getOSVersion=0;}
#endif
#define SetSumInfFMTID(fmtId) {(fmtId)->dword=0XF29F85E0; *(long FAR *)&(fmtId)->words=0X10684FF9;\
            *(long FAR *)&(fmtId)->bytes[0]=0X000891AB; *(long FAR *)&(fmtId)->bytes[4]=0XD9B3272B;}
#define FEqSumInfFMTID(fmtId) ((fmtId)->dword==0XF29F85E0&&*((long FAR *)&(fmtId)->words)==0X10684FF9&&\
            *((long FAR *)&(fmtId)->bytes[0])==0X000891AB&&*((long FAR *)&(fmtId)->bytes[4])==0XD9B3272B)
#define FSzEqPropStreamName(sz) _fstricmp(sz, PropStreamName)
#define ClearSumInf(lpsuminf, cb) {_fmemset(lpsuminf,0,cb); (lpsuminf)->byteOrder=0xfffe;\
            SetOs(lpsuminf, osWinOnDos);}
typedef void FAR *LPOLEUISUMINFO;
typedef LPSTR LPSTZR;
typedef void FAR *THUMBNAIL;  
#define API __far __pascal
extern "C" {
STDAPI_(STDEXPORT LPOLEUISUMINFO) OleStdInitSummaryInfo(int reserved);
STDAPI_(STDEXPORT void) OleStdFreeSummaryInfo(LPOLEUISUMINFO FAR *lplp);
STDAPI_(STDEXPORT void) OleStdClearSummaryInfo(LPOLEUISUMINFO lp);
STDAPI_(STDEXPORT int) OleStdReadSummaryInfo(LPSTREAM lpStream, LPOLEUISUMINFO lp);
STDAPI_(STDEXPORT int) OleStdWriteSummaryInfo(LPSTREAM lpStream, LPOLEUISUMINFO lp);
STDAPI_(STDEXPORT DWORD) OleStdGetSecurityProperty(LPOLEUISUMINFO lp);
STDAPI_(STDEXPORT int) OleStdSetSecurityProperty(LPOLEUISUMINFO lp, DWORD security);
STDAPI_(STDEXPORT LPSTR) OleStdGetStringProperty(LPOLEUISUMINFO lp, DWORD pid);
STDAPI_(STDEXPORT int) OleStdSetStringProperty(LPOLEUISUMINFO lp, DWORD pid, LPASWOLESTR lpsz);
STDAPI_(STDEXPORT LPSTZR) OleStdGetStringZProperty(LPOLEUISUMINFO lp, DWORD pid);
STDAPI_(STDEXPORT void) OleStdGetDocProperty(
      LPOLEUISUMINFO  lp,
      DWORD FAR*      nPage,
      DWORD FAR*      nWords,
      DWORD FAR*      nChars
);
STDAPI_(STDEXPORT int) OleStdSetDocProperty(
      LPOLEUISUMINFO  lp,
      DWORD           nPage,
      DWORD           nWords,
      DWORD           nChars
);
STDAPI_(STDEXPORT int) OleStdGetThumbNailProperty(
      LPSTREAM        lps,
      LPOLEUISUMINFO  lp,
      DWORD FAR*      clipFormatNo,
      LPASWOLESTR FAR*lpszName,
      THUMBNAIL FAR*  clip,
      DWORD FAR*      byteCount,
      BOOL            transferClip
);
STDAPI_(STDEXPORT int) OleStdSetThumbNailProperty(
      LPSTREAM        lps,
      LPOLEUISUMINFO  lp,
      int             vtcfNo,
      DWORD           clipFormatNo,
      LPASWOLESTR     lpszName,
      THUMBNAIL       clip,
      DWORD           byteCount
);
STDAPI_(STDEXPORT void) OleStdGetDateProperty(
      LPOLEUISUMINFO  lp,
      DWORD           pid,
      int FAR*        yr,
      int FAR*        mo,
      int FAR*        dy,
      DWORD FAR*      sc
);
STDAPI_(STDEXPORT int) OleStdSetDateProperty(
      LPOLEUISUMINFO  lp,
      DWORD           pid,
      int             yr,
      int             mo,
      int             dy,
      int             hr,
      int             mn,
      int             sc
);
} 
#include <aspop.h>   
#endif  
