/*******************************************************************************
* FILE NAME: ibidiset.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions used for the    *
*   bidirectional support.                                                     *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

#define INCL_WINCOUNTRY
#define INCL_WINHOOKS
extern "C" {
   #include <iwindefs.h>
}

#include <ibidiset.hpp>
#include <ireslib.hpp>
#include <iwindow.hpp>
#include <iexcept.hpp>
#include <icconst.h>
#include <itbpriv.hpp>
#include <igrafctx.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IBIDISET_CPP_
  #include <ipagetun.h>
#endif

#ifdef IC_PM
/*------------------------------------------------------------------------------
| The following and defined as part of the OS/2 bidectional support toolkit    |
| (pmbidi.h).  These values are not currently a part of the base OS/2 toolkit  |
| but are planned to be added in the future.  In the meantime, define them     |
| here.                                                                        |
------------------------------------------------------------------------------*/
#ifndef PM_BIDI_INCLUDED
#define WM_SETBIDIATTR          0x0BD0
#define WM_QUERYBIDIATTR        0x0BD1
#define BDAM_INIT               0x80000000
#define BDAM_LEVEL              0x70000000
#define BDAM_TEXTTYPE           0x01000000
#define BDAM_TEXT_ORIENTATION   0x00030000
#define BDAM_WND_ORIENTATION    0x00100000
#define BDAM_NUMERALS           0x00003000
#define BDAM_SYM_SWAP           0x00000100
#define BDAM_WORD_BREAK         0x00000200
#define BDAM_TEXT_SHAPE         0x000000FF
#define BDA_INIT                0x80000000
#define BDA_LEVEL               0x30000000
#define BDA_TEXTTYPE_VISUAL     0x00000000
#define BDA_TEXTTYPE_IMPLICIT   0x01000000
#define BDA_WND_ORIENT_LTR      0x00000000
#define BDA_WND_ORIENT_RTL      0x00100000
#define BDA_TEXT_ORIENT_LTR     0x00000000
#define BDA_TEXT_ORIENT_RTL     0x00010000
#define BDA_TEXT_ORIENT_DYNAMIC 0x00020000
#define BDA_TEXT_ORIENT_CONTEXT 0x00020000
#define BDA_NUMERALS_NOMINAL    0x00000000
#define BDA_NUMERALS_PASSTHRU   0x00001000
#define BDA_NUMERALS_NATIONAL   0x00002000
#define BDA_NUMERALS_CONTEXTUAL 0x00003000
#define BDA_SYM_SWAP_OFF        0x00000000
#define BDA_SYM_SWAP_ON         0x00000100
#define BDA_WORD_BREAK_OFF      0x00000000
#define BDA_WORD_BREAK_ON       0x00000200
#define BDA_TEXT_DISPLAY_SHAPED 0x00000000
#define BDA_TEXT_SAVE_SHAPED    0x00000001
#define BDA_TEXT_NOMINAL        0x00000010
#define BDA_TEXT_INITIAL        0x00000011
#define BDA_TEXT_MIDDLE         0x00000012
#define BDA_TEXT_FINAL          0x00000013
#define BDA_TEXT_ISOLATED       0x00000014
#define LI_BD_WND_ATTR          0x00000020
#define LIF_CHILD_INHERIT       0x00000002
#define LIF_WND_REFRESH         0x00000004
#endif

typedef unsigned long
     (_System  IWinSetLangInfo)( unsigned long,   // hwnd
                                 unsigned long,   // effect
                                 unsigned long,   // data
                                 unsigned long,   // mask
                                 unsigned long,   // flags
                                 unsigned long ); // reserved

typedef unsigned long
     (_System  IWinQueryLangInfo)( unsigned long,   // hwnd
                                   unsigned long,   // effect
                                   unsigned long,   // flags
                                   unsigned long ); // reserved

typedef unsigned long
     (_System  IGpiQueryBidiAttr)( HPS ); // hps

typedef unsigned long
     (_System  IGpiSetBidiAttr)( HPS,             // hps
                                 unsigned long ); // attributes

#endif /* IC_PM */

/*------------------------------------------------------------------------------
| class IBidiData                                                              |
|                                                                              |
| Private class used for implementation of bidi support                        |
------------------------------------------------------------------------------*/
class IBidiData : public IBase {
typedef IBase
  Inherited;
public:
  IBidiData ( );
Boolean
  support;
unsigned long
  codepage;
#ifdef IC_PM
IDynamicLinkLibrary
  *pmBidiDynLinkLib;
IWinSetLangInfo
  *setInfo;
IWinQueryLangInfo
  *queryInfo;
IGpiSetBidiAttr
  *setBidiAttr;
IGpiQueryBidiAttr
  *queryBidiAttr;
#endif
};

IBidiData :: IBidiData ( )
           : support(false),
#ifdef IC_PM
             pmBidiDynLinkLib(0),
             setInfo(0),
             queryInfo(0),
             setBidiAttr(0),
             queryBidiAttr(0),
#endif
             codepage(0)
{
}

/*------------------------------------------------------------------------------
| class IBidiSettingsData                                                      |
|                                                                              |
| Private class used for implementation of bidi support                        |
------------------------------------------------------------------------------*/
class IBidiSettingsData : public IBase {
typedef IBase
  Inherited;
public:
unsigned long
  data;
static IBidiData
 *bidi;
};

/*------------------------------------------------------------------------------
| static data                                                                  |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLNonConst)
IBidiData* IBidiSettingsData::bidi(0);
#pragma data_seg()

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IBidiSettings::isBidiSupported ()
{
#ifdef IC_PM
  if ( !IBidiSettingsData::bidi )
  {
    IBidiSettingsData::bidi = new IBidiData;

    // If the codepage is 862(Hebrew) or 864(Arabic)
    IBidiSettingsData::bidi->codepage = WinQueryCp(HMQ_CURRENT);

    if (( IBidiSettingsData::bidi->codepage == 862 ) ||
        ( IBidiSettingsData::bidi->codepage == 864 ))
    {
      // Try to load the PMBIDI.DLL and get the address of APIs
      try
      {
        IBidiSettingsData::bidi->pmBidiDynLinkLib = new IDynamicLinkLibrary("pmbidi");
        IBidiSettingsData::bidi->setInfo = (IWinSetLangInfo*)
                           IBidiSettingsData::bidi->pmBidiDynLinkLib->procAddress
                           ("Win32SetLangInfo");
        IBidiSettingsData::bidi->queryInfo = (IWinQueryLangInfo*)
                           IBidiSettingsData::bidi->pmBidiDynLinkLib->procAddress
                           ("Win32QueryLangInfo");
        IBidiSettingsData::bidi->setBidiAttr = (IGpiSetBidiAttr*)
                           IBidiSettingsData::bidi->pmBidiDynLinkLib->procAddress
                           ("Gpi32SetBidiAttr");
        IBidiSettingsData::bidi->queryBidiAttr = (IGpiQueryBidiAttr*)
                           IBidiSettingsData::bidi->pmBidiDynLinkLib->procAddress
                           ("Gpi32QueryBidiAttr");
        IBidiSettingsData::bidi->support = true;
      }
      catch ( IException& exc ) {}
    }
  }
  return IBidiSettingsData::bidi->support;
#endif

#ifndef IC_PM
  return false; // Bidi is currently not supported for other platforms
#endif

}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings& IBidiSettings::apply ( const IWindow& window,
                                      Boolean childInherit, Boolean refresh )
{
  if ( isBidiSupported() == false )
  {
    ITHROWLIBRARYERROR(IC_UNEXPECTED_ERROR,
                       IBaseErrorInfo::invalidRequest,
                       IException::recoverable);
  }

#ifdef IC_PM
  //-----------------------------------------------------------------
  // There is a bug in PM that does not update frame extensions
  // when the BIDI attributes of the frame have changed (and the
  // child inherit flag is true).
  //
  // A fix for this problem is to send a private message to the frame
  // before we change the BIDI attributes.  This will enable the
  // window to decide if it should force the frame extensions to be
  // updated.  The following cases are covered:
  //
  // 1. IWindow::applyBidiSettings (no child inherit)
  //    For this case we first send the private message (to set a
  //    flag in the IFrameWindowData).  When IFrameHandler receives
  //    the BIDI attribute change, it will not update frame
  //    extensions (but will turn off the IFrameWindowData flag)
  //
  // 2. IWindow::applyBidiSettings (child inherit)
  //    We do not send the private message and when IFrameHandler
  //    received the BIDI attribute change, it will update all
  //    of the frame extensions.
  //
  // 3. IWindow::applyBidiSettings (to parent of the frame)
  //    (same processing as item 2)
  //
  // 4. User update through Language Viewer (ALT-ScrollLock) popup
  //    (same processing as item 2)
  //-----------------------------------------------------------------
  if ( !childInherit && window.isFrameWindow() )
     window.sendEvent ( IC_UM_BIDI_CHANGE );

  //-----------------------------------------------------------------
  // Let the base window handle case do the real work
  //-----------------------------------------------------------------
  apply ( window.handle(), childInherit, refresh );

  //-----------------------------------------------------------------
  // If the window is a frame, we need to update any toolbars as well
  // in case they are floating or hidden.  We do this by sending a
  // private message to the frame and letting the tool bar handler
  // (attached to the frame automatically by the tool bar) do the
  // work of updating BIDI attributes.
  //-----------------------------------------------------------------
  if ( childInherit && window.isFrameWindow() )
     window.sendEvent ( IC_UM_TOOLBAR, IC_UM_TBAR_BIDI_CHANGE );
#endif

  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings& IBidiSettings::apply ( const IWindowHandle windowHandle,
                                      Boolean childInherit, Boolean refresh )
{

#ifdef IC_PM
  unsigned long ulFlags = 0;

  if (refresh)
    ulFlags |= LIF_WND_REFRESH;

  if (childInherit)
    ulFlags |= LIF_CHILD_INHERIT;

  IBidiSettingsData::bidi->setInfo( windowHandle.asUnsigned(),
                                    LI_BD_WND_ATTR,
                                    fBidiSettingsData->data |
                                    BDA_INIT | BDA_LEVEL,
                                    BDAM_INIT | BDAM_LEVEL |
                                    BDAM_TEXTTYPE | BDAM_TEXT_ORIENTATION |
                                    BDAM_WND_ORIENTATION | BDAM_NUMERALS |
                                    BDAM_SYM_SWAP | BDAM_WORD_BREAK |
                                    BDAM_TEXT_SHAPE,
                                    ulFlags, 0 );
#endif

  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings& IBidiSettings::apply ( const IGraphicContext& graphicContext )
{
  return apply ( graphicContext.handle() );
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings& IBidiSettings::apply ( const IPresSpaceHandle presSpace )
{
#ifdef IC_PM
  IBidiSettingsData::bidi->setBidiAttr( presSpace.asUnsigned(),
                                        fBidiSettingsData->data );
#endif
  return *this;
}


/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings::IBidiSettings ( const IWindow& window )
              :fBidiSettingsData(0)
{
  if ( isBidiSupported() == false )
  {
    ITHROWLIBRARYERROR(IC_UNEXPECTED_ERROR,
                       IBaseErrorInfo::invalidRequest,
                       IException::recoverable);
  }

  fBidiSettingsData = new IBidiSettingsData;
#ifdef IC_PM
  fBidiSettingsData->data = IBidiSettingsData::bidi->queryInfo
                    ( window.handle().asUnsigned(), LI_BD_WND_ATTR, 0L, 0L);
#endif
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings::IBidiSettings ( const IWindowHandle windowHandle )
              :fBidiSettingsData(0)
{
  if ( isBidiSupported() == false )
  {
    ITHROWLIBRARYERROR(IC_UNEXPECTED_ERROR,
                       IBaseErrorInfo::invalidRequest,
                       IException::recoverable);
  }

  fBidiSettingsData = new IBidiSettingsData;
#ifdef IC_PM
  fBidiSettingsData->data = IBidiSettingsData::bidi->queryInfo
                    ( windowHandle.asUnsigned(), LI_BD_WND_ATTR, 0L, 0L);
#endif
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings::IBidiSettings ( const IGraphicContext& graphicContext )
              :fBidiSettingsData(0)
{
  if ( isBidiSupported() == false )
  {
    ITHROWLIBRARYERROR(IC_UNEXPECTED_ERROR,
                       IBaseErrorInfo::invalidRequest,
                       IException::recoverable);
  }

  fBidiSettingsData = new IBidiSettingsData;
#ifdef IC_PM
  fBidiSettingsData->data = IBidiSettingsData::bidi->queryBidiAttr
                    ( graphicContext.handle().asUnsigned() );
#endif
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings::IBidiSettings ( const IPresSpaceHandle presSpace )
              :fBidiSettingsData(0)
{
  if ( isBidiSupported() == false )
  {
    ITHROWLIBRARYERROR(IC_UNEXPECTED_ERROR,
                       IBaseErrorInfo::invalidRequest,
                       IException::recoverable);
  }

  fBidiSettingsData = new IBidiSettingsData;
#ifdef IC_PM
  fBidiSettingsData->data = IBidiSettingsData::bidi->queryBidiAttr
                    ( presSpace.asUnsigned() );
#endif
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings::~IBidiSettings ()
{
   delete fBidiSettingsData;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings& IBidiSettings::setTextType
               ( IBidiSettings::BidiTextType textType )
{
#ifdef IC_PM
  switch ( textType )
  {
    case visual:
      fBidiSettingsData->data &= ~BDA_TEXTTYPE_IMPLICIT;
      fBidiSettingsData->data |= BDA_TEXTTYPE_VISUAL;
      break;
    case implicit:
      fBidiSettingsData->data &= ~BDA_TEXTTYPE_VISUAL;
      fBidiSettingsData->data |= BDA_TEXTTYPE_IMPLICIT;
      break;
  }
#endif
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings& IBidiSettings::setTextOrientation
               ( IBidiSettings::BidiTextOrientation textOrientation )
{
#ifdef IC_PM
  switch ( textOrientation )
  {
    case textLeftToRight:
      fBidiSettingsData->data &= ~BDA_TEXT_ORIENT_RTL & ~BDA_TEXT_ORIENT_DYNAMIC;
      fBidiSettingsData->data |= BDA_TEXT_ORIENT_LTR;
      break;
    case textRightToLeft:
      fBidiSettingsData->data &= ~BDA_TEXT_ORIENT_LTR & ~BDA_TEXT_ORIENT_DYNAMIC;
      fBidiSettingsData->data |= BDA_TEXT_ORIENT_RTL;
      break;
    case textContextual:
      fBidiSettingsData->data &= ~BDA_TEXT_ORIENT_LTR & ~BDA_TEXT_ORIENT_RTL;
      fBidiSettingsData->data |= BDA_TEXT_ORIENT_DYNAMIC;
      break;
  }
#endif
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings& IBidiSettings::setWindowLayout
               ( IBidiSettings::BidiLayout windowLayout )
{
#ifdef IC_PM
  switch ( windowLayout )
  {
    case layoutLeftToRight:
      fBidiSettingsData->data &= ~BDA_WND_ORIENT_RTL;
      fBidiSettingsData->data |= BDA_WND_ORIENT_LTR;
      break;
    case layoutRightToLeft:
      fBidiSettingsData->data &= ~BDA_WND_ORIENT_LTR;
      fBidiSettingsData->data |= BDA_WND_ORIENT_RTL;
      break;
  }
#endif
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings& IBidiSettings::setNumeralDisplay
               ( IBidiSettings::BidiNumeralType numeralDisplay )
{

#ifdef IC_PM
  fBidiSettingsData->data &= ~BDA_NUMERALS_NOMINAL & ~BDA_NUMERALS_PASSTHRU &
                             ~BDA_NUMERALS_NATIONAL & ~BDA_NUMERALS_CONTEXTUAL;

  switch ( numeralDisplay )
  {
    case arabic:
      fBidiSettingsData->data |= BDA_NUMERALS_NOMINAL;
      break;
    case asStored:
      fBidiSettingsData->data |= BDA_NUMERALS_PASSTHRU;
      break;
    case national:
      fBidiSettingsData->data |= BDA_NUMERALS_NATIONAL;
      break;
    case contextual:
      fBidiSettingsData->data |= BDA_NUMERALS_CONTEXTUAL;
      break;
  }
#endif
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings& IBidiSettings::enableSymmetricSwapping ( Boolean enable )
{
#ifdef IC_PM
  if ( enable )
  {
    fBidiSettingsData->data &= ~BDA_SYM_SWAP_OFF;
    fBidiSettingsData->data |= BDA_SYM_SWAP_ON;
  }
  else
  {
    fBidiSettingsData->data &= ~BDA_SYM_SWAP_ON;
    fBidiSettingsData->data |= BDA_SYM_SWAP_OFF;
  }
#endif
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings& IBidiSettings::disableSymmetricSwapping ( )
{
   return enableSymmetricSwapping(false);
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings& IBidiSettings::enableWordByWordReordering ( Boolean enable )
{
#ifdef IC_PM
  if ( enable )
  {
    fBidiSettingsData->data &= ~BDA_WORD_BREAK_OFF;
    fBidiSettingsData->data |= BDA_WORD_BREAK_ON;
  }
  else
  {
    fBidiSettingsData->data &= ~BDA_WORD_BREAK_ON;
    fBidiSettingsData->data |= BDA_WORD_BREAK_OFF;
  }
#endif
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings& IBidiSettings::disableWordByWordReordering ( )
{
   return enableWordByWordReordering(false);
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings& IBidiSettings::setTextShape
               ( IBidiSettings::BidiTextShape textShape )
{

#ifdef IC_PM
  fBidiSettingsData->data &= ~BDA_TEXT_DISPLAY_SHAPED & ~BDA_TEXT_SAVE_SHAPED &
                             ~BDA_TEXT_NOMINAL & ~BDA_TEXT_MIDDLE & ~BDA_TEXT_FINAL &
                             ~BDA_TEXT_ISOLATED;

  switch ( textShape )
  {
    case displayShaped:
      fBidiSettingsData->data |= BDA_TEXT_DISPLAY_SHAPED;
      break;
    case saveShaped:
      fBidiSettingsData->data |= BDA_TEXT_SAVE_SHAPED;
      break;
    case nominalShape:
      fBidiSettingsData->data |= BDA_TEXT_NOMINAL;
      break;
    case initialShape:
      fBidiSettingsData->data |= BDA_TEXT_INITIAL;
      break;
    case middleShape:
      fBidiSettingsData->data |= BDA_TEXT_MIDDLE;
      break;
    case finalShape:
      fBidiSettingsData->data |= BDA_TEXT_FINAL;
      break;
    case isolatedShape:
      fBidiSettingsData->data |= BDA_TEXT_ISOLATED;
      break;
  }
#endif
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings::BidiTextType IBidiSettings::textType () const
{
#ifdef IC_PM
  if ( fBidiSettingsData->data & BDA_TEXTTYPE_IMPLICIT )
    return implicit;
#endif
  return visual;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings::BidiTextOrientation IBidiSettings::textOrientation () const
{
#ifdef IC_PM
  if ( fBidiSettingsData->data & BDA_TEXT_ORIENT_DYNAMIC )
    return textContextual;
  if ( fBidiSettingsData->data & BDA_TEXT_ORIENT_RTL )
    return textRightToLeft;
#endif
  return textLeftToRight;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings::BidiLayout IBidiSettings::windowLayout () const
{
#ifdef IC_PM
  if ( fBidiSettingsData->data & BDA_WND_ORIENT_RTL )
    return layoutRightToLeft;
#endif
  return layoutLeftToRight;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings::BidiNumeralType IBidiSettings::numeralDisplay () const
{
#ifdef IC_PM
  if (( fBidiSettingsData->data & BDA_NUMERALS_CONTEXTUAL ) == BDA_NUMERALS_CONTEXTUAL )
    return contextual;
  if (( fBidiSettingsData->data & BDA_NUMERALS_PASSTHRU ) == BDA_NUMERALS_PASSTHRU )
    return asStored;
  if (( fBidiSettingsData->data & BDA_NUMERALS_NATIONAL ) == BDA_NUMERALS_NATIONAL )
    return national;
#endif
  return arabic;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IBidiSettings::isSymmetricSwappingEnabled () const
{
#ifdef IC_PM
  if ( fBidiSettingsData->data & BDA_SYM_SWAP_ON )
    return true;
#endif
  return false;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IBidiSettings::isWordByWordReorderingEnabled () const
{
#ifdef IC_PM
  if ( fBidiSettingsData->data & BDA_WORD_BREAK_ON )
    return true;
#endif
  return false;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings::BidiTextShape IBidiSettings::textShape () const
{
#ifdef IC_PM
  if (( fBidiSettingsData->data & BDA_TEXT_ISOLATED ) == BDA_TEXT_ISOLATED )
    return isolatedShape;
  if (( fBidiSettingsData->data & BDA_TEXT_FINAL ) == BDA_TEXT_FINAL )
    return finalShape;
  if (( fBidiSettingsData->data & BDA_TEXT_MIDDLE ) == BDA_TEXT_MIDDLE )
    return middleShape;
  if (( fBidiSettingsData->data & BDA_TEXT_INITIAL ) == BDA_TEXT_INITIAL )
    return initialShape;
  if (( fBidiSettingsData->data & BDA_TEXT_NOMINAL ) == BDA_TEXT_NOMINAL )
    return nominalShape;
  if (( fBidiSettingsData->data & BDA_TEXT_SAVE_SHAPED ) == BDA_TEXT_SAVE_SHAPED )
    return saveShaped;
#endif
  return displayShaped;
}

