/* NOSHIP */
#ifndef _IBMPDATA_
#define _IBMPDATA_
/*******************************************************************************
* FILE NAME: ibmpdata.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IBitmapData                                                              *
*     IBitmapSet                                                               *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ibase.hpp>
#include <istring.hpp>
#include <ihandle.hpp>
#include <ipoint.hpp>
#include <iset.h>

#pragma pack(4)

class IBitmapData : public IBase {
/*******************************************************************************
* The IBitmapData class provides reference counting mechanism for the IBitmap  *
* and IBitmapHandle class.  This class contains the bitmap handle, the         *
* bitmap resource id, the file the from which the bitmap was loaded, and the   *
* size of the bitmap.                                                          *
*                                                                              *
*******************************************************************************/
public:
/*------------------------- Constructors/Destructor ----------------------------
| There are two ways to construct an IBitmapData object.                       |
|                                                                              |
| You can construct an IBitmapData object from its resource id, the bitmap     |
| handle, the file from which the bitmap was loaded and optionally the         |
| size of the bitmap.  If the size is not passed on the ctor, then the         |
| size defaults to 0,0.                                                        |
------------------------------------------------------------------------------*/
   IBitmapData( unsigned long ulId,
                unsigned long handle, const IString& strFileName);
   IBitmapData( unsigned long ulId,
                unsigned long handle, const IString& strFileName,
                const ISize& sizBitmap);
   ~IBitmapData();

/*-------------------------------- Accessors -----------------------------------
| addRef           - Increment the use count and return the current use count. |
| removeRef        - Decrement the use count and return the current use count. |
| useCount         - Return the current use count.                             |
| id               - return the resource id for the bitmap.                    |
| handle           - return the bitmap handle.                                 |
| setHandle        - Replace the current handle with the new handle.           |
| setSize          - Replace the current size with the new size.               |
| resourceFileName - Return the name of the file from which the bitmap was     |
|                     loaded.                                                  |
| size             - Return the size of the bitmap.  This man not reflect      |
|                    the actual size of the bitmap.                            |
------------------------------------------------------------------------------*/
unsigned long
  addRef           ( ),
  removeRef        ( ),
  useCount         ( ) const,
  id               ( ) const,
  handle           ( ) const;

IBitmapData
 &setHandle        ( const unsigned long ulNewHandle ),
 &setSize          ( const ISize& newSize            ),
 &setChanged       ( Boolean changed = true          );

IString
  resourceFileName ( ) const;
ISize
  size             ( ) const;
Boolean
  changed          ( ) const;

/*--------------------------- Comparison Operators -----------------------------
| operator == - Returns true if the id size and resource file names are        |
|               identical.                                                     |
------------------------------------------------------------------------------*/
Boolean operator ==(const IBitmapData& bmpData ) const;

private:

  unsigned long ulClId;
  unsigned long ulClHandle;
  IString strClFileName;
  unsigned long ulClUseCount;
  ISize sizClBitmap;
  Boolean chngd;
};

class IBitmapSet : public  ISet<IBitmapData*>
{
/*******************************************************************************
* The IBitmapSet is a set of pointers to IBitmapData objects.  There is only   *
* one such set per process pointed to by a static data member in IReslibStatic *
*******************************************************************************/
public:
/*------------------------- Constructors/Destructor ----------------------------
| Objects of this class must be created using the default constructor.         |
------------------------------------------------------------------------------*/
  IBitmapSet();
  ~IBitmapSet();
/*-------------------------------- Accessors -----------------------------------
------------------------------------------------------------------------------*/
  IBitmapData* find( const unsigned long ulBitmapId,
                     const IString& fileName,
                     const ISize& sizeBitmap );
  IBitmapData* find( const IBitmapHandle& bmh );
  IBitmapData* find( const IString& fileName  );
  IBitmapSet&  remove( IBitmapData* pBmpData );
};

#pragma pack()

#ifndef I_NO_INLINES
  #include <ibmpdata.inl>
#endif

#endif // _IBMPDATA_
