/*******************************************************************************
* FILE NAME: icustbev.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icustbev.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C" {
  #define INCL_WINBUTTONS
  #include <iwindefs.h>
}

#include <icustbev.hpp>
#include <icustbut.hpp>
#include <ictlevt.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICUSTBEV_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| ICustomButtonDrawEvent::ICustomButtonDrawEvent                               |
------------------------------------------------------------------------------*/
ICustomButtonDrawEvent :: ICustomButtonDrawEvent ( const IEvent& event )
                        : IControlEvent ( event ),
#ifndef IC_WIN
                          fGraphicContext ( controlWindow()->handle() )
#else
                          fGraphicContext ( (IPresSpaceHandle)((LPDRAWITEMSTRUCT)event.parameter2().asUnsignedLong())->hDC )
#endif
{
  fRectangle = controlWindow()->size();
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawEvent::ICustomButtonDrawEvent                               |
------------------------------------------------------------------------------*/
ICustomButtonDrawEvent :: ICustomButtonDrawEvent
                          ( const ICustomButtonDrawEvent& event )
                        : IControlEvent ( event ),
#ifndef IC_WIN
                          fGraphicContext ( controlWindow()->handle() )
#else
                          fGraphicContext ( (IPresSpaceHandle)((LPDRAWITEMSTRUCT)event.parameter2().asUnsignedLong())->hDC )
#endif
{
  fRectangle = controlWindow()->size();
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawEvent::ICustomButtonDrawEvent                               |
------------------------------------------------------------------------------*/
ICustomButtonDrawEvent :: ICustomButtonDrawEvent
                          ( const IEvent& event, const ISize& size )
                        : IControlEvent  ( event ),
                          fGraphicContext( size )
{
  fRectangle = size;
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawEvent::~ICustomButtonDrawEvent                              |
------------------------------------------------------------------------------*/
ICustomButtonDrawEvent :: ~ICustomButtonDrawEvent ()
{
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawEvent::graphicContext                                       |
------------------------------------------------------------------------------*/
IGraphicContext& ICustomButtonDrawEvent :: graphicContext () const
{
  return (((ICustomButtonDrawEvent*)this)->fGraphicContext);
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawEvent::ButtonState ICustomButtonDrawEvent :: buttonState    |
------------------------------------------------------------------------------*/
ICustomButtonDrawEvent::ButtonState ICustomButtonDrawEvent :: buttonState () const
{
#ifdef IC_WIN
  DRAWITEMSTRUCT *pUser = (DRAWITEMSTRUCT *)parameter2().asUnsignedLong();
  if ( pUser->itemState & ODS_SELECTED )
#else
  USERBUTTON *pUser = (USERBUTTON *)parameter2().asUnsignedLong();
  if ( pUser->fsState & BDS_HILITED )
#endif
    return buttonDown;
  if ( customButton()->isLatched() )
    return buttonLatched;
  return buttonUp;
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawEvent::drawUp                                               |
------------------------------------------------------------------------------*/
Boolean ICustomButtonDrawEvent :: drawUp () const
{
#ifdef IC_WIN
  DRAWITEMSTRUCT *pUser = (DRAWITEMSTRUCT *)parameter2().asUnsignedLong();
  if ( pUser->itemState & ODS_SELECTED )
#else
  USERBUTTON *pUser = (USERBUTTON *)parameter2().asUnsignedLong();
  if ( pUser->fsState & BDS_HILITED )
#endif
    return false;
  if ( customButton()->isLatched() )
    return false;
  return true;
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawEvent::drawDown                                             |
------------------------------------------------------------------------------*/
Boolean ICustomButtonDrawEvent :: drawDown () const
{
#ifdef IC_WIN
  DRAWITEMSTRUCT *pUser = (DRAWITEMSTRUCT *)parameter2().asUnsignedLong();
  if ( pUser->itemState & ODS_SELECTED )
#else
  USERBUTTON *pUser = (USERBUTTON *)parameter2().asUnsignedLong();
  if ( pUser->fsState & BDS_HILITED )
#endif
    return true;
  else
    return false;
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawEvent::drawLatched                                          |
------------------------------------------------------------------------------*/
Boolean ICustomButtonDrawEvent :: drawLatched () const
{
#ifdef IC_WIN
  DRAWITEMSTRUCT *pUser = (DRAWITEMSTRUCT *)parameter2().asUnsignedLong();
  if ( pUser->itemState & ODS_SELECTED )
#else
  USERBUTTON *pUser = (USERBUTTON *)parameter2().asUnsignedLong();
  if ( pUser->fsState & BDS_HILITED )
#endif
    return false;
  if ( customButton()->isLatched() )
    return true;
  return false;
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawEvent::isButtonEnabled                                      |
------------------------------------------------------------------------------*/
Boolean ICustomButtonDrawEvent :: isButtonEnabled () const
{
#ifdef IC_WIN
  DRAWITEMSTRUCT *pUser = (DRAWITEMSTRUCT *)parameter2().asUnsignedLong();
  if ( pUser->itemState & ODS_DISABLED )
#else
  USERBUTTON *pUser = (USERBUTTON *)parameter2().asUnsignedLong();
  if ( pUser->fsState & BDS_DISABLED )
#endif
    return false;
  return true;
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawEvent::customButton                                         |
------------------------------------------------------------------------------*/
ICustomButton* ICustomButtonDrawEvent :: customButton () const
{
  return ((ICustomButton*)controlWindow());
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawEvent::drawingArea                                          |
------------------------------------------------------------------------------*/
IRectangle ICustomButtonDrawEvent :: drawingArea () const
{
  return fRectangle;
}

/*------------------------------------------------------------------------------
| ICustomButtonDrawEvent::setDrawingArea                                       |
------------------------------------------------------------------------------*/
ICustomButtonDrawEvent& ICustomButtonDrawEvent :: setDrawingArea
                                                  ( const IRectangle& rectangle )
{
  fRectangle = rectangle;
  return *this;
}

