/* NOSHIP */
#ifndef _IDMDITM_
#define _IDMDITM_
/*******************************************************************************
* FILE NAME: idmditm.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IDragItem                                                                *
*     IDragItemWrapper                                                         *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ihandle.hpp>
#include <ipoint.hpp>
#include <istring.hpp>
#include <imarsha.hpp>

#pragma pack(4)

class IDragItem : public IDMMarshall {
typedef IVBase
  Inherited;
public:
/*------------------------------- Constructors -------------------------------*/
  IDragItem ( );

virtual
 ~IDragItem ( );

virtual unsigned long
  marshall              ( unsigned long startPos=0, char* outStr=0 ) const;

virtual const char
 *unmarshall            ( const char* inStr );

private:
/*--------------------------------- Private ----------------------------------*/
friend class IDragItemWrapper;

IString
  strTypes,
  strRMFs,
  strContainerName,
  strSourceName,
  strTargetName;

ISize
  sizeImageOffset;

IWindowHandle
  hwndItem;

unsigned long
  ulItemID;

unsigned short
  fsControl,
  fsSupportedOps;

}; // IDragItem

#if defined(IC_MOTIF) || defined(IC_WIN)
  /*---------------------------------------------------------------------------
  | This allows us to keep the IC_PM interfaces that use _DRAGITEM* arguments.
  ---------------------------------------------------------------------------*/
  typedef IDragItem _DRAGITEM;
  typedef IDragItem DRAGITEM;
#endif
#ifdef IC_PM
  struct _DRAGITEM;
  class IStringHandle;
#endif

class IDragItemWrapper  {
public:
/*------------------------------- Constructors -------------------------------*/
  IDragItemWrapper ( _DRAGITEM& aDragItem );

  IDragItemWrapper ( const IDragItemWrapper& item );

virtual
 ~IDragItemWrapper ( );

_DRAGITEM
 *asDragItemPtr         ( ) const;

IWindowHandle
  sourceWindowHandle    ( ) const;

unsigned long
  itemID                ( ) const;

unsigned short
  dragAttrs             ( ) const,
  dragSupOps            ( ) const;

/*----------------------------------------------------------------------------*/
/* These return const& for performance, to avoid IString and ISize ctrs/dtrs. */
/* It forces our data representation, but that is OK since this is a NOSHIP   */
/* file and we can change this to something else if we ever have to.          */
/*----------------------------------------------------------------------------*/

const ISize
 &imageOffset           ( ) const;

const IString
 &types                 ( ) const,
 &rmfs                  ( ) const,
 &containerName         ( ) const,
 &sourceName            ( ) const,
 &targetName            ( ) const;

IDragItemWrapper
 &setSourceWindowHandle ( const IWindowHandle& window ),
 &setItemID             ( unsigned long ulID),
 &setTypes              ( const IString& types ),
 &setRMFs               ( const IString& rmfs ),
 &setContainerName      ( const IString& containerName ),
 &setTargetName         ( const IString& targetName ),
 &setSourceName         ( const IString& sourceName ),
 &setImageOffset        ( const ISize& offset),
 &setDragAttrs          ( unsigned short attrs),
 &setDragSupOps         ( unsigned short supOps),
 &clearStrings          ( );

#ifdef IC_PM
static void
  setHSTR          ( unsigned long&       oldHSTR,
                     const IStringHandle& newHSTR );
static IString
  stringFromHandle ( const IStringHandle& hstr );

static IStringHandle
  handleFromString ( const IString& str );
#endif

private:
/*--------------------------------- Private ----------------------------------*/
// Dissallow assignment.
IDragItemWrapper
 &operator = ( const IDragItemWrapper& item );

_DRAGITEM
 &theDragItem;

#ifdef IC_PM
IDragItem
 &workDragItem;
#endif
}; // IDragItemWrapper

#pragma pack()

#ifndef I_NO_INLINES
  #include <idmditm.inl>
#endif

#endif // _IDMDITM_
