/* NOSHIP */
#ifndef _IFNTPRIV_
#define _IFNTPRIV_
/*******************************************************************************
* FILE NAME: ifntpriv.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the definition of classes that are private              *
*   to the implementation of ifont.                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ibase.hpp>
#include <ipoint.hpp>
#include <ihandle.hpp>

/*----------------------------------------------------------------------------*/
/* Align classes on four byte boundary.                                       */
/*----------------------------------------------------------------------------*/
#pragma pack(4)

#ifdef IC_WIN

/*--------------------------------------------------------------------*/
/* Need a wrapper around font handles, allowing us to ref count those */
/* that we allocated so that we can delete them when we are done      */
/*--------------------------------------------------------------------*/
class IFontHandle : public IHandle {
typedef IHandle
  Inherited;
public:
/*------------------------------- Constructors -------------------------------*/
  IFontHandle ( Value hFont = 0,
                Boolean allocated = false );

 ~IFontHandle ( );

unsigned long
  refCount() const { return refs; }

IFontHandle
  &addRef() { refs++; return *this; }

void
  removeRef();

private:
/*-------------------------------- Private -----------------------------------*/
unsigned long
  refs;

Boolean
  fAllocatedByIFont;
}; // class IFontHandle

/*--------------------------------------------------------------------*/
/* Structure declarations for caching font enumeration callback info  */
/*--------------------------------------------------------------------*/
typedef struct _FONTINFO {   /* fi */
  char   szFaceName[LF_FACESIZE];  /* face name                       */
  int    iPointSize;               /* point size of font (if fBitmap) */
  struct _FONTINFO *pfiNext;       /* next FONTINFO in the list       */
} FONTINFO, *PFONTINFO;

typedef struct _FACEINFO {   /* faci */
  char   szFaceName[LF_FACESIZE];  /* face name                       */
  struct _FACEINFO *pfaciNext;     /* next FACEINFO in the list       */
} FACEINFO, *PFACEINFO;

typedef struct _FONTCOUNT {   /* fcnt */
  PFONTINFO pFontInfo;             /* Pointer to linked list of fonts */
  int       matchCount;            /* Count of fonts matching parms   */
  DWORD     dwType;                /* Type of font to match           */
  long      horzRes;               /* Resolution to match if type     */
  long      vertRes;               /*  of font is a bitmap            */
  BOOL      bPointSizes;
} FONTCOUNT, *PFONTCOUNT;

int CALLBACK EnumAllFonts( LPENUMLOGFONT, LPNEWTEXTMETRIC, int, LPARAM );
int CALLBACK EnumAllFaces( LPENUMLOGFONT, LPNEWTEXTMETRIC, int, LPARAM );
#endif  // IC_WIN

class IFontPrivateData : public IBase {
typedef IBase
  Inherited;
/*******************************************************************************
* Objects of IFontData contain the private data for the IFont class.           *
* In addition, IFontData provides reference counting for IFont objects.        *
*******************************************************************************/
friend class IFont;
public:
/*------------------------------- Constructors ---------------------------------
| There are n ways to construct instances of this class:                       |
|   1. default                                                                 |
|   2. Copy constructor.                                                       |
------------------------------------------------------------------------------*/
  IFontPrivateData( );

  IFontPrivateData( const IFontPrivateData& fpd );

unsigned long
  refCount() const { return refs; }

IFontPrivateData
  &addRef() { refs++; return *this; }
void
  removeRef();

private:
/*------------------------------ Data Members ----------------------------------
|  The following data members are used to different font values:               |
|   pfontmetCl - Holds the font metrics for the font.                          |
|   plClCharWidths - Holds the array of character widths for the font.         |
|   pfattrsCl - Holds the font attributes structure.                           |
|   fxClPointSize - Holds the point size for the font.                         |
|   fxClHeight - Holds the character height.                                   |
|   fxClWidth - Holds the character width.                                     |
|   lClFontDir - Holds the direction to draw the font.                         |
|   lClXAngle - Holds the X portion of the font angle.                         |
|   lClYAngle - Holds the Y portion of the font angle.                         |
|   ptlClShear - Holds the font shear.                                         |
|   lClLCId - Holds the ID uses for the logical font.                          |
------------------------------------------------------------------------------*/
int
  flClStyles;

IPoint
  pntClAngle,
  pntClShear;

long
  fxClPointSize,
  lClFontDir;

#ifdef IC_PM
struct _FONTMETRICS
 *pfontmetCl;

struct _FATTRS
 *pfattrsCl;

long
 *plClCharWidths,
  fxClHeight,
  fxClWidth,
  lClLCId;
#endif

#ifdef IC_WIN
IFontHandle
 *fpFontHandle;

HFONT
  prevFont,
  systemFont;

BYTE
  fcharSet;

struct tagLOGFONTA
 *plogfontCl;
struct tagTEXTMETRICA
 *ptextmetricCl;
#endif

unsigned long
  inUseCount,
  refs;

IFontPrivateData
 &operator=( const IFontPrivateData&);
}; // IFontData

/*----------------------------------------------------------------------------*/
/* Resume compiler default packing.                                           */
/*----------------------------------------------------------------------------*/
#pragma pack()

/*--------------------------------- INLINES ----------------------------------*/

#endif /* _IFNTPRIV_ */
