/*******************************************************************************
* FILE NAME: ifonthdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes declared in ifonthdr.hpp                        *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define INCL_WINMESSAGEMGR
#define INCL_WINSTDFONT
#define INCL_WINWINDOWMGR
extern "C" {
   #include <iwindefs.h>
}

#include <ifonthdr.hpp>
#include <ifontdlg.hpp>
#include <ifont.hpp>
#include <ievent.hpp>
#include <ihandle.hpp>
#ifdef IC_PM
#include <ipmexsta.hpp>
#endif
#include <istring.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IFONTHDR_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IFontDialogHandler::IFontDialogHandler                                      |
------------------------------------------------------------------------------*/
IFontDialogHandler::IFontDialogHandler() {}

/*------------------------------------------------------------------------------
| IFontDialogHandler::~IFontDialogHandler                                      |
------------------------------------------------------------------------------*/
IFontDialogHandler::~IFontDialogHandler() {}


/*------------------------------------------------------------------------------
| IFontDialogHandler::dispatchHandlerEvent                                     |
------------------------------------------------------------------------------*/
Boolean IFontDialogHandler::dispatchHandlerEvent( IEvent& evt )
{
  unsigned long idOfEvent = evt.eventId();
  switch (idOfEvent)
  {
#ifdef IC_WIN
    case IC_UM_DLG_APPLY_PRESSED:
#endif
    case WM_DESTROY:
    {
#ifdef IC_PM
      /*****************************************************************/
      /* Get the "this" pointer for window object from the window word */
      /* which is set during dialog initialization                     */
      /*****************************************************************/
      FONTDLG* pfontData = (FONTDLG*)WinQueryWindowPtr(evt.handle(), QWL_USER);
      IFontDialog* pfontdlg = (IFontDialog*)pfontData->ulUser;
#else
      IFontDialog* pfontdlg = (IFontDialog*)
                            IQUERYWINDOWULONG( evt.handle(), DWL_USER );
#endif
      /*****************************************************************/
      /* Verify we have a non-null object to reference before using    */
      /*****************************************************************/
      if (pfontdlg != (IFontDialog*)0)
      {
        /***************************************************************/
        /* If modeless, call handler function, dependent on event type */
        /***************************************************************/
        if (pfontdlg->isModeless())
        {
          /*************************************************************/
          /* Even if dialog cancelled, still call results on destroy   */
          /*************************************************************/
          if (idOfEvent == WM_DESTROY)
            modelessResults(pfontdlg);

          /*************************************************************/
          /* otherwise this is apply button, so notify of font to use  */
          /*************************************************************/
          else if (idOfEvent == IC_UM_DLG_APPLY_PRESSED)
            modelessApply( pfontdlg,
                           (IFont*)evt.parameter1().asUnsignedLong() );
        }
      }
    }
  } /* endswitch */

  evt.setResult(IEventResult(false));
  return false;
}

/*------------------------------------------------------------------------------
| IFontDialogHandler::modelessResults                                          |
|                                                                              |
| This member is called whenever a modeless dialog ends, regardless of whether |
| the dialog was closed via the OK or Cancel button                            |
------------------------------------------------------------------------------*/
Boolean IFontDialogHandler::modelessResults( IFontDialog* endingDialog )
{
  return true;
}

/*------------------------------------------------------------------------------
| IFontDialogHandler::modelessApply                                            |
|                                                                              |
| This member is called whenever an apply button on a modeless dialog is       |
| pressed, passing the current font dialog object and the new font to be used  |
------------------------------------------------------------------------------*/
Boolean IFontDialogHandler::modelessApply( IFontDialog* modelessDialog,
                                           IFont*       appliedFont )
{
  return true;
}

