/*

IGBM.C  General Bitmap Code

*/

/*...sincludes:0:*/
#include <stdio.h>
#include <ctype.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <memory.h>
#include <igbmstnd.hpp>
#include <igbm.hpp>
#include <igbmhelp.hpp>

#include <igbmpgm.hpp>
#include <igbmppm.hpp>
#include <igbmbmp.hpp>
#include <igbmtga.hpp>
#include <igbmkps.hpp>
#include <igbmiax.hpp>
#include <igbmpcx.hpp>
#include <igbmtif.hpp>
#include <igbmlbm.hpp>
#include <igbmvid.hpp>
#include <igbmgif.hpp>
#include <igbmxbm.hpp>
#include <igbmspr.hpp>
#include <igbmpsg.hpp>
#include <icconst.h>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IGBM_CPP_
  #include <ipagetun.h>
#endif


typedef struct
        {
        GBM_ERR (*query_filetype)(GBMFT *gbmft);
        GBM_ERR (*read_header )(char *fn, int fd, GBM *gbm, char *opt);
        GBM_ERR (*read_palette)(int fd, GBM *gbm, GBMRGB *gbmrgb);
        GBM_ERR (*read_data   )(int fd, GBM *gbm, byte *data);
        GBM_ERR (*write       )(char *fn, int fd, GBM *gbm, GBMRGB *gbmrgb, byte *data, char *opt);
        unsigned long (*err   )(GBM_ERR rc);
        } FT;

static FT fts[] =
        {
        bmp_qft, bmp_rhdr, bmp_rpal, bmp_rdata, bmp_w, bmp_err,
        gif_qft, gif_rhdr, gif_rpal, gif_rdata, gif_w, gif_err,
        pcx_qft, pcx_rhdr, pcx_rpal, pcx_rdata, pcx_w, pcx_err,
        tif_qft, tif_rhdr, tif_rpal, tif_rdata, tif_w, tif_err,
        tga_qft, tga_rhdr, tga_rpal, tga_rdata, tga_w, tga_err,
        lbm_qft, lbm_rhdr, lbm_rpal, lbm_rdata, lbm_w, lbm_err,
        vid_qft, vid_rhdr, vid_rpal, vid_rdata, vid_w, vid_err,
        pgm_qft, pgm_rhdr, pgm_rpal, pgm_rdata, pgm_w, pgm_err,
        ppm_qft, ppm_rhdr, ppm_rpal, ppm_rdata, ppm_w, ppm_err,
        kps_qft, kps_rhdr, kps_rpal, kps_rdata, kps_w, kps_err,
        iax_qft, iax_rhdr, iax_rpal, iax_rdata, iax_w, iax_err,
        xbm_qft, xbm_rhdr, xbm_rpal, xbm_rdata, xbm_w, xbm_err,
        spr_qft, spr_rhdr, spr_rpal, spr_rdata, spr_w, spr_err,
        psg_qft, psg_rhdr, psg_rpal, psg_rdata, psg_w, psg_err,
        };

#define N_FT    (sizeof(fts) / sizeof(fts[0]))
/*...e*/

/*...sextension:0:*/
static char *extension(char *fn)
        {
        char *dot, *slash;

        if ( (dot = strrchr(fn, '.')) == NULL )
                return NULL;

        if ( (slash = strpbrk(fn, "/\\")) == NULL )
                return dot + 1;

        return ( slash < dot ) ? dot + 1 : NULL;
        }
/*...e*/

/*...sgbm_init:0:*/
GBM_ERR gbm_init(void)
        {
        return GBM_ERR_OK;
        }
/*...e*/
/*...sgbm_deinit:0:*/
GBM_ERR gbm_deinit(void)
        {
        return GBM_ERR_OK;
        }
/*...e*/
/*...sgbm_query_n_filetypes:0:*/
GBM_ERR gbm_query_n_filetypes(int *n_ft)
        {
        *n_ft = N_FT;
        return GBM_ERR_OK;
        }
/*...e*/
/*...sgbm_guess_filetype:0:*/
GBM_ERR gbm_guess_filetype(char *fn, int *ft)
        {
        int i;
        char *ext = extension(fn);

        if ( ext == NULL )
                ext = "";

        for ( i = 0; i < N_FT; i++ )
                {
                GBMFT   gbmft;
                char    buf[100+1], *s;

                fts[i].query_filetype(&gbmft);
                for ( s  = strtok(strcpy(buf, gbmft.extensions), " \t,");
                      s != NULL;
                      s  = strtok(NULL, " \t,") )
                        if ( gbm_same(s, ext, strlen(ext) + 1) )
                                {
                                *ft = i;
                                return GBM_ERR_OK;
                                }
                }
        return GBM_ERR_NOT_FOUND;
        }
/*...e*/

/*...sgbm_query_filetype:0:*/
GBM_ERR gbm_query_filetype(int ft, GBMFT *gbmft)
        {
        return (*fts[ft].query_filetype)(gbmft);
        }
/*...e*/
/*...sgbm_read_header:0:*/
GBM_ERR gbm_read_header(char *fn, int fd, int ft, GBM *gbm, char *opt)
        {
        return (*fts[ft].read_header)(fn, fd, gbm, opt);
        }
/*...e*/
/*...sgbm_read_palette:0:*/
GBM_ERR gbm_read_palette(int fd, int ft, GBM *gbm, GBMRGB *gbmrgb)
        {
        return (*fts[ft].read_palette)(fd, gbm, gbmrgb);
        }
/*...e*/
/*...sgbm_read_data:0:*/
GBM_ERR gbm_read_data(int fd, int ft, GBM *gbm, byte *data)
        {
        return (*fts[ft].read_data)(fd, gbm, data);
        }
/*...e*/
/*...sgbm_write:0:*/
GBM_ERR gbm_write(char *fn, int fd, int ft, GBM *gbm, GBMRGB *gbmrgb, byte *data, char *opt)
        {
        return (*fts[ft].write)(fn, fd, gbm, gbmrgb, data, opt);
        }
/*...e*/
/*...sgbm_err:0:*/
unsigned long gbm_err(GBM_ERR rc)
        {
        int ft;

        switch ( (int) rc )
                {
                case GBM_ERR_OK:
                        return 0;
                case GBM_ERR_MEM:
                        return IC_HEAP_EXHAUSTED;
                case GBM_ERR_NOT_SUPP:
                        return IC_GBM_NOT_SUPPORTED;
                case GBM_ERR_BAD_OPTION:
                        return IC_GBM_BAD_OPTION;
                case GBM_ERR_NOT_FOUND:
                        return IC_GBM_NOT_FOUND;
                case GBM_ERR_BAD_MAGIC:
                        return IC_GBM_BAD_MAGIC;
                case GBM_ERR_BAD_SIZE:
                        return IC_GBM_BAD_SIZE;
                case GBM_ERR_READ:
                        return IC_GBM_READ_FAILED;
                case GBM_ERR_WRITE:
                        return IC_GBM_WRITE_FAILED;
                }

        for ( ft = 0; ft < N_FT; ft++ )
                {
                unsigned err;

                if ( (err = (*fts[ft].err)(rc)) != 0 )
                        return err;
                }

        return IC_GBM_UNKNOWN_ERROR;
        }
