/*******************************************************************************
* FILE NAME: ihelpsta.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ihelpsta.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Note we depend on ireslib.cpp being first
#pragma priority( -2147482102 )
extern "C"
  {
  #define INCL_WINHELP
  #include <iwindefs.h>
#ifdef IC_WIN
  extern LRESULT CALLBACK _IHelpMsgFilterProc ( UINT, WPARAM, LPARAM );
  extern LRESULT CALLBACK _IHelpKeyboardProc ( UINT, WPARAM, LPARAM );
#endif
  }

#ifdef IC_WIN
  #include <iwindow.hpp>
  #include <imenu.hpp>
  #include <iframe.hpp>
  #include <iplatfrm.hpp>
#endif

#include <ihelpsta.hpp>
#include <itrace.hpp>
#include <ireslib.hpp>
#include <iexcept.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IHELPSTA_CPP_
  #include <ipagetun.h>
#endif

#ifdef IC_PMWIN
#pragma data_seg(ICLNonConst)
  IHelpStatics helpMgr;
#pragma data_seg()
#endif

ICreateInstance*     IHelpStatics::entryCreate        = 0;
IAssocInstance*      IHelpStatics::entryAssoc         = 0;
IQueryInstance*      IHelpStatics::entryQuery         = 0;
IDestroyInstance*    IHelpStatics::entryDestroy       = 0;
ILoadHelpTable*      IHelpStatics::entryLoadHelpTable = 0;
IDynamicLinkLibrary* IHelpStatics::helpDynLinkLib     = 0;

#ifdef IC_WIN
ICallIPF*            IHelpStatics::entryCallIPF       = 0;
HHOOK                IHelpStatics::keyboardHook       = 0;
HHOOK                IHelpStatics::msgFilterHook      = 0;
unsigned short       IHelpStatics::activeMenuItem     = 0;
HWND                 IHelpStatics::activeMenu         = 0;
#endif

/*------------------------------------------------------------------------------
| IHelpStatics::~IHelpStatics                                                  |
------------------------------------------------------------------------------*/
IHelpStatics ::  ~IHelpStatics ( )
  {
#ifdef IC_WIN
    if (keyboardHook)
      UnhookWindowsHookEx( keyboardHook );
    if (msgFilterHook)
      UnhookWindowsHookEx( msgFilterHook );
#endif
    if (helpDynLinkLib)
      delete helpDynLinkLib;
  }

/*------------------------------------------------------------------------------
| IHelpStatics::helpDLL                                                        |
------------------------------------------------------------------------------*/
IDynamicLinkLibrary& IHelpStatics :: helpDLL ( )
  {
     if (!helpDynLinkLib)
#ifdef IC_PM
        helpDynLinkLib = new IDynamicLinkLibrary("helpmgr");
#endif
#ifdef IC_WIN
        helpDynLinkLib = new IDynamicLinkLibrary("libipf32.dll");
#endif

     return *helpDynLinkLib;
  }

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IHelpStatics::addHooks                                                       |
------------------------------------------------------------------------------*/
void IHelpStatics :: addHooks ( )
  {
    if (!msgFilterHook)
      msgFilterHook = SetWindowsHookEx(WH_MSGFILTER,
                                       (HOOKPROC)&(_IHelpMsgFilterProc),
                                       0,
                                       GetCurrentThreadId());

    if (!keyboardHook)
      keyboardHook = SetWindowsHookEx(WH_KEYBOARD,
                                      (HOOKPROC)&(_IHelpKeyboardProc),
                                      0,
                                      GetCurrentThreadId());

  }
#endif

/*------------------------------------------------------------------------------
| IHelpStatics::createHelpInstance                                             |
------------------------------------------------------------------------------*/
#ifdef IC_PM
IWindowHandle
  IHelpStatics :: createHelpInstance ( const IAnchorBlockHandle& hab,
                                       void* helpstruct )
  {
     if (!entryCreate)
        entryCreate =
          (ICreateInstance*)helpDLL().procAddress( "Win32CreateHelpInstance");
     return (*entryCreate)(hab, helpstruct);
  }
#endif
#ifdef IC_WIN
IWindowHandle
  IHelpStatics :: createHelpInstance ( const IWindowHandle& windowHandle,
                                       void* helpstruct )
  {
     HWND hwndHelp=0;
     long rc=-1;
     if (!entryCreate)
      entryCreate =
             (ICreateInstance*)helpDLL().procAddress( "XhCreateHelpInstance");
     if (entryCreate)
     {
       rc = (*entryCreate)(windowHandle, (PHELPINIT)helpstruct, &hwndHelp );
     }
     switch (rc)
     {
       case 0:
         break;
       case HMERR_INVALID_APPL_WIDGET:
       {
         ITHROWGUIERROR("WinCreateHelpInstance - Invalid Appl Handle");
         break;
       }
       case HMERR_INVALID_HELP_STRUCTURE:
       {
         ITHROWGUIERROR("WinCreateHelpInstance - Invalid Help Structure");
         break;
       }
       case HMERR_MEMORY_ERROR:
       {
         ITHROWGUIERROR("WinCreateHelpInstance - Insufficient Memory");
         break;
       }
       case HMERR_INVALID_RES_FILE:
       {
         ITHROWGUIERROR("WinCreateHelpInstance - Invalid DLL IPF.DLL");
         break;
       }
       case HMERR_INVALID_HELPTABLE:
       {
         ITHROWGUIERROR("WinCreateHelpInstance - Invalid Help Table");
         break;
       }
       case HMERR_OPEN_LIB_FILE:
       case HMERR_INVALID_LIB_FILE:
       case HMERR_READ_LIB_FILE:
       {
         ITHROWGUIERROR("WinCreateHelpInstance - Help File Cannot Be Opened");
         break;
       }
       default:
       {
         ITHROWGUIERROR("WinCreateHelpInstance - Unknown Error");
         break;
       }
     }

     return hwndHelp;
  }
#endif

/*------------------------------------------------------------------------------
| IHelpStatics::associateHelpInstance                                          |
------------------------------------------------------------------------------*/
Boolean
  IHelpStatics :: associateHelpInstance ( const IWindowHandle& helpHandle,
                                          const IWindowHandle& activeWindowHandle )
  {
#ifdef IC_PM
     if (!entryAssoc)
        entryAssoc =
          (IAssocInstance*)helpDLL().procAddress("Win32AssociateHelpInstance");
     return (*entryAssoc)(helpHandle, activeWindowHandle);
#endif
#ifdef IC_WIN
     if (!entryAssoc)
        entryAssoc =
          (IAssocInstance*)helpDLL().procAddress("XhAssociateHelpInstance");

     long rc = (*entryAssoc)(helpHandle, activeWindowHandle);

     if ( rc == 0 )
      return true;
     else
      return false;
#endif
  }

/*------------------------------------------------------------------------------
| IHelpStatics::queryHelpInstance                                              |
------------------------------------------------------------------------------*/
IWindowHandle
  IHelpStatics :: queryHelpInstance ( const IWindowHandle& windowHandle )
  {
#ifdef IC_PM
     if (!entryQuery)
        entryQuery =
          (IQueryInstance*)helpDLL().procAddress("Win32QueryHelpInstance");
     return (*entryQuery)(windowHandle);
#endif
#ifdef IC_WIN
     HWND hwndHelp=0;

     if (!entryQuery)
        entryQuery =
          (IQueryInstance*)helpDLL().procAddress("XhQueryHelpInstance");

     if (entryQuery)
     {
        (*entryQuery)(windowHandle, &hwndHelp);
     }

     return hwndHelp;
#endif
  }

/*------------------------------------------------------------------------------
| IHelpStatics::destroyHelpInstance                                            |
------------------------------------------------------------------------------*/
Boolean IHelpStatics :: destroyHelpInstance ( const IWindowHandle& helpHandle ) // $HT!c
  {
#ifdef IC_PM
     if (!entryDestroy)
        entryDestroy =
          (IDestroyInstance*)helpDLL().procAddress("Win32DestroyHelpInstance");
     return (*entryDestroy)(helpHandle);
#endif
#ifdef IC_WIN
     if (!entryDestroy)
        entryDestroy =
          (IDestroyInstance*)helpDLL().procAddress("XhDestroyHelpInstance");

     long rc = (*entryDestroy)(helpHandle);

     if ( rc == 0 )
      return true;
     else
      return false;
#endif
  }

/*------------------------------------------------------------------------------
| IHelpStatics::loadHelpTable                                                  |
------------------------------------------------------------------------------*/
Boolean IHelpStatics :: loadHelpTable ( const IWindowHandle& helpHandle,
                                        unsigned long id,
                                        const IModuleHandle& hmod )
  {
#ifdef IC_PM
     if (!entryLoadHelpTable)
        entryLoadHelpTable =
          (ILoadHelpTable*)helpDLL().procAddress("Win32LoadHelpTable");
     return (*entryLoadHelpTable)(helpHandle, id, hmod);
#endif
#ifdef IC_WIN
     if (!entryLoadHelpTable)
        entryLoadHelpTable =
          (ILoadHelpTable*)helpDLL().procAddress("XhLoadHelpTable");

     long rc = (*entryLoadHelpTable)(helpHandle, id, hmod);

     if ( rc == 0 )
      return true;
     else
      return false;
#endif
  }

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IHelpStatics::callIPF                                                        |
------------------------------------------------------------------------------*/
long IHelpStatics :: callIPF ( const IWindowHandle& helpHandle,
                                  unsigned short usMsgType,
                                  void* param1, void* param2 )
{
  IFUNCTRACE_DEVELOP();
  ITRACE_DEVELOP(" MsgType= " + IString((unsigned long)usMsgType));
  ITRACE_DEVELOP(" param1 = " + IString((unsigned long)param1));

  if (!entryCallIPF)
    entryCallIPF =
          (ICallIPF*)helpDLL().procAddress("XhCallIPF");

  long rc = (*entryCallIPF)(helpHandle, usMsgType, param1, param2);


  return rc;
//if ( rc == 0 )
//  return true;
//else
//  return false;
}

/*------------------------------------------------------------------------------
| ihelpwindowSearchChain                                                       |
| Routine to search parent and owner chain for window.                         |
| If a validated window is found, its handle is returned.  Otherwise           |
| IFrameWindow(0) is returned.                                                 |
------------------------------------------------------------------------------*/
IFrameWindow* ihelpwindowSearchChain( HWND contextHwnd, HWND lastActiveFrame)
{
  IBoolean foundFrame = false;
  IFrameWindow *activeFrame = 0;
  IWindow* tempWin = 0;
  if (lastActiveFrame)
    tempWin = IWindow::windowWithHandle(lastActiveFrame);

  if ( (tempWin != 0) && (tempWin->isFrameWindow()) )
    return (IFrameWindow *)tempWin;

  if (contextHwnd==0)
    return 0;

  HWND currentHwnd = contextHwnd;

  // If the first IWindow is not the desktop search the chain.  Otherwise
  // return 0.
  while ( !foundFrame && ( currentHwnd !=0 ) &&
        ( currentHwnd != (HWND)IWindow::desktopWindow()->handle() ) )
  {
    IWindow *currentWindow = IWindow::windowWithHandle( currentHwnd );
    if ( currentWindow && currentWindow->isFrameWindow() )
    {
      foundFrame = true;
      activeFrame = (IFrameWindow*)currentWindow;
    }
    else
    {
      currentHwnd = IPARENTOF(currentHwnd);
    }
  }

  currentHwnd = contextHwnd;
  // search for a suitable window in the owner tree
  while ( !foundFrame && ( (currentHwnd=IOWNEROF(currentHwnd)) !=0 )
                      && ( currentHwnd != IWindow::desktopWindow()->handle() ) )
  {
    IWindow *currentWindow = IWindow::windowWithHandle( currentHwnd );
    if ( currentWindow && currentWindow->isFrameWindow() )
    {
      foundFrame = true;
      activeFrame = (IFrameWindow*)currentWindow;
    }
  }
  return activeFrame;

}

/*------------------------------------------------------------------------------
| _IHelpMsgFilterHook                                                          |
|                                                                              |
| Hook WM_KEYDOWN, VK_F1 messages on win32S.                                   |
------------------------------------------------------------------------------*/
LRESULT CALLBACK _IHelpMsgFilterProc (UINT msgFilterCode,
                                      WPARAM unused,
                                      LPARAM lpMsg)
{
  MSG input = *(LPMSG)lpMsg;
  if ( (msgFilterCode == MSGF_MENU) &&
       (input.message == WM_MENUSELECT) )
  {
    IFrameWindow *frame=ihelpwindowSearchChain(input.hwnd, 0);
    if (frame)
    {
      unsigned short itemID = (unsigned short)(unsigned long)input.wParam;
      unsigned short flags  = (unsigned short)((unsigned long)input.wParam>>16);
      if (flags==0xFFFF && input.lParam == 0)
      {
        IHelpStatics::activeMenuItem = 0;
        IHelpStatics::activeMenu = 0;
      }
      else
      {
        if (flags & MF_POPUP)
        {
          HWND subMenuHandle = GetSubMenu( (HWND)input.lParam, itemID );
          IMenu *subMenu = new IMenu( subMenuHandle );
          if (subMenu)
            itemID = subMenu->id();
        }
        IHelpStatics::activeMenuItem = itemID;
        IHelpStatics::activeMenu = (HWND) input.lParam;
      }
    }
  }

  return CallNextHookEx(IHelpStatics::msgFilterHook,msgFilterCode,unused,lpMsg);
}

/*------------------------------------------------------------------------------
| _IHelpKeyboardHook                                                           |
|                                                                              |
| Hook WM_KEYDOWN, VK_F1 messages on win32S.                                   |
------------------------------------------------------------------------------*/
LRESULT CALLBACK _IHelpKeyboardProc (UINT hookCode,
                                     WPARAM vkey,
                                     LPARAM flags)
{
  unsigned short keyFlags = (unsigned short)((unsigned long)flags>>16);
  if ( (hookCode == HC_ACTION) &&
     ( !(keyFlags & KF_UP) && vkey == VK_F1 ) )
  {
    HWND focus = IQUERYFOCUS(NULL);
    IFrameWindow *frame=ihelpwindowSearchChain(focus, 0);
    if (focus && frame)
    {
      IEventParameter1 parm1(0);
      IEventParameter2 parm2(focus);
      if (IHelpStatics::activeMenu)
      {
        parm1=IEventData(IHelpStatics::activeMenuItem,0);
        parm2=IEventData(IHelpStatics::activeMenu);
      }
      return IPOSTMESSAGE((HWND)frame->handle(),
                          IC_UM_CONTEXT_HELP,
                          parm1,
                          parm2);
    }
  }

  return CallNextHookEx(IHelpStatics::keyboardHook,hookCode,vkey,flags);
}
#endif
