/*******************************************************************************
* FILE NAME: imcelcvh.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in imcelcvh.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
  {
  #define INCL_WININPUT
  #define INCL_WINWINDOWMGR
  #define INCL_GPIPRIMITIVES
  #define INCL_GPILOGCOLORTABLE
  #define INCL_WINSTATICS            // SS_BKGNDRECT
  #define INCL_WINSYS
  #define INCL_WINPOINTERS
  #define INCL_WINTRACKRECT
  #define INCL_WINMESSAGEMGR
  #include <iwindefs.h>
  #ifdef IC_PMWIN
    #include <stdio.h>
  #endif
  }

#include <imcelcvh.hpp>
#include <icconst.h>
#include <icolor.hpp>
#include <iexcept.hpp>
#include <imcelcv.hpp>
#ifdef IC_PMWIN
#include <imcelprv.hpp>
#endif
#include <imcelrch.hpp>
#include <ipoint.hpp>
#ifdef IC_PMWIN
  #include <isizeevt.hpp>
#endif
#ifdef IC_MOTIF
  #include <ixconst.h>
#endif
#include <icoordsy.hpp>
#include <igrafctx.hpp>
#include <igline.hpp>
#include <igrect.hpp>
#include <icmnfun.hpp>
#include <itrace.hpp>
#include <igregion.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMCELCVH_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
 Default constructor, empty destructor for page tuning
------------------------------------------------------------------------------*/
IMultiCellCanvasHandler :: IMultiCellCanvasHandler ( )
  : IMultiCellCanvasHandler::Inherited ( )
  { }

IMultiCellCanvasHandler :: ~IMultiCellCanvasHandler ( )
  { }


/*------------------------------------------------------------------------------
  dispatchHandlerEvent - Dispatch a handler event,
  performing significant built-in processing for some events.

------------------------------------------------------------------------------*/
IBase::Boolean IMultiCellCanvasHandler::dispatchHandlerEvent(IEvent& evt)
{
 switch (evt.eventId()) {
#ifdef IC_PMWIN
   case WM_PAINT:
   {
     IMultiCellCanvas* canvas = (IMultiCellCanvas*)evt.window();
     if (canvas->extendedStyle() &
         (IMultiCellCanvas::gridLines.asExtendedUnsignedLong() |
          IMultiCellCanvas::dragLines.asExtendedUnsignedLong()))
     {
       ITRACE_ALL("IMultiCellCanvasHandler - WM_PAINT");
       HWND hwnd=evt.handle();
#ifdef IC_PM
       IPoint ptMouse(evt.parameter1().lowNumber(),
                      evt.parameter1().highNumber());
#endif
#ifdef IC_WIN
       IPoint ptMouse(evt.parameter2().lowNumber(),
                      evt.parameter2().highNumber());
#endif
#ifdef IC_PM
       RECTL paintrect;
#endif
#ifdef IC_WIN
       PAINTSTRUCT paintrect;
#endif
       ISize sizeCanvas = canvas->size();
       IRectangle rect( IPoint(0,0), sizeCanvas );

       canvas->layout();

       IPresSpaceHandle hps(IBEGINPAINT(hwnd, 0L, &paintrect));
       {
         IGraphicContext pshCanvas( hps );

         IColor clrBgnd = canvas->backgroundColor();
         IGRectangle grect( rect );
         pshCanvas.setFillColor( clrBgnd );
         pshCanvas.setDrawOperation (IGraphicBundle::fill ); //AndFrame );

         // The clip region is to make sure we don't draw drag lines outside
         // of the multicell.
         pshCanvas.setClipRegion( IGRegion( hps, IRectangle(IPoint(0,0),
           canvas->visibleRectangle().size())));
         ITRACE_DEVELOP( "visible rect is " + canvas->visibleRectangle().asString() );

         grect.drawOn( pshCanvas );

         pshCanvas.setMixMode( IGraphicBundle::invert );

                 /* Erase the current cross hair by drawing it    */
                 /* again which will cause the previous colours   */
                 /* to be displayed since it is being drawn in    */
                 /* invert mix mode                               */

                 /* Save the new position of the mouse pointer    */
                 /* within the cross hair point array             */

         unsigned long width  = 0;
         unsigned long height = sizeCanvas.height();
         unsigned long i;
         for(i = 1; i <= canvas->pRowColHeaderList->columns(); i++)
         {
            IRCHeader& colHeader = (*canvas->pRowColHeaderList)(i, 0);
//          width += colHeader.width();
            width = colHeader.offset()+colHeader.width();

            IGLine line( IPoint( width, 0 ),
                         IPoint( width, height ) );
            line.drawOn( pshCanvas );
         }

         width  = sizeCanvas.width();
         for(i = 1; i <= canvas->pRowColHeaderList->rows(); i++)
         {
            IRCHeader& rowHeader = (*canvas->pRowColHeaderList)(0, i);
//          height -= rowHeader.height()+1;
#ifdef IC_ORIGINLOWERLEFT
            height = rowHeader.offset()-1;
#else
            height = rowHeader.offset()+rowHeader.height();
#endif

            IGLine line( IPoint( 0,     height ),
                         IPoint( width, height ) );
            line.drawOn( pshCanvas );
         }
       }  /* brackets to ensure ~IGraphicContext called before IENDPAINT */

       IENDPAINT (hwnd, pshCanvas.handle(), &paintrect);

       evt.setResult((unsigned long)false);
       return(true);                    // do not pass on to next handler
     }
   }/* end WM_PAINT */
   break;

#ifdef IC_WIN
   // This keeps the drag cursor when dragging the lines
   // but we are still getting WM_SETCURSORs which cause the cursor to
   // flash to the default and back to drag cursor when moving the cursor
   // along a drag line.
   case WM_SETCURSOR:
   {
      if ( ( evt.handle().asUnsigned() == evt.parameter1().asUnsignedLong() ) &&
           ( ( evt.parameter2().highNumber() == WM_BUTTON1DOWN ) ||
             ( evt.parameter2().highNumber() == WM_BUTTON2DOWN ) ||
             ( evt.parameter2().highNumber() == WM_BUTTON3DOWN ) ) )
         {
         // Eat this message to prevent resetting of cursor to default
         evt.setResult( true );
         return true;
         }
   }
   break;
#endif

   case WM_MOUSEMOVE:
      /*********************************************************/
      //Return sizeHorz/sizeVert pointer handle if on the split-bar
      //Use system for now - need to get container split drag pointer
      /*********************************************************/
   {
      IMultiCellCanvas* canvas = (IMultiCellCanvas*)evt.window();

      if (canvas->extendedStyle() &
                          IMultiCellCanvas::dragLines.asExtendedUnsignedLong())
      {
#ifdef IC_PM
        IPoint ptMouse( evt.parameter1().lowNumber(),
                        evt.parameter1().highNumber());
#endif
#ifdef IC_WIN
        IPoint ptMouse( evt.parameter2().lowNumber(),
                        evt.parameter2().highNumber());
#endif
        ISize sizeCanvas = canvas->size();
        IRectangle rclGridLine;
        Boolean bOverVertGridLine = false;
        Boolean bOverHorzGridLine = false;
        unsigned long width  = 0;
        unsigned long height = sizeCanvas.height();
        unsigned long i;

        for(i = 1; i <= canvas->pRowColHeaderList->columns(); i++)
        {
           IRCHeader& colHeader = (*canvas->pRowColHeaderList)(i, 0);
//         width += colHeader.width();
           width = colHeader.offset()+colHeader.width();

           rclGridLine = IRectangle (width-1, 0, width+1, height);

           if (rclGridLine.contains(ptMouse))
           {
             bOverVertGridLine = true;
             break;            // leave enumeration loop
           }
        }
        if (!bOverVertGridLine)
        {
          width  = sizeCanvas.width();
          for(i = 1; i <= canvas->pRowColHeaderList->rows(); i++)
          {
             IRCHeader& rowHeader = (*canvas->pRowColHeaderList)(0, i);
//           height -= rowHeader.height();
#ifdef IC_ORIGINLOWERLEFT
             height = rowHeader.offset()-1;
#else
             height = rowHeader.offset()+rowHeader.height();
#endif

             rclGridLine = IRectangle (0, height-1, width, height+1);

             if (rclGridLine.contains(ptMouse))
             {
               bOverHorzGridLine = true;
               break;            // leave enumeration loop
             }
          }
        }
        if (bOverVertGridLine )
        {
          ISETPOINTER( ISystemPointerHandle(ISystemPointerHandle::sizeHorizontal));
          evt.setResult(true);
          return(true);
        }
        else if (bOverHorzGridLine )
        {
          ISETPOINTER( ISystemPointerHandle(ISystemPointerHandle::sizeVertical));
          evt.setResult(true);
          return(true);
        }
      }
      evt.setResult(false);
      return(false);
     break;
   }/* end WM_MOUSEMOVE */

   case WM_BUTTON1DOWN:
   case WM_BUTTON2DOWN:
   case WM_BUTTON3DOWN:
      /*********************************************************/
      //If button down on the split-bar, start tracking the mouse
      //and resize controls on both sides of split-bar according to
      //where mouse button was released.
      /*********************************************************/
   {
      HWND hwnd=evt.handle();
      IMultiCellCanvas* canvas = (IMultiCellCanvas*)evt.window();

      if (canvas->extendedStyle() &
                          IMultiCellCanvas::dragLines.asExtendedUnsignedLong())
      {
        POINTL ptScreen;

        /*********************************************************/
        //compute mouse location within the canvas window
        /*********************************************************/
        IQUERYMSGPOS(IQUERYANCHOR(hwnd),&ptScreen);
        IMAPWINDOWPOINTS(HWND_DESKTOP,hwnd,&ptScreen,1);
        IPoint ptMouse(ptScreen.x,ptScreen.y);

        IRectangle rclGridLine;
        Boolean bOverVertGridLine = false;
        Boolean bOverHorzGridLine = false;
        unsigned long width  = 0;
        ISize sizeCanvas = canvas->size();
        unsigned long height = sizeCanvas.height();
        unsigned long i, column=0, columnPrev=0, row=0, rowPrev=0;

        for(i = 1; i <= canvas->pRowColHeaderList->columns(); i++)
        {
           IRCHeader& colHeader = (*canvas->pRowColHeaderList)(i, 0);
//         width += colHeader.width();
           width = colHeader.offset()+colHeader.width();

           rclGridLine = IRectangle (width-1, 0, width+1, height);

           if (rclGridLine.contains(ptMouse))
           {
             bOverVertGridLine = true;
             column = i+1;
             columnPrev = i;
             rclGridLine = IRectangle (width, 0, width+1, height);
             break;            // leave enumeration loop
           }
        }                  // endfor
        if (!bOverVertGridLine)
        {
          width  = sizeCanvas.width();
          for(i = 1; i <= canvas->pRowColHeaderList->rows(); i++)
          {
             IRCHeader& rowHeader = (*canvas->pRowColHeaderList)(0, i);
//           height -= rowHeader.height();
#ifdef IC_ORIGINLOWERLEFT
             height = rowHeader.offset()-1;
#else
             height = rowHeader.offset()+rowHeader.height();
#endif

             rclGridLine = IRectangle (0, height-1, width, height+1);

             if (rclGridLine.contains(ptMouse))
             {
               bOverHorzGridLine = true;
               row = i+1;
               rowPrev=i;
               rclGridLine = IRectangle (0, height, width, height+1);
               break;            // leave enumeration loop
             }
          }             // endfor
        }             // endif !bOverVertGridLine

        // clear WS_CLIPCHILDREN bit
        /**********************************************************/
        long lStyle=ISTYLEOF( hwnd );
        ISETWINDOWSTYLE(hwnd, lStyle & ~WS_CLIPCHILDREN);
        // start tracking
        TRACKINFO trk;
        /********************************************************************/
        /* The Splitbar should track below the titlebar and not go any      */
        /* farther right than the right control's right and any farther left*/
        /* than the left control's left.                                    */
        /********************************************************************/
        if (bOverVertGridLine)
        {
          IRCHeader& colHeader = (*canvas->pRowColHeaderList)(columnPrev, 0);
          trk.rclBoundary.MINX = rclGridLine.minX()-colHeader.width();
          trk.rclBoundary.MINY = rclGridLine.minY();
          trk.rclBoundary.MAXX = sizeCanvas.width();
          trk.rclBoundary.MAXY = rclGridLine.maxY();

          /********************************************************/
          /* This specifies the width of the Splitbar, so         */
          /* WinTrackRect knows how wide of a thing to track. The */
          /* Splitbar is the width of a Size Border.              */
          /********************************************************/
          trk.cxBorder = 1;

          /********************************************************/
          /* This specifies the height of the splitbar so         */
          /* WinTrackRect knows how tall of a thing to track.     */
          /********************************************************/
          trk.cyBorder = trk.rclBoundary.MAXY;

          /********************************************************/
          /* This specifies the tracking rectangle used by        */
          /* WinTrackRect.                                        */
          /********************************************************/
          trk.rclTrack.MINX = rclGridLine.minX();
          trk.rclTrack.MINY = rclGridLine.minY();
          trk.rclTrack.MAXX = rclGridLine.maxX();
          trk.rclTrack.MAXY = trk.rclBoundary.MAXY;

        }
        else if (bOverHorzGridLine)
        {
          IRCHeader& rowHeader = (*canvas->pRowColHeaderList)(0,rowPrev);
          trk.rclBoundary.MINX = rclGridLine.minX();
          trk.rclBoundary.MAXX = rclGridLine.maxX();
#ifdef IC_ORIGINLOWERLEFT
          trk.rclBoundary.MINY = 0;
          trk.rclBoundary.MAXY = rclGridLine.maxY()+rowHeader.height();
#else
          trk.rclBoundary.MINY = rclGridLine.minY()-rowHeader.height();
          trk.rclBoundary.MAXY = sizeCanvas.height();
#endif

          /********************************************************/
          /* This specifies the width of the gridline, so         */
          /* WinTrackRect knows how wide of a thing to track.     */
          /********************************************************/
          trk.cxBorder = trk.rclBoundary.MAXX;

          /********************************************************/
          /* This specifies the height of the gridline so         */
          /* WinTrackRect knows how tall of a thing to track.     */
          /********************************************************/
          trk.cyBorder = 1;

          /********************************************************/
          /* This specifies the tracking rectangle used by        */
          /* WinTrackRect.                                        */
          /********************************************************/
          trk.rclTrack.MINX = rclGridLine.minX();
          trk.rclTrack.MINY = rclGridLine.minY();
          trk.rclTrack.MAXX = rclGridLine.maxX();
          trk.rclTrack.MAXY = rclGridLine.maxY();
        }

        if (bOverHorzGridLine || bOverVertGridLine)
        {
          trk.cxGrid = 0;
          trk.cyGrid = 0;
          trk.cxKeyboard = 0;
          trk.cyKeyboard = 0;

          /**********************************************************/
          /* Specify the minimum and maximum tracking size.         */
          /**********************************************************/
          trk.ptlMinTrackSize.x = trk.cxBorder;
          trk.ptlMinTrackSize.y = trk.cyBorder;
          trk.ptlMaxTrackSize.x = trk.cxBorder;
          trk.ptlMaxTrackSize.y = trk.cyBorder;

          /**********************************************************/
          /* These tracking options specify:  TF_MOVE - to center   */
          /* the pointer in the tracking rectangle. TF_ALLINBOUNDARY*/
          /* performs tracking such that no part of the tracking    */
          /* rectangle ever falls outside the bounding rectangle.   */
          /**********************************************************/
          trk.fs = TF_MOVE | TF_ALLINBOUNDARY;

          BOOL bSuc=WinTrackRect(hwnd,0,&trk);

          /**********************************************************/
          /* restore WS_CLIPCHILDREN bit                            */
          /**********************************************************/
          ISETWINDOWSTYLE(hwnd, lStyle );

          if (row)
          {
            IRCHeader& rowHeader = (*canvas->pRowColHeaderList)(0, rowPrev);
            long  rowDelta;
#ifdef IC_ORIGINLOWERLEFT
            rowDelta = rclGridLine.maxY() - trk.rclTrack.MAXY ;
#else
            rowDelta = trk.rclTrack.MAXY - rclGridLine.maxY() ;
#endif
            canvas->setRowHeight(rowPrev,
                                 rowDelta + rowHeader.height());
            canvas->setLayoutDistorted(IWindow::layoutChanged |
                                       IWindow::immediateUpdate, 0);
          }

          if (column)
          {
            IRCHeader& colHeader = (*canvas->pRowColHeaderList)(columnPrev, 0);
            canvas->setColumnWidth(columnPrev,
                            (trk.rclTrack.MINX - rclGridLine.minX()) +
                             colHeader.width());
            canvas->setLayoutDistorted(IWindow::layoutChanged |
                                       IWindow::immediateUpdate, 0);
          }


        }       // endif bOverHorzGridLine or ...
      }       // endif extendedStyle() & ...
      break;
   }/*end WM_BUTTON...DOWN */
#endif   //IC_PMWIN
   case WM_SIZE:
   {
      IMultiCellCanvas* canvas = (IMultiCellCanvas*)evt.window();

      // Canvas needs new layout if either:
      // - Sized vertically and it has expandable rows or
      //   its new height is not what was used to position
      //   its child windows.
      // - Sized horizontally and it has expandable columns.
      long
        newWidth = evt.parameter2().number1(),
        newHeight = evt.parameter2().number2();
      if ( newWidth  &&
           newHeight  &&
#ifdef IC_ORIGINLOWERLEFT
           ( ( newHeight !=
                 canvas->fMultiCellCanvasData->sizeForLayout.height() )  ||
#endif
#ifndef IC_ORIGINLOWERLEFT
           ( ( newHeight !=
                 canvas->fMultiCellCanvasData->sizeForLayout.height()  &&
               *( (unsigned short*)&(canvas->fExpand) ) )  ||
#endif
             ( newWidth !=
                 canvas->fMultiCellCanvasData->sizeForLayout.width()  &&
               *( ((unsigned short*)&(canvas->fExpand)) + 1 ) ) ) )
      {
         canvas->setLayoutDistorted( IWindow::sizeChanged |
                                     IWindow::layoutChanged, 0 );
         canvas->layout();
      }
      else
      {
         // Notification for backwards compatibility.
         canvas->setLayoutDistorted( IWindow::sizeChanged, 0 );
      }
      break;
   }/*end WM_SIZE */

 } /* endswitch */
 return false;
}


/*------------------------------------------------------------------------------
   handleEventsFor - begin handling events for the specified canvas.
------------------------------------------------------------------------------*/
IMultiCellCanvasHandler &  IMultiCellCanvasHandler :: handleEventsFor
                                                 ( IMultiCellCanvas *canvas )
{
   IASSERTPARM(canvas != 0);
   Inherited::handleEventsFor(canvas);
   return *this;
}


/*------------------------------------------------------------------------------
   stopHandleEventsFor - end handling of events for the specified canvas.
------------------------------------------------------------------------------*/
IMultiCellCanvasHandler &  IMultiCellCanvasHandler :: stopHandlingEventsFor
                                                 ( IMultiCellCanvas *canvas )
{
   IASSERTPARM(canvas != 0);
   Inherited::stopHandlingEventsFor(canvas);
   return *this;
}


/*------------------------------------------------------------------------------
   handleEventsFor - throw an error if someone attempts to attach
   this handler to any other IWindow besides an IMultiCellCanvas (which
   would call "handleEventsFor(IMultiCellCanvas*)" above, not this method).
------------------------------------------------------------------------------*/
IHandler &  IMultiCellCanvasHandler :: handleEventsFor ( IWindow * )
{                              // private to hide version in IHandler
   ITHROWLIBRARYERROR(IC_MEMBER_ACCESS_ERROR,
                      IBaseErrorInfo::invalidRequest,
                      IException::recoverable);
   return *this;
}


/*------------------------------------------------------------------------------
   stopHandlingEventsFor - throw an error if someone attempts to call
   this method for any other IWindow besides an IMultiCellCanvas (which
   would call "stopHandlingEventsFor(IMultiCellCanvas*)" above, not this method).
------------------------------------------------------------------------------*/
IHandler &  IMultiCellCanvasHandler :: stopHandlingEventsFor ( IWindow * )
{                              // private to hide version in IHandler
   ITHROWLIBRARYERROR(IC_MEMBER_ACCESS_ERROR,
                      IBaseErrorInfo::invalidRequest,
                      IException::recoverable);
   return *this;
}
