/*******************************************************************************
* FILE NAME: imle1.cpp                                                         *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in imlenhdr.hpp                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

#define INCL_WINMLE

extern "C" {
   #include <iwindefs.h>
}

#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif

#ifndef _IMLE_
  #include <imle.hpp>
#endif

#ifndef _IMLENHDR_
  #include <imlenhdr.hpp>
#endif

#ifndef _IEVENT_
  #include <ievent.hpp>
#endif

#ifndef _INOTIFEV_
  #include <inotifev.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMLE1_CPP_
  #include <ipagetun.h>
#endif

// Class to clean up static objects
class IMultiLineEditNotifyStatics
{
public:

~IMultiLineEditNotifyStatics ()
{
  delete (multiLineEditNotifyHandler);
  multiLineEditNotifyHandler = 0;
}
static IMultiLineEditNotifyHandler
 &notificationHandler ( );

static IMultiLineEditNotifyHandler
  *multiLineEditNotifyHandler;
};

// Initialize static objects and notification strings.
#pragma data_seg(ICLStaticConst)
static IMultiLineEditNotifyStatics MultiLineEditStatics;
INotificationId const IMultiLineEdit::dataUpdateId = "IMultiLineEdit::dataUpdate";
INotificationId const IMultiLineEdit::limitId      = "IMultiLineEdit::limit";
#pragma data_seg()

#pragma data_seg(ICLNonConst)
IMultiLineEditNotifyHandler*
  IMultiLineEditNotifyStatics::multiLineEditNotifyHandler = 0;
#pragma data_seg()

/*------------------------------------------------------------------------------
| IMultiLineEditNotifyStatics::notificationHandler                             |
|                                                                              |
| Static function to create and return a notification handler for              |
| IMultiLineEdit.                                                              |
------------------------------------------------------------------------------*/
IMultiLineEditNotifyHandler& IMultiLineEditNotifyStatics::notificationHandler ()
{
  if (!(IMultiLineEditNotifyStatics::multiLineEditNotifyHandler))
  {
    IMultiLineEditNotifyStatics::multiLineEditNotifyHandler =
                                   new IMultiLineEditNotifyHandler();
  }
  return (*IMultiLineEditNotifyStatics::multiLineEditNotifyHandler);
}

/*------------------------------------------------------------------------------
| IMultiLineEdit::enableNotification                                           |
|                                                                              |
| Flags this part as being ready to accept notifications.  A Notifier, like    |
| IMultiLineEdit, does not send notifications to observer objects until it is  |
| enabled.                                                                     |
|                                                                              |
| In the window hierarchy we override this function to allocate the correct    |
| notification handler.                                                        |
------------------------------------------------------------------------------*/
IMultiLineEdit& IMultiLineEdit::enableNotification (Boolean enabled)
{
  if (!notificationHandler())
  {
    setNotificationHandler(
               &(IMultiLineEditNotifyStatics::notificationHandler()));
  }
  IWindow::enableNotification(enabled);
  return *this;
}

/*------------------------------------------------------------------------------
| IMultiLineEditNotifyHandler::IMultiLineEditNotifyHandler                     |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
IMultiLineEditNotifyHandler::IMultiLineEditNotifyHandler()
{}

/*------------------------------------------------------------------------------
| IMultiLineEditNotifyHandler::~IMultiLineEditNotifyHandler                    |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IMultiLineEditNotifyHandler::~IMultiLineEditNotifyHandler()
{}

/*------------------------------------------------------------------------------
| IMultiLineEditNotifyHandler::dispatchHandlerEvent                            |
|                                                                              |
| This notifies IMultiLineEdit Windows of the following events:                |
|                                                                              |
| dataUpdate(Id)                                                               |
| limit(Id)                                                                    |
------------------------------------------------------------------------------*/
#ifdef IC_PMWIN

Boolean IMultiLineEditNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
  IWindow* thisWindow = anEvent.window();

  if (thisWindow)
  {
    switch (anEvent.eventId())
    {
      case MLM_SETREADONLY:

        /**************************************************************/
        /* Post private message to delay notification until after     */
        /* the readonly state has been modified                       */
        /**************************************************************/
        thisWindow->postEvent( IC_UM_DELAYNOTIFY,
                               IEventParameter1( anEvent.parameter1() ),
                               IEventParameter2( IC_NOTDEL_MLE_DATAUPD ));
        break;

      case MLM_SETTEXTLIMIT:

        /**************************************************************/
        /* Post private message to delay notification until after     */
        /* the limit has actually been set                            */
        /**************************************************************/
        thisWindow->postEvent( IC_UM_DELAYNOTIFY,
                               IEventParameter1( anEvent.parameter1() ),
                               IEventParameter2( IC_NOTDEL_MLE_LIMIT ));
        break;

    /******************************************************************/
    /* This is the delayed notification message.  Check to see which  */
    /* notification should be sent to the observers based on info     */
    /* stored in parameter 2 of the private message.  Parameter 1     */
    /* contains the information passed in the original event that     */
    /* needs to be sent with the notification                         */
    /******************************************************************/
    case IC_UM_DELAYNOTIFY:

      if ( anEvent.parameter2().asUnsignedLong() == IC_NOTDEL_MLE_DATAUPD )
      {
        thisWindow->notifyObservers(INotificationEvent(
                                    IMultiLineEdit::dataUpdateId,
                                    *thisWindow, true,
                    (void*)(!(anEvent.parameter1().asUnsignedLong()))));
      }
      else if ( anEvent.parameter2().asUnsignedLong() == IC_NOTDEL_MLE_LIMIT )
      {
        thisWindow->notifyObservers(INotificationEvent(
                                    IMultiLineEdit::limitId,
                                    *thisWindow, true,
                                    (void*)anEvent.parameter1()));
      }
      break;

      default:
        break;
    }  /* end switch */
  }
  return (Inherited::dispatchHandlerEvent(anEvent));
}

#endif // IC_PMWIN

