/*******************************************************************************
* FILE NAME: imoushdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in imoushdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#define INCL_WININPUT
extern "C"
  {
  #include <iwindefs.h>
  }

#include <imoushdr.hpp>
#include <iwindow.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMOUSHDR_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| IMouseHandler::IMouseHandler                                                 |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
IMouseHandler::IMouseHandler ( )
  : IMouseHandler::Inherited ( )
{}


/*------------------------------------------------------------------------------
| IMouseHandler::~IMouseHandler                                                |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IMouseHandler::~IMouseHandler ( )
{}


/*------------------------------------------------------------------------------
| IMouseHandler::dispatchHandlerEvent                                          |
|                                                                              |
| Dispatch mouse click events for this handler.                                |
------------------------------------------------------------------------------*/
Boolean  IMouseHandler::dispatchHandlerEvent ( IEvent & evt )
{
   Boolean bRc = false;
   switch (evt.eventId())
   {
      case WM_BUTTON1DOWN:
      case WM_BUTTON1UP:
      case WM_BUTTON1DBLCLK:
      case WM_BUTTON2DOWN:
      case WM_BUTTON2UP:
      case WM_BUTTON2DBLCLK:
      case WM_BUTTON3DOWN:
      case WM_BUTTON3UP:
      case WM_BUTTON3DBLCLK:
      case WM_BUTTON1CLICK:
      case WM_BUTTON2CLICK:
      case WM_BUTTON3CLICK:
      case WM_CHORD:
      {
         IMouseClickEvent mevt(evt);
         bRc = mouseClicked(mevt);  // call virtual function
         if (bRc)
         {                        // processing done
            evt.setResult(mevt.result());
         }
         break;
      }
      case WM_MOUSEMOVE:
      {
         IMouseEvent mevt(evt);
         bRc = mouseMoved(mevt);
         if (bRc)
         {
           this->defaultProcedure( evt );
           evt.setResult(mevt.result());
         }
         break;
      }

      case WM_CONTROLPOINTER:
      {
        IMousePointerEvent mpevt(evt);
        bRc = mousePointerChange(mpevt);
        evt.setResult(mpevt.result());
        break;
      }
      default:
         break;
   } /* endswitch */

   return bRc;
}

/*------------------------------------------------------------------------------
| IMouseHandler::mouseClicked                                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMouseHandler::mouseClicked( IMouseClickEvent& event )
{
  return false;
}

/*------------------------------------------------------------------------------
| IMouseHandler::mouseMoved                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMouseHandler::mouseMoved( IMouseEvent& event )
{
  return false;
}

/*------------------------------------------------------------------------------
| IMouseHandler::changeMousePointer                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMouseHandler::mousePointerChange( IMousePointerEvent& event )
{
  return false;
}
