/*******************************************************************************
* FILE NAME: ireslib.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IResourceLibrary                                                         *
*     IDynamicLinkLibrary                                                      *
*     IResourceId                                                              *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147482112 )

extern "C" {
  #define INCL_DOSMODULEMGR
  #define INCL_DOSRESOURCES    /* Resource types */
  #define INCL_DOSINFOSEG
  #define INCL_DOSPROCESS
  #define INCL_WINERRORS
  #define INCL_WINPOINTERS
  #define INCL_WINMENUS
  #define INCL_WINHELP
  #define INCL_WINACCELERATORS
  #define INCL_DEV                /* Device Function definitions  */
  #define INCL_GPICONTROL         /* GPI control Functions        */
  #define INCL_GPIBITMAPS
  #include <iwindefs.h>
}

#include <ibase.hpp>
#include <ihandle.hpp>
#include <ipoint.hpp>
#include <istring.hpp>
#include <iset.h>
#include <ireslib.hpp>
#include <itrace.hpp>
#include <iexcept.hpp>
#include <iwindow.hpp>
#include <iapp.hpp>
#include <ithread.hpp>
#include <ireslock.hpp>
#include <iresstat.hpp>
#include <ihelpsta.hpp>
#include <iplatfrm.hpp>
#include <imenuprv.hpp>
#include <iacceltb.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IRESLIB_CPP_
  #include <ipagetun.h>
#endif

#pragma data_seg(ICLNonConst)
 static IResLibStaticPtr ResLibStatics;
#pragma data_seg()

IPointerSet* IResLibStaticPtr::pPtrSet = 0;
IBitmapSet* IResLibStaticPtr::pBmpSet = 0;
IDLLSet* IResLibStaticPtr::pDLLSet = 0;
IResourceLibrary* IResLibStaticPtr::pResDefault = 0;
IPrivateResource* IResLibStaticPtr::pLibKey = 0;



/*------------------------------------------------------------------------------
| IResourceLibrary::IResourceLibrary                                           |
------------------------------------------------------------------------------*/
IResourceLibrary :: IResourceLibrary()
  : ppd(0)
{
  IMODTRACE_ALL("ResLib::ResLib");
#ifdef IC_WIN
  hInstance = GetModuleHandle( 0 );
#else
  hInstance = 0;
#endif
}

/*------------------------------------------------------------------------------
| IResourceLibrary::IResourceLibrary                                           |
| Copy constructor                                                             |
------------------------------------------------------------------------------*/
IResourceLibrary :: IResourceLibrary(const IResourceLibrary& resLibrary)
{
  IMODTRACE_ALL("ResLib::ResLib(const &)");
  ppd = resLibrary.ppd;  // When IResLibPrivateData is defined be sure to copy
                         // contents.
  hInstance = resLibrary.hInstance;
}

/*------------------------------------------------------------------------------
| IResourceLibrary::operator=                                                  |
------------------------------------------------------------------------------*/
IResourceLibrary& IResourceLibrary :: operator= (const IResourceLibrary& resLibrary)
{
  IMODTRACE_ALL("ResLib::operator=");
  if (this != &resLibrary)
  {
     ppd = resLibrary.ppd; // When IResLibPrivateData is defined be sure to copy
                           // contents.
     hInstance = resLibrary.hInstance;
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IResourceLibrary::~IResourceLibrary                                          |
------------------------------------------------------------------------------*/
IResourceLibrary :: ~IResourceLibrary   ( )
{
// delete ppd; when IResLibPrivateData is defined.
}

/*------------------------------------------------------------------------------
| IResourceLibrary::loadText                                                   |
------------------------------------------------------------------------------*/
char* IResourceLibrary ::  loadText(Boolean isString,
                                unsigned long ulStringID, char* pszBuffer,
                                unsigned long ulLength,
                                Boolean       throwException) const
{
  IMODTRACE_ALL("ResLib::loadText");
  unsigned long ulReturnLength;

  // Win32 LoadString() is failing for some DBCS strings when buffer
  // length does not accommodate the string.  Increasing buffer to work around
  // the problem.  Only happens on Windows '95.
  // It was later discovered that this also occurs on Win32S.
  unsigned long ulBufferLength =
     (IPlatform::isWin95() || IPlatform::isWin32S()) ? 1024 : 100;

  char* pszString = 0;
  IAnchorBlockHandle habTemp = IQUERYANCHOR(HWND_DESKTOP);

  if (pszBuffer == 0)
  {
     pszString = new char[ulBufferLength];      /* work buffer  */

     // New does not give us a clean chunk on Win32S. We null it out for
     // all platforms just to safe.
     memset(pszString,0,ulBufferLength);
  }
  else
  {
     pszString = pszBuffer;
     ulBufferLength = ulLength;
  }

  while (1)
  {
     if(isString==true)
     {
        ulReturnLength = ILOADSTRING( habTemp,
                          handle(), ulStringID, (int)ulBufferLength,
                          (PSZ)pszString);
        /* if an error, save it and quit */
        if (!ulReturnLength)
        {
           /* Free memory if we allocated it. */
           if ((!pszBuffer) && (pszString))
           {
             delete (pszString);
           }
           if(throwException)
              ITHROWGUIERROR(IString("WinLoadString(")+
                        "IResourceId:"+IString(ulStringID)+")");
        }
     }
     else
     {
        ulReturnLength = ILOADMESSAGE(habTemp,
                          handle(), ulStringID, ulBufferLength,
                          (PSZ)pszString);
        /* if an error, save it and quit */
        if (!ulReturnLength)
        {
           /* Free memory if we allocated it. */
           if ((!pszBuffer) && (pszString))
           {
             delete (pszString);
           }
           if(throwException)
              ITHROWGUIERROR(IString("WinLoadMessage(")+
                        "IResourceId:"+IString(ulStringID)+")");
        }
     }



     /* if (possibly) truncated, get more buffer and retry */
     if (((ulReturnLength == (ulBufferLength-1)) ||
          (ulReturnLength == (ulBufferLength-2))) &&
           pszBuffer==0)
     {
       delete(pszString);
       ulBufferLength += ulBufferLength;
       pszString = new char[ulBufferLength];
     }
     else  /* All O.K. get out  */
        break;
  }
  if (ulReturnLength)
  {
    return(pszString);
  }
  else
    return(0);
}

/*------------------------------------------------------------------------------
| IResourceLibrary::loadString                                                 |
------------------------------------------------------------------------------*/
IString IResourceLibrary ::  loadString(unsigned long ulStringId) const
{
  IMODTRACE_ALL("ResLib::loadString");
  char *pszTemp;
  pszTemp = loadText(true, ulStringId);
  IString stringTemp(pszTemp);
  delete(pszTemp);
  return(stringTemp);
}


/*------------------------------------------------------------------------------
| IResourceLibrary::tryToLoadString                                            |
------------------------------------------------------------------------------*/
IString IResourceLibrary ::  tryToLoadString(unsigned long ulStringId) const
{
  IMODTRACE_ALL("ResLib::tryToLoadString");
  char *pszTemp;
  IString stringTemp;
  pszTemp = loadText(true, ulStringId, 0, 0, false);
  if(pszTemp)
  {
     stringTemp = IString(pszTemp);
     delete(pszTemp);
  }
  return(stringTemp);
}

/*------------------------------------------------------------------------------
| IResourceLibrary::loadMessage                                                |
------------------------------------------------------------------------------*/
IString IResourceLibrary ::  loadMessage(unsigned long ulMessageId) const
{
  IMODTRACE_ALL("ResLib::loadMessage");
  char *pszTemp;
  pszTemp = loadText(false, ulMessageId);
  IString stringTemp(pszTemp);
  delete(pszTemp);
  return(stringTemp);
}


/*------------------------------------------------------------------------------
| IResourceLibrary::tryToLoadMessage                                           |
------------------------------------------------------------------------------*/
IString IResourceLibrary ::  tryToLoadMessage(unsigned long ulMessageId) const
{
  IMODTRACE_ALL("ResLib::tryToLoadMessage");
  char *pszTemp;
  IString stringTemp;
  pszTemp = loadText(true, ulMessageId, 0, 0, false);
  if(pszTemp)
  {
     stringTemp = IString(pszTemp);
     delete(pszTemp);
  }
  return(stringTemp);
}

/*------------------------------------------------------------------------------
| IResourceLibrary::loadMenu                                                   |
------------------------------------------------------------------------------*/
IMenuHandle IResourceLibrary :: loadMenu ( unsigned long ulMenuID,
                                           IWindow* pwinMenuOwner) const
{
  IMODTRACE_ALL("ResLib::loadMenu");
  IMenuHandle hwndMenu = 0;

  //*******************************************************************
  // If Win32s, use our own API to load the extended menu resource since
  //  Win32s does not support the new resource format
  //*******************************************************************
  if (IPlatform::isWin32S())
    hwndMenu = IMenuPrivate::loadMenuForWin32s( handle(), ulMenuID );
  else
    hwndMenu = ILOADMENU(pwinMenuOwner->handle(), handle(), ulMenuID);

  if (!hwndMenu)
  {
     ITHROWGUIERROR(IString("WinLoadMenu(")+
                  "IResourceId:"+IString(ulMenuID)+")");
  }

  return (hwndMenu);
}


/*------------------------------------------------------------------------------
| IResourceLibrary::loadDialog                                                 |
------------------------------------------------------------------------------*/
IWindowHandle IResourceLibrary :: loadDialog(unsigned long ulDlgID,
                      IWindow* pwinDlgParent, IWindow* pwinDlgOwner,
                      IWinProc* pDlgProc,
                      void* pDlgCreateParams) const
{
  IMODTRACE_ALL("ResLib::loadDialog");
  IWindowHandle hwndDlg = 0L;

  IWindowHandle hwndOwner = 0;
  IWindowHandle hwndParent = HWND_DESKTOP;
   if (pwinDlgOwner != 0) {
      hwndOwner = pwinDlgOwner->handle();
   } /* endif */
   if (pwinDlgParent != 0) {
      hwndParent = pwinDlgParent->handle();
   } /* endif */
#ifdef IC_WIN
   hwndDlg = ILOADDLG (hwndParent, hwndOwner,
                         (DLGPROC)pDlgProc,
                         handle(), ulDlgID, (LPARAM)pDlgCreateParams);
#else
   hwndDlg = ILOADDLG (hwndParent, hwndOwner,
                         pDlgProc,
                         handle(), ulDlgID, pDlgCreateParams);
#endif
  if (!hwndDlg)
  {
     ITHROWGUIERROR(IString("WinLoadDlg(")+
                  "IResourceId:"+IString(ulDlgID)+")");
  }
  return (hwndDlg);

}

/*------------------------------------------------------------------------------
| IResourceLibrary::loadAccelTable                                             |
------------------------------------------------------------------------------*/
IAccelTblHandle IResourceLibrary ::
                  loadAccelTable(unsigned long ulAccelTableId) const
{
  IMODTRACE_ALL("ResLib::loadAccelTable");
  IAccelTblHandle haccel;
  IAnchorBlockHandle habTemp = IQUERYANCHOR(HWND_DESKTOP);

  haccel = ILOADACCELTABLE(habTemp, handle(), ulAccelTableId);
#ifdef IC_WIN

  if ( IPlatform::isWin95() )
  {
     // For Windows 95, avoid the use of LoadAccelerators.  MSDN
     // documentation warns against using DestroyAcceleratorTable on
     // handles created with LoadAccelerators (which returns the same
     // handle when called twice for the same accelerator table resource),
     // but requires DestroyAcceleratorTable be called for handles
     // created by CreateAcceleratorTable.  To be able to do clean-up in
     // a reasonable fashion, remap any handle created by LoadAccelerators
     // into one created by CreateAcceleratorTable.  Windows NT seems to
     // allow DestroyAcceleratorTable to be used with LoadAccelerators.
     IAcceleratorTable accelTable( haccel );
     haccel = accelTable.handle();
  }

  // We should use the Windows 95 code for Win32s as well, but
  // CopyAcceleratorTable (called by IAcceleratorTable) does not seem
  // to work.
#endif // IC_WIN

  if (!haccel)
  {
     ITHROWGUIERROR(IString("WinLoadAccelTable(")+
                  "IResourceId:"+IString(ulAccelTableId)+")");
  }

  return (haccel);

}

/*------------------------------------------------------------------------------
| IResourceLibrary::loadHelpTable                                              |
------------------------------------------------------------------------------*/
IResourceLibrary& IResourceLibrary :: loadHelpTable(IWindow* pwinHelpInstance,
                          unsigned long ulHelpTableId) const
{
  IMODTRACE_ALL("ResLib::loadHelpTable");

  if (IHelpStatics::loadHelpTable
         (pwinHelpInstance->handle(), ulHelpTableId, handle())==0)
  {
     ITHROWGUIERROR(IString("WinLoadHelpTable(")+
                  "IResourceId:"+IString(ulHelpTableId)+")");
  }

  return (IResourceLibrary&)*this;
}


/*------------------------------------------------------------------------------
| IResourceLibrary::fileName                                                   |
|   Return the Dll File Name                                                   |
------------------------------------------------------------------------------*/
IString IResourceLibrary::fileName() const
{

  IMODTRACE_ALL("ResLib::fileName");

#ifdef IC_PM
  PTIB    ptib;      /*  Thread Information Block  */
  PPIB    ppib;      /*  Process Information Block */
  APIRET  rc;         /* Return code */
  rc = DosGetInfoBlocks(&ptib, &ppib);
  IString str(ppib->pib_pchcmd);
#endif
#ifdef IC_WIN
  IString str(GetCommandLine());
#endif
  return (str);
};


/*------------------------------------------------------------------------------
| IResourceLibrary::isOpen                                                     |
------------------------------------------------------------------------------*/
IBase::Boolean IResourceLibrary::isOpen() const
{
  return true;
}

/*------------------------------------------------------------------------------
| IResourceLibrary::handle                                                     |
------------------------------------------------------------------------------*/
IModuleHandle IResourceLibrary::handle() const
{ return hInstance;};


/*------------------------------------------------------------------------------
| IResLibStaticPtr::~IResLibStaticPtr                                          |
------------------------------------------------------------------------------*/
IResLibStaticPtr :: ~IResLibStaticPtr ()
{
  deletePointerSet ( );
  deleteBitmapSet  ( );
  deleteDLLSet     ( );
  if(pResDefault)
    delete pResDefault;
}


/*------------------------------------------------------------------------------
| IResLibStaticPtr::defaultResource                                            |
------------------------------------------------------------------------------*/
IResourceLibrary& IResLibStaticPtr :: defaultResource()
{
  if(pResDefault==0)
    pResDefault = new IResourceLibrary;
  return *pResDefault;
}

/*------------------------------------------------------------------------------
| IResLibStaticPtr::libraryKey                                                 |
------------------------------------------------------------------------------*/
IPrivateResource& IResLibStaticPtr :: libraryKey()
{
  if(pLibKey==0)
    pLibKey = new IPrivateResource;
  return *pLibKey;
}
