/*******************************************************************************
* FILE NAME: iscroll1.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in isclnhdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

#define INCL_WINSCROLLBARS
extern "C" {
   #include <iwindefs.h>
}

#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif

#ifndef _ISCROLL_
  #include <iscroll.hpp>
#endif

#ifndef _ISCLNHDR_
  #include <isclnhdr.hpp>
#endif

#ifndef _IEVENT_
  #include <ievent.hpp>
#endif

#ifndef _INOTIFEV_
  #include <inotifev.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISCROLL1_CPP_
  #include <ipagetun.h>
#endif

// Class to clean up static objects
class IScrollBarNotifyStatics
{
public:

  ~IScrollBarNotifyStatics ()
  {
    delete scrollBarNotifyHandler;
    scrollBarNotifyHandler = 0;
  }

static IScrollBarNotifyHandler
  &notificationHandler ( );

static IScrollBarNotifyHandler
  *scrollBarNotifyHandler;
};

// Initialize static objects and notification strings.
#pragma data_seg(ICLStaticConst)
static IScrollBarNotifyStatics scrollBarStatics;
INotificationId const IScrollBar::scrollBoxPositionId
                                       = "IScrollBar::scrollBoxPosition";
#pragma data_seg()

#pragma data_seg(ICLNonConst)
IScrollBarNotifyHandler*
  IScrollBarNotifyStatics::scrollBarNotifyHandler = 0;
#pragma data_seg()

/*------------------------------------------------------------------------------
| IScrollBarNotifyStatics::notificationHandler                                 |
|                                                                              |
| Static function to create and return a notification handler for IScrollBar.  |
------------------------------------------------------------------------------*/
IScrollBarNotifyHandler& IScrollBarNotifyStatics::notificationHandler ()
{
  if (!(IScrollBarNotifyStatics::scrollBarNotifyHandler))
  {
    IScrollBarNotifyStatics::scrollBarNotifyHandler =
                                   new IScrollBarNotifyHandler();
  }
  return(*IScrollBarNotifyStatics::scrollBarNotifyHandler);
}

/*------------------------------------------------------------------------------
| IScrollBar::enableNotification                                               |
|                                                                              |
| Flags this part as being ready to accept notifications.  A Notifier, like    |
| IScrollBar, does not send notifications to observer objects until it is      |
| enabled.                                                                     |
|                                                                              |
| In the window hierarchy we override this function to allocate the correct    |
| notification handler.                                                        |
------------------------------------------------------------------------------*/
IScrollBar& IScrollBar::enableNotification (Boolean enabled)
{
  if (!notificationHandler())
  {
    setNotificationHandler(
               &(IScrollBarNotifyStatics::notificationHandler()));
  }
  IWindow::enableNotification(enabled);
  return(*this);
}

/*------------------------------------------------------------------------------
| IScrollBarNotifyHandler::IScrollBarNotifyHandler                             |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
IScrollBarNotifyHandler::IScrollBarNotifyHandler()
{}

/*------------------------------------------------------------------------------
| IScrollBarNotifyHandler::~IScrollBarNotifyHandler                            |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IScrollBarNotifyHandler::~IScrollBarNotifyHandler()
{}

/*------------------------------------------------------------------------------
| IScrollBarNotifyHandler::dispatchHandlerEvent                                |
|                                                                              |
| This notifies IScrollBar Windows of the following events:                    |
|                                                                              |
| scrollBoxPosition(Id)                                                        |
------------------------------------------------------------------------------*/
#ifdef IC_PMWIN
Boolean IScrollBarNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
  IWindow* thisWindow = anEvent.window();

  if (thisWindow)
  {
    switch (anEvent.eventId())
    {
#ifdef IC_WIN
      case WM_CTLCOLORSCROLLBAR:
      {
        thisWindow->notifyObservers(INotificationEvent(
                                    IScrollBar::backgroundColorId,
                                    *thisWindow));
      break;
      }
#endif

#ifdef IC_WIN
      /* IC_SBM_SETPOS_WIN32S added for wins32s */
      case IC_SBM_SETPOS_WIN32S:
#endif
      case SBM_SETPOS:

        /**************************************************************/
        /* Post private message to delay notification until after     */
        /* the selection state has been modified                      */
        /**************************************************************/
        thisWindow->postEvent( IC_UM_DELAYNOTIFY,
                               IEventParameter1( anEvent.parameter1() ),
                               IEventParameter2( IC_NOTDEL_SCRL_POS ));
        break;


      /****************************************************************/
      /* This is the delayed notification message.  Check to see which*/
      /* notification should be sent to the observers based on info   */
      /* stored in parameter 2 of the private message.  Parameter 1   */
      /* contains the information passed in the original event that   */
      /* needs to be sent with the notification                       */
      /****************************************************************/
      case IC_UM_DELAYNOTIFY:

        if (anEvent.parameter2().asUnsignedLong() == IC_NOTDEL_SCRL_POS)
        {
          thisWindow->notifyObservers(INotificationEvent(
                                      IScrollBar::scrollBoxPositionId,
                                      *thisWindow, true,
                                      (void*)anEvent.parameter1()));
        }
        break;

      default:
        break;
    }  /* end switch */
  }
  return (Inherited::dispatchHandlerEvent(anEvent));
}
#endif // IC_PMWIN

