/*******************************************************************************
* FILE NAME: isetbut.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in isetbut.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C"
{
  #define INCL_WINBUTTONS
  #include <iwindefs.h>
}

#ifndef _ISETBUT_
  #include <isetbut.hpp>
#endif

#ifndef _INOTIFEV_
  #include <inotifev.hpp>
#endif

#ifdef IC_PAGETUNE
  #define _ISETBUT_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| ISettingButton::ISettingButton                                               |
------------------------------------------------------------------------------*/
ISettingButton :: ISettingButton() {;}

/*------------------------------------------------------------------------------
| ISettingButton::~ISettingButton                                              |
------------------------------------------------------------------------------*/
ISettingButton :: ~ISettingButton() {;}

/*------------------------------------------------------------------------------
| ISettingButton::isSelected                                                   |
------------------------------------------------------------------------------*/
Boolean  ISettingButton :: isSelected() const
{
  IEventResult evt = handle().sendEvent(BM_QUERYCHECK,
                             IEventParameter1(0),
                             IEventParameter2(0));
  if (evt.asUnsignedLong())
     return true;
  else
     return false;
}


/*------------------------------------------------------------------------------
| ISettingButton::select                                                       |
------------------------------------------------------------------------------*/
ISettingButton&  ISettingButton :: select(Boolean sel)
{
  if ((sel && (!(this->isSelected()))) ||
      (!sel && (this->isSelected())))
  {
    ISize siz = size();
    Boolean wasvisible=isVisible();

    if (wasvisible && ((siz.width()==0) || (siz.height()==0)))
      hide();

    handle().sendEvent(BM_SETCHECK,
                       IEventParameter1(sel),
                       IEventParameter2(0));
#ifdef IC_WIN
    IWindowHandle hwnd = this->handle();
    if ((sel) &&
        (hwnd.sendEvent(WM_QUERYDLGCODE, 0, 0) & DLGC_RADIOBUTTON))
       {
       // We have to uncheck any other radio buttons in the group
       // ourselves.  Note that this code does not handle disabled
       // radio buttons properly.  There is code in the iradiobt.cpp
       // to do that since we have to handle that case when the button
       // is clicked by the user as well as set manually.
       IWindowHandle
          parent  ( IPARENTOF( hwnd ) ) ,
          first   (0),
          current (0);

       first = current = GetNextDlgGroupItem( parent, hwnd, FALSE);
       if (current != 0)
          {
          do
             {
             if (current != hwnd)
                {
                // Uncheck the button.
                current.sendEvent(BM_SETCHECK,
                                  IEventParameter1(0),
                                  IEventParameter2(0));
                // If tabstop is set turn it off.  Otherwise we have
                // key navigation problems.
                unsigned long style = ISTYLEOF(current);
                if (style & WS_TABSTOP)
                   {
                   style &= (unsigned long)~WS_TABSTOP;
                   ISETWINDOWSTYLE(current, style);
                   }
                }
             current = GetNextDlgGroupItem( parent, current, FALSE);
             }
          while (current != first );
          }
       }  // if radio button
#endif

    if (wasvisible && ((siz.width()==0) || (siz.height()==0)))
      show();

    this->notifyObservers(INotificationEvent(IButton::buttonClickId,
                                             *this, false));
  }
  return *this;
}


/*------------------------------------------------------------------------------
| ISettingButton::deselect                                                     |
------------------------------------------------------------------------------*/
ISettingButton&  ISettingButton :: deselect()
{
  this->select(false);
  return *this;
}
