/*******************************************************************************
* FILE NAME: isetcv.cpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in isetcv.hpp.                                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C" {
  #define INCL_GPIPRIMITIVES        // GpiQueryTextBox, TXTBOX_xxx
  #define INCL_WINSTATICS           // SS_GROUPBOX
  #define INCL_WINSYS               // PP_BACKGROUNDCOLOR
  #include <iwindefs.h>
}

#include <isetcv.hpp>

#include <icconst.h>
#include <icolor.hpp>
#include <iexcept.hpp>
#include <ifont.hpp>
#include <inotifev.hpp>
#include <ipoint.hpp>
#include <ireslib.hpp>
#include <isetcvob.hpp>         // for ISetCanvasObject
#include <isizehdr.hpp>
#include <istring.hpp>
#include <itrace.hpp>
#include <iwposbuf.hpp>
#include <iseq.h>               // for ISequence

#ifdef IC_MOTIF
  #ifndef _IXCONST_
    #include <ixconst.h>
  #endif
  #ifndef _IMSTRING_
     #include <imstring.hpp>
  #endif
  extern "C"
    {
    #include <X11/Intrinsic.h>
    #include <Xm/Frame.h>
    #include <Xm/LabelG.h>
    #include <Xm/BulletinB.h>
    }
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISETCV_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| Public set canvas styles.                                                    |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLStaticConst)
const ISetCanvas::Style
  ISetCanvas::horizontalDecks     ( 0, ICNV_HORIZONTALDECKS ),
  ISetCanvas::verticalDecks       ( 0, ICNV_VERTICALDECKS ),
  ISetCanvas::packTight           ( 0, ICNV_PACKTIGHT ),
  ISetCanvas::packEven            ( 0, ICNV_PACKEVEN ),
  ISetCanvas::packExpanded        ( 0, ICNV_PACKEXPANDED ),
  ISetCanvas::leftAlign           ( 0, ICNV_LEFTALIGN ),
  ISetCanvas::rightAlign          ( 0, ICNV_RIGHTALIGN ),
  ISetCanvas::centerAlign         ( 0, ICNV_CENTERALIGN ),
  ISetCanvas::topAlign            ( 0, ICNV_TOPALIGN ),
  ISetCanvas::bottomAlign         ( 0, ICNV_BOTTOMALIGN ),
  ISetCanvas::centerVerticalAlign ( 0, ICNV_CENTERVERTICALALIGN ),
  ISetCanvas::decksByGroup        ( 0, ICNV_DECKSBYGROUP ),
#ifdef IC_MOTIF
  ISetCanvas::border              ( 0, ICNV_BORDER ),
#endif
  ISetCanvas::classDefaultStyle   ( WS_VISIBLE, (ICNV_HORIZONTALDECKS |
                                                 ICNV_PACKTIGHT       |
                                                 ICNV_LEFTALIGN       |
                                                 ICNV_TOPALIGN) );
#pragma data_seg()

/*------------------------------------------------------------------------------
| Default style for new objects (initial value).                               |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLNonConst)
ISetCanvas::Style
  ISetCanvas::currentDefaultStyle ( WS_VISIBLE, (ICNV_HORIZONTALDECKS |
                                                 ICNV_PACKTIGHT       |
                                                 ICNV_LEFTALIGN       |
                                                 ICNV_TOPALIGN) );
#pragma data_seg()

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| ISetCanvasShowHandler                                                        |
|                                                                              |
| Motif does not get the first WM_RESIZE message so we need a handler to look  |
| for the first time the window is shown.                                      |
------------------------------------------------------------------------------*/
class ISetCanvasShowHandler : public IHandler {
protected:
virtual Boolean
  dispatchHandlerEvent ( IEvent& event );
}; // ISetCanvasShowHandler
#endif

/*------------------------------------------------------------------------------
| ISetCanvasData                                                               |
|                                                                              |
| Private data class for ISetCanvas.                                           |
------------------------------------------------------------------------------*/
class ISetCanvasData {
public:
ISetCanvasData ( )
#ifdef IC_PMWIN
  : layoutInProgress( 0 )
#endif
#ifdef IC_PM
    ,
    positioningHeight( 0 )
#endif
#ifdef IC_MOTIF
  : frame( 0 ),
    title( 0 ),
    needsFrame( false )
#endif
  { }
#ifdef IC_MOTIF
IWindowHandle
  frame,
  title;
IBase::Boolean
  needsFrame;
ISetCanvasShowHandler
 *showHandler;
#endif
#ifdef IC_PMWIN
unsigned long
  layoutInProgress;
#endif
#ifdef IC_PM
unsigned long
  positioningHeight;
#endif
}; // ISetCanvasData


/*------------------------------------------------------------------------------
| ISetCanvasResizeHandler                                                      |
|                                                                              |
| Handle resize requests for the tool bars frame.                              |
------------------------------------------------------------------------------*/
class ISetCanvasResizeHandler : public IResizeHandler {
public:
  ISetCanvasResizeHandler ( );
virtual
 ~ISetCanvasResizeHandler ( );
protected:
virtual Boolean
  windowResize ( IResizeEvent& event );
}; // ISetCanvasResizeHandler

/*------------------------------------------------------------------------------
| ISetCanvasResizeHandler::ISetCanvasResizeHandler                             |
|                                                                              |
------------------------------------------------------------------------------*/
ISetCanvasResizeHandler::ISetCanvasResizeHandler ( )
{ }

/*------------------------------------------------------------------------------
| ISetCanvasResizeHandler::~ISetCanvasResizeHandler                            |
|                                                                              |
------------------------------------------------------------------------------*/
ISetCanvasResizeHandler::~ISetCanvasResizeHandler ( )
{ }


/*------------------------------------------------------------------------------
| ISetCanvasObjectList                                                         |
|                                                                              |
| Implementation class.                                                        |
------------------------------------------------------------------------------*/
class ISetCanvasObjectList : public ISequence < ISetCanvasObject* > {
public:
  ISetCanvasObjectList ( );
 ~ISetCanvasObjectList ( );
}; // ISetCanvasObjectList

/*------------------------------------------------------------------------------
| ISetCanvasObjectList::ISetCanvasObjectList                                   |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
ISetCanvasObjectList::ISetCanvasObjectList ( )
  : ISequence < ISetCanvasObject* > ( )
{ }

/*------------------------------------------------------------------------------
| ISetCanvasObjectList::~ISetCanvasObjectList                                  |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
ISetCanvasObjectList::~ISetCanvasObjectList ( )
{ }


IBase::Boolean ISetCanvasDeleter ( ISetCanvasObject* const& element,
                                   void* )
{
  delete element;
  return true;
}


/*------------------------------------------------------------------------------
| ISetCanvas::ISetCanvas                                                       |
|                                                                              |
| Constructor to create a set canvas on a standard window.                     |
------------------------------------------------------------------------------*/
ISetCanvas::ISetCanvas ( unsigned long windowIdentifier,
                         IWindow* parent,
                         IWindow* owner,
                         const IRectangle& initial,
                         const Style& style )
  : ISetCanvas::Inherited( ),
    ulClNumDecks( 1 ), ulClGroupPad( 0 ),
    sizeHandler( new ISetCanvasResizeHandler ),
    fSetCanvasData( new ISetCanvasData )
{
  // Check for valid style combination.
  if ( ((style & horizontalDecks)  &&  (style & verticalDecks))        ||
       ((style & packTight)        &&  (style & packEven))             ||
       ((style & packTight)        &&  (style & packExpanded))         ||
       ((style & packEven)         &&  (style & packExpanded))         ||
       ((style & leftAlign)        &&  (style & rightAlign))           ||
       ((style & leftAlign)        &&  (style & centerAlign))          ||
       ((style & rightAlign)       &&  (style & centerAlign))          ||
       ((style & topAlign)         &&  (style & bottomAlign))          ||
       ((style & topAlign)         &&  (style & centerVerticalAlign))  ||
       ((style & bottomAlign)      &&  (style & centerVerticalAlign)) )
  {                                   // Mutually exclusive styles.
     ITHROWLIBRARYERROR( IC_INVALIDSTYLE,
                         IBaseErrorInfo::invalidParameter,
                         IException::recoverable );
  } /* endif */

  // Save the extended style to make sure we have a copy of it stored.
  setExtendedStyle( extendedStyle() | style.asExtendedUnsignedLong() );

#ifdef IC_MOTIF
  Arg            args[8];
  int            n = 0;

  XtSetArg(args[n], XmNmarginHeight, 0 ); n++;
  XtSetArg(args[n], XmNmarginWidth, 0 ); n++;

  // ResizeNone is necessary to stop the bulletin board from resize itself
  // smaller around the controls. XmRESIZE_GROW might work as well.
  XtSetArg( args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
  XtSetArg( args[n], XmNautoUnmanage, false); n++;
  if (style & border )
  {
    fSetCanvasData->needsFrame = true;

    // the args set before this if statement are for the bulletin board created
    // below. If any args are to be set for the frame use frameArgs.
    Arg frameArgs[1];
    int frameN = 0;

    fSetCanvasData->frame = XmCreateFrame( parent->handle(),
       "frame_" + IString( windowIdentifier ),
       frameArgs, frameN );

    Widget whndCanvas = XmCreateBulletinBoard( fSetCanvasData->frame,
       IString( windowIdentifier ),
       args, n );
    XtManageChild( whndCanvas );

    fSetCanvasData->title = XtVaCreateManagedWidget( "Label_" +
                   IString( windowIdentifier ),
                   xmLabelGadgetClass, fSetCanvasData->frame,
                   XmNchildType, XmFRAME_TITLE_CHILD,
                   XmNchildVerticalAlignment, XmALIGNMENT_CENTER,
                   NULL );
    IMString mString("");
    XtVaSetValues( fSetCanvasData->title,
      XmNlabelString, (XmString) mString,
      NULL );
    prepareForUse( whndCanvas );

    // Process styles
    convertToGUIStyle( style );

    moveSizeTo( initial );
  }
  else
  {
    Inherited::create( windowIdentifier, NULL, convertToGUIStyle( style ),
        (IXmCreateFunction ) XmCreateBulletinBoard,
         parent->handle(), owner->handle(),
         initial, args, n );
  }
#else // !IC_MOTIF
  // Create the presentation system window.
  this->initialize( windowIdentifier,
                    parent,
                    owner,
                    initial,
                    this->convertToGUIStyle( style ),
                    this->convertToGUIStyle( style, true ) );
#endif

  /*****************************************************************/
  /* Set the margin and pad defaults using the current font (use   */
  /* the average character width and half the maximum character    */
  /* height).                                                      */
  /*****************************************************************/
  sizClMargin = this->characterSize();
  sizClMargin.setHeight( sizClMargin.height() / 2 );
  sizClPad = sizClMargin;

  sizeHandler->handleEventsFor( this );
#ifdef IC_MOTIF
  fSetCanvasData->showHandler = new ISetCanvasShowHandler;
  fSetCanvasData->showHandler->handleEventsFor( this );
#endif
}

/*------------------------------------------------------------------------------
| ISetCanvas::~ISetCanvas                                                      |
|                                                                              |
| Clean-up.                                                                    |
------------------------------------------------------------------------------*/
ISetCanvas::~ISetCanvas ( )
{
  if ( sizeHandler )
  {
     sizeHandler->stopHandlingEventsFor( this );
     delete sizeHandler;
  }
  if ( fSetCanvasData )
  {
#ifdef IC_MOTIF
     if ( fSetCanvasData->showHandler )
     {
        fSetCanvasData->showHandler->stopHandlingEventsFor( this );
        delete fSetCanvasData->showHandler;
     }
#endif
     delete fSetCanvasData;
  }
}

/*------------------------------------------------------------------------------
| ISetCanvas::defaultStyle                                                     |
|                                                                              |
| Returns the current default style.                                           |
| Notes: Can not inline this since it is in the constructor declaration.       |
------------------------------------------------------------------------------*/
ISetCanvas::Style ISetCanvas::defaultStyle ( )
{
  return currentDefaultStyle;
}

/*------------------------------------------------------------------------------
| ISetCanvas::convertToGUIStyle                                                |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long ISetCanvas::convertToGUIStyle ( const IBitFlag& guiStyle,
                                              Boolean bExtOnly ) const
{
  // Obtain the style from the class (ICanvas) that we inherit from.
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if ( bExtOnly )
  {
    // Use mask to only return extended styles in the user defined range.
    ulStyle |= guiStyle.asExtendedUnsignedLong() & IS_EXTMASK;
  }
  else
  {
    // All the styles for the setcanvas are extended styles, so we do not mask.
    ulStyle |= guiStyle.asUnsignedLong();

    // No additional processing required since ICanvas::convertToGUIStyle is
    // doing all the required work ...
  }

  return ulStyle;
}


/*------------------------------------------------------------------------------
| ISetCanvas::setDeckOrientation                                               |
|                                                                              |
| Sets the orientation to be used for laying out the contents of the canvas    |
| (vertical vs. horizontal).                                                   |
------------------------------------------------------------------------------*/
ISetCanvas& ISetCanvas::setDeckOrientation ( DeckOrientation orValue )
{
  unsigned long ulExtStyle = this->extendedStyle();
  unsigned long ulOldExtStyle = ulExtStyle;

  switch ( orValue )
  {
     case horizontal:
        ulExtStyle &= ~verticalDecks.asExtendedUnsignedLong();
        ulExtStyle |= horizontalDecks.asExtendedUnsignedLong();
        break;
     case vertical:
        ulExtStyle &= ~horizontalDecks.asExtendedUnsignedLong();
        ulExtStyle |= verticalDecks.asExtendedUnsignedLong();
        break;
     default:                          // Uninitialized enum
        ITHROWLIBRARYERROR( IC_INVALIDENUMVALUE,
                            IBaseErrorInfo::invalidParameter,
                            IException::recoverable );
        break;
  }

  if ( ulExtStyle != ulOldExtStyle )
  {                                    // Orientation change.
     this->setExtendedStyle( ulExtStyle );
     this->setLayoutDistorted( IWindow::layoutChanged |
                               IWindow::immediateUpdate, 0 );
                                       // Cause canvas to re-layout.
     this->notifyObservers( INotificationEvent( ISetCanvas::deckOrientationId,
                                                *this, true,
                                                (void*)orValue ));
  }
  return *this;
}

/*------------------------------------------------------------------------------
| ISetCanvas::deckOrientation                                                  |
|                                                                              |
| Returns the current orientation setting.                                     |
------------------------------------------------------------------------------*/
ISetCanvas::DeckOrientation ISetCanvas::deckOrientation ( ) const
{
  return( (extendedStyle() & verticalDecks.asExtendedUnsignedLong()) ? vertical
                                                                : horizontal );
}

/*------------------------------------------------------------------------------
| ISetCanvas::setPackType                                                      |
|                                                                              |
| Replaces the packing type to be used for laying out the contents of the      |
| canvas (tight vs. even vs. expanded).                                        |
------------------------------------------------------------------------------*/
ISetCanvas& ISetCanvas::setPackType ( PackType packValue )
{
  unsigned long ulExtStyle = this->extendedStyle();
  unsigned long ulOldExtStyle = ulExtStyle;

  switch ( packValue )
  {
    case tight:
       ulExtStyle &= ~(packEven.asExtendedUnsignedLong() |
                       packExpanded.asExtendedUnsignedLong());
       ulExtStyle |= packTight.asExtendedUnsignedLong();
       break;
    case even:
       ulExtStyle &= ~(packTight.asExtendedUnsignedLong() |
                       packExpanded.asExtendedUnsignedLong());
       ulExtStyle |= packEven.asExtendedUnsignedLong();
       break;
    case expanded:
       ulExtStyle &= ~(packTight.asExtendedUnsignedLong() |
                       packEven.asExtendedUnsignedLong());
       ulExtStyle |= packExpanded.asExtendedUnsignedLong();
       break;
    default:                          // Uninitialized enum
       ITHROWLIBRARYERROR( IC_INVALIDENUMVALUE,
                           IBaseErrorInfo::invalidParameter,
                           IException::recoverable );
       break;
  }

  if ( ulExtStyle != ulOldExtStyle )
  {                                    // Packing change.
     this->setExtendedStyle( ulExtStyle );
     this->setLayoutDistorted( IWindow::layoutChanged |
                               IWindow::immediateUpdate, 0 );
                                      // Cause canvas to re-layout.
  }
  return *this;
}

/*------------------------------------------------------------------------------
| ISetCanvas::packType                                                         |
|                                                                              |
| Returns the current packing type.                                            |
------------------------------------------------------------------------------*/
ISetCanvas::PackType ISetCanvas::packType ( ) const
{
  unsigned long ulExtStyle = this->extendedStyle();
  PackType packValue = tight;          // Default value.

  if ( ulExtStyle & packEven.asExtendedUnsignedLong() )
  {
     packValue = even;
  }
  else if ( ulExtStyle & packExpanded.asExtendedUnsignedLong() )
  {
     packValue = expanded;
  }
  return packValue;
}

/*------------------------------------------------------------------------------
| ISetCanvas::setAlignment                                                     |
|                                                                              |
| Replaces the alignment used to position controls within their grids          |
| (topLeft, topCenter, topRight, centerLeft, etc).                             |
------------------------------------------------------------------------------*/
ISetCanvas& ISetCanvas::setAlignment ( Alignment alignValue )
{
  unsigned long ulExtStyle = this->extendedStyle();
  unsigned long ulOldExtStyle = ulExtStyle;

  switch ( alignValue )
  {
    case topLeft:
       ulExtStyle &= ~(centerVerticalAlign.asExtendedUnsignedLong() |
                       bottomAlign.asExtendedUnsignedLong()         |
                       centerAlign.asExtendedUnsignedLong()         |
                       rightAlign.asExtendedUnsignedLong());
       ulExtStyle |= (topAlign.asExtendedUnsignedLong() |
                      leftAlign.asExtendedUnsignedLong());
       break;
    case topCenter:
       ulExtStyle &= ~(centerVerticalAlign.asExtendedUnsignedLong() |
                       bottomAlign.asExtendedUnsignedLong()         |
                       leftAlign.asExtendedUnsignedLong()           |
                       rightAlign.asExtendedUnsignedLong());
       ulExtStyle |= (topAlign.asExtendedUnsignedLong() |
                      centerAlign.asExtendedUnsignedLong());
       break;
    case topRight:
       ulExtStyle &= ~(centerVerticalAlign.asExtendedUnsignedLong() |
                       bottomAlign.asExtendedUnsignedLong()         |
                       leftAlign.asExtendedUnsignedLong()           |
                       centerAlign.asExtendedUnsignedLong());
       ulExtStyle |= (topAlign.asExtendedUnsignedLong() |
                      rightAlign.asExtendedUnsignedLong());
       break;
    case centerLeft:
       ulExtStyle &= ~(topAlign.asExtendedUnsignedLong()    |
                       bottomAlign.asExtendedUnsignedLong() |
                       rightAlign.asExtendedUnsignedLong()  |
                       centerAlign.asExtendedUnsignedLong());
       ulExtStyle |= (centerVerticalAlign.asExtendedUnsignedLong() |
                      leftAlign.asExtendedUnsignedLong());
       break;
    case centerCenter:
       ulExtStyle &= ~(topAlign.asExtendedUnsignedLong()    |
                       bottomAlign.asExtendedUnsignedLong() |
                       rightAlign.asExtendedUnsignedLong()  |
                       leftAlign.asExtendedUnsignedLong());
       ulExtStyle |= (centerVerticalAlign.asExtendedUnsignedLong() |
                      centerAlign.asExtendedUnsignedLong());
       break;
    case centerRight:
       ulExtStyle &= ~(topAlign.asExtendedUnsignedLong()    |
                       bottomAlign.asExtendedUnsignedLong() |
                       centerAlign.asExtendedUnsignedLong() |
                       leftAlign.asExtendedUnsignedLong());
       ulExtStyle |= (centerVerticalAlign.asExtendedUnsignedLong() |
                      rightAlign.asExtendedUnsignedLong());
       break;
    case bottomLeft:
       ulExtStyle &= ~(centerVerticalAlign.asExtendedUnsignedLong() |
                       topAlign.asExtendedUnsignedLong()            |
                       rightAlign.asExtendedUnsignedLong()          |
                       centerAlign.asExtendedUnsignedLong());
       ulExtStyle |= (bottomAlign.asExtendedUnsignedLong() |
                      leftAlign.asExtendedUnsignedLong());
       break;
    case bottomCenter:
       ulExtStyle &= ~(centerVerticalAlign.asExtendedUnsignedLong() |
                       topAlign.asExtendedUnsignedLong()            |
                       rightAlign.asExtendedUnsignedLong()          |
                       leftAlign.asExtendedUnsignedLong());
       ulExtStyle |= (bottomAlign.asExtendedUnsignedLong() |
                      centerAlign.asExtendedUnsignedLong());
       break;
    case bottomRight:
       ulExtStyle &= ~(centerVerticalAlign.asExtendedUnsignedLong() |
                       topAlign.asExtendedUnsignedLong()            |
                       centerAlign.asExtendedUnsignedLong()         |
                       leftAlign.asExtendedUnsignedLong());
       ulExtStyle |= (bottomAlign.asExtendedUnsignedLong() |
                      rightAlign.asExtendedUnsignedLong());
       break;
    default:                          // Uninitialized enum
       ITHROWLIBRARYERROR( IC_INVALIDENUMVALUE,
                           IBaseErrorInfo::invalidParameter,
                           IException::recoverable );
       break;
  }

  if ( ulExtStyle != ulOldExtStyle )
  {                                    // Alignment change.
     this->setExtendedStyle( ulExtStyle );
     this->setLayoutDistorted( IWindow::layoutChanged |
                               IWindow::immediateUpdate, 0);
                                      // Cause canvas to re-layout.
  }
  return *this;
}

/*------------------------------------------------------------------------------
| ISetCanvas::alignment                                                        |
|                                                                              |
| Returns the current alignment of controls in the canvas.                     |
------------------------------------------------------------------------------*/
ISetCanvas::Alignment ISetCanvas::alignment ( ) const
{
  unsigned long ulExtStyle = this->extendedStyle();
  Alignment alignValue = topLeft;      // Default value.

  if ( ulExtStyle & centerVerticalAlign.asExtendedUnsignedLong() )
  {
     if ( ulExtStyle & centerAlign.asExtendedUnsignedLong() )
     {
        alignValue = centerCenter;
     }
     else if ( ulExtStyle & rightAlign.asExtendedUnsignedLong() )
     {
        alignValue = centerRight;
     }
     else                              // leftAlign.
     {
        alignValue = centerLeft;
     }
  }
  else if ( ulExtStyle & bottomAlign.asExtendedUnsignedLong() )
  {
     if ( ulExtStyle & centerAlign.asExtendedUnsignedLong() )
     {
        alignValue = bottomCenter;
     }
     else if ( ulExtStyle & rightAlign.asExtendedUnsignedLong() )
     {
        alignValue = bottomRight;
     }
     else                              // leftAlign.
     {
        alignValue = bottomLeft;
     }
  }
  else                                 // topAlign.
  {
     if ( ulExtStyle & centerAlign.asExtendedUnsignedLong() )
     {
        alignValue = topCenter;
     }
     else if ( ulExtStyle & rightAlign.asExtendedUnsignedLong() )
     {
        alignValue = topRight;
     }
     else                              // leftAlign.
     {
        alignValue = topLeft;
     }
  }

  return alignValue;
}

/*------------------------------------------------------------------------------
| ISetCanvas::setDeckCount                                                     |
|                                                                              |
| Sets the maximum number of decks used by the canvas during layout.           |
------------------------------------------------------------------------------*/
ISetCanvas& ISetCanvas::setDeckCount ( unsigned long decks )
{
  IASSERTPARM( decks != 0 );

  if ( ulClNumDecks != decks )
  {                                    // Deck number change.
     ulClNumDecks = decks;
     this->setLayoutDistorted( IWindow::layoutChanged |
                               IWindow::immediateUpdate, 0 );
                                       // Cause canvas to re-layout.
  }
  return *this;
}

/*------------------------------------------------------------------------------
| ISetCanvas::setMargin                                                        |
|                                                                              |
| Sets the margin width and height which is the space between the edge of the  |
| canvas and the outermost child window.                                       |
------------------------------------------------------------------------------*/
ISetCanvas& ISetCanvas::setMargin ( const ISize& margin )
{
  IASSERTPARM( margin.width() >= 0  &&  margin.height() >= 0 );

  if ( sizClMargin != margin )
  {                                    // New margin specified.
     sizClMargin = margin;
     this->setLayoutDistorted( IWindow::layoutChanged |
                               IWindow::immediateUpdate, 0 );
                                       // Cause canvas to re-layout.
  }
  return *this;
}

/*------------------------------------------------------------------------------
| ISetCanvas::setPad                                                           |
|                                                                              |
| Sets the pad width and height which is the space between child windows in a  |
| deck and between mulitiple decks.                                            |
------------------------------------------------------------------------------*/
ISetCanvas& ISetCanvas::setPad ( const ISize& pad )
{
  IASSERTPARM( pad.width() >= -1  &&  pad.height() >= -1 );

  if ( sizClPad != pad )
  {                                    // New pad specified.
     sizClPad = pad;
     this->setLayoutDistorted( IWindow::layoutChanged |
                               IWindow::immediateUpdate, 0 );
                                       // Cause canvas to re-layout.
  }
  return *this;
}

/*------------------------------------------------------------------------------
| ISetCanvas::setGroupPad                                                      |
|                                                                              |
| Sets the pad width or height.                                                |
------------------------------------------------------------------------------*/
ISetCanvas& ISetCanvas::setGroupPad ( unsigned long pad )
{
  if ( ulClGroupPad != pad )
  {                                    // New pad specified.
     ulClGroupPad = pad;
     this->setLayoutDistorted( IWindow::layoutChanged |
                               IWindow::immediateUpdate, 0 );
                                       // Cause canvas to re-layout.
  }
  return *this;
}

/*------------------------------------------------------------------------------
| ISetCanvas::groupPad                                                         |
|                                                                              |
| Return the group pad.                                                        |
------------------------------------------------------------------------------*/
unsigned long ISetCanvas::groupPad ( ) const
{
  return ulClGroupPad;
}

/*------------------------------------------------------------------------------
| ISetCanvas::setText                                                          |
|                                                                              |
| Causes a group box with the given text to be drawn around the canvas.  If    |
| necessary, the canvas is grown to fill the width of the group box.  If no    |
| text is specified, an existing group box is removed.                         |
------------------------------------------------------------------------------*/
ISetCanvas& ISetCanvas::setText ( const char* text )
{
#ifdef IC_PMWIN

#ifdef IC_WIN
  // We can't use a real groupbox in Windows since it is a button instead
  // of a static.  We paint the frame and text ourselves in the canvas
  // handler, ICanvasHandler.   We use the SS_LEFT style to allow storing
  // the group text in the control.
#define SS_GROUPBOX   SS_LEFT
#endif   //IC_WIN

  unsigned long ulStyle = this->style();
  IString strOrigText = this->text();

  if ( text  &&  *text  &&
       (ulStyle & 0x000F) != SS_GROUPBOX )
  {                                    // Text and is not a group box.
     ulStyle &= (unsigned long)(~0x000F);
     ulStyle |= SS_GROUPBOX;           // Make it a group box.
     this->setStyle( ulStyle );
  }
  else if ( (text == 0  ||  *text == 0)  &&
            (ulStyle & 0x000F) == SS_GROUPBOX )
  {                                    // No text and is a group box.
     ulStyle &= (unsigned long)(~SS_GROUPBOX);
     ulStyle |= SS_BKGNDRECT;          // Make it a fill box.
     this->setStyle( ulStyle );
  }

  if ( strOrigText != text )
  {                                    // Change in text.
     ISETWINDOWTEXT( this->handle(), (PSZ)text );
     if ( text == 0 )
     {                                 // No text now.
        this->setLayoutDistorted( IWindow::layoutChanged, 0 );
        this->refresh();         // Force repaint to remove group box.
     }
     else
     {                                 // Cause canvas to re-layout.
        this->setLayoutDistorted( IWindow::layoutChanged |
                                  IWindow::immediateUpdate, 0 );
     }
     this->notifyObservers( INotificationEvent( ISetCanvas::textId,
                                                *this ));
  } /* endif */

#endif   //IC_PMWIN
#ifdef IC_MOTIF
  // If the setCanvas was set up with the correct style.
  if ( fSetCanvasData->needsFrame )
  {
     IMString mString( "" );
     if ( text != 0  &&  *text != 0 )
     {
        mString = IMString( text );
     }
     XtVaSetValues( fSetCanvasData->title,
                    XmNlabelString, (XmString) mString,
                    NULL );

     this->setLayoutDistorted( IWindow::layoutChanged |
                               IWindow::immediateUpdate, 0 );
  }
  this->notifyObservers( INotificationEvent( ISetCanvas::textId,
                                             *this ));
#endif

  return *this;
}

/*------------------------------------------------------------------------------
| ISetCanvas::setText                                                          |
|                                                                              |
| Adds/removes text to/from the canvas window.                                 |
------------------------------------------------------------------------------*/
ISetCanvas& ISetCanvas::setText ( const IResourceId& residText)
{
  IString aString = residText.resourceLibrary().loadString( residText );
  return this->setText( (char*)aString );
}

/*------------------------------------------------------------------------------
| ISetCanvas::text                                                             |
|                                                                              |
| Returns the text of the group box.                                           |
------------------------------------------------------------------------------*/
IString ISetCanvas::text ( ) const
{
#ifdef IC_PMWIN

  unsigned long ulClBufLen =
           (unsigned long)IQUERYWINDOWTEXTLENGTH( handle() ) + 1;

  char* pszClBuf = new char[ ulClBufLen ];

  IQUERYWINDOWTEXT( this->handle(), ulClBufLen, (PSZ)pszClBuf );
  IString str( pszClBuf );
  delete [] pszClBuf;
  return str;
#endif   //IC_PMWIN

#ifdef IC_MOTIF
  if ( fSetCanvasData->needsFrame )
  {
     return IMString( fSetCanvasData->title, XmNlabelString ).asString();
  }
  return IString( "" );
#endif
}

/*------------------------------------------------------------------------------
| ISetCanvas::expandForText                                                    |
|                                                                              |
| Grow the layout size of the canvas if necessary to be able to include the    |
| group box text of the canvas and its border.                                 |
------------------------------------------------------------------------------*/
ISetCanvas  &ISetCanvas :: expandForText ( )
{
#ifdef IC_PMWIN
  IMODTRACE_DEVELOP("ISetCanvas::expandForText()");

  IString strText = text();            // Text returned by canvas

  if (*(char*)strText)                 // Text is specified
  {                                    // Canvas is a group box control
#ifdef   IC_PM
     POINTL aptl[TXTBOX_COUNT];
     IPresSpaceHandle hps = presSpace();
     if (GpiQueryTextBox(hps, strText.length(),
                         (char*)strText, TXTBOX_COUNT, aptl) == false)
     {                                 // error from GpiQueryTextBox
        releasePresSpace(hps);
        ITHROWGUIERROR("GpiQueryTextBox");
     }
     releasePresSpace(hps);

     /**************************************************************/
     /* Determine the min and max of "x" for each corner.  This    */
     /* will enable support for sheared (e.g. italic) or rotated   */
     /* strings.                                                   */
     /* This code is modeled off of ITextControl::displaySize().   */
     /**************************************************************/
     long lLeft = aptl[TXTBOX_TOPLEFT].x;
     long lRight = lLeft;
     long lCompare = aptl[TXTBOX_BOTTOMLEFT].x;
     if (lCompare < lLeft)
     {                                 // Keep minimum x-coordinate
        lLeft = lCompare;
     }
     else if (lCompare > lRight)
     {                                 // Keep maximum x-coordinate
        lRight = lCompare;
     }
     lCompare = aptl[TXTBOX_TOPRIGHT].x;
     if (lCompare < lLeft)
     {                                 // Keep minimum x-coordinate
        lLeft = lCompare;
     }
     else if (lCompare > lRight)
     {                                 // Keep maximum x-coordinate
        lRight = lCompare;
     }
     lCompare = aptl[TXTBOX_BOTTOMRIGHT].x;
     if (lCompare < lLeft)
     {                                 // Keep minimum x-coordinate
        lLeft = lCompare;
     }
     else if (lCompare > lRight)
     {                                 // Keep maximum x-coordinate
        lRight = lCompare;
     }

     IFont fontmgr(this);
     long lAvgCharWidth = fontmgr.avgCharWidth();
     long lTextWidth = lRight - lLeft +
                         lAvgCharWidth * 2 + lAvgCharWidth / 2;
     ISize textSize( lTextWidth, fontmgr.maxCharHeight() );
#endif   //IC_PM
#ifdef IC_WIN
     SIZE   lpSize;
     IPresSpaceHandle hdc = presSpace();
     if (!GetTextExtentPoint(hdc,
                             (char*)strText,
                             strText.length(),
                             &lpSize ) )
     {                                 // error
        releasePresSpace(hdc);
        ITHROWGUIERROR("GetTextExtentPoint");
     }
     releasePresSpace(hdc);
     ISize textSize( lpSize.cx, lpSize.cy );
#endif

     /**************************************************************/
     /* Now add length needed for group box text onto the size of  */
     /* the set canvas, and also add in what is needed for the     */
     /* group box border.                                          */
     /**************************************************************/
     ISize sizLayout = layoutSize();
     sizLayout = sizLayout.maximum(ISize(textSize.width(), 0));
                                       // Include width of group box text

     sizLayout += ISize(1, 1);         // For WinDrawBorder top/right
     sizLayout += ISize(0, textSize.height());
         // Add right and bottom borders and height of group box text
     setLayoutSize(sizLayout);   // Grow if group box is expanding it
  }
#endif   //IC_PMWIN

#ifdef IC_MOTIF
  if ( fSetCanvasData->needsFrame )
    {
    ISize sizLayout = layoutSize();
    Dimension mHeight, mWidth, shadow;
    XtVaGetValues( fSetCanvasData->frame,
      XmNmarginHeight, &mHeight,
      XmNmarginWidth, &mWidth,
      XmNshadowThickness, &shadow,
      NULL );
    Dimension titleHeight;
    XtVaGetValues( fSetCanvasData->title,
      XmNheight, &titleHeight,
      NULL );
    sizLayout += ISize( (2 * mWidth) + (2 * shadow),
                        (2 * mHeight) + (2 * shadow) + titleHeight );
    setLayoutSize( sizLayout );
    }
#endif
  return *this;
}


/*------------------------------------------------------------------------------
| ISetCanvas::setLayoutDistorted                                               |
|                                                                              |
| Provide the response specific for a set canvas to various distortion         |
| notifications.  Some will cause the canvas to need to update how it has      |
| layed out its child windows.                                                 |
------------------------------------------------------------------------------*/
ISetCanvas &  ISetCanvas :: setLayoutDistorted(
                                            unsigned long layoutAttributeOn,
                                            unsigned long layoutAttributeOff )
{
  unsigned long ulFlagsOn =
       (layoutAttributeOn & (unsigned long)~IWindow::childMinimumSizeChanged);

#ifdef IC_PMWIN
  // Ignore child minimum size changes only during a layout (during a
  // layout, assume these are a by-product of calling calcMinimumSize,
  // as is the case for a child set or multicell canvas).
  if ( ( layoutAttributeOn & IWindow::childMinimumSizeChanged )  &&
       fSetCanvasData->layoutInProgress == 0 )
#endif
#ifndef IC_PMWIN
  if (layoutAttributeOn & IWindow::childMinimumSizeChanged)
#endif
  {                                    // Change to child window size
     ulFlagsOn |= IWindow::layoutChanged;  // Flag for ICanvas
     if (!(layoutAttributeOn & IWindow::fontPropogated)  &&
         !isLayoutDistorted(IWindow::fontPropogated))
     {                                 // Change buffered til now?
        ulFlagsOn |= IWindow::immediateUpdate;
     }
  }

  if (layoutAttributeOn & IWindow::fontChanged)
  {                                    // Expect minimum size change
     IString strText = text();         // Text returned by set canvas
     if (*(char*)strText)              // Text is specified
     {                                 // Text display size will change
        ulFlagsOn |= IWindow::layoutChanged;  // Flag for ICanvas
     }
  }
  Inherited::setLayoutDistorted(ulFlagsOn, layoutAttributeOff);

  return *this;
}


/*------------------------------------------------------------------------------
| ISetCanvas::layout                                                           |
|                                                                              |
| Layout management routine.                                                   |
------------------------------------------------------------------------------*/
ISetCanvas  &ISetCanvas :: layout ( )
{
  IMODTRACE_DEVELOP("ISetCanvas::layout()");

  if (isLayoutDistorted(IWindow::layoutChanged))
  {
     ITRACE_DEVELOP("isLayoutDistorted(layoutChanged)==true");
#ifdef IC_PMWIN
     fSetCanvasData->layoutInProgress++;
#endif
     /**************************************************************/
     /* The layout is distorted so calculate the position and size */
     /* of each of the child controls.                             */
     /*                                                            */
     /* For the time being, one of two simple rules is used to     */
     /* divide child controls among multiple decks: (1) The child  */
     /* controls are divied up to best keep the decks even in the  */
     /* number of controls in each, or (2) The child windows with  */
     /* the IControl::group style are used to start new decks.     */
     /* Neither algorithm takes into consideration the size of the */
     /* child controls or how best to balance the decks in terms   */
     /* of screen size.                                            */
     /*                                                            */
     /* Note: Until the controls are actually repositioned, the    */
     /*       position value kept in the ISetCanvasObject elements */
     /*       are maintained in a sort of "flipped" state          */
     /*       (top-left corners are actually the bottom-left       */
     /*       corners and uppermost controls are placed at the     */
     /*       bottom of the canvas window), since the coordinate   */
     /*       system is treated as if it were:                     */
     /*                                                            */
     /*            0    5   10   15   20   25   30   35   40       */
     /*          0 +----+----+----+----+----+----+----+----+-      */
     /*            |                                               */
     /*            |                                               */
     /*         10 |                                               */
     /*                                                            */
     /*        All this is adjusted at the end.                    */
     /**************************************************************/

     /**************************************************************/
     /* Build a list of the canvas' child controls, as enumerated  */
     /* from PM.  This list will be used as a workspace for        */
     /* positioning and sizing the child controls.                 */
     /*                                                            */
     /* Note that child controls are enumerated in the z-order     */
     /* (which is the default order that they are layed out and    */
     /* tabbed through).                                           */
     /**************************************************************/
     ITRACE_DEVELOP("Start getting all child controls");
     ISize sizLayout = layoutSize();  // Save original size for children
     unsigned long ulChildCount = 0;
     unsigned long ulExtStyle = extendedStyle();

     unsigned long
       ulCountPerDeck = 0,
       ulCountLeft = 0;

     ISetCanvasObjectList* pscvClObjList = new ISetCanvasObjectList;

     IWindow::ChildCursor childCursor(*this);
     IWindowHandle whChild;
     for (childCursor.setToFirst();
          childCursor.isValid();
          childCursor.setToNext())
     {                                 // Enumerate all child controls
        whChild = childAt(childCursor);
        IWindow* pwinChild = IWindow::windowWithHandle(whChild);
        if (pwinChild)
        {                              // Have child control object
           ISetCanvasObject* pscvChild =
                                     new ISetCanvasObject(pwinChild);

           if (areChildrenReversed() == false)
           {                         // enum is in reverse layout order
              pscvClObjList->addAsFirst(pscvChild);
           }
           else
           {                           // enum is in layout order
              pscvClObjList->addAsLast(pscvChild);
           }
           ulChildCount++;

           if (ulExtStyle & decksByGroup.asExtendedUnsignedLong())
           {            // Use group style to divide decks.
              if (pwinChild->isGroup())
              {         // This starts a new deck.
                 pscvChild->setFirstInDeck();
                 ulChildCount = 1;
              }
              if (ulChildCount > ulCountPerDeck)
              {         // Track deck with most children.
                 ulCountPerDeck = ulChildCount;
              }
           }            // End decksByGroup.
        }               // Else ignore non-wrappered child.
     }                  // End for all child windows.

     if (!(ulExtStyle & decksByGroup.asExtendedUnsignedLong()))
     {                  // Divide decks evenly by child count.
        ulCountPerDeck = ulChildCount / ulClNumDecks;
        ulCountLeft = ulChildCount % ulClNumDecks;
     }

     ITRACE_DEVELOP("End getting all child controls");

     /**************************************************************/
     /* Start with an initial position and size for each.          */
     /**************************************************************/
     ITRACE_DEVELOP("Start layout pass 1");

     long lXMargin = sizClMargin.width();   // White space from edge
     long lYMargin = sizClMargin.height();  //   to outer controls
     long lYTextHeight = 0;            // Assume no group box text.

     if (*(char*)text())               // Text returned by canvas
     {                                 // Canvas is a group box control
        lXMargin++;                    // Be sure child controls are
        lYMargin += 2;                 //   placed inside the border
        lYTextHeight = characterSize().height() + 1;
        lYMargin += lYTextHeight;
     }
     long lXPad = sizClPad.width();    // White space between elements
     long lYPad = sizClPad.height();
     long lNextX = lXMargin;           // Offset to first child control
     long lNextY = lYMargin;

     ISize szMax(0, 0);       // Biggest child so far (for pack expanded)
     long lCanvasWidth = 0;            // Size of the canvas window
     long lCanvasHeight = 0;

     Boolean bRowLayout = (deckOrientation() == horizontal);
     Boolean bFirstInDeck = true;
     Boolean bFirstDeck = true;

     ISetCanvasObject* pscvFirstInDeck = 0;  // For pack tight/expanded
     unsigned long ulCountInDeck = 0;
     long lDeckWidth = 0;
     long lMaxDeckLength = 0;

     unsigned long ulCount = ulCountPerDeck;  // For pack even

     if (ulCountLeft)              // Need number of windows in a deck
     {                             // Adjust for overflow in some decks
        ulCount++;
     } /* endif */

     unsigned long *pulDeckCellLengths = new unsigned long [ulCount];
     unsigned long *pulDeckCellLength = pulDeckCellLengths;
     for (; ulCount; ulCount--)
     {                                 // Zero out the array of lengths
        *pulDeckCellLength++ = 0;
     }
     pulDeckCellLength = pulDeckCellLengths;  // Reset to start

     long lNextLPos = lNextX;          // Initialize direction-independent
     long lNextWPos = lNextY;          //   variables assuming
     long lLMargin  = lXMargin;        //   horizontal decks
     long lLPad     = lXPad;
     long lWPad     = lYPad;

     if (!bRowLayout)
     {                                 // Reset direction-independent
        lNextLPos = lNextY;            //   variables for vertical decks
        lNextWPos = lNextX;
        lLMargin  = lYMargin;
        lLPad     = lYPad;
        lWPad     = lXPad;
     }

     /**************************************************************/
     /* Divide child windows across decks, storing the following   */
     /* information in the ISetCanvasObjectList:                   */
     /*  - Store the minimum size of each window.                  */
     /*  - Mark each child window that starts a deck.              */
     /*  - Store the position of each window based on whether it   */
     /*    starts a new deck or is part of the current deck.       */
     /*  - Update the size of the current deck, and the position   */
     /*    for the next child window in the current deck.          */
     /**************************************************************/
     ISetCanvasObjectList::Cursor cursor(*pscvClObjList);
     for (cursor.setToFirst(); cursor.isValid(); cursor.setToNext())
     {                                 // Process all contents

        ISetCanvasObject* pscvObj = pscvClObjList->elementAt(cursor);
        ISize szObj = pscvObj->window()->minimumSize();
                                       // Get screen size for each control
        pscvObj->sizeTo(szObj);
        long lHeight = szObj.height(); // Size of the
        long lWidth = szObj.width();   //   child control
        long lLength = lHeight;        // Assume vertical decks

        if (bRowLayout)
        {                              // Horizontal decks
           lLength = lWidth;           // Initialize direction-independent
           lWidth  = lHeight;          //   variables
        } /* endif */

        /***********************************************************/
        /* Position control, adding appropriate white space.       */
        /***********************************************************/
        if (bFirstInDeck  ||  pscvObj->isFirstInDeck())
        {                              // Start of new deck
           pscvObj->setFirstInDeck();
           pscvFirstInDeck = pscvObj;
           lNextWPos += lDeckWidth;    // Move past last row
           lDeckWidth = 0;
           pulDeckCellLength = pulDeckCellLengths;  // New deck
           ulCountInDeck = 1;
           lNextLPos = lLMargin;
           if (bFirstDeck)
           {                           // First deck
              bFirstDeck = false;      // Unset it for next time
           }
           else
           {                           // Not first deck
              lNextWPos += lWPad;      // Need pad before this deck
           }
           bFirstInDeck = false;       // Reset for next time.
        }
        else
        {                              // Follows another child window
           ulCountInDeck++;
           lNextLPos += lLPad;
           if (pscvObj->window()->isGroup())
           {            // Add in group pad.
              lNextLPos += groupPad();
           }
        }

        IPoint pt;
        if (bRowLayout)
        {                              // Horizontal decks
           pt.setX(lNextLPos).setY(lNextWPos);
        }
        else
        {                              // Vertical decks
           pt.setX(lNextWPos).setY(lNextLPos);
        }
        pscvObj->moveTo(pt);

        lNextLPos += lLength;          // Position for next child window
        if(lLength > *pulDeckCellLength)
        {                              // Length of the window
           *pulDeckCellLength = lLength;
        }

        if(lWidth > lDeckWidth)
        {                              // Track thickness of the deck
           lDeckWidth = lWidth;
           pscvFirstInDeck->setDeckWidth(lDeckWidth);
        }

        if (lNextLPos > lMaxDeckLength)
        {                              // Track longest deck
           lMaxDeckLength = lNextLPos;
        }
        pulDeckCellLength++;   // For the next window in the deck.

        if (!(ulExtStyle & decksByGroup.asExtendedUnsignedLong()))
        {               // Divide decks evenly by child count.
           if ((ulCountInDeck == ulCountPerDeck  &&
                ulCountLeft == 0)  ||
               ulCountInDeck > ulCountPerDeck)
           {            // Last child window in the deck.
              bFirstInDeck = true;   // Set flag for new row.
           }
           else if (ulCountInDeck == ulCountPerDeck)
           {            // Extra control for the deck.
              ulCountLeft--;
           }
        }

        szMax = szMax.maximum(szObj);  // New biggest control
     }                                 // End for all child controls
     lNextWPos += lDeckWidth;          // Move past last row.

     ITRACE_DEVELOP("End layout pass 1");

     /**************************************************************/
     /* Process specified pack and alignment styles now:           */
     /* - pack tight                                               */
     /*   Child windows are already packed tight, so the only      */
     /*   adjustment needed is aligning within the deck            */
     /*   (vertically in a horizontal deck, horizontally in a      */
     /*   vertical deck).                                          */
     /* - pack even                                                */
     /*   Child windows are aligned within the deck, as done for   */
     /*   pack tight, as well as horizontally in a horizontal deck */
     /*   and vertically in a vertical deck since each cell in a   */
     /*   deck may be expanded.                                    */
     /* - pack expanded                                            */
     /*   Child windows are resized to the largest child window,   */
     /*   and repositioned accordingly.  No alignment can be done  */
     /*   since everything is of "maximum" size.                   */
     /**************************************************************/
     if (bRowLayout)
     {                                 // Horizontal decks
        lNextX = lNextLPos;            // Map direction-independent
        lNextY = lNextWPos;            //   variables back out
     }
     else
     {                                 // Vertical decks
        lNextX = lNextWPos;            // Map direction-independent
        lNextY = lNextLPos;            //   variables back out
     }

     if (packType() == tight  ||  packType() == even)
     {
        ITRACE_DEVELOP("Start layout pass 2 -- packType()==tight/even");

        if (bRowLayout  &&
            (packType() == even  ||
             (packType() == tight  &&
              !(ulExtStyle & topAlign.asExtendedUnsignedLong()))))
        {                       // Need to reposition in a horizontal deck
           unsigned long ulDeckHeight = 0;
           unsigned long ulWidth = 0;
           unsigned long ulYDivisor = 0;  // For top align

           if (ulExtStyle & bottomAlign.asExtendedUnsignedLong())
           {                           // Need to adjust fully down
              ulYDivisor = 1;
           }
           else if (ulExtStyle & centerVerticalAlign.asExtendedUnsignedLong())
           {                           // Need to adjust halfway
              ulYDivisor = 2;
           }

           unsigned long ulXDivisor = 0;  // For leftAlign
           if (ulExtStyle & rightAlign.asExtendedUnsignedLong())
           {                           // Need to adjust fully over
              ulXDivisor = 1;
           }
           else if (ulExtStyle & centerAlign.asExtendedUnsignedLong())
           {                           // Need to adjust halfway
              ulXDivisor = 2;
           }

           for (cursor.setToFirst(); cursor.isValid(); cursor.setToNext())
           {                           // Process all contents
              ISetCanvasObject* pscvObj =
                                    pscvClObjList->elementAt(cursor);

              if (pscvObj->isFirstInDeck())
              {                        // Deck height stored here
                 ulDeckHeight = pscvObj->deckWidth();
                 lNextX = lXMargin;    // For pack even only
                 pulDeckCellLength = pulDeckCellLengths;
              }
              else
              {                        // Preceding entry in deck
                 lNextX += lXPad;      //   for pack even only
              }

              if (ulYDivisor)
              {                   // Vertically reposition child window
                 pscvObj->moveTopTo(pscvObj->topLeft().y() +
                                    (ulDeckHeight -
                                     pscvObj->size().height())
                                                  / ulYDivisor);
              }

              if (packType() == even)
              {                   // Horizontally reposition child window
                 ulWidth = *pulDeckCellLength++;
                 unsigned long ulDelta =
                     (ulXDivisor ? ((ulWidth -
                                      pscvObj->size().width())
                                                  / ulXDivisor) : 0);
                 pscvObj->moveLeftTo(lNextX + ulDelta);
                 lNextX += ulWidth;    // For next child window

                 if (lNextX > lMaxDeckLength)
                 {                     // Track longest row
                    lMaxDeckLength = lNextX;
                 }
              }
           }                           // End all contents
        }                              // End vertical aligning
        else if (!bRowLayout  &&
                 (packType() == even  ||
                  (packType() == tight  &&
                   !(ulExtStyle & leftAlign.asExtendedUnsignedLong()))))
        {                             // Need to reposition in a vert deck
           unsigned long ulDeckWidth = 0;
           unsigned long ulHeight = 0;
           unsigned long ulXDivisor = 0;  // For leftAlign

           if (ulExtStyle & rightAlign.asExtendedUnsignedLong())
           {                           // Need to adjust fully over
              ulXDivisor = 1;
           }
           else if (ulExtStyle & centerAlign.asExtendedUnsignedLong())
           {                           // Need to adjust halfway
              ulXDivisor = 2;
           }
           unsigned long ulYDivisor = 0;  // For top align
           if (ulExtStyle & bottomAlign.asExtendedUnsignedLong())
           {                           // Need to adjust fully down
              ulYDivisor = 1;
           }
           else if (ulExtStyle & centerVerticalAlign.asExtendedUnsignedLong())
           {                           // Need to adjust halfway
              ulYDivisor = 2;
           }

           for (cursor.setToFirst(); cursor.isValid(); cursor.setToNext())
           {                           // Process all contents
              ISetCanvasObject* pscvObj =
                                  pscvClObjList->elementAt(cursor);
              if (pscvObj->isFirstInDeck())
              {                        // Deck width stored here
                 ulDeckWidth = pscvObj->deckWidth();
                 lNextY = lYMargin;    // For pack even only
                 pulDeckCellLength = pulDeckCellLengths;
              }
              else
              {                        // Preceding entry in deck
                 lNextY += lYPad;      // For pack even only
              }

              if (ulXDivisor)
              {                    // Horizontally reposition child window
                 pscvObj->moveLeftTo(pscvObj->topLeft().x() +
                                     (ulDeckWidth -
                                      pscvObj->size().width())
                                                  / ulXDivisor);
              }

              if (packType() == even)
              {                      // Vertically reposition child window
                 ulHeight = *pulDeckCellLength++;
                 unsigned long ulDelta =
                    (ulYDivisor ? ((ulHeight -
                                    pscvObj->size().height())
                                                  / ulYDivisor) : 0);
                 pscvObj->moveTopTo(lNextY + ulDelta);
                 lNextY += ulHeight;   // For next child window

                 if (lNextY > lMaxDeckLength)
                 {                     // Track longest column
                    lMaxDeckLength = lNextY;
                 }
              }
           }                           // End all contents
        }                              // End horizontal aligning

        ITRACE_DEVELOP("End layout pass 2");
     }                                 // End pack tight/even
     else if (packType() == expanded)
     {
        ITRACE_DEVELOP("Start layout pass 2 -- packType()==expanded");
        long lWidth = szMax.width();   // Use largest for size
        long lHeight = szMax.height(); //   of all child controls
        bFirstDeck = true;

        if (bRowLayout)
        {                              // Lay out everything in rows
           for (cursor.setToFirst(); cursor.isValid(); cursor.setToNext())
           {                           // Process all contents
              ISetCanvasObject* pscvObj =
                                  pscvClObjList->elementAt(cursor);
              pscvObj->sizeTo(szMax);  // New bigger size

              /*****************************************************/
              /* Position control, adding appropriate white space. */
              /*****************************************************/
              if (pscvObj->isFirstInDeck())
              {                        // First in the row
                 lNextX = lXMargin;
                 if (bFirstDeck)
                 {                     // First deck
                    lNextY = lYMargin;   // Offset to first row
                    bFirstDeck = false;  // Unset it for next time
                 }
                 else
                 {                     // Not first deck
                    lNextY += (lHeight + lYPad);  // Offset to row
                 }
              }
              else
              {
                 lNextX += lXPad;
              }

              pscvObj->moveTo(IPoint(lNextX, lNextY));
              lNextX += lWidth;        // For next control

              if(lNextX > lMaxDeckLength)
              {                        // Track longest row
                 lMaxDeckLength = lNextX;
              }
           }                           // End all contents
           lNextY += lHeight;          // Bottom of last row
        }                              // End horizontal decks
        else
        {                              // Lay out everything in columns
           for (cursor.setToFirst(); cursor.isValid(); cursor.setToNext())
           {                           // Process all contents
              ISetCanvasObject* pscvObj =
                                  pscvClObjList->elementAt(cursor);
              pscvObj->sizeTo(szMax);  // New bigger size

              /*****************************************************/
              /* Position control, adding appropriate white space. */
              /*****************************************************/
              if (pscvObj->isFirstInDeck())
              {                        // First in the column
                 lNextY = lYMargin;
                 if (bFirstDeck)
                 {                     // First deck
                    lNextX = lXMargin;   // Offset to first column
                    bFirstDeck = false;  // Unset it for next time
                 }
                 else
                 {                     // Not first deck
                    lNextX += (lWidth + lXPad);  // Offset to column
                 }
              }
              else
              {
                 lNextY += lYPad;
              }

              pscvObj->moveTo(IPoint(lNextX, lNextY));
              lNextY += lHeight;       // For next control

              if(lNextY > lMaxDeckLength)
              {                        // Track longest column
                 lMaxDeckLength = lNextY;
              }
           }                           // End all contents
           lNextX += lWidth;           // End of last column
        }                              // End vertical decks

        ITRACE_DEVELOP("End layout pass 2");
     }                                 // End pack expanded

     /**************************************************************/
     /* Have size of the canvas window.                            */
     /**************************************************************/
     if (bRowLayout)
     {                                 // Lay out everything in rows
        lNextX = lMaxDeckLength;
     }
     else
     {                                 // Lay out everything in columns
        lNextY = lMaxDeckLength;
     }

     lCanvasWidth = lNextX + lXMargin;
     lCanvasHeight = lNextY + lYMargin - lYTextHeight;
                        // (Don't add text height twice.)

     /**************************************************************/
     /* If this is a group box, need to adjust size to include the */
     /* width of the text and grow the height to not overwrite any */
     /* child controls with the text.  Also expand to include the  */
     /* group box text.                                            */
     /**************************************************************/
     setLayoutSize(ISize(lCanvasWidth, lCanvasHeight));
     expandForText();   // Grow sizClLayout if necessary.
     lCanvasWidth = layoutSize().width();
     ISize newLayoutSize(lCanvasWidth, lCanvasHeight);

     // Keep child windows at top of canvas.
     lCanvasHeight = size().height();

     ITRACE_DEVELOP(IString("Canvas size = (") +
                    IString(lCanvasWidth) + ", " +
                    IString(lCanvasHeight) + ")");

#ifdef IC_PM
     // Position child windows based on the current height of the
     // canvas, unless it has no height.  In this case, position
     // them based on the calculated minimum height to optimize the
     // case when/if the canvas is next sized to its minimum height.
     if ( lCanvasHeight == 0 )
     {
        lCanvasHeight = newLayoutSize.height();
     }
     fSetCanvasData->positioningHeight = lCanvasHeight;

#endif
     /**************************************************************/
     /* Size and position each child control, and clean up the     */
     /* sequential set.                                            */
     /**************************************************************/
     ITRACE_DEVELOP("Start layout pass 3 -- move/size controls");

     enableUpdate(false);              // Surpress updates
     IWindowPosBuffer wposbuf = fixupChildren();

     for (cursor.setToFirst(); cursor.isValid(); cursor.setToNext())
     {                                 // Process all contents
        ISetCanvasObject* pscvObj = pscvClObjList->elementAt(cursor);

#ifdef IC_ORIGINLOWERLEFT
        // Convert offsets from top-relative to native bottom-relative.
        pscvObj->moveTopTo(lCanvasHeight -
                           pscvObj->topLeft().y() -
                           pscvObj->size().height());
                                       // Adjust coordinate system
#endif

        // Adjust the control if necessary (primarily drop-down combos
        IRectangle rectAdjust = pscvObj->window()->layoutAdjustment();

        pscvObj->sizeTo(pscvObj->size()+rectAdjust.size());
#ifdef IC_ORIGINLOWERLEFT
        // With bottom-relative offsets, growing a child window
        // means moving its bottom edge down.
        pscvObj->moveTo(pscvObj->rect().bottomLeft()+rectAdjust.bottomLeft());
#endif

        wposbuf.moveSizeTo((IWindow*)(pscvObj->window()),
                           pscvObj->rect());  // Cast away const
     }                                 // End all contents

     pscvClObjList->removeAll(&ISetCanvasDeleter);
     delete pscvClObjList;
     wposbuf.apply();                  // Reposition/size child controls

     ITRACE_DEVELOP("End layout pass 3");

     /**************************************************************/
     /* New size after adjusting for group box.                    */
     /**************************************************************/
     ITRACE_DEVELOP(IString("Canvas size = (") +
                    IString(layoutSize().width()) + ", " +
                    IString(layoutSize().height()) + ")");

     setLayoutDistorted(0, IWindow::layoutChanged);

     /**************************************************************/
     /* The below follows setLayoutDistorted(), since calling      */
     /* WinEnableWindowUpdate may cause another WM_PAINT message   */
     /* and we don't want an infinite loop problem.                */
     /**************************************************************/
     enableUpdate(true);               // Show updates
     setLayoutSize(newLayoutSize);     // Reset after fixupChildren.
     if (sizLayout != newLayoutSize)   // New minimum size for children.
     {                                 // Cause layout in parent canvas
        setLayoutDistorted(IWindow::minimumSizeChanged, 0);
     }
     delete (pulDeckCellLengths);

#ifdef IC_PMWIN
     fSetCanvasData->layoutInProgress--;
#endif
  }   /* end isDistorted */

  return *this;
}


/*------------------------------------------------------------------------------
| ISetCanvasResizeHandler::windowResize                                        |
|                                                                              |
------------------------------------------------------------------------------*/
IBase::Boolean ISetCanvasResizeHandler::windowResize ( IResizeEvent& event )
{
  ISetCanvas* window = (ISetCanvas*)event.dispatchingWindow();
#ifdef IC_MOTIFPM
  long newHeight = event.newSize().height();
#ifdef IC_MOTIF
  if ( newHeight  &&
       newHeight != event.oldSize().height() )
#endif
#ifdef IC_PM
  if ( newHeight  &&
       newHeight != event.oldSize().height()  &&
       newHeight != window->fSetCanvasData->positioningHeight )
#endif
  {  // Force layout to run, since the canvas must update the y positions
     // of its child windows (the child windows not already positioned for
     // the new height).
#ifdef IC_MOTIF
     // Post a canvas paint message to run layout after the
     // new size is in effect.
#endif
     window->setLayoutDistorted( IWindow::layoutChanged, 0 );
     window->postEvent( IC_UM_CANVAS_PAINT );
#ifdef IC_PM
     // (Can't use the immediateUpdate flag or IWindow::refresh, since for
     // some reason these don't cause a toolbar container to get a WM_PAINT
     // message it needs.)
#endif
  }
  else
#endif
#ifdef IC_PMWIN
  if ( window  &&
       IBidiSettings::isBidiSupported()  &&
       event.newSize().width() != event.oldSize().width() )
  {
     IBidiSettings settings( *window );
     if ( settings.windowLayout() == IBidiSettings::layoutRightToLeft )
     {
        window->setLayoutDistorted( IWindow::layoutChanged, 0 );
        window->postEvent( IC_UM_CANVAS_PAINT );
     }
  }
#endif
  return false;
}

/*------------------------------------------------------------------------------
| ISetCanvas::setDefaultStyle                                                  |
|                                                                              |
| Sets the default style                                                       |
|                                                                              |
| Note: This function is inlined on all platforms except Windows due to        |
|       variables it references not being exported.                            |
|------------------------------------------------------------------------------*/
#ifdef IC_WIN
void  ISetCanvas::setDefaultStyle ( const ISetCanvas::Style& style )
  {
  currentDefaultStyle = style;
  }
#endif //IC_WIN

#ifdef IC_MOTIF
/*------------------------------------------------------------------------------
| ISetCanvasShowHandler::dispatchHandlerEvent                                  |
|                                                                              |
| This is motif specific because we don't get the first WM_RESIZE message.     |
| In motif we mapped the configureNotify event to the WM_RESIZE when we detect |
| the size has changed but the configureNotify event is not generated the      |
| first time a window is created.                                              |
------------------------------------------------------------------------------*/
IBase::Boolean ISetCanvasShowHandler::dispatchHandlerEvent( IEvent& event )

  {
  if ( event.eventType() == IEvent::Xevent )
    {
    if ( event.eventId() == VisibilityNotify &&
         event.parameter1().asUnsignedLong() == true )
      {
      IWindow* window = event.controlWindow();
      if (window )
        {                // Canvas needs to update y positions of child windows.
        window->setLayoutDistorted(IWindow::layoutChanged |
                                   IWindow::immediateUpdate, 0);
        }
      }
    }
  return false;
  }

/*------------------------------------------------------------------------------
| ISetCanvas::moveSizeTo                                                       |
|                                                                              |
| Special case is when a frame is specified. The position of the frame and the |
| set canvas must be calculated and then the widget placed in the correct      |
| position. The tricky part to all this is that the frame does not have any    |
| children to size itself around. The frame's and bulletin board both have the |
| same parent.                                                                 |
------------------------------------------------------------------------------*/
ISetCanvas& ISetCanvas::moveSizeTo( const IRectangle& rect)

  {
  if ( fSetCanvasData->needsFrame )
    {
    Inherited::moveSizeTo( rect );
    if ( rect.width() != 0 && rect.height() != 0 )
      {
      Dimension width, height, mWidth, mHeight, shadow, border;
      XtVaGetValues( fSetCanvasData->frame,
        XmNmarginWidth, &mWidth,
        XmNmarginHeight, &mHeight,
        XmNshadowThickness, &shadow,
        NULL );
      Dimension tHeight;
      XtVaGetValues( fSetCanvasData->title,
        XmNheight, &tHeight,
        NULL );
      IRectangle adjustedRect = rect.sizedTo(
         ISize( rect.width() - (2 * mWidth) - (2 * shadow ),
                rect.height() - (2 * mHeight) - (2 * shadow ) - (tHeight)) );
      XtVaSetValues( handle(),
        XmNx, mWidth + shadow,
        XmNy, tHeight + mHeight + shadow,
        XmNwidth, adjustedRect.width(),
        XmNheight, adjustedRect.height(),
        NULL );
      // need to recoordinate the children after handle has been sized.
      Inherited::moveSizeTo( rect );
      }
    }
  else
    {
    Inherited::moveSizeTo( rect        );
    }
  return *this;
  }

/*------------------------------------------------------------------------------
| ISetCanvas::size                                                             |
|                                                                              |
| Calculates the size.                                                         |
------------------------------------------------------------------------------*/
ISize ISetCanvas::size( ) const

  {
  return Inherited::size();
  }

/*------------------------------------------------------------------------------
| ISetCanvas::position                                                         |
|                                                                              |
| Calculates the position.                                                     |
------------------------------------------------------------------------------*/
IPoint ISetCanvas::position( ) const

  {
  return Inherited::position();
  }

/*------------------------------------------------------------------------------
| ISetCanvas::topHandle                                                        |
|                                                                              |
| Return the top handle for the window.                                        |
------------------------------------------------------------------------------*/
IWindowHandle ISetCanvas::topHandle() const
{
  if (fSetCanvasData->needsFrame)
     return fSetCanvasData->frame;
  else
     return handle();
}
#endif
