/*******************************************************************************
* FILE NAME: isizeevt.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in isizeevt.hpp.  This encapsulates a WM_WINDOWPOSCHANGED                  *
*    message.  It provides member functions for querying the previous and      *
*    new size of the window.                                                   *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
  #include <iwindefs.h>           // for PSWP
}

#include <isizeevt.hpp>
#include <ievtdata.hpp>
#include <ipoint.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISIZEEVT_CPP_
  #include <ipagetun.h>
#endif


/*******************************************************************/
/* Constructor here for page tuning.                               */
/*******************************************************************/
IResizeEvent :: IResizeEvent ( const IEvent & event )
  : IEvent ( event )
{ ; }


/*******************************************************************/
/* Empty destructor here for page tuning.                          */
/*******************************************************************/
IResizeEvent :: ~IResizeEvent ( )
{ ; }


/*******************************************************************/
/* oldSize - Return the previous size of the window.               */
/*******************************************************************/
ISize  IResizeEvent :: oldSize ( ) const
{
#ifdef IC_WIN
   // Windows does not provide the old size on WM_WINDOWPOSCHANGED.
   // We save the old size in IWindow private data and use it on a user message
   // which is sent and processed by IResizeHandler.  User message is
   // as follows
   //    msgid    IC_UM_WINDOWPOSCHANGED
   //    wparam   (unused)
   //    lparam   address of array of 2 SIZE structures with new size
   //             and old size in that order
   SIZE*  size = (SIZE*) (char*)parameter2();
   size++;
   return ISize(size->cx, size->cy);    // old size
#else
   PSWP pswp = (PSWP)(char*)parameter1();
                                  // array of two SWP structures
   pswp++;                        // old window is second SWP struct
   return ISize(pswp->cx, pswp->cy);    // old size
#endif
}


/*******************************************************************/
/* newSize - Return the new size of the window.                    */
/*******************************************************************/
ISize  IResizeEvent :: newSize ( ) const
{

#ifdef IC_WIN
   // Windows does not provide the old size on WM_WINDOWPOSCHANGED.
   // We save the old size in IWindow private data and use it on a user message
   // which is sent and processed by IResizeHandler.  User message is
   // as follows
   //    msgid    IC_UM_WINDOWPOSCHANGED
   //    wparam   (unused)
   //    lparam   address of array of 2 SIZE structures with new size
   //             and old size in that order
   SIZE*  size = (SIZE*) (char*)parameter2();
   return ISize(size->cx, size->cy);    // new size
#else
   PSWP pswp = (PSWP)(char*)parameter1();
                                  // array of two SWP structures
   return ISize(pswp->cx, pswp->cy);    // new size
#endif
}
