/*******************************************************************************
* FILE NAME: islider1.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the initialization of static strings used in the        *
*   IProgressIndicator class.                                                  *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

#define INCL_WINSTDSLIDER

extern "C" {
   #include <iwindefs.h>
}
#include <inotify.hpp>
#include <islider.hpp>
#include <islidenh.hpp>
#include <inotifev.hpp>
#include <ictlevt.hpp>
#if (!defined(IC_MOTIF) && !defined(_IWCNAME_))
  #include <iwcname.hpp>
#endif
#ifdef IC_WIN
#include <iclsldw.h>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISLIDER1_CPP_
  #include <ipagetun.h>
#endif

// Class to clean up static objects
class IProgressIndicatorNotifyStatics
{
public:

  ~IProgressIndicatorNotifyStatics ()
  {
    delete progressIndicatorNotifyHandler;
    progressIndicatorNotifyHandler = 0;
  }

static IProgressIndicatorNotifyHandler
  &notificationHandler ( );

static IProgressIndicatorNotifyHandler
  *progressIndicatorNotifyHandler;
};

// Initialize static objects and notification strings.
#pragma data_seg(ICLStaticConst)
INotificationId const IProgressIndicator::scaleId          =
                                    "IProgressIndicator::scale";
INotificationId const IProgressIndicator::armChangeId      =
                                    "IProgressIndicator::armChange";
INotificationId const ISlider::armTrackId = "ISlider::armTrack";
#pragma data_seg()

#pragma data_seg(ICLNonConst)
IProgressIndicatorNotifyHandler*
  IProgressIndicatorNotifyStatics::progressIndicatorNotifyHandler = 0;
#pragma data_seg()

/*------------------------------------------------------------------------------
| IProgressIndicatorNotifyStatics::notificationHandler                         |
|                                                                              |
| Static function to create and return a notification handler for              |
| IProgressIndicator.                                                          |
------------------------------------------------------------------------------*/
IProgressIndicatorNotifyHandler& IProgressIndicatorNotifyStatics::
                                                       notificationHandler()
{
  if (!(IProgressIndicatorNotifyStatics::progressIndicatorNotifyHandler))
  {
    IProgressIndicatorNotifyStatics::progressIndicatorNotifyHandler =
                                   new IProgressIndicatorNotifyHandler();
  }
  return( *IProgressIndicatorNotifyStatics::progressIndicatorNotifyHandler );
}

/*------------------------------------------------------------------------------
| IProgressIndicator::enableNotification                                       |
|                                                                              |
| Flags this part as being ready to accept notifications.  A Notifier, like    |
| IProgressIndicator, does not send notifications to observer objects until it |
| is enabled.                                                                  |
|                                                                              |
| In the window hierarchy we override this function to allocate the correct    |
| notification handler.                                                        |
------------------------------------------------------------------------------*/
IProgressIndicator& IProgressIndicator::enableNotification(Boolean enabled)
{
  if (!notificationHandler())
  {
    setNotificationHandler(
               &(IProgressIndicatorNotifyStatics::notificationHandler()));
  }
  IWindow::enableNotification(enabled);
  return *this;
}

/*------------------------------------------------------------------------------
| IProgressIndicatorNotifyHandler::IProgressIndicatorNotifyHandler             |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
IProgressIndicatorNotifyHandler::IProgressIndicatorNotifyHandler()
{}

/*------------------------------------------------------------------------------
| IProgressIndicatorNotifyHandler::~IProgressIndicatorNotifyHandler            |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IProgressIndicatorNotifyHandler::~IProgressIndicatorNotifyHandler()
{}

/*------------------------------------------------------------------------------
| IProgressIndicatorNotifyHandler::dispatchHandlerEvent                        |
|                                                                              |
| This notifies IProgressIndicator Windows of the following events:            |
|                                                                              |
| armChange(Id)                                                                |
| armTrack(Id)                                                                 |
------------------------------------------------------------------------------*/
#ifdef IC_PMWIN
Boolean IProgressIndicatorNotifyHandler::dispatchHandlerEvent(IEvent& anEvent)
{
  switch (anEvent.eventId())
  {
    case WM_CONTROL:
    {
      IControlEvent ctlevt(anEvent);
      IWindow* pwndControl = ctlevt.controlWindow();
      unsigned long ulMsg  = ctlevt.parameter1().highNumber();
      IWindow* thisWindow;
      IWindowHandle hwndControl;

      if (pwndControl)
      {
        hwndControl = pwndControl->handle();
        thisWindow = pwndControl;
      }
      else
      {
        hwndControl = IWindow::handleWithParent(ctlevt.controlId(),
                                                ctlevt.handle());
        thisWindow = IWindow::windowWithOwner(ctlevt.controlId(),
                                              ctlevt.window());
      }

      if (hwndControl && thisWindow)
      {
        IWindowClassName className(hwndControl);
#ifdef IC_WIN
        if (className == WC_SLIDER ||
            className == WC_NATIVESLIDER)
#else
        if (className == WC_SLIDER)
#endif
        {
          if (ulMsg == SLN_CHANGE)
          {
            thisWindow->notifyObservers(INotificationEvent(
                                        IProgressIndicator::armChangeId,
                                        *thisWindow, true,
                                        (void *)anEvent.parameter2()));
          }
          else if (ulMsg == SLN_SLIDERTRACK)
          {
            thisWindow->notifyObservers(INotificationEvent(
                                        ISlider::armTrackId,
                                        *thisWindow, true,
                                        (void *)anEvent.parameter2()));
          }
        }
      }
    break;
    } /* end WM_CONTROL */

    default:
      break;
  }  /* end switch */

  return (Inherited::dispatchHandlerEvent(anEvent));
}

#endif // IC_PMWIN

