/*******************************************************************************
* FILE NAME: isplitch.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in isplitch.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define INCL_WINFRAMEMGR
#define INCL_WININPUT
#define INCL_WINMESSAGEMGR
#define INCL_WINPOINTERS
#define INCL_WINSYS
#define INCL_WINTRACKRECT
#define INCL_WINWINDOWMGR
extern "C" {
  #include <iwindefs.h>
}

#include <isplitch.hpp>

#include <icconst.h>
#include <icmnfun.hpp>
#include <icolor.hpp>
#include <icoordsy.hpp>
#include <ievent.hpp>
#include <iexcept.hpp>
#include <igbundle.hpp>
#include <igrafctx.hpp>
#include <igrect.hpp>
#include <ipoint.hpp>
#include <irect.hpp>
#include <isizeevt.hpp>
#include <isplitcv.hpp>
#include <itrace.hpp>


// Segment definitions.
#ifdef IC_PAGETUNE
  #define _ISPLITCH_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| ISplitCanvasHandler::ISplitCanvasHandler                                     |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
ISplitCanvasHandler::ISplitCanvasHandler ( )
  : ISplitCanvasHandler::Inherited ( )
{ }

/*------------------------------------------------------------------------------
| ISplitCanvasHandler::~ISplitCanvasHandler                                    |
|                                                                              |
| Empy destructor here for page tuning.                                        |
------------------------------------------------------------------------------*/
ISplitCanvasHandler::~ISplitCanvasHandler ( )
{ }

/*------------------------------------------------------------------------------
| ISplitCanvasHandler::dispatchHandlerEvent                                    |
|                                                                              |
| Processes events as appropriate for a split canvas.                          |
------------------------------------------------------------------------------*/
Boolean ISplitCanvasHandler::dispatchHandlerEvent ( IEvent& evt )
{
  switch ( evt.eventId() )
  {
     case WM_PAINT:
     /***************************************************************/
     /* Paint any invalidated split bars (let the default canvas    */
     /* handler paint the background of the canvas if it has no     */
     /* child windows).                                             */
     /***************************************************************/
     {
        HWND hwnd = evt.handle();
        ISplitCanvas* splitcv = (ISplitCanvas*)evt.window();
        IWindow::ChildCursor cwCursor( *splitcv );
        if ( cwCursor.setToFirst() )
        {          // Have at least one child window.

           IPresSpaceHandle pshCanvas;
           unsigned long ulExtStyle = splitcv->extendedStyle();

           splitcv->layout();
#ifdef IC_PM
           RECTL rect;
           pshCanvas = IBEGINPAINT( hwnd, 0L, &rect );
           IRectangle paintRect(rect);
#endif
#ifndef IC_PM
           PAINTSTRUCT rect;
           pshCanvas = IBEGINPAINT( hwnd, 0L, &rect );
           IRectangle paintRect( rect.rcPaint.left,
                                 rect.rcPaint.top,
                                 rect.rcPaint.right,
                                 rect.rcPaint.bottom);
#endif
           if ( ! (ulExtStyle &
                     ISplitCanvas::noSplitBars.asExtendedUnsignedLong()) )
           {
              IWindowHandle whChild;
              IWindow* pwinChild     = 0;
              IWindow* pwinChildPrev = 0;
              ISize sizSplitCV = splitcv->size();

              Boolean rightToLeft = false;
#ifdef IC_PMWIN
              if ( IBidiSettings::isBidiSupported() )
              {
                IBidiSettings bidi(*splitcv);
                if ( bidi.windowLayout() == IBidiSettings::layoutRightToLeft )
                  rightToLeft = true;
              }
#endif
              for ( ; cwCursor.isValid(); cwCursor.setToNext() )
              {                   // Enumerate all child windows.
                 whChild = splitcv->childAt( cwCursor );
                 pwinChild = IWindow::windowWithHandle( whChild );
                 if ( pwinChild )
                 {                // Have child window object.
                    if ( pwinChildPrev )
                    {
                       IRectangle rclSplit;
                       if ( ulExtStyle &
                              ISplitCanvas::vertical.asExtendedUnsignedLong() )
                       {
                          IRectangle rclControl = pwinChildPrev->nativeRect();
                          if ( rightToLeft )
                            rclControl = pwinChild->nativeRect();
                          rclSplit =
                            IRectangle( rclControl.maxX(),
                                        0,
                                        rclControl.maxX() +
                                          (splitcv->ulClEdgeThickness * 2) +
                                          splitcv->ulClMiddleThickness,
                                        sizSplitCV.height() );
                       }
                       else
                       {
#ifdef  IC_PM   // originLowerLeft
                          IRectangle rclControl = pwinChild->nativeRect();
#endif
#ifndef IC_PM   // originUpperLeft
                          IRectangle rclControl = pwinChildPrev->nativeRect();
#endif
                          rclSplit =
                            IRectangle( 0,
                                        rclControl.maxY(),
                                        sizSplitCV.width(),
                                        rclControl.maxY() +
                                          (splitcv->ulClEdgeThickness * 2) +
                                          splitcv->ulClMiddleThickness );
                       }

                       if ( rclSplit.intersects( paintRect ) )
                       {
                          ITRACE_DEVELOP( IString( "Split Rect : " ) +
                                            rclSplit.asString() );

                          this->paintSplitBar( splitcv, pshCanvas, rclSplit );
                       }
                    }
                    pwinChildPrev = pwinChild;
                 }
              }
           }
           IENDPAINT( hwnd, pshCanvas, &rect );
           evt.setResult( (unsigned long)false );
           return true;      // Do not pass on to next handler.
        }               // End have at least one child window.
        return false;   // Let default canvas handler paint.
     } /* end WM_PAINT */

#ifdef IC_PM
     case WM_MOUSEMOVE:
#endif
#ifdef IC_WIN
     case WM_SETCURSOR:
#endif
     /***************************************************************/
     /* If the mouse pointer is over a split bar, set it to the     */
     /* sizeHorizontal/sizeVertical pointer.                        */
     /***************************************************************/
     {
        ISplitCanvas* splitcv = (ISplitCanvas*)evt.window();
        if ( splitcv->isEnabled() )
        {               // Canvas and its split bars are enabled.
#ifdef IC_WIN
           if ( splitcv->handle().asUnsigned() ==
                     evt.parameter1().asUnsignedLong() )
           {
#endif
           IWindow::ChildCursor cwCursor( *splitcv );
           if ( cwCursor.setToFirst() )
           {            // Has at least one child window.
              // Assume the mouse pointer is over a split bar.
              // Now determine the correct pointer to use.
#ifdef IC_PM
              ISystemPointerHandle
                splitPtr( ISystemPointerHandle::sizeHorizontal );

#endif
              if ( splitcv->extendedStyle() &
                     ISplitCanvas::horizontal.asExtendedUnsignedLong() )
              {         // Horizontal split bars.
#ifdef IC_PM
                 splitPtr =
                   ISystemPointerHandle( ISystemPointerHandle::sizeVertical );
#endif
#ifndef IC_PM
                 ISETPOINTER( ISystemPointerHandle(
                                 ISystemPointerHandle::sizeVertical ));
#endif
              }
#ifndef IC_PM
              else
              {         // Vertical split bars.
                 ISETPOINTER( ISystemPointerHandle(
                                 ISystemPointerHandle::sizeHorizontal ));
              }
#endif
#ifdef IC_PM

              // Give the owner or frame window the opportunity to change
              // the mouse pointer via the WM_CONTROLPOINTER message,
              // before setting it.
              IWindowHandle wHandle = splitcv->owner() ?
                                        splitcv->owner()->handle() :
                                        splitcv->handle();
              IEventResult newPtr =
                 wHandle.sendEvent( WM_CONTROLPOINTER,
                                    IEventParameter1( splitcv->id() ),
                                    IEventParameter2( splitPtr ));
              ISETPOINTER( newPtr );
#endif

              evt.setResult( true );
              return true;
           }            // End have at least one child window.
#ifdef IC_WIN
           }
#endif
        }               // End canvas is enabled.
        return false;   // No mouse pointer processing needed.
#ifdef IC_PM
     } /* end WM_MOUSEMOVE */
#endif
#ifdef IC_WIN
     } /* end WM_SETCURSOR */
#endif

     case WM_BUTTON1DOWN:
     case WM_BUTTON2DOWN:
     case WM_BUTTON3DOWN:
     /***************************************************************/
     /* If mouse button down on a split bar, start tracking the     */
     /* mouse, and resize the child windows on both sides of split  */
     /* bar according to where the mouse button was released.       */
     /***************************************************************/
     {
        HWND hwnd = evt.handle();
        ISplitCanvas* splitcv = (ISplitCanvas*)evt.window();
        POINTL ptScreen;
        IWindow* pwinChild     = 0;
        IWindow* pwinChildPrev = 0;
        IWindow* pwinSizeChild = 0;
        unsigned long ulSplitBarWidth = (2 * splitcv->ulClEdgeThickness) +
                                           splitcv->ulClMiddleThickness;
        unsigned long ulExtStyle = splitcv->extendedStyle();

        /**********************************************************/
        /* Compute mouse location within the split canvas window. */
        /**********************************************************/
        IQUERYMSGPOS( IQUERYANCHOR( hwnd ), &ptScreen );
        IMAPWINDOWPOINTS( IWindow::desktopWindow()->handle(),
                          hwnd, &ptScreen, 1 );
        IPoint ptMouse( ptScreen.x,ptScreen.y );
        ITRACE_ALL( IString("ptMouse=") + ptMouse.asString() );

        /*********************************************************/
        /* Used to flag if button down was over a split bar.     */
        /*********************************************************/
        Boolean bOverSplitBar = false;

#ifdef IC_PMWIN
        /*********************************************************/
        /* Check for BIDI right-to-left layout.                  */
        /*********************************************************/
        Boolean rightToLeft = false;
        if ( IBidiSettings::isBidiSupported() )
        {
          IBidiSettings bidi(*splitcv);
          if ( bidi.windowLayout() == IBidiSettings::layoutRightToLeft )
            rightToLeft = true;
        }
#endif

        IWindow::ChildCursor cwCursor( *splitcv );
        IWindowHandle whChild;
        IRectangle rclChild;
        IRectangle rclChildPrev;

        for ( cwCursor.setToFirst(); cwCursor.isValid(); cwCursor.setToNext() )
        {                         // Enumerate all child windows.
           whChild = splitcv->childAt( cwCursor );
           pwinChild = IWindow::windowWithHandle( whChild );
           if ( pwinChild )
           {
              /******************************************************/
              /* Vertical split child enumeration                   */
              /*                                                    */
              /*        ------||----||------                        */
              /*        | ctrl||ctrl||ctrl |                        */
              /*        |  1  || 2  || 3   |                        */
              /*        |     ||    ||     |                        */
              /*        ------||----||------                        */
              /* Child windows are enumerated from left to right,   */
              /* so ctrl 1 is pwinChildPrev and ctrl2 is pwinChild. */
              /******************************************************/

              /******************************************************/
              /* Horizontal split child enumeration                 */
              /*                                                    */
              /*        ---------------------------                 */
              /*        |  ctrl 1                 |                 */
              /*        |_________________________|                 */
              /*        |-------------------------|                 */
              /*        |  ctrl 2                 |                 */
              /*        |_________________________|                 */
              /*        |-------------------------|                 */
              /*        |  ctrl 3                 |                 */
              /*        ---------------------------                 */
              /* Child windows are enumerated from top to bottom,   */
              /* so ctrl 1 is pwinChildPrev and ctrl2 is pwinChild. */
              /******************************************************/

              /******************************************************/
              /* Must have two controls to have a split bar.        */
              /******************************************************/
              if ( pwinChildPrev )
              {
                 IRectangle rclSplitBar;
                 rclChild = pwinChild->nativeRect();
                 rclChildPrev = pwinChildPrev->nativeRect();

                 if ( ulExtStyle &
                        ISplitCanvas::vertical.asExtendedUnsignedLong() )
                 {
                    if ( !rightToLeft )
                    {
                      rclSplitBar = IRectangle( rclChildPrev.maxX(),
                                                rclChildPrev.minY(),
                                                rclChild.minX(),
                                                rclChildPrev.maxY() );
                    }
                    else
                    {
                      rclSplitBar = IRectangle( rclChild.maxX(),
                                                rclChild.minY(),
                                                rclChildPrev.minX(),
                                                rclChild.maxY()
                                                );
                    }
                 }
                 else
#ifdef  IC_PM   // originLowerLeft
                    rclSplitBar = IRectangle( rclChild.minX(),
                                              rclChild.maxY(),
                                              rclChild.maxX(),
                                              rclChildPrev.minY() );
#endif
#ifndef IC_PM   // originUpperLeft
                    rclSplitBar = IRectangle( rclChild.minX(),
                                              rclChildPrev.maxY(),
                                              rclChild.maxX(),
                                              rclChild.minY() );
#endif

                 ITRACE_ALL( IString("rclChild    =") + rclChild.asString());
                 ITRACE_ALL( IString("rclChildPrev=") + rclChildPrev.asString());
                 ITRACE_ALL( IString("rclSplitBar =") + rclSplitBar.asString());

                 if ( rclSplitBar.contains( ptMouse ))
                 {
                    bOverSplitBar = true;
                    break;        // Leave enumeration loop.
                 }
              }

              pwinChildPrev = pwinChild;
           }
        }

        if ( ! bOverSplitBar )  // Button down was not over a split bar.
           return false;

        /*********************************/
        /* Clear WS_CLIPCHILDREN bit.    */
        /*********************************/
        long lStyle = ISTYLEOF( hwnd );
        ISETWINDOWSTYLE( hwnd, (lStyle & ~WS_CLIPCHILDREN) );

        // Start tracking of mouse.
        TRACKINFO trk;
        /************************************************************/
        /* The split bar should track below the titlebar, and not   */
        /* go any further right than the right edge of the child    */
        /* window to the right of the split bar, and not go any     */
        /* further left than the left edge of the child window to   */
        /* left of the split bar.                                   */
        /************************************************************/
        if ( ulExtStyle & ISplitCanvas::vertical.asExtendedUnsignedLong() )
        {
           if (!rightToLeft)
           {
             trk.rclBoundary.MINX = rclChildPrev.minX();
             trk.rclBoundary.MINY = rclChild.minY();
             trk.rclBoundary.MAXX = rclChild.maxX();
             trk.rclBoundary.MAXY = rclChildPrev.maxY();
           }
           else
           {
             trk.rclBoundary.MINX = rclChild.minX();
             trk.rclBoundary.MINY = rclChildPrev.minY();
             trk.rclBoundary.MAXX = rclChildPrev.maxX();
             trk.rclBoundary.MAXY = rclChild.maxY();
           }

           /********************************************************/
           /* This specifies the width of the split bar, so        */
           /* WinTrackRect knows how wide of a thing to track. The */
           /* split bar is the width of a size border.             */
           /********************************************************/
           trk.cxBorder = (int)ulSplitBarWidth;

           /********************************************************/
           /* This specifies the height of the split bar, so       */
           /* WinTrackRect knows how tall of a thing to track.     */
           /********************************************************/
           trk.cyBorder = (int)trk.rclBoundary.MAXY;

           /********************************************************/
           /* This specifies the tracking rectangle used by        */
           /* WinTrackRect.                                        */
           /********************************************************/
           if (!rightToLeft)
           {
             trk.rclTrack.MINX = rclChildPrev.maxX();
             trk.rclTrack.MINY = rclChild.minY();
             trk.rclTrack.MAXX = rclChild.minX();
             trk.rclTrack.MAXY = trk.rclBoundary.MAXY;
           }
           else
           {
             trk.rclTrack.MINX = rclChild.maxX();
             trk.rclTrack.MINY = rclChildPrev.minY();
             trk.rclTrack.MAXX = rclChildPrev.minX();
             trk.rclTrack.MAXY = trk.rclBoundary.MAXY;
           }
        }
        else
        {          // Horizontal split bars.
#ifdef  IC_PM   // originLowerLeft
           trk.rclBoundary.MINY = rclChild.minY();
           trk.rclBoundary.MAXY = rclChildPrev.maxY();
#endif
#ifndef IC_PM   // originUpperLeft
           trk.rclBoundary.MINY = rclChildPrev.minY();
           trk.rclBoundary.MAXY = rclChild.maxY();
#endif

           trk.rclBoundary.MINX = rclChild.minX();
           trk.rclBoundary.MAXX = rclChild.maxX();

           /********************************************************/
           /* This specifies the length of the split bar, so       */
           /* WinTrackRect knows how wide of a thing to track.     */
           /* The split bar is the width of a size border.         */
           /********************************************************/
           trk.cxBorder = (int)trk.rclBoundary.MAXX;

           /********************************************************/
           /* This specifies the height of the split bar, so       */
           /* WinTrackRect knows how tall of a thing to track.     */
           /* The split bar is the width of a size border.         */
           /********************************************************/
           trk.cyBorder = (int)ulSplitBarWidth;

           /********************************************************/
           /* This specifies the tracking rectangle used by        */
           /* WinTrackRect.                                        */
           /********************************************************/
           trk.rclTrack.MINX = rclChild.minX();
           trk.rclTrack.MAXX = rclChild.maxX();
#ifdef  IC_PM   // originLowerLeft
           trk.rclTrack.MINY = rclChild.maxY();
           trk.rclTrack.MAXY = rclChildPrev.minY();
#endif
#ifndef IC_PM   // originUpperLeft
           trk.rclTrack.MINY = rclChildPrev.maxY();
           trk.rclTrack.MAXY = rclChild.minY();
#endif
        }

        trk.cxGrid = 0;
        trk.cyGrid = 0;
        trk.cxKeyboard = 0;
        trk.cyKeyboard = 0;

        /**********************************************************/
        /* Specify the minimum and maximum tracking size.         */
        /**********************************************************/
        trk.ptlMinTrackSize.x = trk.cxBorder;
        trk.ptlMinTrackSize.y = trk.cyBorder;
        trk.ptlMaxTrackSize.x = trk.cxBorder;
        trk.ptlMaxTrackSize.y = trk.cyBorder;

        /**********************************************************/
        /* These tracking options specify:  TF_MOVE - to center   */
        /* the pointer in the tracking rectangle. TF_ALLINBOUNDARY*/
        /* performs tracking such that no part of the tracking    */
        /* rectangle ever falls outside the bounding rectangle.   */
        /**********************************************************/
        trk.fs = TF_MOVE | TF_ALLINBOUNDARY;

        WinTrackRect( hwnd, 0, &trk );

        /**********************************************************/
        /* Restore WS_CLIPCHILDREN bit.                           */
        /**********************************************************/
        ISETWINDOWSTYLE( hwnd, lStyle );

        /************************************************************/
        /* Resize the two child windows affected by the movement of */
        /* the split bar.  The child to the right of the split bar  */
        /* split bar has to be moved and resized.  The child to the */
        /* left of the split bar only has to be resized.            */
        /* In the case of a horizontal split bar we have to move    */
        /* and size both controls to insure correct operation when  */
        /* the application and native coordinate systems are not    */
        /* the same.                                                */
        /************************************************************/
        if ( ulExtStyle & ISplitCanvas::vertical.asExtendedUnsignedLong() )
        {
           ISize cvSize = splitcv->size();

           if (!rightToLeft)
           {
             pwinChildPrev->sizeTo( ISize( trk.rclTrack.MINX -
                                             rclChildPrev.minX(),
                                           rclChildPrev.maxY() ));

             pwinChild->moveSizeTo(
                ICoordinateSystem::convertToApplication(
                   IRectangle( trk.rclTrack.MAXX,
                               rclChild.minY(),
                               rclChild.maxX(),
                               rclChild.maxY() ),
                   cvSize ));
           }
           else
           {
             pwinChild->sizeTo( ISize( trk.rclTrack.MINX -
                                             rclChild.minX(),
                                           rclChild.maxY() ));

             pwinChildPrev->moveSizeTo(
                ICoordinateSystem::convertToApplication(
                   IRectangle( trk.rclTrack.MAXX,
                               rclChildPrev.minY(),
                               rclChildPrev.maxX(),
                               rclChildPrev.maxY() ),
                   cvSize ));
           }

           /*************************************************************/
           /* Save new window sizes.                                    */
           /*************************************************************/
           for ( cwCursor.setToFirst();
                 cwCursor.isValid();
                 cwCursor.setToNext() )
           {            // Enumerate all child windows.
              whChild = splitcv->childAt( cwCursor );
              pwinSizeChild = IWindow::windowWithHandle( whChild );
              splitcv->setResolvedPercentage( pwinSizeChild,
                                              pwinSizeChild->size().width() );
           }
        }
        else
        {
           ISize cvSize = splitcv->size();
#ifdef  IC_PM   // originLowerLeft
           pwinChild->moveSizeTo(
              ICoordinateSystem::convertToApplication(
                 IRectangle( rclChild.minXMinY(),
                             ISize( rclChild.maxX(),
                                    trk.rclTrack.MINY -
                                    rclChild.minY() ) ),
                 cvSize ));

           ITRACE_ALL( IString("pwinChild new rect=") +
                       IRectangle( rclChild.minXMinY(),
                                   ISize( rclChild.maxX(),
                                          trk.rclTrack.MINY -
                                          rclChild.minY() ) ).asString() );

           pwinChildPrev->moveSizeTo(
              ICoordinateSystem::convertToApplication(
                 IRectangle( rclChildPrev.minX(),
                             trk.rclTrack.MAXY,
                             rclChildPrev.maxX(),
                             rclChildPrev.maxY() ) ,
                 cvSize ));

           ITRACE_ALL( IString("pwinChildPrev new rect=") +
                       IRectangle( rclChildPrev.minX(),
                                   trk.rclTrack.MAXY,
                                   rclChildPrev.maxX(),
                                   rclChildPrev.maxY() ).asString() );
#endif
#ifndef IC_PM   // originUpperLeft
           pwinChild->moveSizeTo(
              ICoordinateSystem::convertToApplication(
                 IRectangle( rclChild.minX(),
                             trk.rclTrack.MAXY,
                             rclChild.maxX(),
                             rclChild.maxY() ),
                 cvSize ));

           ITRACE_ALL( IString("pwinChild new rect=") +
                       IRectangle( rclChild.minX(),
                                   trk.rclTrack.MAXY,
                                   rclChild.maxX(),
                                   rclChild.maxY() ).asString() );

           pwinChildPrev->moveSizeTo(
              ICoordinateSystem::convertToApplication(
                 IRectangle( rclChildPrev.minXMinY(),
                             ISize( rclChildPrev.width(),
                                    trk.rclTrack.MINY -
                                    rclChildPrev.minY() )),
                 cvSize ) );

           ITRACE_ALL( IString("pwinChildPrev new rect=") +
                       IRectangle( rclChildPrev.minXMinY(),
                                   ISize( rclChildPrev.width(),
                                          trk.rclTrack.MINY -
                                          rclChildPrev.minY() )).asString() );
#endif

           /*************************************************************/
           /* Save new window sizes.                                    */
           /*************************************************************/
           for ( cwCursor.setToFirst();
                 cwCursor.isValid();
                 cwCursor.setToNext() )
           {            // Enumerate all child windows.
              whChild = splitcv->childAt( cwCursor );
              pwinSizeChild = IWindow::windowWithHandle( whChild );
              splitcv->setResolvedPercentage( pwinSizeChild,
                                              pwinSizeChild->size().height() );
           }
        }

        /*********************************************************/
        /* Resolve rations base on 100%.                         */
        /*********************************************************/
        splitcv->resolveRatios();
        splitcv->setLayoutDistorted( 0, IWindow::layoutChanged );
                             // Prevent re-layout of controls.

        /*********************************************************/
        /* Say we processed event.                               */
        /*********************************************************/
        IINVALIDATERECT( hwnd, &(trk.rclTrack), false );
        return false;
     } /* end WM_BUTTONxDOWN */

     case WM_SIZE:
     {
        /************************************************************/
        /* Resize the child controls based on the percentage of the */
        /* window they occupied when the window was resized, thus   */
        /* maintaining their ratios to each other.                  */
        /************************************************************/
        ISplitCanvas* splitcv = (ISplitCanvas*)evt.window();
        splitcv->sizClCanvasSize = ISize( evt.parameter2().number1(),
                                          evt.parameter2().number2() );
        splitcv->setLayoutDistorted( IWindow::sizeChanged, 0 );
        splitcv->layout();

        break;
     }
  } /* endswitch */

  return false;
}

/*------------------------------------------------------------------------------
| ISplitCanvasHandler::paintSplitBar                                           |
------------------------------------------------------------------------------*/
void ISplitCanvasHandler::paintSplitBar ( const ISplitCanvas* splitcv,
                                          const IPresSpaceHandle& pshCanvas,
                                          const IRectangle& rect )
{
  /********************************************************************/
  /* Get the width of a window border, and the width of a window size */
  /* border.  These widths will be used to make up the split bar. The */
  /* split bar is painted in 3 steps.  The left and right sides are   */
  /* the width of a window border, with the center being the width of */
  /* a window sizing border, minus the width of 2 window borders.     */
  /* A rough picture may help:                                        */
  /*                                                                  */
  /*           Width of a size border                                 */
  /*           minus the width of 2                                   */
  /*           window borders                                         */
  /*                   |                                              */
  /*   Width of a      |      Width of a                              */
  /*   Window     \    |    / Window                                  */
  /*   Border      ||     ||  Border                                  */
  /*               ||     ||                                          */
  /*               ||     ||                                          */
  /*               ||     ||                                          */
  /********************************************************************/

  IGraphicContext gc(pshCanvas);
  gc.setDrawOperation( IGraphicBundle::fill );
  if ( splitcv->extendedStyle() &
         ISplitCanvas::vertical.asExtendedUnsignedLong() )
  {
     IRectangle box = rect.sizedTo(
                        ISize(splitcv->ulClEdgeThickness, rect.height() ));

     /***************************************************************/
     /* Paint the left edge of the split bar.                       */
     /***************************************************************/
     gc.setFillColor( splitcv->clrClEdgeColor );
     gc.draw( IGRectangle(box) );


     /***************************************************************/
     /* Paint the middle of the split bar.                          */
     /***************************************************************/
     box.moveTo( box.maxXMinY() );
     box.sizeTo( ISize( splitcv->ulClMiddleThickness, box.height() ));
     gc.setFillColor( splitcv->clrClMiddleColor );
     gc.draw( IGRectangle(box) );


     /***************************************************************/
     /* Paint the right edge of the split bar.                      */
     /***************************************************************/
     box.moveTo( box.maxXMinY() );
     box.sizeTo( ISize( splitcv->ulClEdgeThickness, box.height() ));
     gc.setFillColor( splitcv->clrClEdgeColor );
     gc.draw( IGRectangle(box) );
  }
  else
  {      // Must be a horizontal split bar.
     /***************************************************************/
     /* Paint the minimum edge of the split bar.                    */
     /***************************************************************/
     IRectangle box = rect.sizedTo(
                        ISize(rect.width(), splitcv->ulClEdgeThickness ));
     gc.setFillColor( splitcv->clrClEdgeColor );
     gc.draw( IGRectangle(box) );

     /***************************************************************/
     /* Paint the middle of the split bar.                          */
     /***************************************************************/
     box.moveTo( box.minXMaxY() );
     box.sizeTo( ISize( box.width(), splitcv->ulClMiddleThickness ));
     gc.setFillColor( splitcv->clrClMiddleColor );
     gc.draw( IGRectangle(box) );

     /***************************************************************/
     /* Paint the maximum edge of the split bar.                    */
     /***************************************************************/
     box.moveTo( box.minXMaxY() );
     box.sizeTo( ISize( box.width(), splitcv->ulClEdgeThickness ));
     gc.setFillColor( splitcv->clrClEdgeColor );
     gc.draw( IGRectangle(box) );
  }
}

/*------------------------------------------------------------------------------
| ISplitCanvasHandler::handleEventsFor                                         |
|                                                                              |
| Attach the handler to the canvas.                                            |
------------------------------------------------------------------------------*/
ISplitCanvasHandler& ISplitCanvasHandler::handleEventsFor
                                            ( ISplitCanvas *canvas )
{
  IASSERTPARM( canvas != 0 );
  Inherited::handleEventsFor( canvas );
  return *this;
}

/*------------------------------------------------------------------------------
| ISplitCanvasHandler::stopHandlingEventsFor                                   |
|                                                                              |
| Detack the handler to the canvas.                                            |
------------------------------------------------------------------------------*/
ISplitCanvasHandler& ISplitCanvasHandler::stopHandlingEventsFor
                                             ( ISplitCanvas *canvas )
{
  IASSERTPARM( canvas != 0 );
  Inherited::stopHandlingEventsFor( canvas );
  return *this;
}

/*------------------------------------------------------------------------------
| ISplitCanvasHandler::handleEventsFor                                         |
------------------------------------------------------------------------------*/
IHandler& ISplitCanvasHandler::handleEventsFor ( IWindow* )
{                              // Private to hide version in IHandler.
  ITHROWLIBRARYERROR( IC_MEMBER_ACCESS_ERROR,
                      IBaseErrorInfo::invalidRequest,
                      IException::recoverable );
  return *this;
}

/*------------------------------------------------------------------------------
| ISplitCanvasHandler::stopHandlingEventsFor                                   |
------------------------------------------------------------------------------*/
IHandler& ISplitCanvasHandler::stopHandlingEventsFor ( IWindow* )
{                              // Private to hide version in IHandler.
  ITHROWLIBRARYERROR( IC_MEMBER_ACCESS_ERROR,
                      IBaseErrorInfo::invalidRequest,
                      IException::recoverable );
  return *this;
}

