/*******************************************************************************
* FILE NAME: istathdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in istathdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
{
  #define INCL_WINSYS
  #define INCL_GPIPRIMITIVES
  #define INCL_GPILOGCOLORTABLE
  #include <iwindefs.h>
}

#include <istathdr.hpp>

#include <icontrol.hpp>
#include <iexcept.hpp>
#include <ifont.hpp>
#include <igbundle.hpp>
#include <igrafctx.hpp>
#include <igrect.hpp>
#include <ihandle.hpp>
#include <ipainevt.hpp>
#include <istattxt.hpp>
#include <istring.hpp>
#include <iswp.hpp>
#include <itrace.hpp>
#include <iwcname.hpp>

#ifdef IC_PM
  #include <ipmbidi.hpp>        // OS/2 specific BIDI functions
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISTATHDR_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IStaticTextHandler::IStaticTextHandler                                       |
------------------------------------------------------------------------------*/
IStaticTextHandler :: IStaticTextHandler()
{ }

/*------------------------------------------------------------------------------
| IStaticTextHandler::~IStaticTextHandler                                      |
------------------------------------------------------------------------------*/
IStaticTextHandler :: ~IStaticTextHandler()
{ }

/*------------------------------------------------------------------------------
| IStaticTextHandler::paintWindow                                              |
|                                                                              |
| Process the paint events for a static window.                                |
------------------------------------------------------------------------------*/
Boolean IStaticTextHandler::paintWindow( IPaintEvent& evt )
{

  IStaticText* pStatic = (IStaticText *)evt.window();
  ISize winsize = pStatic->size();
  unsigned long ulLength = pStatic->textLength();
  unsigned long ulExtStyles = pStatic->extendedStyle();
  RECTL lprc ;

  lprc.MINX = 0;
  lprc.MINY = 0;
  lprc.MAXX = winsize.width();
  lprc.MAXY = winsize.height();

#ifndef IC_WIN
  ICREATELOGCOLORTABLE( evt.presSpaceHandle(), 0L,
                        LCOLF_RGB, 0L, 0L, 0 );
#endif

  IColor clrFgnd = pStatic->foregroundColor();
  IColor clrBgnd = pStatic->backgroundColor();

  unsigned long ulStyles = pStatic->style();
#ifdef IC_MOTIFPM
  if (ulStyles & IStaticText::fillBackground.asUnsignedLong())
  {
    IColor clrFill = pStatic->fillColor();
    WinFillRect( evt.presSpaceHandle(), &lprc, clrFill.asRGBLong() );
  }
#endif

#ifdef IC_WIN
#ifdef IC_TRACE_ALL
  ITRACE_ALL(IString("IStaticTextHandler ")+
             IString("HWND=") + IString(evt.handle().asUnsigned()).d2x()+
             IString(", ulLength=") + IString(ulLength)+
             IString(", ulStyles=") + IString(ulStyles).d2x()+
             IString(", ulExtStyles=") + IString(ulExtStyles).d2x() );
  ITRACE_ALL(IString("IStaticTextHandler ")+
             IString("rect=(") +
             IString(lprc.MINX)+ IString(",") +
             IString(lprc.MINY)+ IString(",") +
             IString(lprc.MAXX)+ IString(",") +
             IString(lprc.MAXY)+IString(")") );
  ITRACE_ALL(IString("IStaticTextHandler ")+
             IString(" clrFgnd=") + IString( clrFgnd.asRGBLong() ).d2x() +
             IString(" clrBgnd=") + IString( clrBgnd.asRGBLong() ).d2x() );
#endif

  IGraphicContext  gc( evt.presSpaceHandle() );

  if (ulExtStyles & IStaticText::fillBackground.asExtendedUnsignedLong())
     {
     gc.setFillColor( pStatic->fillColor() );
     gc.setPenColor( pStatic->fillColor() );    // need for Win95
     gc.setDrawOperation( IGraphicBundle::fillAndFrame );
     gc.draw( IGRectangle( IRectangle(lprc) ) );
     gc.setPenColor( clrFgnd );                 // put back real foregrond
    }
#endif

  if (ulLength)
  {
    unsigned long ulStyle = 0;

#ifdef IC_PM
    IString*      bidiText         = 0;
    LAYOUT_OBJECT bidiLayoutObject = 0;
#endif

#ifdef IC_PMWIN
    //---------------------------------------------------------------
    // Check for BIDI support
    //---------------------------------------------------------------
    if ( IBidiSettings::isBidiSupported() )
    {
      //-------------------------------------------------------------
      // Query the current BIDI attributes
      //-------------------------------------------------------------
      IBidiSettings settings(*pStatic);

#ifdef IC_PM
      //-------------------------------------------------------------
      // BIDI text strings need to be converted before display, so
      // we need to make a copy of the text (to work with)
      //-------------------------------------------------------------
      bidiText = new IString(pStatic->text());

      //-------------------------------------------------------------
      // Create a layout object to use when drawing BIDI text
      //-------------------------------------------------------------
      bidiLayoutObject = bidiCreateLayoutObject();

      //-------------------------------------------------------------
      // Convert the BIDI text for display
      // Note: We need to turn off symmetric swapping because during
      //       conversion (it will be handled later)
      //-------------------------------------------------------------
      {
        IBidiSettings tmpSettings(*pStatic);
        unsigned long tilde = bidiText->indexOf('~');
        if (tilde)
           bidiText->remove(tilde,1);
        tmpSettings.disableSymmetricSwapping();
        bidiPrepareForDisplay(bidiLayoutObject,*bidiText,tmpSettings,tilde);
        if (tilde)
           bidiText->insert("~",tilde);
      }
#endif

      //-------------------------------------------------------------
      // If right-to-left layout, swap left and right justify flags
      //-------------------------------------------------------------
      if ( settings.windowLayout() == IBidiSettings::layoutRightToLeft )
      {
#ifndef IC_WIN
        if ( ulStyles & IStaticText::right.asUnsignedLong() )
        {
          ulStyles &= ~IStaticText::right.asUnsignedLong();
          ulExtStyles |= IStaticText::left.asExtendedUnsignedLong();
        }
        else
        {
          if ( !(ulStyles & IStaticText::center.asUnsignedLong()) )
          {
            ulStyles |= IStaticText::right.asUnsignedLong();
            ulExtStyles &= ~IStaticText::left.asExtendedUnsignedLong();
          }
        }
#endif
#ifdef IC_WIN
        if ( ulExtStyles & IStaticText::right.asExtendedUnsignedLong() )
        {
          ulExtStyles &= ~IStaticText::right.asExtendedUnsignedLong();
          ulExtStyles |= IStaticText::left.asExtendedUnsignedLong();
        }
        else
        {
          if ( !(ulExtStyles & IStaticText::center.asExtendedUnsignedLong()) )
          {
            ulExtStyles |= IStaticText::right.asExtendedUnsignedLong();
            ulExtStyles &= ~IStaticText::left.asExtendedUnsignedLong();
          }
        }
#endif
      }
    } // isBidiSupported
#endif

#ifdef IC_MOTIFPM
    if (ulExtStyles & IStaticText::underscore.asExtendedUnsignedLong())
      ulStyles |= DT_UNDERSCORE;
    if (ulExtStyles & IStaticText::strikeout.asExtendedUnsignedLong())
      ulStyles |= DT_STRIKEOUT;

    ulStyle |= (ulStyles & (DT_RIGHT | DT_CENTER | DT_BOTTOM |
                            DT_VCENTER | DT_WORDBREAK | DT_MNEMONIC |
                            DT_HALFTONE | DT_UNDERSCORE | DT_STRIKEOUT));

    if (!pStatic->isEnabled())
      ulStyle |= DT_HALFTONE;

    // Draw the text
    IString pszText = pStatic->text();
#ifdef IC_PM
    if ( bidiText )
       pszText = *bidiText;
#endif

    ISize fm = pStatic->characterSize();
    unsigned long cchDrawn = 0;

    // Loop to handle multiple line static text
    while (cchDrawn != ulLength)
    {
      RECTL lprc2;
      lprc2.MINX = lprc.MINX;
      lprc2.MAXX = lprc.MAXX;
      lprc2.MINY = lprc.MINY;
      lprc2.MAXY = lprc.MAXY;

#ifdef IC_PM
      //-------------------------------------------------------------
      // Check for BIDI support
      //-------------------------------------------------------------
      if ( IBidiSettings::isBidiSupported() )
      {
         // For BIDI systems, call specialized draw routine
         IBidiSettings settings(*pStatic);
         cchDrawn += bidiDrawText( evt.presSpaceHandle(),
                                   pszText,
                                   cchDrawn,
                                   bidiLayoutObject,
                                   settings,
                                   ulStyle,
                                   clrFgnd,
                                   clrBgnd,
                                   IRectangle(lprc2) );
      }
      else
#endif  //IC_PM
      {
        // Calculate the size of the drawing rectangle
        WinDrawText( evt.presSpaceHandle(),
                     ulLength - cchDrawn,
                     (PCHAR)pszText + cchDrawn,
                     &lprc2,
                     0,
                     0,
                     ulStyle | DT_QUERYEXTENT );
        WinFillRect( evt.presSpaceHandle(), &lprc2, clrBgnd.asRGBLong() );

        // Draw the static text
        cchDrawn += WinDrawText( evt.presSpaceHandle(),
                                 ulLength - cchDrawn,
                                 (PCHAR)pszText + cchDrawn,
                                 &lprc,
                                 clrFgnd.asRGBLong(),
                                 clrBgnd.asRGBLong(),
                                 ulStyle );
      }

       // Don't loop if no word breaking, or if it is not top aligned.
      if (!(ulStyle & DT_WORDBREAK))
        break;

      if (ulStyle & (DT_VCENTER | DT_BOTTOM))
        break;

       // If out of room for drawing, exit.
      if ((lprc.MAXY -= fm.height()) <= lprc.MINY)
        break;
    } /* end while */
#endif  // IC_MOTIFPM

#ifdef IC_WIN
    ulStyle = DT_NOPREFIX ;     // implied DT_LEFT | DT_TOP
    // Set up the DrawText flags based on the control extended styles
    if (ulExtStyles & IDT_RIGHT)
       ulStyle |= DT_RIGHT;
    if (ulExtStyles & IDT_CENTER)
       ulStyle |= DT_CENTER ;
    if (ulExtStyles & IDT_BOTTOM)
       ulStyle |= DT_BOTTOM | DT_SINGLELINE;
    if (ulExtStyles & IDT_VCENTER)
       ulStyle |= DT_VCENTER | DT_SINGLELINE;
    if (ulExtStyles & IDT_WORDBREAK)
       ulStyle |= DT_WORDBREAK;
    if (ulExtStyles & IDT_MNEMONIC)
       ulStyle &= (unsigned long)~DT_NOPREFIX;

    Boolean grayText = ( (ulExtStyles & IDT_HALFTONE ) ||
                         (!pStatic->isEnabled()) );

    // Draw the text
    IString pszText = pStatic->text();
    COLORREF fgOld, bgOld;
    if (!grayText)
       {
       fgOld = SetTextColor( gc.handle(), clrFgnd.asRGBLong() );
       }
    else
       {
       DWORD grayColor = GetSysColor( COLOR_GRAYTEXT );
       fgOld = SetTextColor( gc.handle(), grayColor );
       }
    if (fgOld == (COLORREF)CLR_INVALID)
       {
       ITHROWGUIERROR2( "SetTextColor",
                        IBaseErrorInfo::accessError,
                        IException::unrecoverable);
       }
    bgOld = SetBkColor( gc.handle(), clrBgnd.asRGBLong() );
    int bmOld = SetBkMode( gc.handle(), OPAQUE );
    if (bgOld == (COLORREF)CLR_INVALID)
       {
       ITHROWGUIERROR2( "SetTextColor",
                        IBaseErrorInfo::accessError,
                        IException::unrecoverable);
       }

    // select the font to use...need a copy of the font.  We use the font
    // to implement strikeout and underscore.
    IFont winFont = pStatic->font();
    if (pStatic->isUnderscore())
       winFont.setUnderscore();
    if (pStatic->isStrikeout())
       winFont.setStrikeout();
    winFont.beginUsingFont( gc.handle() );

    // Draw the static text
    DrawText( gc.handle(),
              (PCHAR)pszText,
              (int)ulLength ,
              (RECT*)&lprc,
              (UINT)ulStyle );

    winFont.endUsingFont( gc.handle() );

    fgOld = SetTextColor( gc.handle(), fgOld);
    bgOld = SetBkColor( gc.handle(), bgOld);
    SetBkMode( gc.handle(), bmOld );
#endif

    //---------------------------------------------------------------
    // Clean up any temporary storage used for BIDI support
    //---------------------------------------------------------------
#ifdef IC_PM
    if ( bidiText )
      delete bidiText;
    if ( bidiLayoutObject )
        bidiDeleteLayoutObject(bidiLayoutObject);
#endif

  }  // if length > 0
  return( true );
}

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IStaticTextHandler::dispatchHandlerEvent                                     |
|                                                                              |
| We override this function to detect resizing of the static text window.      |
| When the window is resized we need to invalidate the entire window so that   |
| text (and graphics) get positioned consistently.  Without this, painting     |
| problems appear in some cases (for example, the asplitcv sample).            |
| We could use a resize handler to do this but it is done this way to          |
| keep only one default handler.                                               |
------------------------------------------------------------------------------*/
IBase::Boolean  IStaticTextHandler :: dispatchHandlerEvent ( IEvent& event )
{
   Boolean result = false;
   if (event.eventId() == WM_WINDOWPOSCHANGED)
      {
      PWINDOWPOS pswp = (PWINDOWPOS)(char*)event.parameter2();
      if (!(pswp->flags & SWP_NOSIZE))
         {
         // Invalidate the entire window
         IWindow* win = event.dispatchingWindow();
         if (win)
            win->refresh( IWindow::paintAll );
         }
      }
   else if (event.eventId() == WM_ENABLE)
      {
      // Bypass the Static class window procedure, since this seems to
      // do direct painting (no WM_PAINT generated) to give its text a
      // disabled/enabled look.  This direct painting does use colors
      // assigned by the application to the IStaticText object.
      // Invalidate the entire window to allow the IStaticText paint
      // handler to repaint the text appropriately.
      IWindow* win = event.dispatchingWindow();
      if (win)
         {
         win->refresh( IWindow::paintAll );
         DefWindowProc( event.handle(),
                        (UINT)event.eventId(),
                        event.parameter1(),
                        event.parameter2() );
         result = true;
         }
      }
   else if (event.eventId() == WM_NCHITTEST)
      {
      IWindowHandle parent = IPARENTOF( event.handle() );
      if (( !parent ) ||
          (IWindowClassName( parent ) != WC_BUTTON ) )
         {
         // Avoid HT_TRANSPARENT from WC_STATIC window procedure to
         // allow mouse button presses on static controls.
         MRESULT mr = IDEFWINDOWPROC( event.handle(),
                                      (UINT)event.eventId(),
                                      event.parameter1(),
                                      event.parameter2() );
         event.setResult( IEventResult( (unsigned long)mr ) );
         result = true;
         }
      else
         {
         // We want the default behavior when the parent is a button
         // such as IGraphicPushButton.   Can't click in graphic otherwise.
         result = Inherited::dispatchHandlerEvent( event );
         }
      }
    else
      {
      result = Inherited::dispatchHandlerEvent( event );
      }
   return result;
}

#endif
