/*******************************************************************************
* FILE NAME: iswp.cpp                                                          *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iswp.hpp.                                                               *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
  {
  #define INCL_WINWINDOWMGR
  #include <iwindefs.h>
  }

#include <iswp.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISWP_CPP_
  #include <ipagetun.h>
#endif

/*******************************************************************************
* Implementation note:                                                         *
*                                                                              *
* The types and order of the data members of this class must exactly match the *
* layout of the Presentation Manager SWP structure (on OS/2) or the WINDOWPOS  *
* structure (on Windows).  The data members use the corresponding field names  *
* used in the SWP structure.                                                   *
*                                                                              *
*   Data member order   SWP field         Use                                  *
*   ---------------------------------------------------------------------      *
*   fl                  fl                Flags.                               *
*   cy                  cy                Width.                               *
*   cx                  cx                Height.                              *
*   x                   x                 The X-coordinate position.           *
*   y                   y                 The Y-coordinate position.           *
*   behind              behind            Handle for the "behind" window.      *
*   hwnd                hwnd              Handle for this window.              *
*   ulReserved1         ulReserved1       Reserved.                            *
*   ulReserved2         ulReserved2       Reserved.                            *
*                                                                              *
*   Data member order   WINDOWPOS field   Use                                  *
*   ---------------------------------------------------------------------      *
*   hwnd                hwnd              Handle for this window.              *
*   behind              hwndInsertAfter   Handle for the "behind" window.      *
*   x                   x                 The X-coordinate position.           *
*   y                   y                 The Y-coordinate position.           *
*   cx                  cx                Height.                              *
*   cy                  cy                Width.                               *
*   fl                  flags             Flags.                               *
*******************************************************************************/


/*------------------------------------------------------------------------------
| ISWPArray::indexOf                                                           |
|                                                                              |
| Step through the swp array looking for an entry for an hwnd whose id         |
| matches the search argument.  Returns "size" if not found.                   |
------------------------------------------------------------------------------*/
unsigned ISWPArray::indexOf ( unsigned long id ) const
{
  int
    result = 0;
  while ( result < dim )
    {
       if (IIDOF(p[result].windowHandle()) == id )
          break;
    else
      result++;
    }
  return result;
}

/*------------------------------------------------------------------------------
| IBase::Boolean ISWP::isMove                                                  |
------------------------------------------------------------------------------*/
IBase::Boolean ISWP::isMove( ) const
{
#ifdef   IC_PM
  return (fl & SWP_MOVE)   ? true  : false;
#endif
#ifdef   IC_WIN
  return (fl & SWP_NOMOVE) ? false : true;
#endif
}

/*------------------------------------------------------------------------------
| IBase::Boolean ISWP::isSize                                                  |
------------------------------------------------------------------------------*/
IBase::Boolean ISWP::isSize( ) const
{
#ifdef   IC_PM
  return (fl & SWP_SIZE)   ? true  : false;
#endif
#ifdef   IC_WIN
  return (fl & SWP_NOSIZE) ? false : true;
#endif
}

/*------------------------------------------------------------------------------
| IBase::Boolean ISWP::isShow                                                  |
------------------------------------------------------------------------------*/
IBase::Boolean ISWP::isShow( ) const
{
#ifdef   IC_PM
  return (fl & SWP_SHOW)   ? true  : false;
#endif
#ifdef   IC_WIN
  return (fl & SWP_SHOWWINDOW) ? true : false;
#endif
}

/*------------------------------------------------------------------------------
| IBase::Boolean ISWP::isHide                                                  |
------------------------------------------------------------------------------*/
IBase::Boolean ISWP::isHide( ) const
{
#ifdef   IC_PM
  return (fl & SWP_HIDE)   ? true  : false;
#endif
#ifdef   IC_WIN
  return (fl & SWP_HIDEWINDOW) ? true : false;
#endif
}

/*------------------------------------------------------------------------------
| IBase::Boolean ISWP::isZOrder                                                |
------------------------------------------------------------------------------*/
IBase::Boolean ISWP::isZOrder( ) const
{
#ifdef   IC_PM
  return (fl & SWP_ZORDER) ? true  : false;
#endif
#ifdef   IC_WIN
  return (fl & SWP_NOZORDER) ? false : true;
#endif
}

/*------------------------------------------------------------------------------
| ISWP::setMove                                                                |
------------------------------------------------------------------------------*/
ISWP& ISWP::setMove ( Boolean enable )
{
#ifdef   IC_PM
  if (enable)
     fl |= SWP_MOVE;
  else
     fl &= ~SWP_MOVE;
#endif
#ifdef   IC_WIN
  if (enable)
     fl &= ~SWP_NOMOVE;
  else
     fl |= SWP_NOMOVE;
#endif

  return *this;
}

/*------------------------------------------------------------------------------
| ISWP::setSizeFlag                                                            |
------------------------------------------------------------------------------*/
ISWP& ISWP::setSizeFlag ( Boolean enable )
{
#ifdef   IC_PM
  if (enable)
     fl |= SWP_SIZE;
  else
     fl &= ~SWP_SIZE;
#endif
#ifdef   IC_WIN
  if (enable)
     fl &= ~SWP_NOSIZE;
  else
     fl |= SWP_NOSIZE;
#endif

  return *this;
}

/*------------------------------------------------------------------------------
| ISWP::setShow                                                                |
------------------------------------------------------------------------------*/
ISWP& ISWP::setShow ( Boolean enable )
{
#ifdef   IC_PM
  if (enable)
     fl |= SWP_SHOW;
  else
     fl &= ~SWP_SHOW;
#endif
#ifdef   IC_WIN
  if (enable)
     fl |= SWP_SHOWWINDOW;
  else
     fl &= ~SWP_SHOWWINDOW;
#endif

  return *this;
}

/*------------------------------------------------------------------------------
| ISWP::setHide                                                                |
------------------------------------------------------------------------------*/
ISWP& ISWP::setHide ( Boolean enable )
{
#ifdef   IC_PM
  if (enable)
     fl |= SWP_HIDE;
  else
     fl &= ~SWP_HIDE;
#endif
#ifdef   IC_WIN
  if (enable)
     fl |= SWP_HIDEWINDOW;
  else
     fl &= ~SWP_HIDEWINDOW;
#endif

  return *this;
}

/*------------------------------------------------------------------------------
| ISWP::setZOrder                                                              |
------------------------------------------------------------------------------*/
ISWP& ISWP::setZOrder( Boolean enable )
{
#ifdef   IC_PM
  if (enable)
     fl |= SWP_ZORDER;
  else
     fl &= ~SWP_ZORDER;
#endif
#ifdef   IC_WIN
  if (enable)
     fl &= ~SWP_NOZORDER;
  else
     fl |= SWP_NOZORDER;
#endif

  return *this;
}

/*------------------------------------------------------------------------------
| ISWP::setNoAdjust                                                            |
------------------------------------------------------------------------------*/
ISWP& ISWP::setNoAdjust( Boolean enable )
{
#ifdef   IC_PM
  if (enable)
     fl |= SWP_NOADJUST;
  else
     fl &= ~SWP_NOADJUST;
#endif
#ifdef   IC_WIN
  if (enable)
     fl &= ~SWP_FRAMECHANGED;
  else
     fl |= SWP_FRAMECHANGED;
#endif

  return *this;
}
