/*******************************************************************************
* FILE NAME: isysmenu.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in isysmenu.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
  #define INCL_WINFRAMEMGR
  #define INCL_WINMENUS
  #include <iwindefs.h>
}

#include <isysmenu.hpp>
#include <iframe.hpp>
#include <iexcept.hpp>
#include <icconst.h>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISYSMENU_CPP_
  #include <ipagetun.h>
#endif

#pragma data_seg(ICLStaticConst)
#ifndef IC_WIN
const unsigned long ISystemMenu :: idHide(SC_HIDE);
#endif
#ifdef IC_WIN
// Not really supported in Windows 32...simulate with minimize.
const unsigned long ISystemMenu :: idHide(SC_MINIMIZE);
#endif
const unsigned long ISystemMenu :: idPulldown(SC_SYSMENU);
const unsigned long ISystemMenu :: idRestore(SC_RESTORE);
const unsigned long ISystemMenu :: idMove(SC_MOVE);
const unsigned long ISystemMenu :: idSize(SC_SIZE);
const unsigned long ISystemMenu :: idMinimize(SC_MINIMIZE);
const unsigned long ISystemMenu :: idMaximize(SC_MAXIMIZE);
const unsigned long ISystemMenu :: idClose(SC_CLOSE);
const unsigned long ISystemMenu :: idWindowList(SC_TASKMANAGER);
#pragma data_seg()

/*------------------------------------------------------------------------------
| ISystemMenu::ISystemMenu                                                     |
------------------------------------------------------------------------------*/
ISystemMenu::ISystemMenu( IFrameWindow* owner )
                        : fSystemMenuData( 0 )
{
   IASSERTPARM(owner != 0);

#ifdef IC_WIN
   fowner = owner->handle();
   fmenuFrame = owner;
   fhMenu = GetSystemMenu( fowner, false );
   if (!fhMenu)
#else
   IWindowHandle hwndSysMenu = handleWithParent(FID_SYSMENU,owner->handle());

   if (hwndSysMenu)
   {
      // Since we are only wrappering the underlying PM window
      setAutoDestroyWindow(false);
      startHandlingEventsFor(hwndSysMenu);
   }
   else
#endif
   {
     ITHROWLIBRARYERROR(IC_NO_SYSTEMMENU,
                        IBaseErrorInfo::invalidRequest,
                        IException::recoverable);
   } /* endif */
}


/*------------------------------------------------------------------------------
| ISystemMenu::~ISystemMenu                                                    |
------------------------------------------------------------------------------*/
ISystemMenu::~ISystemMenu()
{}
