/*******************************************************************************
* FILE NAME: itbarfrm.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in itbarfrm.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C"
{
  #define INCL_WINSYS
  #define INCL_WINFRAMEMGR
  #define INCL_WINFRAMECTLS
  #define INCL_WINWINDOWMGR
  #define INCL_WINCOUNTRY
  #define INCL_WINHOOKS
  #include <iwindefs.h>
}

#include <itbarfrm.hpp>
#include <itbfpriv.hpp>
#include <itbar.hpp>
#include <iswp.hpp>
#include <icoordsy.hpp>
#include <ifont.hpp>
#include <igstring.hpp>
#include <igbitmap.hpp>
#include <igregion.hpp>
#include <igbundle.hpp>
#include <itbpriv.hpp>
#include <itrace.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ITBARFRM_CPP_
  #include <ipagetun.h>
#endif

#pragma data_seg(ICLStaticConst)
static const unsigned long titleBarHeight(19);
#pragma data_seg()

#pragma data_seg(ICLNonConst)
unsigned long IToolBarTitle::offSet(0);
#pragma data_seg()

/*------------------------------------------------------------------------------
| IToolBarFrameData::IToolBarFrameData                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarFrameData::IToolBarFrameData( IToolBarFrameWindow* toolBarFrameWindow )
  : systemMenu( toolBarFrameWindow ),
    toolBarTitle( toolBarFrameWindow, "" ),
#ifndef IC_WIN
    titleFrameHandler(),
    systemMenuDrawHandler(),
#endif
    titleBarCommandHandler()
#ifndef IC_WIN
    ,
    showHideButton( IC_ID_SHOWHIDE,
                    toolBarFrameWindow,
                    toolBarFrameWindow,
                    IRectangle(),
                    IToolBarButton::bitmapVisible |
                    IWindow::visible              |
                    IButton::noPointerFocus ),
    moveButton( IC_ID_OWNERMOVE,
                toolBarFrameWindow,
                toolBarFrameWindow,
                IRectangle(),
                IToolBarButton::bitmapVisible  |
                ICustomButton::latchable       |
                IWindow::visible               |
                IButton::noPointerFocus )
#endif
{ }

/*------------------------------------------------------------------------------
| IToolBarFrameData::~IToolBarFrameData                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarFrameData::~IToolBarFrameData( )
{ }

/*------------------------------------------------------------------------------
| IToolBarFrameWindow::IToolBarFrameWindow                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarFrameWindow::IToolBarFrameWindow( unsigned long windowId,
                                          IFrameWindow* owner )
  : IToolBarFrameWindow::Inherited( windowId,
                                    IWindow::desktopWindow(),
                                    owner,
                                    IRectangle(),
                                    IFrameWindow::titleBar      |
                                    IFrameWindow::systemMenu    |
                                    IFrameWindow::border        |
#ifdef IC_WIN
                                    IFrameWindow::maximizeButton |
#endif
                                    IFrameWindow::noMoveWithOwner ),
    fToolBarFrameData( new IToolBarFrameData( this ))
{
  IToolBarTitle::offSet = IQUERYSYSVALUE( SV_CYTITLEBAR ) - titleBarHeight;

#ifdef IC_WIN
  // Remove all the unnecessary menu options.
  fToolBarFrameData->systemMenu.deleteItem(ISystemMenu::idSize)
                               .deleteItem(ISystemMenu::idMinimize);

  // Windows does not use the showHideButton, the moveButton,  the
  // titleFrameHandler, or the systemMenuDrawHandler.
#else
  // Make the system menu bitmap owner draw &
  // Remove all the unnecessary menu options.
  fToolBarFrameData->systemMenu.setItem(
                fToolBarFrameData->systemMenu.menuItem(SC_SYSMENU).setDrawItem())
                              .deleteItem(ISystemMenu::idRestore)
                              .deleteItem(ISystemMenu::idMinimize)
                              .deleteItem(ISystemMenu::idMaximize)
                              .deleteItem(ISystemMenu::idHide);

  // Set up bitmaps for move with owner button
  fToolBarFrameData->moveButton.setBitmap(IApplication::current().
                    resourceLibrary().tryToLoadBitmap(IC_ID_NOOWNERMOVE));
  fToolBarFrameData->moveButton.setLatchedBitmap(IApplication::current().
                    resourceLibrary().tryToLoadBitmap(IC_ID_OWNERMOVE));

  fToolBarFrameData->showHideButton.setUserData( false );

  fToolBarFrameData->showHideButton.setBitmap(IApplication::current()
                                   .resourceLibrary().tryToLoadBitmap(IC_ID_HIDE));

//.setLatchedColor( fToolBarFrameData->showHideButton.backgroundColor(), false );

  fToolBarFrameData->titleFrameHandler.handleEventsFor( this );
  fToolBarFrameData->systemMenuDrawHandler.handleEventsFor(&fToolBarFrameData->systemMenu);
#endif

  fToolBarFrameData->titleBarCommandHandler.handleEventsFor( this );

#ifndef IC_WIN
  // These buttons are not shown in Windows, since title bar extensions
  // are not supported.  We use the maximize and restore buttons for the
  // collapse/expand functions
  addExtension( &fToolBarFrameData->showHideButton,
                IFrameWindow::rightOfTitleBar, titleBarHeight );
  addExtension( &fToolBarFrameData->moveButton,
                IFrameWindow::rightOfTitleBar, titleBarHeight );
#endif

}

/*------------------------------------------------------------------------------
| IToolBarFrameWindow::~IToolBarFrameWindow                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarFrameWindow::~IToolBarFrameWindow( )
{
#ifndef IC_WIN
  fToolBarFrameData->titleFrameHandler.stopHandlingEventsFor( this );
  fToolBarFrameData->systemMenuDrawHandler.stopHandlingEventsFor(&fToolBarFrameData->systemMenu);
#endif
  fToolBarFrameData->titleBarCommandHandler.stopHandlingEventsFor( this );
  delete fToolBarFrameData;
}

/*------------------------------------------------------------------------------
| IToolBarFrameWindow::setTitleText                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarFrameWindow& IToolBarFrameWindow::setTitleText( const char* text )
{
  fToolBarFrameData->toolBarTitle.setText( text );
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBarFrameWindow::setTitleText                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarFrameWindow& IToolBarFrameWindow::setTitleText( const IResourceId& text)
{
  fToolBarFrameData->toolBarTitle.setText( text );
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBarFrameWindow::titleText                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IString IToolBarFrameWindow::titleText( ) const
{
  return fToolBarFrameData->toolBarTitle.text();
}

/*------------------------------------------------------------------------------
| ITitleBarFrameHandler::ITitleBarFrameHandler                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITitleBarFrameHandler::ITitleBarFrameHandler()
 : ITitleBarFrameHandler::Inherited()
{}

/*------------------------------------------------------------------------------
| ITitleBarFrameHandler::~ITitleBarFrameHandler                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITitleBarFrameHandler::~ITitleBarFrameHandler()
{}

/*------------------------------------------------------------------------------
| ITitleBarFrameHandler::format                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean ITitleBarFrameHandler::format( IFrameFormatEvent &formatEvent )
{
#ifdef IC_WIN
   // Since we use the default title bar size, we can use the standard
   // format as well.
   return Inherited::format( formatEvent );
#else

  RECTL
    temp = { 0, 0 };

  IFrameFormatEvent
    localEvent = formatEvent;
#ifdef __BORLANDC__
  if ( (unsigned long)formatEvent.parameter2() == 0 )
#else
  if ( formatEvent.parameter2() == 0 )
#endif
  {
    localEvent = IEvent( formatEvent.handle(),
                         formatEvent.eventId(),
                         formatEvent.parameter1(),
                         &temp );
  }

  // Get count of standard controls (and thus index of extensions)...
  unsigned long
    controls(this -> defaultProcedure( localEvent ));

  ISWPArray
   swpArray( formatEvent.swpArray(),
             controls + this->numberOfExtensions( formatEvent.frame() ) );

  ISWP &titleSWP     (swpArray[ swpArray.indexOf( FID_TITLEBAR )]);
  ISWP &clientSWP    (swpArray[ swpArray.indexOf( FID_CLIENT )]);
  ISWP &systemMenuSWP(swpArray[ swpArray.indexOf( FID_SYSMENU )]);

  // make the titlebar wider and smaller in height
  titleSWP.setSize( ISize( titleSWP.size().width() + IToolBarTitle::offSet,
                           titleSWP.size().height() - IToolBarTitle::offSet));

  // move the title bar up and over
  titleSWP.setPosition( IPoint( titleSWP.position().x() - IToolBarTitle::offSet,
                                titleSWP.position().y() + IToolBarTitle::offSet));

  // make the system menu smaller in both x and y direction
  systemMenuSWP.setSize( ISize( systemMenuSWP.size().width() - IToolBarTitle::offSet,
                                systemMenuSWP.size().height() - IToolBarTitle::offSet));
  // move the system menu up
  systemMenuSWP.setPosition( IPoint( systemMenuSWP.position().x(),
                                     systemMenuSWP.position().y() + IToolBarTitle::offSet));

  // make the client taller
  clientSWP.setSize( ISize( clientSWP.size().width(),
                            clientSWP.size().height() + IToolBarTitle::offSet));

  unsigned long widthBeforePositioningCtrls(titleSWP.size().width());

  // position the remaining extensions
  formatEvent.setResult( this->positionExtensions( localEvent, controls ));

  // Remove the two buttons to the left of the titlebar if there
  // is insufficient room to display them.
  if (widthBeforePositioningCtrls < 2 * titleBarHeight)
  {
    ISWP &moveButton    (swpArray[controls]);
    ISWP &showHideButton(swpArray[controls+1]);
    moveButton.setSize(ISize(0,0));
    showHideButton.setSize(ISize(0,0));
    titleSWP.setSize( ISize(widthBeforePositioningCtrls, titleSWP.size().height()));
  } /* endif */
  return true;
#endif
}

/*------------------------------------------------------------------------------
| ITitlePaintHandler::ITitlePaintHandler                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITitlePaintHandler::ITitlePaintHandler()
 : ITitlePaintHandler::Inherited()
{}

/*------------------------------------------------------------------------------
| ITitlePaintHandler::~ITitlePaintHandler                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITitlePaintHandler::~ITitlePaintHandler()
{}

/*------------------------------------------------------------------------------
| ITitlePaintHandler::paintWindow                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean ITitlePaintHandler::paintWindow( IPaintEvent& event )
{
  IMODTRACE_ALL("ITitlePaintHandler::paintWindow");
  IToolBarTitle* toolBarTitle((IToolBarTitle*)event.window());
  IGraphicContext gc(event.presSpaceHandle());

  this->defaultProcedure( event );
  paintTitleText( *toolBarTitle, gc );

  return true;
}

/*------------------------------------------------------------------------------
| ITitlePaintHandler::dispatchHandlerEvent                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean ITitlePaintHandler::dispatchHandlerEvent( IEvent& event )
{

#ifdef IC_PM
  // If the BIDI attributes change, repaint the title bar
  if (event.eventId() == 0xBD0)
     event.window()->postEvent(TBM_SETHILITE);
#endif

#ifndef IC_WIN
  if (event.eventId() == TBM_SETHILITE)
  {
    IMODTRACE_ALL("ITitlePaintHandler::dispatchHandlerEvent - activated");
    IToolBarTitle* toolBarTitle((IToolBarTitle*)event.window());
    IGraphicContext gc(event.window()->handle());
    this->defaultProcedure( event );

    // Get the title bar rectangle and set it as the
    // clip region to ensure the title text string does
    // not paint outside the title bar window.
    IRectangle toolBarTitleRect(toolBarTitle->rect());
    toolBarTitleRect.moveTo( IPoint(0,0) );

    // Set the clip region to the window rect and destroy
    // any existing clip region.
    IGRegion titleBarRegion( gc, toolBarTitleRect );

    gc.setClipRegion( titleBarRegion );

    // Paint the title text string.
    paintTitleText( *toolBarTitle, gc );

    // Clear the clip region
    gc.clearClipRegion();

    return true;
  }
  else
#endif
  {
    return ITitlePaintHandler::Inherited::dispatchHandlerEvent( event );
  }
}

/*------------------------------------------------------------------------------
| ITitlePaintHandler::paintWindow                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITitlePaintHandler& ITitlePaintHandler::paintTitleText( const IToolBarTitle& toolBarTitle,
                                                        IGraphicContext& gc )
{
  IMODTRACE_ALL("ITitlePaintHandler::paintTitleText");
#ifndef IC_WIN
  //Function not needed in Windows since we use the standard title bar.

  IGraphicBundle titleStr;

  IFont titleFont(&toolBarTitle);

  if (IQUERYACTIVEWINDOW( HWND_DESKTOP ) == toolBarTitle.owner()->handle())
  {
    titleStr.setPenColor( IGUIColor(IGUIColor::activeTitleText ))
            .setBackgroundColor( IGUIColor(IGUIColor::activeTitleTextBgnd ));
  }
  else
  {
    titleStr.setPenColor( IGUIColor(IGUIColor::inactiveTitleText ))
            .setBackgroundColor( IGUIColor(IGUIColor::inactiveTitleTextBgnd ));
  }
  titleStr.setBackgroundMixMode( IGraphicBundle::backOverPaint );

  // Assume we should start the text from the left side but check for
  // right-to-left layout
  IPoint pt(titleFont.avgCharWidth(), titleFont.maxLowercaseDescender()+1);

#ifdef IC_PM
  if ( IBidiSettings::isBidiSupported() )
  {
    IBidiSettings bidi(toolBarTitle);
    if ( bidi.windowLayout() == IBidiSettings::layoutRightToLeft )
    {
      unsigned long txtWidth = titleFont.textWidth(toolBarTitle.text(),bidi);
      pt.setX(toolBarTitle.size().width() - txtWidth - pt.x());
    }
  }
#endif
  IGString titleText(toolBarTitle.text(), pt);

  titleText.setGraphicBundle( titleStr );
  titleText.drawOn(gc);
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBarTitle::IToolBarTitle                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
#ifdef IC_WIN

IToolBarTitle::IToolBarTitle( IWindow* owner, char* titleText )
                            : IToolBarTitle::Inherited( owner, titleText )
{
   // In Windows, we use the default system painting.  Therefore,
   // we let the base class (ITitle) handle everything.
}

#else

IToolBarTitle::IToolBarTitle( IWindow* owner, char* titleText )
                            : IToolBarTitle::Inherited( owner, "" ),
                              paintHandler(),
                              title(titleText)
{
  this->Inherited::setText("");
  paintHandler.handleEventsFor( this );

  // Set the default font to be Helv.8
  // Note: DBCS OS/2 systems should use the default font instead of Helv
  //       since the double-byte characters do not exist fot this font

#ifdef IC_PM
  unsigned long codepage = WinQueryCp(HMQ_CURRENT);
  switch (codepage)
  {
    // Note: The following list represents the DBCS code pages supported
    //       by OS/2 2.1 and Warp as of June 1995

    case  949: // Korean                      (replaces older 934/944)
    case  944: // Korean
    case  934: // Korean
    case 1381: // People's Republic of China  (replaces older 936/946)
    case  936: // People's Republic of China
    case  946: // People's Republic of China
    case  950: // Taiwan                      (replaces older 938/948)
    case  938: // Taiwan
    case  948: // Taiwan
    case  932: // Japanese
    case  942: // Japanese
    {
      IFont fnt(this);
      fnt.setPointSize(8);
      fnt.setWindowFont(this);
    }
    break;
    default:
    {
#endif
      IFont fnt("Helv",8);
      fnt.setWindowFont(this);
#ifdef IC_PM
    }
    break;
  }
#endif
}

#endif

/*------------------------------------------------------------------------------
| IToolBarTitle::~IToolBarTitle                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarTitle::~IToolBarTitle( )
{
#ifndef IC_WIN
  paintHandler.stopHandlingEventsFor( this );
#endif
}

/*------------------------------------------------------------------------------
| IToolBarTitle::setText                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarTitle& IToolBarTitle::setText( const char* text )
{
#ifdef IC_WIN
  // Not needed since we use standard title bar
  Inherited::setText(text);
#else
  title = text;
  refresh();
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBarTitle::setText                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarTitle& IToolBarTitle::setText( const IResourceId& residText )
{
#ifdef IC_WIN
  Inherited::setText( residText );
#else
  title = residText.resourceLibrary().loadString(residText);
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBarTitle::text                                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IString IToolBarTitle::text( ) const
{
#ifdef IC_WIN
  return Inherited::text();
#else
  return title;
#endif
}

/*------------------------------------------------------------------------------
| ISystemMenuDrawHandler::ISystemMenuDrawHandler                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ISystemMenuDrawHandler::ISystemMenuDrawHandler() :
  ISystemMenuDrawHandler::Inherited()
{}

/*------------------------------------------------------------------------------
| ISystemMenuDrawHandler::~ISystemMenuDrawHandler                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ISystemMenuDrawHandler::~ISystemMenuDrawHandler()
{}

/*------------------------------------------------------------------------------
| ISystemMenuDrawHandler::draw                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean ISystemMenuDrawHandler::draw(IMenuDrawItemEvent& evt, DrawFlag& flag)
{
  IBitmapHandle bmp = IApplication::current().resourceLibrary()
                      .tryToLoadBitmap ( IC_SMALLSYSTEMMENU );
  if ( bmp )
  {
    IGraphicContext gc(evt.itemPresSpaceHandle());
    IGBitmap systemMenu( bmp );
    systemMenu.drawOn( gc );
    return true;
  }
  return false;
}

/*------------------------------------------------------------------------------
| ISystemMenuDrawHandler::setSize                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean ISystemMenuDrawHandler::setSize(IMenuDrawItemEvent& evt, ISize& newSize)
{
  newSize.setWidth(IQUERYSYSVALUE( SV_CYTITLEBAR )).setHeight( titleBarHeight );
  return true;
}


/*------------------------------------------------------------------------------
| ITitleBarCommandHandler::ITitleBarCommandHandler                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITitleBarCommandHandler::ITitleBarCommandHandler() {}

/*------------------------------------------------------------------------------
| ITitleBarCommandHandler::~ITitleBarCommandHandler                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITitleBarCommandHandler::~ITitleBarCommandHandler() {}

/*------------------------------------------------------------------------------
| ITitleBarCommandHandler::command                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean ITitleBarCommandHandler::command( ICommandEvent& event )
{
  switch (event.commandId())
  {
    case IC_ID_SHOWHIDE:
    {
      IToolBar* toolBar(((IToolBar*)((IToolBarFrameWindow*)event.window())->client()));
#ifdef IC_WIN
      if (toolBar)
      {
        toolBar->expand( !toolBar->isExpanded() );
        // toggle state of tool bar expand/collapse
      }
#else
      IToolBarButton *showHideButton((IToolBarButton*)
                     IWindow::windowWithOwner( IC_ID_SHOWHIDE, event.window()));
      if (showHideButton->userData())
      {
        showHideButton->setBitmap( IC_ID_HIDE );
        showHideButton->setUserData( false );
        toolBar->expand( );
        // show the tool bar
      }
      else
      {
        showHideButton->setBitmap( IC_ID_SHOW );
        showHideButton->setUserData( true );
        toolBar->collapse( );
        // hide the tool bar
      }
#endif
    }
    break;
#ifndef IC_WIN
    // IFrameWindow::noMoveWithOwner ignored in Windows (always on)
    case IC_ID_OWNERMOVE:
    {
      IToolBarFrameWindow *toolBarFrame((IToolBarFrameWindow*)event.window());
      IToolBarButton *moveButton((IToolBarButton*)
                   IWindow::windowWithOwner( IC_ID_OWNERMOVE, event.window()));

      unsigned long tbfStyle(ISTYLEOF(toolBarFrame->handle()));
      if (moveButton->isLatched())
      {
        ISETWINDOWULONG(toolBarFrame->handle(), QWL_STYLE,
                        (tbfStyle & ~FS_NOMOVEWITHOWNER ));
      }
      else
      {
        ISETWINDOWULONG(toolBarFrame->handle(), QWL_STYLE,
                        (tbfStyle | FS_NOMOVEWITHOWNER ));
        // move with owner
      }
    }
    break;
#endif
    default:
    {
      // Route the command event to the frame which owns the toolbar frame,
      // if there is one.
      IToolBarFrameWindow *toolBarFrame((IToolBarFrameWindow*)event.window());
      IWindow             *owner (toolBarFrame ? toolBarFrame->owner() : 0);
      if (owner)
         owner->sendEvent( event );
    }
    break;
  } /* endswitch */
  return false;
}

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| ITitleBarCommandHandler::systemCommand                                       |
------------------------------------------------------------------------------*/
IBase::Boolean ITitleBarCommandHandler::systemCommand( ICommandEvent& event )
{
  // We use restore and maximize to collapse and expand the toolbar,
  // respectively
  switch (event.commandId() & 0xfffffff0)
  {
     case  SC_RESTORE:
     {
        IToolBar* toolBar(((IToolBar*)((IToolBarFrameWindow*)event.window())->client()));
        if (toolBar && toolBar->isExpanded())
           {
           // Need to set the normal position so the toolbar will not
           // move during the restore.
           WINDOWPLACEMENT wp;
           RECT            position;
           POINT           delta;
           wp.length = sizeof( WINDOWPLACEMENT );
           GetWindowRect( event.handle(), &position);    // current position
           GetWindowPlacement( event.handle(), &wp);     // current placement
           delta.x =  position.left - wp.rcNormalPosition.left;
           delta.y =  position.top  - wp.rcNormalPosition.top ;
           wp.rcNormalPosition.left   += delta.x;
           wp.rcNormalPosition.right  += delta.x;
           wp.rcNormalPosition.top    += delta.y;
           wp.rcNormalPosition.bottom += delta.y;
           SetWindowPlacement( event.handle(), &wp);
           event.handle().postEvent( WM_COMMAND,
                                     IEventData(IC_ID_SHOWHIDE),
                                     IEventData(0) );
           }
     }
     break;
     case  SC_MAXIMIZE:
     {
        IToolBar* toolBar(((IToolBar*)((IToolBarFrameWindow*)event.window())->client()));
        if (toolBar && !toolBar->isExpanded())
           {
           // We ensure the maximize position is correct in
           // dispatchHandlerEvent since we need to set the size also.
           event.handle().postEvent( WM_COMMAND,
                                     IEventData(IC_ID_SHOWHIDE),
                                     IEventData(0) );
           }
     }
     break;
  } /* endswitch */
  return false;
}
#endif

/*------------------------------------------------------------------------------
| ITitleBarCommandHandler::dispatchHandlerEvent                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean ITitleBarCommandHandler::dispatchHandlerEvent( IEvent& event )
{
#ifndef IC_WIN
  if ( event.eventId() == IC_UM_TOOLBAR &&
       event.parameter1().asUnsignedLong() == IC_UM_TBAR_SETEXPAND_BTN )
  {
    IToolBarButton *showHideButton((IToolBarButton*)
                   IWindow::windowWithOwner( IC_ID_SHOWHIDE, event.window()));
    if (event.parameter2().asUnsignedLong() == true)
    {
      showHideButton->setUserData( false );
      showHideButton->setBitmap( IC_ID_HIDE );
    }
    else
    {
      showHideButton->setUserData( true );
      showHideButton->setBitmap( IC_ID_SHOW );
    }
    return false;
  } // if
#else
  if ( event.eventId() == WM_GETMINMAXINFO)
  {
    // We basically allow no changes in the size/position for maximize,
    // since all we really want changed by the system is the visuals.
    // The setLayoutDistorted function in IToolBar handles actual sizing.
    RECT rect;
    GetWindowRect( event.handle(), &rect );
    MINMAXINFO* mmi( (MINMAXINFO*)(event.parameter2().asUnsignedLong()) );
    mmi->ptMaxSize.x = rect.right - rect.left;
    mmi->ptMaxSize.y = rect.bottom - rect.top;
    mmi->ptMaxPosition.x = rect.left;
    mmi->ptMaxPosition.y = rect.top;
    // other fields allowed to default
    return true;
  } // if
#endif
  else
  {
    return Inherited::dispatchHandlerEvent( event );
  }
}

/*------------------------------------------------------------------------------
| IToolBarFrameWindow::convertToGUIStyle                                       |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
| In Windows, this routine causes WS_EX_TOOLWINDOW style to be set.            |
| This gives us the system toolbar frame behavior.                             |
------------------------------------------------------------------------------*/
unsigned long IToolBarFrameWindow::convertToGUIStyle ( const IBitFlag& style,
                                                Boolean extendedOnly ) const
{
  // Obtain the style from the class (IFrameWindow) that we inherit from
  unsigned long ulStyle =
             Inherited::convertToGUIStyle( style, extendedOnly );

#ifdef IC_WIN
  if (extendedOnly)
  {
    // Turn on the WS_EX_TOOLWINDOW style
    ulStyle |= WS_EX_TOOLWINDOW;
  }
#endif
  return( ulStyle );
}

