/*******************************************************************************
* FILE NAME: itextct1.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in itextcnh.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

#define INCL_WINWINDOWMGR
#define INCL_WINENTRYFIELDS
#define INCL_WINMLE
extern "C" {
   #include <iwindefs.h>
}

#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif

#ifndef _ICONTROL_
  #include <icontrol.hpp>
#endif

#ifndef _ITEXTCTL_
  #include <itextctl.hpp>
#endif

#ifndef _ITEXTCNH_
  #include <itextcnh.hpp>
#endif

#ifndef _IEVENT_
  #include <ievent.hpp>
#endif

#ifndef _ICTLEVT_
  #include <ictlevt.hpp>
#endif

#ifndef _INOTIFEV_
  #include <inotifev.hpp>
#endif

#if (!defined(IC_MOTIF) && !defined(_IWCNAME_))
  #include <iwcname.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ITEXTCT1_CPP_
  #include <ipagetun.h>
#endif

// Class to clean up static objects
class ITextControlNotifyStatics
{
public:

~ITextControlNotifyStatics ()
{
  delete (textControlNotifyHandler);
  textControlNotifyHandler = 0;
}
static ITextControlNotifyHandler
 &notificationHandler ( );

static ITextControlNotifyHandler
 *textControlNotifyHandler;
};

// Initialize static objects and notification strings.
#pragma data_seg(ICLStaticConst)
static ITextControlNotifyStatics textControlStatics;
INotificationId const ITextControl::textId = "ITextControl::text";
#pragma data_seg()

#pragma data_seg(ICLNonConst)
ITextControlNotifyHandler*
  ITextControlNotifyStatics::textControlNotifyHandler = 0;
#pragma data_seg()

/*------------------------------------------------------------------------------
| ITextControlNotifyHandler::ITextControlNotifyHandler                         |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
ITextControlNotifyHandler::ITextControlNotifyHandler()
{}

/*------------------------------------------------------------------------------
| ITextControlNotifyHandler::~ITextControlNotifyHandler                        |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
ITextControlNotifyHandler::~ITextControlNotifyHandler()
{}

/*------------------------------------------------------------------------------
| ITextControlNotifyHandler::dispatchHandlerEvent                              |
|                                                                              |
| This notifies ITextControl Windows of the following events:                  |
|                                                                              |
| text(Id)                                                                     |
------------------------------------------------------------------------------*/
#ifdef IC_PMWIN
Boolean ITextControlNotifyHandler::dispatchHandlerEvent ( IEvent& anEvent )
{
  IWindow* thisWindow = 0;

  switch (anEvent.eventId())
  {
    case WM_CONTROL:
    {
      /**************************************************************/
      /* Never notify if we receive this message from the setText() */
      /* function.  The setText() function will send a notification */
      /* itself and we do not want to notify twice.                 */
      /**************************************************************/
      IControlEvent ctlevt(anEvent);
      IWindow* pwndControl = ctlevt.controlWindow();
      unsigned long ulMsg  = ctlevt.parameter1().highNumber();
      IWindowHandle hwndControl;

      if (pwndControl)
      {
        hwndControl = pwndControl->handle();
        thisWindow  = pwndControl;
      }
      else
      {
        hwndControl = IWindow::handleWithParent(ctlevt.controlId(),
                                                ctlevt.handle());
        thisWindow  = IWindow::windowWithOwner(ctlevt.controlId(),
                                               ctlevt.window());
      }

      if (hwndControl && thisWindow)
      {
        if (!(((ITextControl*)thisWindow)->fromSetText()))
        {
          IWindowClassName className(hwndControl);
          if (((className == WC_ENTRYFIELD) && (ulMsg == EN_CHANGE)) ||
#ifdef IC_PM
              ((className == WC_MLE) && (ulMsg == MLN_CHANGE))       ||
#endif
              ((className == WC_COMBOBOX)
#ifdef IC_WIN
                && (ulMsg == CBN_EFCHANGE || ulMsg == IC_UM_CBN_EFCHANGE)))
#else
                && (ulMsg == CBN_EFCHANGE)))
#endif
          {
            thisWindow->notifyObservers(INotificationEvent(
                                        ITextControl::textId,
                                        *thisWindow));
          }
        }
      }
    break;
    } /* endcase */

    default:
      break;
  }  /* end switch */

  return (Inherited::dispatchHandlerEvent(anEvent));
}
#endif //IC_PMWIN

