/* NOSHIP */
/*******************************************************************************
* FILE NAME: icompimp.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IAswServSpace                                                            *
*     IComponentImp                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*   Portions or this component are based on work originating from Lotus.       *
*   (C) Copyright Lotus Development Corporation. 1996 - All Rights Reserved.   *
*                                                                              *
*******************************************************************************/
#ifndef _ICOMPIMP_
#define _ICOMPIMP_

#include <icompnen.hpp>
#include <idocstor.hpp>
#include <iguibndl.hpp>
#include <iview.hpp>
#include <iframe.hpp>

/*******************************************************************************
*                                                                              *
*   Class IAswServSpace                                                        *
*                                                                              *
*   Implementation of CAswServSpace - server-side toolbar space negotiation    *
*                                                                              *
*******************************************************************************/

class ASWPUBCLASS IAswServSpace : public CAswServSpace
{
public:
                                IAswServSpace(PASWCONTROL pControl);
                                ~IAswServSpace(void);

    void                        Setup( IGUIBundle& bundle );

    virtual CAswResult ASWCLIENTMETHOD AddTools( LPRECT prectBorder,
                                                 BOOL fIsMDIContainer );

    virtual void ASWCLIENTMETHOD RemoveTools( void );

    virtual CAswResult ASWCLIENTMETHOD ResizeBorder( LPCRECT prcBorder );

private:

    IGUIBundle*                 fpBundle;

};


/*******************************************************************************
*                                                                              *
*   Class IComponentImp                                                        *
*                                                                              *
*******************************************************************************/

class IComponentImp : public CAswObjectInstance
{
public:
                                IComponentImp(IComponent*);
   virtual                      ~IComponentImp();
    
   // Setup a CAswObjectInstance object
   CAswResult                   Setup(IGUIBundle*, IModel*);
   void                         SetStorage(IDocumentStorage* s) { fpStorage = s; }

   // Shutdown a CAswObjectInstance object
   virtual CAswResult           Shutdown(BOOL fEmergency = FALSE);

   // ------------------------
   // Public Methods
   // ------------------------
   CAswSourceFormatSet*         formatSet() { return fpAswFormatSet; }
   BOOL                         isEmbedded() { return fEmbedded; }

   // Host names
   const IString&               hostAppName() { return fstrHostAppName; }
   const IString&               hostObjName() { return fstrHostObjName; }
   const IString&               hostName()
                                {
                                    if( fstrHostObjName.length() > 0 )
                                        return fstrHostObjName;
                                    else
                                        return fstrHostAppName;
                                }

   //--------------------------------------------------
   // STANDARD SERVER METHODS
   //--------------------------------------------------

   // method called to notify an application that an object is closing
   virtual CAswResult ASWCLIENTMETHOD OnClose(DWORD dwSaveOption);

   // notification from framework to shut down this instance
   virtual void ObjectReleased();

   // informs the application that an object is being activated for
   // out-of-place editing.
   virtual CAswResult ASWCLIENTMETHOD Launch(LONG iVerb);

   // inform an application that an object is being hidden
   virtual CAswResult ASWCLIENTMETHOD Hide(LONG iVerb);

   // method called to provide container name and object name to server for
   // display
   virtual CAswResult ASWCLIENTMETHOD
   SetHostNames(LPASWCOLESTR pszContainerApp, LPASWCOLESTR pszContainerObj);


   //--------------------------------------------------
   // INPLACE SERVER METHODS
   //--------------------------------------------------

   // asks the server to return its window handle. Users should also re-parent.
   // In-Place objects must override this method
   virtual HWND ASWPUBMETHOD GetInPlaceServerWindow(HWND    hwndParent,
                                                    RECT    rcClip,
                                                    LPRECT  prcPos);

   // tell the server object to hide its window (InPlace editing only)
   virtual BOOL ASWPUBMETHOD HideInPlaceWindow(void);

   // tell the server object to show its window (InPlace editing only)
   virtual BOOL ASWPUBMETHOD ShowInPlaceWindow(void);

   // tell the server object that it's going inplace active
   virtual void ASWPUBMETHOD OnInPlaceActivate(void);

   // tell the server object that it's doing and inplace deactivate
   // the server object should reparent its window
   virtual void ASWPUBMETHOD OnInPlaceDeactivate(void);

   // adjust the object rect during resize operations.
   virtual void ASWPUBMETHOD AdjustObjectRect(LPRECT prcObject);


   //--------------------------------------------------
   // SERVER RESIZE METHODS
   //--------------------------------------------------

   // set the rectangular limits for an object
   virtual CAswResult ASWCLIENTMETHOD SetExtent(DWORD dwDrawAspect,
                                                LPSIZEL lpsizel);

   // method to get the current extent that an object will actually use
   virtual CAswResult ASWCLIENTMETHOD GetExtent(DWORD dwDrawAspect,
                                                LPSIZEL lpsizel);

   //--------------------------------------------------
   // SERVER STORAGE METHODS - implementation moved to DocStorg.cpp
   //--------------------------------------------------


   // Loads the document that is contained in the given file name.
   virtual CAswResult ASWCLIENTMETHOD LoadFile(LPASWCOLESTR pszFileName,
                                               DWORD dwReserved);

   // Saves a copy of the object to the indicated file name.
   virtual CAswResult ASWCLIENTMETHOD SaveFile(LPASWCOLESTR pszFileName,
                                               BOOL fRemember);

   // indicates that the caller has saved the file with a call to SaveFile and
   // is finished working
   virtual CAswResult ASWCLIENTMETHOD SaveFileCompleted(LPASWCOLESTR pszFileName);

   // Returns absolute path name of the document's currently-associated file.
   virtual CAswResult ASWCLIENTMETHOD GetCurFile(LPASWOLESTR pszFileName,UINT cchMax);

   // Initializes the specified AswStorage instance.
   virtual CAswResult ASWCLIENTMETHOD InitNewStorage(PASWSTORAGE pAswStg);

   // asks the object to load itself
   virtual CAswResult ASWCLIENTMETHOD LoadStorage(PASWSTORAGE pAswStg);

   // Saves an object into its AswStorage instance, which includes saving any
   // nested objects.
   virtual CAswResult ASWCLIENTMETHOD SaveStorage(PASWSTORAGE   pAswStgSave,
                                                  BOOL          fSameAsLoad);

   // Signals the completion of a SaveStorage.
   virtual CAswResult ASWCLIENTMETHOD SaveStorageCompleted(PASWSTORAGE pAswStgNew);

   // causes the object to release its storage object and enter the HANDS OFF
   // storage mode
   virtual CAswResult ASWCLIENTMETHOD HandsOffStorage(void);


   //--------------------------------------------------
   // DATA METHODS
   //--------------------------------------------------

   // Get the server's data in the specified medium
   virtual CAswResult ASWCLIENTMETHOD GetData( LPFORMATETC     lpFormatEtc,
                                               PASWSTGMEDUNION pAswStgMedium );

   // Return the native clipboard format.
   virtual CAswResult ASWCLIENTMETHOD GetNativeClipFormat(LPCLIPFORMAT pFormat);

    // render metafile into global memory
    HGLOBAL                     RenderMetafilePict( Boolean inPlaceView = true );
protected:

    //--------------------------------------------------
    // helper functions
    //--------------------------------------------------
    HWND                        frameWindowHandle()
                                { return (HWND)(void*)fpBundle->frameWindow().handle(); }
    HWND                        inPlaceViewHandle()
                                { return (HWND)(void*)fpBundle->inPlaceView().handle(); }

    // Clipboardo support methods 
    HGLOBAL                     RenderFormat(CLIPFORMAT cf);
    HGLOBAL                     GetModelData();


private:

    IComponentImp&              operator=( const IComponentImp& );
    
    //--------------------------------------------------
    // data
    //--------------------------------------------------

    // The component that owns us
    IComponent*                 fpComponent;

    // The bundle we're associated with
    IGUIBundle*                 fpBundle;
    
    // The storage to use
    IDocumentStorage*           fpStorage;
    
    // In process of shutting down
    BOOL                        fShuttingDown;

    // Our local stream is shut down
    BOOL                        fIsStreamShutdown;

    // Component is embedded in a container
    BOOL                        fEmbedded;

    // Aswan stream. This is our local storage
    CAswStream                  fAswStream;

    // supported clipboard formats
    CLIPFORMAT                  fcfNative;              // our model (absorbable)
    CLIPFORMAT                  fcfEmbedSource;         // our model as an embed source

    // The set of formats that we handle
    CAswSourceFormatSet*        fpAswFormatSet;

    // inplace menu handler
    CAswServMenu                fServMenu;
    ASWSHAREDMENU*              fSharedMenu;

    // inplace tool bar space
    IAswServSpace               fServFrameSpace;

    // host names
    IString                     fstrHostAppName;
    IString                     fstrHostObjName;

    // zoom factor (for extent scaling)
    double                      fdZoomX;
    double                      fdZoomY;

};

#endif
