/*******************************************************************************
* FILE NAME: immmaud.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immmaud.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
extern "C"
   {
   #include <string.h>
   #define INCL_MCIOS2
   #define IC_MULTIMEDIA
   #include <immdefs.h>
   }

#ifndef _ITRACE_
   #include <itrace.hpp>
#endif
#ifndef _IMMEXCPT_
   #include <immexcpt.hpp>
#endif

#include <immmaud.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IFRAME_CPP_
  #include <ipagetun.h>
#endif

#ifdef IC_PMWIN

/*------------------------------------------------------------------------------
| IMMMasterAudioData                                                           |
| IMMDevice proxy class for IMMMasterAudio.                                    |
|                                                                              |
------------------------------------------------------------------------------*/
class IMMMasterAudioData : public IMMDevice
{
public:
  IMMMasterAudioData()
    : IMMDevice(0, IString()) // master audio is device id 0
     { }
public:
IMMDevice
  &sendCommand(unsigned short message, unsigned long param1, void *param2)
         {return IMMDevice::sendCommand(message,param1,param2);};

IMMDevice
  &setLastError  (unsigned long errorId)
         {return IMMDevice::setLastError(errorId);};

unsigned long
  lastError() const
         {return IMMDevice::lastError();};

#ifdef IC_WIN
IMMDevice
  &setVolume   (unsigned long volume)
         {return IMMDevice::setVolume(volume);};

unsigned long
  volume       (void) const
         {return IMMDevice::volume(all);};

Boolean mixerControlValues(
                         unsigned long dwComponentType,
                         unsigned long dwControlType,
                         unsigned short *usDetails,
                         Boolean uniform,
                         unsigned long multipleItems=0)
  {return IMMDevice::mixerControlValues(
                         dwComponentType,
                         dwControlType,
                         usDetails,
                         uniform,
                         multipleItems);};

Boolean setMixerControlValues(
                         unsigned long dwComponentType,
                         unsigned long dwControlType,
                         unsigned short *usDetails,
                         Boolean uniform,
                         unsigned long multipleItems=0)
  {return IMMDevice::setMixerControlValues(
                         dwComponentType,
                         dwControlType,
                         usDetails,
                         uniform,
                         multipleItems);};
#endif

};

/*------------------------------------------------------------------------------
| IMMMasterAudio::IMMMasterAudio                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMMasterAudio::IMMMasterAudio()
              : fMasterAudioData(new IMMMasterAudioData())
{
  IMODTRACE_DEVELOP("IMMMasterAudio::ctor");
  // JAD - we should verify that a mixer device exists here.
}

/*------------------------------------------------------------------------------
| IMMMasterAudio::~IMMMasterAudio                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMMasterAudio::~IMMMasterAudio()
{
  IMODTRACE_DEVELOP("IMMMasterAudio::dtor");

  if (fMasterAudioData)
     delete fMasterAudioData;
}

/*------------------------------------------------------------------------------
| IMMMasterAudio::setVolume                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMMasterAudio& IMMMasterAudio::setVolume(unsigned long vol,
                                          IMMDevice::CallType call)
{
  IMODTRACE_DEVELOP("IMMMasterAudio::setVolume");

#ifdef IC_PM
  MCI_MASTERAUDIO_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  parms.ulMasterVolume = (unsigned long)vol;
  fMasterAudioData->sendCommand(MCI_MASTERAUDIO, MCI_MASTERVOL | call, &parms);
  if (fMasterAudioData->lastError())
     ITHROWMMERROR(fMasterAudioData->lastError(),"sendCommand");
#endif
#ifdef IC_WIN
  fMasterAudioData->setVolume(vol);
#endif

  return *this;
}

/*------------------------------------------------------------------------------
| IMMMasterAudio::enableSpeakers                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMMasterAudio& IMMMasterAudio::enableSpeakers(Boolean enable,
                                               IMMDevice::CallType call)
{
  IMODTRACE_DEVELOP("IMMMasterAudio::enableSpeakers");

#ifdef IC_PM
  MCI_MASTERAUDIO_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  fMasterAudioData->sendCommand(MCI_MASTERAUDIO,
              MCI_SPEAKERS | call | (unsigned long) enable, &parms);
  if (fMasterAudioData->lastError())
     ITHROWMMERROR(fMasterAudioData->lastError(),"sendCommand");
#endif
#ifdef IC_WIN
  unsigned short      usDetails[2];

  if (enable)
     usDetails[0] = 0; // for mute, 0=off
  else
     usDetails[0] = 1; // for mute, 1=on

  if (!fMasterAudioData->setMixerControlValues(
              MIXERLINE_COMPONENTTYPE_DST_SPEAKERS,       // master mute
              MIXERCONTROL_CONTROLTYPE_MUTE,
              usDetails, true))    // multiple items = 0 for mute
     ITRACE_DEVELOP("IMMMasterAudio::disableSpeakers - setMixerControlValues error");
#endif

  return *this;
}

/*------------------------------------------------------------------------------
| IMMMasterAudio::disableSpeakers                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMMasterAudio& IMMMasterAudio::disableSpeakers(IMMDevice::CallType call)
{
  IMODTRACE_DEVELOP("IMMMasterAudio::disableSpeakers");

#ifdef IC_PMWIN
  enableSpeakers(false,call);
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| IMMMasterAudio::enableHeadphones                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMMasterAudio& IMMMasterAudio::enableHeadphones(Boolean enable,
                                                 IMMDevice::CallType call)
{
  IMODTRACE_DEVELOP("IMMMasterAudio::enableHeadphones");

#ifdef IC_PM
  MCI_MASTERAUDIO_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  fMasterAudioData->sendCommand(MCI_MASTERAUDIO, MCI_HEADPHONES |
              call | (unsigned long)enable, &parms);
  if (fMasterAudioData->lastError())
     ITHROWMMERROR(fMasterAudioData->lastError(),"sendCommand");
#endif
#ifdef IC_WIN
  unsigned short      usDetails[2];

  if (enable)
     usDetails[0] = 0; // for mute, 0=off
  else
     usDetails[0] = 1; // for mute, 1=on

  if (!fMasterAudioData->setMixerControlValues(
              MIXERLINE_COMPONENTTYPE_DST_HEADPHONES,
              MIXERCONTROL_CONTROLTYPE_MUTE,
              usDetails, true))    // multiple items = 0 for mute
  {
     ITRACE_DEVELOP("IMMMasterAudio::enableHeadphones - setMixerControlValues error");
     // If invalid line, work with speakers (since headphones are tied to
     // the speaker in most cases, this is a valid assumption).
     enableSpeakers(enable);
  }
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| IMMMasterAudio::disableHeadphones                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMMasterAudio& IMMMasterAudio::disableHeadphones( IMMDevice::CallType call)
{
  IMODTRACE_DEVELOP("IMMMasterAudio::disableHeadphones");

#ifdef IC_PMWIN
  enableHeadphones(false,call);
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| IMMMasterAudio::saveVolume                                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMMasterAudio& IMMMasterAudio::saveVolume( IMMDevice::CallType call)
{
  IMODTRACE_DEVELOP("IMMMasterAudio::saveVolume");

#ifdef IC_PM
  MCI_MASTERAUDIO_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  fMasterAudioData->sendCommand(MCI_MASTERAUDIO,
              MCI_MASTERVOL | call | MCI_SAVESETTING, &parms);
  if (fMasterAudioData->lastError())
     ITHROWMMERROR(fMasterAudioData->lastError(),"sendCommand");
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| IMMMasterAudio::saveSpeakersSetting                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMMasterAudio& IMMMasterAudio::saveSpeakersSetting( IMMDevice::CallType call)
{
  IMODTRACE_DEVELOP("IMMMasterAudio::saveSpeakersSetting");

#ifdef IC_PM
  MCI_MASTERAUDIO_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  fMasterAudioData->sendCommand(MCI_MASTERAUDIO,
              MCI_SPEAKERS | call | MCI_SAVESETTING, &parms);
  if (fMasterAudioData->lastError())
     ITHROWMMERROR(fMasterAudioData->lastError(),"sendCommand");
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| IMMMasterAudio::saveHeadphonesSetting                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMMasterAudio& IMMMasterAudio::saveHeadphonesSetting( IMMDevice::CallType call)
{
  IMODTRACE_DEVELOP("IMMMasterAudio::saveHeadphonesSetting");

#ifdef IC_PM
  MCI_MASTERAUDIO_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  fMasterAudioData->sendCommand(MCI_MASTERAUDIO,
              MCI_HEADPHONES | call | MCI_SAVESETTING, &parms);
  if (fMasterAudioData->lastError())
     ITHROWMMERROR(fMasterAudioData->lastError(),"sendCommand");
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| IMMMasterAudio::volume                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMMasterAudio::volume(SettingSource source,
                                               IMMDevice::CallType call ) const
{
  IMODTRACE_DEVELOP("IMMMasterAudio::volume");

#ifdef IC_PM
  MCI_MASTERAUDIO_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  fMasterAudioData->sendCommand(MCI_MASTERAUDIO,
              MCI_MASTERVOL | call | source,
              &parms);
  if (fMasterAudioData->lastError())
     ITHROWMMERROR(fMasterAudioData->lastError(),"sendCommand");
  return parms.ulReturn;
#endif
#ifdef IC_WIN
  return fMasterAudioData->volume();
#endif
}

/*------------------------------------------------------------------------------
| IMMMasterAudio::areSpeakersEnabled                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMMasterAudio::areSpeakersEnabled(SettingSource source,
                                           IMMDevice::CallType call ) const
{
  IMODTRACE_DEVELOP("IMMMasterAudio::areSpeakersEnabled");

#ifdef IC_PM
  MCI_MASTERAUDIO_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  fMasterAudioData->sendCommand(MCI_MASTERAUDIO,
              MCI_SPEAKERS | call | source,
              &parms);
  if (fMasterAudioData->lastError())
     ITHROWMMERROR(fMasterAudioData->lastError(),"sendCommand");
  if (parms.ulReturn)
     return true;
#endif
#ifdef IC_WIN
  unsigned short      usDetails[2];

  if (!fMasterAudioData->mixerControlValues(
              MIXERLINE_COMPONENTTYPE_DST_SPEAKERS,       // master mute
              MIXERCONTROL_CONTROLTYPE_MUTE,
              usDetails, true))    // multiple items = 0 for mute
     ITRACE_DEVELOP("IMMMasterAudio::areSpeakersEnabled - mixerControlValues error");

  if (usDetails[0])
     return false;     // mute is on; speakers are not enabled
  else
     return true;
#endif
}

/*------------------------------------------------------------------------------
| IMMMasterAudio::areHeadphonesEnabled                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMMasterAudio::areHeadphonesEnabled(SettingSource source,
                                             IMMDevice::CallType call ) const
{
  IMODTRACE_DEVELOP("IMMMasterAudio::areHeadphonesEnabled");

#ifdef IC_PM
  MCI_MASTERAUDIO_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  fMasterAudioData->sendCommand(MCI_MASTERAUDIO,
                                MCI_HEADPHONES | call | source,
                                &parms);
  if (fMasterAudioData->lastError())
     ITHROWMMERROR(fMasterAudioData->lastError(),"sendCommand");
  if (parms.ulReturn)
     return true;
#endif
#ifdef IC_WIN
  unsigned short      usDetails[2];

  if (!fMasterAudioData->mixerControlValues(
              MIXERLINE_COMPONENTTYPE_DST_HEADPHONES,
              MIXERCONTROL_CONTROLTYPE_MUTE,
              usDetails, true))    // multiple items = 0 for mute
  {
     ITRACE_DEVELOP("IMMMasterAudio::areHeadphonesEnabled - mixerControlValues error");
     // If invalid line, use state of speakers (since headphones are tied to
     // the speaker in most cases, this is a valid assumption).
     return areSpeakersEnabled();
  }
  else
  {
     // If valid line, return actual state
     if (usDetails[0])
        return false;     // mute is on; speakers are not enabled
     else
        return true;
  }
#endif
}
#endif /* IC_PMWIN */
