/*******************************************************************************
* FILE NAME: igregion.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in igregion.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
{
  #include <iwindefs.h>
  #include <math.h>
  #include <stdlib.h>
}

#include <iwindow.hpp>
#include <igregion.hpp>
#include <igrafctx.hpp>
#include <igcdata.hpp>
#include <igrect.hpp>
#include <igpie.hpp>
#include <igpyline.hpp>
#include <igelipse.hpp>
#include <iexcept.hpp>
#include <itrnsfrm.hpp>
#include <igbundle.hpp>
#include <iplatfrm.hpp>

/*------------------------------------------------------------------------------
| IGRegion::IGRegion                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion::IGRegion( )
  : IGRegion::Inherited(), fRegion(0)
{
  if ((fRegion = CreateRectRgn(0,0,0,0)) == 0)
    ITHROWGUIERROR("CreateRectRgn");
}

/*------------------------------------------------------------------------------
| IGRegion::IGRegion                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion::IGRegion( const IRegionHandle& regionHandle )
  : IGRegion::Inherited(), fRegion(regionHandle)
{
}

/*------------------------------------------------------------------------------
| IGRegion::IGRegion                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion::IGRegion( const IGRegion& region )
  : IGRegion::Inherited( region ), fRegion(0)
{
  if ((fRegion = CreateRectRgn(0,0,0,0)) == 0)
    ITHROWGUIERROR("CreateRectRgn");

  CombineRgn( fRegion, region.fRegion, region.fRegion, RGN_COPY );
}

/*------------------------------------------------------------------------------
| IGRegion::IGRegion                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion::IGRegion( const IGraphicContext& graphicContext,
                    const IRectangle& rectangle )
  : IGRegion::Inherited()
{
  fRegion = CreateRectRgn( rectangle.topLeft().x(),
                           rectangle.topLeft().y(),
                           rectangle.bottomRight().x(),
                           rectangle.bottomRight().y());
  if ( fRegion == 0 )
  {
    ITHROWGUIERROR2("CreateRectRgn",
                    IBaseErrorInfo::invalidRequest,
                    IException::recoverable);
  }
}

/*------------------------------------------------------------------------------
| IGRegion::IGRegion                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion::IGRegion( const IWindowHandle& windowHandle )
  : IGRegion::Inherited()
{
  IGraphicContext graphicContext( IWindow::desktopWindow()->handle() );
  RECTL rcl;

  if ((fRegion = CreateRectRgn(0,0,0,0)) == 0)
    ITHROWGUIERROR("CreateRectRgn");

  if (GetUpdateRgn( windowHandle, fRegion, false ) == ERROR )
  {
    ITHROWGUIERROR2("GetUpdateRgn",
                    IBaseErrorInfo::invalidRequest,
                    IException::recoverable);
  }
}


/*------------------------------------------------------------------------------
| IGRegion::~IGRegion                                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion::~IGRegion( )
{
  if (fRegion)
  {
    if (!DeleteObject(fRegion))
      ITHROWGUIERROR("DeleteObject");
  }
}

/*------------------------------------------------------------------------------
| IGRegion::moveBy                                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::moveBy( const IPoint& point )
{
  if (OffsetRgn( fRegion, point.x(), point.y() ) == ERROR)
  {
    ITHROWGUIERROR2("OffsetRgn",
                    IBaseErrorInfo::invalidRequest,
                    IException::recoverable);
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::clear                                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::clear( )
{
  if (CombineRgn( fRegion, fRegion, fRegion, RGN_XOR ) == ERROR)
  {
    ITHROWGUIERROR2("CombineRgn",
                    IBaseErrorInfo::invalidRequest,
                    IException::recoverable);
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::drawOn                                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::drawOn( IGraphicContext& graphicContext )
{
  graphicContext.draw( *this );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::operator                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion::operator const IRegionHandle( ) const
{
  return fRegion;
}

/*------------------------------------------------------------------------------
| IGRegion::opertor+= (IGRegion)                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::operator+= ( const IGRegion& region )
{
  CombineRgn( fRegion, fRegion, region.fRegion, RGN_OR );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::opertor-= (IGRegion)                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::operator-= ( const IGRegion& region )
{
  CombineRgn( fRegion, fRegion, region.fRegion, RGN_DIFF );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::opertor&= (IGRegion)                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::operator&= ( const IGRegion& region )
{
  CombineRgn( fRegion, fRegion, region.fRegion, RGN_AND );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::opertor^= (IGRegion)                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::operator^= ( const IGRegion& region )
{
  CombineRgn( fRegion, fRegion, region.fRegion, RGN_XOR );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::opertor= (IGRegion)                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::operator= ( const IGRegion& region )
{
  if (this == &region)
    return *this;

  this->Inherited::operator=( region );
  CombineRgn( fRegion, region.fRegion, region.fRegion, RGN_COPY );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::opertor+= (IGRectangle)                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::operator+= ( const IGRectangle& rect )
{
  IGraphicContext graphicContext( IWindow::desktopWindow()->handle() );
  IRegionHandle newRegion(doRect(graphicContext, rect));
  if (CombineRgn( fRegion, fRegion, newRegion, RGN_OR ) == ERROR)
    ITHROWGUIERROR("CombineRgn");
  DeleteObject( newRegion );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::opertor-= (IGRectangle)                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::operator-= ( const IGRectangle& rect )
{
  IGraphicContext graphicContext( IWindow::desktopWindow()->handle() );
  IRegionHandle newRegion(doRect(graphicContext, rect));
  CombineRgn( fRegion, fRegion, newRegion, RGN_DIFF );
  DeleteObject( newRegion );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::opertor&= (IGRectangle)                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::operator&= ( const IGRectangle& rect )
{
  IGraphicContext graphicContext( IWindow::desktopWindow()->handle() );
  IRegionHandle newRegion(doRect(graphicContext, rect));
  CombineRgn( fRegion, fRegion, newRegion, RGN_AND );
  DeleteObject( newRegion );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::opertor^= (IGRectangle)                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::operator^= ( const IGRectangle& rect )
{
  IGraphicContext graphicContext( IWindow::desktopWindow()->handle() );
  IRegionHandle newRegion(doRect(graphicContext, rect));
  CombineRgn( fRegion, fRegion, newRegion, RGN_XOR );
  DeleteObject( newRegion );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::opertor+= (IGPie)                                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::operator+= ( const IGPie& pie )
{
  IGraphicContext graphicContext( IWindow::desktopWindow()->handle() );
  IRegionHandle newRegion(doPie(graphicContext, pie));
  CombineRgn( fRegion, fRegion, newRegion, RGN_OR );
  DeleteObject( newRegion );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::opertor-= (IGPie)                                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::operator-= ( const IGPie& pie )
{
  IGraphicContext graphicContext( IWindow::desktopWindow()->handle() );
  IRegionHandle newRegion(doPie(graphicContext, pie));
  CombineRgn( fRegion, fRegion, newRegion, RGN_DIFF );
  DeleteObject( newRegion );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::opertor&= (IGPie)                                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::operator&= ( const IGPie& pie )
{
  IGraphicContext graphicContext( IWindow::desktopWindow()->handle() );
  IRegionHandle newRegion(doPie(graphicContext, pie));
  CombineRgn( fRegion, fRegion, newRegion, RGN_AND );
  DeleteObject( newRegion );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::opertor^= (IGPie)                                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::operator^= ( const IGPie& pie )
{
  IGraphicContext graphicContext( IWindow::desktopWindow()->handle() );
  IRegionHandle newRegion(doPie(graphicContext, pie));
  CombineRgn( fRegion, fRegion, newRegion, RGN_XOR );
  DeleteObject( newRegion );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::opertor+= (IGChord)                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::operator+= ( const IGChord& chord )
{
  IGraphicContext graphicContext( IWindow::desktopWindow()->handle() );
  IRegionHandle newRegion(doChord(graphicContext, chord));
  CombineRgn( fRegion, fRegion, newRegion, RGN_OR );
  DeleteObject( newRegion );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::opertor-= (IGChord)                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::operator-= ( const IGChord& chord )
{
  IGraphicContext graphicContext( IWindow::desktopWindow()->handle() );
  IRegionHandle newRegion(doChord(graphicContext, chord));
  CombineRgn( fRegion, fRegion, newRegion, RGN_DIFF );
  DeleteObject( newRegion );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::opertor&= (IGChord)                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::operator&= ( const IGChord& chord )
{
  IGraphicContext graphicContext( IWindow::desktopWindow()->handle() );
  IRegionHandle newRegion(doChord(graphicContext, chord));
  CombineRgn( fRegion, fRegion, newRegion, RGN_AND );
  DeleteObject( newRegion );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::opertor^= (IGChord)                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::operator^= ( const IGChord& chord )
{
  IGraphicContext graphicContext( IWindow::desktopWindow()->handle() );
  IRegionHandle newRegion(doChord(graphicContext, chord));
  CombineRgn( fRegion, fRegion, newRegion, RGN_XOR );
  DeleteObject( newRegion );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::opertor+= (IGPolygon)                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::operator+= ( const IGPolygon& polygon )
{
  IGraphicContext graphicContext( IWindow::desktopWindow()->handle() );
  IRegionHandle newRegion(doPolygon(graphicContext, polygon));
  CombineRgn( fRegion, fRegion, newRegion, RGN_OR );
  DeleteObject( newRegion );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::opertor-= (IGPolygon)                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::operator-= ( const IGPolygon& polygon )
{
  IGraphicContext graphicContext( IWindow::desktopWindow()->handle() );
  IRegionHandle newRegion(doPolygon(graphicContext, polygon));
  CombineRgn( fRegion, fRegion, newRegion, RGN_DIFF );
  DeleteObject( newRegion );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::opertor&= (IGPolygon)                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::operator&= ( const IGPolygon& polygon )
{
  IGraphicContext graphicContext( IWindow::desktopWindow()->handle() );
  IRegionHandle newRegion(doPolygon(graphicContext, polygon));
  CombineRgn( fRegion, fRegion, newRegion, RGN_AND );
  DeleteObject( newRegion );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::opertor^= (IGPolygon)                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::operator^= ( const IGPolygon& polygon )
{
  IGraphicContext graphicContext( IWindow::desktopWindow()->handle() );
  IRegionHandle newRegion(doPolygon(graphicContext, polygon));
  CombineRgn( fRegion, fRegion, newRegion, RGN_XOR );
  DeleteObject( newRegion );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::opertor+= (IGEllipse)                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::operator+= ( const IGEllipse& ellipse )
{
  IGraphicContext graphicContext( IWindow::desktopWindow()->handle() );
  IRegionHandle newRegion(doEllipse(graphicContext, ellipse));
  CombineRgn( fRegion, fRegion, newRegion, RGN_OR );
  DeleteObject( newRegion );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::opertor-= (IGEllipse)                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::operator-= ( const IGEllipse& ellipse )
{
  IGraphicContext graphicContext( IWindow::desktopWindow()->handle() );
  IRegionHandle newRegion(doEllipse(graphicContext, ellipse));
  CombineRgn( fRegion, fRegion, newRegion, RGN_DIFF );
  DeleteObject( newRegion );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::opertor&= (IGEllipse)                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::operator&= ( const IGEllipse& ellipse )
{
  IGraphicContext graphicContext( IWindow::desktopWindow()->handle() );
  IRegionHandle newRegion(doEllipse(graphicContext, ellipse));
  CombineRgn( fRegion, fRegion, newRegion, RGN_AND );
  DeleteObject( newRegion );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::opertor^= (IGEllipse)                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::operator^= ( const IGEllipse& ellipse )
{
  IGraphicContext graphicContext( IWindow::desktopWindow()->handle() );
  IRegionHandle newRegion(doEllipse(graphicContext, ellipse));
  CombineRgn( fRegion, fRegion, newRegion, RGN_XOR );
  DeleteObject( newRegion );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::doRect                                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IRegionHandle IGRegion::doRect( IGraphicContext& graphicContext, const IGRectangle& rect )
{
  IRegionHandle newRegion;
  IPair         rounding(rect.rounding());
  IRectangle    enclRect(rect.enclosingRect());

  if (IPlatform::isWin95() || IPlatform::isWin32S())
  {
    if (rect.hasTransformMatrix())
      enclRect = ((IGRectangle&)rect).boundingRect( graphicContext );

    newRegion = CreateRectRgn( enclRect.minX(), enclRect.minY(),
                               enclRect.maxX(), enclRect.maxY() );
  }
  else
  {
    if (!BeginPath(graphicContext.handle()))
      ITHROWGUIERROR2("BeginPath",
                      IBaseErrorInfo::invalidRequest,
                      IException::recoverable);

    if (rect.hasTransformMatrix())
    {
      graphicContext.setWorldTransformMatrix(rect.transformMatrix(), rect.transformMethod());
    }

    if (rect.hasGraphicBundle())
    {
      graphicContext.setGraphicBundle(rect.graphicBundle());
    }

    if (rounding.coord1() == 0 || rounding.coord2())
    {
      if (!Rectangle( graphicContext.handle(),
                      enclRect.minX(), enclRect.minY(),
                      enclRect.maxX(), enclRect.maxY()))
      {
        ITHROWGUIERROR2("Rectangle",
                        IBaseErrorInfo::invalidRequest,
                        IException::recoverable);
      }
    }
    else
      if (!RoundRect( graphicContext.handle(),
                      enclRect.left(), enclRect.top(), enclRect.right(), enclRect.bottom(),
                      rounding.coord1(), rounding.coord2()))
      {
        ITHROWGUIERROR2("RoundRect",
                        IBaseErrorInfo::invalidRequest,
                        IException::recoverable);
      }

    if (!EndPath(graphicContext.handle()))
      ITHROWGUIERROR2("EndPath",
                      IBaseErrorInfo::invalidRequest,
                      IException::recoverable);
    newRegion = PathToRegion(graphicContext.handle());
  }
  return newRegion;
}

/*------------------------------------------------------------------------------
| IGRegion::doPie                                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IRegionHandle IGRegion::doPie( IGraphicContext& graphicContext, const IGPie& pie )
{
  IRegionHandle  newRegion;
  IRectangle     rect(pie.enclosingRect());
  float          startAngle(fmod(pie.startAngle(), 360.0));
  float          endAngle(fmod((startAngle + pie.sweepAngle()), 360.0));

  if (IPlatform::isWin95() || IPlatform::isWin32S())
  {
    if (pie.hasTransformMatrix())
      rect = ((IGPie&)pie).boundingRect(graphicContext);

    newRegion = CreateEllipticRgn( rect.minX(), rect.minY(),
                                   rect.maxX(), rect.maxY() );


    if (startAngle < 0)
    {
      startAngle = -startAngle;
      startAngle = 360.0 - fmod(startAngle, 360.0);
    }
    else
    {
      startAngle = fmod(startAngle, 360.0);
    } /* endif */

    if (endAngle < 0)
    {
      endAngle = -endAngle;
      endAngle = 360.0 - fmod(endAngle, 360.0);
    }
    else
    {
      endAngle = fmod(endAngle, 360.0);
    } /* endif */

    POINTL ptArray[10];
    int    numPts(2);

    if (pie.sweepAngle() < 0)
    {
      float tempAngle(endAngle);
      endAngle = startAngle;
      startAngle = tempAngle;
    }

    Boolean textMode(graphicContext.mappingMode()==IGraphicContext::pels);

    IPoint startRadial(IGCData::radialPoint( textMode, rect, startAngle ));

    IPoint endRadial(IGCData::radialPoint( textMode, rect, endAngle ));

    ptArray[0] = rect.center().asPOINTL();
    ptArray[1] = startRadial.asPOINTL();

    if ((int)startAngle < 45 || (int)startAngle >= 315)
      ptArray[numPts++] = rect.maxXMinY().asPOINTL();
    else if ((int)startAngle < 135)
      ptArray[numPts++] = rect.minXMinY().asPOINTL();
    else if ((int)startAngle < 225 )
      ptArray[numPts++] = rect.minXMaxY().asPOINTL();
    else if ((int)startAngle < 315)
      ptArray[numPts++] = rect.maxXMaxY().asPOINTL();

    int count = abs((int)pie.sweepAngle() / 90);
    endAngle = fmod(startAngle + 90.0, 360.0);

    for (int i=0; i < count; i++, endAngle = fmod(endAngle + 90.0, 360.0))
    {
      if ((int)endAngle < 45 || (int)endAngle >= 315)
        ptArray[numPts++] = rect.maxXMinY().asPOINTL();
      else if ((int)endAngle < 135)
        ptArray[numPts++] = rect.minXMinY().asPOINTL();
      else if ((int)endAngle < 225 )
        ptArray[numPts++] = rect.minXMaxY().asPOINTL();
      else if ((int)endAngle < 315)
        ptArray[numPts++] = rect.maxXMaxY().asPOINTL();
      else if ((int)endAngle > 315)
        ptArray[numPts++] = rect.maxXMinY().asPOINTL();
    }

    ptArray[numPts++] = endRadial.asPOINTL();
    ptArray[numPts++] = rect.center().asPOINTL();

    HRGN    hrgnMask = CreatePolygonRgn((const POINT*)ptArray, numPts, WINDING );
    if ( hrgnMask )
    {
       int  rc = CombineRgn( newRegion, newRegion, hrgnMask, RGN_AND );
       DeleteObject( hrgnMask );
       switch ( rc )
       {
           case COMPLEXREGION:
           case NULLREGION:
           case SIMPLEREGION:
              break;
           default:
              ITHROWGUIERROR2("CombineRgn",     //region probably to big
                              IBaseErrorInfo::invalidRequest,
                              IException::recoverable);
       } /* endswitch */
    }
    else
    {
      ITHROWGUIERROR2("CreatePolygonRgn",       //region probably to big
                      IBaseErrorInfo::invalidRequest,
                      IException::recoverable);
    } /* endif */
  }
  else
  {
    if (!BeginPath(graphicContext.handle()))
      ITHROWGUIERROR2("BeginPath",
                      IBaseErrorInfo::invalidRequest,
                      IException::recoverable);
    if (pie.hasTransformMatrix())
    {
      graphicContext.setWorldTransformMatrix(pie.transformMatrix(), pie.transformMethod());
    }

    if (pie.hasGraphicBundle())
    {
      graphicContext.setGraphicBundle(pie.graphicBundle());
    }

    Boolean textMode(graphicContext.mappingMode()==IGraphicContext::pels);

    IPoint startRadial(IGCData::radialPoint( textMode, rect, startAngle ));

    IPoint endRadial(IGCData::radialPoint( textMode, rect, endAngle ));


    if (!Pie( graphicContext.handle(),
              rect.left(), rect.top(),
              rect.right(), rect.bottom(),
              startRadial.x(),
              startRadial.y(),
              endRadial.x(),
              endRadial.y() ))
    {
      ITHROWGUIERROR2("Pie",
                      IBaseErrorInfo::invalidRequest,
                      IException::recoverable);
    }

    if (!EndPath(graphicContext.handle()))
      ITHROWGUIERROR2("EndPath",
                      IBaseErrorInfo::invalidRequest,
                      IException::recoverable);
    newRegion = PathToRegion(graphicContext.handle());
  }
  return newRegion;
}

/*------------------------------------------------------------------------------
| IGRegion::doChord                                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IRegionHandle IGRegion::doChord( IGraphicContext& graphicContext, const IGChord& chord )
{
  IRegionHandle  newRegion;
  IRectangle     rect(chord.enclosingRect());
  float          startAngle(fmod(chord.startAngle(), 360.0));
  float          endAngle(fmod((startAngle + chord.sweepAngle()), 360.0));

  if (IPlatform::isWin95() || IPlatform::isWin32S())
  {
    if (chord.hasTransformMatrix())
      rect = ((IGChord&)chord).boundingRect(graphicContext);

    newRegion = CreateEllipticRgn( rect.minX(), rect.minY(),
                                   rect.maxX(), rect.maxY() );


    if (startAngle < 0)
    {
      startAngle = -startAngle;
      startAngle = 360.0 - fmod(startAngle, 360.0);
    }
    else
    {
      startAngle = fmod(startAngle, 360.0);
    } /* endif */

    if (endAngle < 0)
    {
      endAngle = -endAngle;
      endAngle = 360.0 - fmod(endAngle, 360.0);
    }
    else
    {
      endAngle = fmod(endAngle, 360.0);
    } /* endif */

    POINTL ptArray[10];
    int    numPts(2);

    if (chord.sweepAngle() < 0)
    {
      float tempAngle(endAngle);
      endAngle = startAngle;
      startAngle = tempAngle;
    }

    Boolean textMode(graphicContext.mappingMode()==IGraphicContext::pels);

    // Use the parametric form of an ellipse to find the starting and ending
    // points of the chord on the ellipse.  This assumes the ellipse is
    // centered at the origin.
    IPoint startRadial((rect.width()/2 * cos( startAngle/57.29577951 )),
                       (rect.height()/2 * sin( startAngle/57.29577951 )));

    IPoint endRadial((rect.width()/2 * cos( endAngle/57.29577951 )),
                     (rect.height()/2 * sin( endAngle/57.29577951 )));


    startRadial.setX( rect.minX() + rect.width()/2 + startRadial.x() );
    startRadial.setY( rect.minY() + rect.height()/2 - startRadial.y() );

    endRadial.setX( rect.minX() + rect.width()/2 + endRadial.x() );
    endRadial.setY( rect.minY() + rect.height()/2 - endRadial.y() );


    ptArray[0] = startRadial.asPOINTL();
    ptArray[1] = IGCData::radialPoint( textMode, rect, startAngle ).asPOINTL();

    if ((int)startAngle < 45 || (int)startAngle >= 315)
      ptArray[numPts++] = rect.maxXMinY().asPOINTL();
    else if ((int)startAngle < 135)
      ptArray[numPts++] = rect.minXMinY().asPOINTL();
    else if ((int)startAngle < 225 )
      ptArray[numPts++] = rect.minXMaxY().asPOINTL();
    else if ((int)startAngle < 315)
      ptArray[numPts++] = rect.maxXMaxY().asPOINTL();

    int count = abs((int)chord.sweepAngle() / 90);
    float tempAngle = fmod(startAngle + 90.0, 360.0);

    for (int i=0; i < count; i++, tempAngle = fmod(tempAngle + 90.0, 360.0))
    {
      if ((int)tempAngle < 45 || (int)tempAngle >= 315)
        ptArray[numPts++] = rect.maxXMinY().asPOINTL();
      else if ((int)tempAngle < 135)
        ptArray[numPts++] = rect.minXMinY().asPOINTL();
      else if ((int)tempAngle < 225 )
        ptArray[numPts++] = rect.minXMaxY().asPOINTL();
      else if ((int)tempAngle < 315)
        ptArray[numPts++] = rect.maxXMaxY().asPOINTL();
      else if ((int)tempAngle > 315)
        ptArray[numPts++] = rect.maxXMinY().asPOINTL();
    }

    ptArray[numPts++] = IGCData::radialPoint( textMode, rect, endAngle ).asPOINTL();
    ptArray[numPts++] = endRadial.asPOINTL();
    ptArray[numPts++] = startRadial.asPOINTL();

    HRGN    hrgnMask = CreatePolygonRgn((const POINT*)ptArray, numPts, WINDING );
    if ( hrgnMask )
    {
       int  rc = CombineRgn( newRegion, newRegion, hrgnMask, RGN_AND );
       DeleteObject( hrgnMask );
       switch ( rc )
       {
           case COMPLEXREGION:
           case NULLREGION:
           case SIMPLEREGION:
              break;
           default:
              ITHROWGUIERROR2("CombineRgn",     //region probably to big
                              IBaseErrorInfo::invalidRequest,
                              IException::recoverable);
       } /* endswitch */
    }
    else
    {
      ITHROWGUIERROR2("CreatePolygonRgn",       //region probably to big
                      IBaseErrorInfo::invalidRequest,
                      IException::recoverable);
    } /* endif */
  }
  else
  {
    if (!BeginPath(graphicContext.handle()))
      ITHROWGUIERROR2("BeginPath",
                      IBaseErrorInfo::invalidRequest,
                      IException::recoverable);

    if (chord.hasTransformMatrix())
    {
      graphicContext.setWorldTransformMatrix(chord.transformMatrix(), chord.transformMethod());
    }

    if (chord.hasGraphicBundle())
    {
      graphicContext.setGraphicBundle(chord.graphicBundle());
    }

    Boolean textMode(graphicContext.mappingMode()==IGraphicContext::pels);

    IPoint startRadial(IGCData::radialPoint( textMode, rect, startAngle ));

    IPoint endRadial(IGCData::radialPoint( textMode, rect, endAngle ));

    if (!Chord( graphicContext.handle(),
                rect.left(), rect.top(),
                rect.right(), rect.bottom(),
                startRadial.x(),
                startRadial.y(),
                endRadial.x(),
                endRadial.y() ))
    {
      ITHROWGUIERROR2("Chord",
                      IBaseErrorInfo::invalidRequest,
                      IException::recoverable);
    }

    if (!EndPath(graphicContext.handle()))
      ITHROWGUIERROR2("EndPath",
                      IBaseErrorInfo::invalidRequest,
                      IException::recoverable);
    newRegion = PathToRegion(graphicContext.handle());
  }
  return newRegion;
}

/*------------------------------------------------------------------------------
| IGRegion::doPolygon                                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IRegionHandle IGRegion::doPolygon( IGraphicContext& graphicContext, const IGPolygon& polygon )
{
  IRegionHandle newRegion;
  LONG fillMode = (polygon.fillMode() == IGPolygon::alternate
                                          ? ALTERNATE
                                          : WINDING);
  if ( !IPlatform::isWin32S())
  {
    if (!BeginPath(graphicContext.handle()))
      ITHROWGUIERROR2("BeginPath",
                      IBaseErrorInfo::invalidRequest,
                      IException::recoverable);

    SetPolyFillMode( graphicContext.handle(), fillMode );
  }

  if (polygon.numberOfPoints() > 1)
  {
    unsigned long     arraySize(polygon.pointArray().size());
    POINTL*           pPtl(new POINTL[arraySize]);

    if (polygon.hasTransformMatrix())
    {
      graphicContext.setWorldTransformMatrix(polygon.transformMatrix(), polygon.transformMethod());
    }
    if (polygon.hasGraphicBundle())
    {
      graphicContext.setGraphicBundle(polygon.graphicBundle());
    }

    for (int i=0; i < arraySize; i++)
    {
      pPtl[i] = polygon.point(i).asPOINTL();
    } /* endfor */

    if ( IPlatform::isWin32S())
    {
      newRegion = CreatePolygonRgn((const POINT*)pPtl, arraySize, fillMode );
      delete [] pPtl;
      if ( !newRegion )
      {
        ITHROWGUIERROR2("CreatePolygonRgn",
                        IBaseErrorInfo::invalidRequest,
                        IException::recoverable);
      }
    }
    else
    {
      if (!Polygon( graphicContext.handle(), (PPOINT)pPtl, arraySize ))
      {
        ITHROWGUIERROR2("Polygon",
                        IBaseErrorInfo::invalidRequest,
                        IException::recoverable);
      }
      delete [] pPtl;
      if (!EndPath(graphicContext.handle()))
        ITHROWGUIERROR2("EndPath",
                        IBaseErrorInfo::invalidRequest,
                        IException::recoverable);

      newRegion = PathToRegion(graphicContext.handle());
    }
  }

  return newRegion;
}

/*------------------------------------------------------------------------------
| IGRegion::doEllipse                                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IRegionHandle IGRegion::doEllipse( IGraphicContext& graphicContext, const IGEllipse& ellipse )
{
  IRegionHandle newRegion;
  IRectangle    enclosingRect(ellipse.enclosingRect());

  if (IPlatform::isWin95() || IPlatform::isWin32S())
  {
    if (ellipse.hasTransformMatrix())
      enclosingRect = ((IGEllipse&)ellipse).boundingRect( graphicContext );

    newRegion = CreateEllipticRgn( enclosingRect.minX(), enclosingRect.minY(),
                                   enclosingRect.maxX(), enclosingRect.maxY() );
  }
  else
  {
    if (!BeginPath(graphicContext.handle()))
      ITHROWGUIERROR2("BeginPath",
                      IBaseErrorInfo::invalidRequest,
                      IException::recoverable);
    if (ellipse.hasTransformMatrix())
    {
      graphicContext.setWorldTransformMatrix(ellipse.transformMatrix(), ellipse.transformMethod());
    }

    if (ellipse.hasGraphicBundle())
    {
      graphicContext.setGraphicBundle(ellipse.graphicBundle());
    }

    if (!Ellipse( graphicContext.handle(), enclosingRect.left(),
                       enclosingRect.top(),
                       enclosingRect.right(),
                       enclosingRect.bottom() ))
    {
      ITHROWGUIERROR2("Ellipse",
                      IBaseErrorInfo::invalidRequest,
                      IException::recoverable);
    }

    if (!EndPath(graphicContext.handle()))
      ITHROWGUIERROR2("EndPath",
                      IBaseErrorInfo::invalidRequest,
                      IException::recoverable);
    newRegion = PathToRegion(graphicContext.handle());
  }
  return newRegion;
}

/*------------------------------------------------------------------------------
| IGRegion::rotateBy                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::rotateBy( double angle, const IPoint &pt )
{
  angle = angle;
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::scaleBy                                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::scaleBy( double xScale, double yScale, const IPoint& pt )
{
  xScale = yScale;
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::translateBy                                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::translateBy( const IPoint& pt )
{
  return *this;
}

/*------------------------------------------------------------------------------
| IGRegion::setTransformMatrix                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGRegion& IGRegion::setTransformMatrix( const ITransformMatrix& transformMatrix )
{
  return *this;
}
