/*******************************************************************************
* FILE NAME: iscroll.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iscroll.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512  (from iwindow.cpp)
#pragma priority( -2147482112 )

extern "C" {
  #define INCL_WINERRORS
  #define INCL_WINMESSAGEMGR
  #define INCL_WINSCROLLBARS
  #define INCL_WINSYS
  #define INCL_WINWINDOWMGR
  #include <iwindefs.h>
}

#include <iscroll.hpp>
#include <icconst.h>
#include <icolor.hpp>
#include <ievent.hpp>
#include <iexcept.hpp>
#include <ihandle.hpp>
#include <ihandler.hpp>
#include <inotifev.hpp>
#include <iplatfrm.hpp>
#include <ipoint.hpp>
#include <irect.hpp>
#include <itrace.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISCROLL_CPP_
  #include <ipagetun.h>
#endif

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| Private data class for IScrollBar                                            |
------------------------------------------------------------------------------*/
class IScrollBarData : public IBase {
public:
  IScrollBarData ( );

virtual
 ~IScrollBarData ( );

HBRUSH
  bkgrndbrush;

  unsigned long fvisibleCount;

}; // IScrollBarData

/*------------------------------------------------------------------------------
| IScrollBarData::IScrollBarData                                               |
|                                                                              |
------------------------------------------------------------------------------*/
IScrollBarData::IScrollBarData ( )
  : bkgrndbrush( 0 )
{ }

/*------------------------------------------------------------------------------
| IScrollBarData::~IScrollBarData                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IScrollBarData::~IScrollBarData ( )
{
  if ( bkgrndbrush != 0 )
    DeleteObject(bkgrndbrush);
}

class IScrollBarHandler : public IHandler {
typedef IHandler
  Inherited;
public:

/*------------------------------- Constructors -------------------------------*/
  IScrollBarHandler ( );

virtual
  ~IScrollBarHandler ( );

/*----------------------------- Event Processing -----------------------------*/
virtual Boolean
  dispatchHandlerEvent ( IEvent& event );

}; // class IScrollBarHandler

/*------------------------------------------------------------------------------
| IScrollBarHandler::IScrollBarHandler                                         |
|                                                                              |
| Empty constructor here for page tuning.                                      |
------------------------------------------------------------------------------*/
IScrollBarHandler::IScrollBarHandler()
{ }

/*------------------------------------------------------------------------------
| IScrollBarHandler::~IScrollBarHandler                                        |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IScrollBarHandler::~IScrollBarHandler()
{ }

/*------------------------------------------------------------------------------
| IScrollBarHandler::dispatchHandlerEvent                                      |
|                                                                              |
| Intercepts control color event                                               |
|    This handler returns false for all other events.                          |
------------------------------------------------------------------------------*/
IBase::Boolean IScrollBarHandler::dispatchHandlerEvent( IEvent& event )
{
  IMODTRACE_ALL("IScrollBarHandler::dispatchHandlerEvent");
  Boolean
    result = false;

  switch( event.eventId() )
  {
    case WM_CTLCOLORSCROLLBAR:
         {
         IWindow* ctlwin =
           IWindow::windowWithHandle( IWindowHandle(event.parameter2()) );
         IColor clrFgnd =
           ((IScrollBar *) ctlwin)->foregroundColor();
         IColor clrBgnd =
           ((IScrollBar *) ctlwin)->backgroundColor();

         // Set foreground and background text colors in device context
         SetBkColor( (HDC)event.parameter1(), clrBgnd.asRGBLong() );
         SetTextColor( (HDC)event.parameter1(), clrFgnd.asRGBLong() );

         // Return HBRUSH for painting controls background
         if ( ((IScrollBar *) ctlwin)->fScrollBarData->bkgrndbrush != 0 )
           DeleteObject( ((IScrollBar *) ctlwin)->fScrollBarData->bkgrndbrush);
         ((IScrollBar *) ctlwin)->fScrollBarData->bkgrndbrush =
           CreateSolidBrush (NEARESTPALETTECOLOR(clrBgnd.asRGBLong()));
         event.setResult( ((IScrollBar *) ctlwin)->fScrollBarData->bkgrndbrush );

         result = true;
         }
      break;

    case WM_LBUTTONDOWN:
      // Need to prevent _pfnwpICWinProc from propagating this event up
      // the owner chain, since the scroll bar window proc does not
      // finish processing this message until the mouse button has been
      // released.  This can cause much havoc on an ISplitCanvas up the
      // owner chain.
      this->defaultProcedure( event );
      result = true;
      break;

    default:
      break;
  }
  return result;
}

/*------------------------------------------------------------------------------
| DefaultHandler                                                               |
|                                                                              |
| This class wrappers an object pointer and destroys the pointer to the object |
| when the wrapper is destructed.                                              |
------------------------------------------------------------------------------*/
static struct DefaultHandler
{
  operator IScrollBarHandler* ();
IScrollBarHandler
 *operator -> ( )
    {
    return *this;
    }
 ~DefaultHandler ( )
    {
    if (ptr)
      {
      delete ptr;
      ptr = 0;
      }
    }
IScrollBarHandler
 *ptr;
} defaultHandler;

DefaultHandler::operator IScrollBarHandler* ( )
{
  if ( !ptr )
    ptr = new IScrollBarHandler;
  return ptr;
}
#endif //IC_WIN


/*------------------------------------------------------------------------------
| Public scroll bar styles.                                                    |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLStaticConst)
const IScrollBar::Style
  IScrollBar::horizontal        ( 0, ISBS_HORIZONTAL ),
  IScrollBar::vertical          = SBS_VERT,
#ifndef IC_WIN
  IScrollBar::autoSize          = SBS_AUTOSIZE,
#endif
  IScrollBar::classDefaultStyle ( WS_VISIBLE | SBS_VERT, 0 );
#pragma data_seg()

/*------------------------------------------------------------------------------
| Default style for new objects (initial value).                               |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLNonConst)
IScrollBar::Style
  IScrollBar::currentDefaultStyle ( WS_VISIBLE | SBS_VERT, 0 );
#pragma data_seg()


/*------------------------------------------------------------------------------
| IScrollBar::systemScrollBarWidth                                             |
|                                                                              |
| Static system value function.                                                |
------------------------------------------------------------------------------*/
unsigned long  IScrollBar :: systemScrollBarWidth ( Boolean bVertical )
{
  return IQUERYSYSVALUE( bVertical ? SV_CXVSCROLL : SV_CYHSCROLL );
}

/*------------------------------------------------------------------------------
| IScrollBar::systemScrollBoxLength                                            |
|                                                                              |
| Static system value function.                                                |
------------------------------------------------------------------------------*/
unsigned long  IScrollBar :: systemScrollBoxLength ( Boolean bVertical )
{
  return IQUERYSYSVALUE( bVertical ? SV_CYVSLIDER : SV_CXHSLIDER );
}

/*------------------------------------------------------------------------------
| IScrollBar::systemScrollButtonLength                                         |
|                                                                              |
| Static system value function.                                                |
------------------------------------------------------------------------------*/
unsigned long  IScrollBar :: systemScrollButtonLength ( Boolean bVertical )
{
  return IQUERYSYSVALUE( bVertical ? SV_CYVSCROLLARROW :
                                     SV_CXHSCROLLARROW );
}


/*------------------------------------------------------------------------------
| IScrollBar::IScrollBar                                                       |
|                                                                              |
| Constructor to create a scroll bar control with defaulted values for         |
| scroll range and scroll box information on a standard window.                |
------------------------------------------------------------------------------*/
IScrollBar :: IScrollBar ( unsigned long windowId,
                           IWindow* parent, IWindow* owner,
                           const IRectangle& initial,
                           const Style& style )
  : ulClMinScrollIncrement(1), ulClPageScrollIncrement(0),
#ifdef IC_WIN
    fScrollBarData (new IScrollBarData() )
#else
    fScrollBarData (0)
#endif
{
  IMODTRACE_DEVELOP("IScrollBar::ctor");
  IASSERTPARM(parent != 0);

  initialize(windowId, parent, owner, initial,
         IRange(1, 100), 25, 1, style);
}


/*------------------------------------------------------------------------------
| IScrollBar::IScrollBar                                                       |
|                                                                              |
| Constructor to create a scroll bar control with scroll range and scroll      |
| box information on a standard window.                                        |
------------------------------------------------------------------------------*/
IScrollBar :: IScrollBar ( unsigned long windowId,
                           IWindow* parent, IWindow* owner,
                           const IRange& scrollableItems,
                           unsigned long visibleItemCount,
                           unsigned long scrollBoxPosition,
                           const IRectangle& initial,
                           const Style& style )
  : ulClMinScrollIncrement(1), ulClPageScrollIncrement(0),
#ifdef IC_WIN
    fScrollBarData (new IScrollBarData() )
#else
    fScrollBarData (0)
#endif
{
  IMODTRACE_DEVELOP("IScrollBar::ctor");
  IASSERTPARM(parent != 0);

  initialize(windowId, parent, owner, initial,
         scrollableItems, visibleItemCount, scrollBoxPosition, style);
}


/*------------------------------------------------------------------------------
| IScrollBar::IScrollBar                                                       |
|                                                                              |
| Constructor to instantiate a scroll bar object for a dialog template.        |
------------------------------------------------------------------------------*/
IScrollBar :: IScrollBar ( unsigned long windowId, IWindow* parent )
  : ulClMinScrollIncrement(1), ulClPageScrollIncrement(0),
#ifdef IC_WIN
    fScrollBarData (new IScrollBarData() )
#else
    fScrollBarData (0)
#endif
{
  IMODTRACE_DEVELOP("IScrollBar::ctor");
  setAutoDestroyWindow(false);
#ifdef IC_WIN
  this->addHandler( defaultHandler );
  if (IPlatform::isWin32S())
    fScrollBarData->fvisibleCount = 25;     //  use same default as ctor 1
#endif
  startHandlingEventsFor(windowId, parent);
}


/*------------------------------------------------------------------------------
| IScrollBar::IScrollBar                                                       |
|                                                                              |
| Constructor to instantiate from an existing scroll bar control.              |
------------------------------------------------------------------------------*/
IScrollBar :: IScrollBar ( const IWindowHandle & wh )
  :   ulClMinScrollIncrement(1), ulClPageScrollIncrement(0),
#ifdef IC_WIN
    fScrollBarData (new IScrollBarData() )
#else
    fScrollBarData (0)
#endif
{
  IMODTRACE_DEVELOP("IScrollBar::ctor");
  setAutoDestroyWindow(false);
#ifdef IC_WIN
  this->addHandler( defaultHandler );
  if (IPlatform::isWin32S())
    fScrollBarData->fvisibleCount = 25;     //  use same default as ctor 1
#endif
  startHandlingEventsFor(wh);
}


/*------------------------------------------------------------------------------
| IScrollBar::~IScrollBar                                                      |
|                                                                              |
| Empty destructor for page tuning.                                            |
------------------------------------------------------------------------------*/
IScrollBar :: ~IScrollBar ( )
{
#ifdef IC_WIN
  removeHandler( defaultHandler );
  if ( fScrollBarData != 0 )
   delete fScrollBarData;
#endif
}


/*------------------------------------------------------------------------------
| IScrollBar::create                                                           |
------------------------------------------------------------------------------*/
void  IScrollBar :: initialize ( unsigned long windowId,
                                 IWindow* pParent,
                                 IWindow* pOwner,
                                 const IRectangle& initial,
                                 const IRange& scrollableItems,
                                 unsigned long visibleItemCount,
                                 unsigned long scrollBoxPosition,
                                 const Style& style )
{
  // Save the extended style to make sure we have a copy of it stored
  setExtendedStyle( extendedStyle() | style.asExtendedUnsignedLong() );

#ifndef IC_WIN
  /*****************************************************************/
  /* Check for autosizing of the scroll bar (PM only autosizes if  */
  /* if the appropriate rectangle dimension is <= 0).  This        */
  /* means that you can't create a scroll bar with a size of       */
  /* (0, 0), unless you size it after you create it.               */
  /*****************************************************************/
  Boolean bAutoWidthSize = false;
  Boolean bAutoHeightSize = false;
  if (style & autoSize)
  {                                    //Scroll bar should be auto-sized
     if (style & vertical)
     {                                 //Vertical scroll bar
        bAutoWidthSize = true;         //Autosize vertical scroll bar's width
     }
     else
     {                                 //Horizontal scroll bar
        bAutoHeightSize = true;        //Autosize horizontal scroll bar's
     }                                 //height
  }

  /*****************************************************************/
  /* Build structure for passing control data.                     */
  /*****************************************************************/
  SBCDATA sbcdata;
  initializeControlData(scrollableItems, visibleItemCount, &sbcdata);
                                       //Initialize the SBCDATA structure
  sbcdata.sHilite = 0;
  sbcdata.posThumb = (short)scrollBoxPosition;

  IRectangle rectInit(initial.bottomLeft(),
                      ISize(bAutoWidthSize ? 0 : initial.width(),
                            bAutoHeightSize ? 0 : initial.height()));
#endif
#ifdef IC_WIN
  if (IPlatform::isWin32S())
    fScrollBarData->fvisibleCount = visibleItemCount;
#endif

  /****************************************************************/
  /* Borland has bug with embedded ? operator so below is needed. */
  /****************************************************************/
#ifdef __BORLANDC__
  IWindowHandle owner;
  if (pOwner == 0)
    owner = IWindowHandle(0);
  else
    owner = pOwner->handle();
#else
  IWindowHandle owner = (pOwner == 0) ? IWindowHandle(0) : pOwner->handle();
#endif

  IWindowHandle wh =
      this -> create( windowId,
                      0,
                      convertToGUIStyle( style ),
                      WC_SCROLLBAR,
                      pParent->handle(),
                      owner,
#ifdef IC_WIN
                      initial,
                      0,
#else
                      rectInit,
                      &sbcdata,
#endif
                      0,
                      defaultOrdering(),
                      convertToGUIStyle( style, true ) );

#ifdef IC_WIN
  this->addHandler( defaultHandler );

  if (!(IPlatform::isWin32S()))
  {
    SCROLLINFO scrollinfo;
    initializeControlData(scrollableItems, visibleItemCount, &scrollinfo);
    scrollinfo.fMask  = SIF_PAGE | SIF_RANGE | SIF_POS;
    scrollinfo.nPos   = (int)scrollBoxPosition;
    SetScrollInfo( wh,
                   SB_CTL,
                   &scrollinfo,
                   TRUE );
  }

#endif
  startHandlingEventsFor(wh);
}


#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IScrollBar::initializeControlData                                            |
|                                                                              |
| Fill in control data structure for a scroll bar control.                     |
------------------------------------------------------------------------------*/
void  IScrollBar :: initializeControlData ( const IRange & scrollableItems,
                                            unsigned long visibleItemCount,
                                            struct tagSCROLLINFO* controlData )
{
  int lower = (int)scrollableItems.lowerBound();
  int upper = (int)scrollableItems.upperBound();
  if (lower > upper)
  {                                    //Swap the bounds
     int ulTemp = lower;
     lower = upper;
     upper = ulTemp;
  }

  /*****************************************************************/
  /* Store control data values now.                                */
  /*****************************************************************/
  controlData->cbSize = sizeof(SCROLLINFO);
  controlData->nMin   = lower;
  controlData->nMax   = upper;
  controlData->nPage    = (unsigned int)visibleItemCount;
}

#else
/*------------------------------------------------------------------------------
| IScrollBar::initializeControlData                                            |
|                                                                              |
| Fill in control data structure for a scroll bar control.                     |
------------------------------------------------------------------------------*/
void  IScrollBar :: initializeControlData ( const IRange & scrollableItems,
                                            unsigned long visibleItemCount,
                                            struct _SBCDATA* controlData )
{
  unsigned long ulLower = scrollableItems.lowerBound();
  unsigned long ulUpper = scrollableItems.upperBound();
  if (ulLower > ulUpper)
  {                                    //Swap the bounds
     unsigned long ulTemp = ulLower;
     ulLower = ulUpper;
     ulUpper = ulTemp;
  }
  unsigned long ulTotalItems = ulUpper - ulLower + 1;

  /*****************************************************************/
  /* Store control data values now.                                */
  /*****************************************************************/
  controlData->cb = sizeof(SBCDATA);
  controlData->posFirst = (short)ulLower;
  controlData->posLast = (short)((visibleItemCount > ulTotalItems) ?
                            ulLower : ulUpper - visibleItemCount + 1);
  controlData->cVisible = (short)visibleItemCount;
  controlData->cTotal = (short)ulTotalItems;
}


/*------------------------------------------------------------------------------
| IScrollBar::windowData                                                       |
|                                                                              |
| Return window parameters and control data for a scroll bar control.  The     |
| caller is responsible for setting up the pointer and count fields for the    |
| data desired.  This function will initialize other fields in the _WNDPARAMS  |
| structure.                                                                   |
------------------------------------------------------------------------------*/
void  IScrollBar :: windowData ( struct _WNDPARAMS* windowData ) const
{
  // Assume caller has set up return buffers and sizes.
  windowData->fsStatus = 0;

  if (windowData->pszText)
  {                                    // Caller wants text
     windowData->fsStatus |= (WPM_TEXT | WPM_CCHTEXT);
  }
  else
  {
     windowData->cchText = 0;
  }

  if (windowData->pPresParams)
  {                           // Caller wants presentation parameters
     windowData->fsStatus |= (WPM_PRESPARAMS | WPM_CBPRESPARAMS);
  }
  else
  {
     windowData->cbPresParams = 0;
  }

  if (windowData->pCtlData)
  {                                    // Caller wants control data
     windowData->fsStatus |= (WPM_CTLDATA | WPM_CBCTLDATA);
  }
  else
  {
     windowData->cbCtlData = 0;
  }

  Boolean success = handle().sendEvent(WM_QUERYWINDOWPARAMS,
                                       windowData, 0);
  if (success == false)
  {                                    // Something went wrong
     ITHROWGUIERROR2("WM_QUERYWINDOWPARAMS",
                     IBaseErrorInfo::accessError,
                     IException::recoverable);
  }
  /*****************************************************************/
  /* Note: windowData->fsStatus does not get cleared on success,   */
  /*       as advertised, so just assume we have all we requested. */
  /*****************************************************************/
}
#endif


/*------------------------------------------------------------------------------
| IScrollBar::defaultStyle                                                     |
|                                                                              |
| Return the current default style for creating new scroll bar objects.        |
------------------------------------------------------------------------------*/
IScrollBar::Style  IScrollBar :: defaultStyle ( )
{
  return currentDefaultStyle;
}

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IScrollBar::setDefaultStyle                                                  |
|                                                                              |
| Sets the current default style for creating new scroll bar objects.          |
------------------------------------------------------------------------------*/
void IScrollBar::setDefaultStyle ( const IScrollBar::Style& style )
{
  currentDefaultStyle = style;
}
#endif

/*------------------------------------------------------------------------------
| IScrollBar::convertToGUIStyle                                                |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long IScrollBar::convertToGUIStyle(const IBitFlag& guiStyle,
                                            Boolean bExtOnly) const
{
  // Obtain the style from the class (IControl) that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= guiStyle.asExtendedUnsignedLong() & IS_EXTMASK;
  }
  else
  {
    // Let the clean SBS_ styles, flow thru
    ulStyle |= guiStyle.asUnsignedLong() & ISBS_MASK;

#ifdef IC_WIN
    // For Windows it is always a child window
    ulStyle |= WS_CHILD;
#else
    // Add the thumb sizing style
    ulStyle |= SBS_THUMBSIZE;
#endif
  }

  return( ulStyle );
}


/*------------------------------------------------------------------------------
| IScrollBar::isHorizontal                                                     |
------------------------------------------------------------------------------*/
Boolean  IScrollBar :: isHorizontal ( ) const
{
  return !isVertical();
}


/*------------------------------------------------------------------------------
| IScrollBar::isVertical                                                       |
------------------------------------------------------------------------------*/
Boolean  IScrollBar :: isVertical ( ) const
{
   return ((style() & vertical.asUnsignedLong()) ? true : false);
}


/*------------------------------------------------------------------------------
| IScrollBar::setScrollBar                                                     |
|                                                                              |
| Set the scroll bar range and scroll box size.                                |
------------------------------------------------------------------------------*/
IScrollBar &  IScrollBar :: setScrollBar ( const IRange & minMax,
                                           unsigned long visibleCount )
{
#ifdef IC_WIN
  if (IPlatform::isWin32S())
  {
    SetScrollRange( handle(), SB_CTL, minMax.lowerBound(),
                    minMax.upperBound(), TRUE );
    // Note thumb size is constant in Windows and cannot be changed
    // visually.  We track it internally.
    fScrollBarData->fvisibleCount = visibleCount;
  }
  else
  {
    SCROLLINFO scrollinfo;
    initializeControlData(minMax, visibleCount, &scrollinfo);
    scrollinfo.fMask  = SIF_PAGE | SIF_RANGE;
    SetScrollInfo( this->handle(),
                   SB_CTL,
                   &scrollinfo,
                   TRUE );
  }

#else
  SBCDATA sbcdata;
  initializeControlData(minMax, visibleCount, &sbcdata);
                                    // Initialize the SBCDATA structure
  unsigned long ulPos = scrollBoxPosition();
  unsigned long ulTotalItems = sbcdata.cTotal;

  handle().sendEvent(SBM_SETSCROLLBAR,
                     IEventParameter1(ulPos),
                     IEventParameter2(sbcdata.posFirst,
                                      sbcdata.posLast));
  handle().sendEvent(SBM_SETTHUMBSIZE,
                     IEventParameter1((unsigned short)visibleCount,
                                      (unsigned short)ulTotalItems),
                     IEventParameter2());
#endif
  enable();       // Don't leave scroll bar disabled if not appropriate
  return *this;
}


/*------------------------------------------------------------------------------
| IScrollBar::setScrollableRange                                               |
|                                                                              |
| Set the range of all items to which the scroll box can scroll.               |
------------------------------------------------------------------------------*/
IScrollBar &  IScrollBar :: setScrollableRange ( const IRange & minMax )
{
  if (this->scrollableRange() != minMax)
  {
    unsigned long ulVisible = visibleCount();
    setScrollBar(minMax, ulVisible);
  }
  return *this;
}


/*------------------------------------------------------------------------------
| IScrollBar::setVisibleCount                                                  |
|                                                                              |
| Set the size of the scroll box.                                              |
------------------------------------------------------------------------------*/
IScrollBar &  IScrollBar :: setVisibleCount ( unsigned long rangeUnits )
{
  if (this->visibleCount() != rangeUnits)
  {
    IRange rangeScroll = scrollableRange();
    setScrollBar(rangeScroll, rangeUnits);
  }
  return *this;
}


/*------------------------------------------------------------------------------
| IScrollBar::moveScrollBoxTo                                                  |
------------------------------------------------------------------------------*/
IScrollBar &  IScrollBar :: moveScrollBoxTo ( unsigned long bottomItem )
{
  if (this->scrollBoxPosition() != bottomItem)
  {
#ifdef IC_WIN
    unsigned long lparam = TRUE;

    if (IPlatform::isWin32S())
    {
      handle().sendEvent( IC_SBM_SETPOS_WIN32S, bottomItem, lparam );
      SetScrollPos( handle(), SB_CTL, bottomItem, lparam );
    }
    else
      handle().sendEvent( SBM_SETPOS, bottomItem, lparam );
#else
    unsigned long lparam = 0;
    handle().sendEvent( SBM_SETPOS, bottomItem, lparam );
#endif

  }
  return *this;
}


/*------------------------------------------------------------------------------
| IScrollBar::scrollableRange                                                  |
|                                                                              |
| Return the range of all scrollable items.                                    |
------------------------------------------------------------------------------*/
IRange  IScrollBar :: scrollableRange ( ) const
{
#ifdef IC_WIN
  INT  minPos, maxPos;

  if (IPlatform::isWin32S())
    GetScrollRange( handle(), SB_CTL, &minPos, &maxPos );
  else
    handle().sendEvent( SBM_GETRANGE, &minPos, &maxPos );
  return IRange(minPos, maxPos);
#else
  SBCDATA sbcdata;
  WNDPARAMS wndparams;

  wndparams.pszText = 0;
  wndparams.pPresParams = 0;
  wndparams.cbCtlData = sizeof(SBCDATA);
  wndparams.pCtlData = &sbcdata;
  windowData(&wndparams);

  unsigned long start = sbcdata.posFirst;
  return IRange(start, start + sbcdata.cTotal - 1);
#endif
}


/*------------------------------------------------------------------------------
| IScrollBar::scrollBoxRange                                                   |
|                                                                              |
| Return the range in which the scroll box can be positioned.                  |
------------------------------------------------------------------------------*/
IRange  IScrollBar :: scrollBoxRange ( ) const
{
#ifdef IC_WIN
  INT  minPos, maxPos;

  if (IPlatform::isWin32S())
    GetScrollRange( handle(), SB_CTL, &minPos, &maxPos );
    // Note that because the thumb size is not variable, the scroll box
    // range should not be adjusted for the thumb size.  The scroll
    // box range should be the entire scrollable range, to allow the
    // scroll box to be positioned anywhere in the range.
  else
    {
    handle().sendEvent( SBM_GETRANGE, &minPos, &maxPos );
    maxPos -= this->visibleCount();
    }
  return IRange( minPos, maxPos );
#else
  IEventResult evtRc = handle().sendEvent(SBM_QUERYRANGE, 0, 0);
  return IRange(evtRc.number1(), evtRc.number2());
#endif
}


/*------------------------------------------------------------------------------
| IScrollBar::visibleCount                                                     |
|                                                                              |
| Return the scroll box size (also used to determine the size of               |
| a page scroll).                                                              |
------------------------------------------------------------------------------*/
unsigned long  IScrollBar :: visibleCount ( ) const
{
#ifdef IC_WIN
  if (IPlatform::isWin32S())
  {
    // Note thumb size is constant in Windows and cannot be changed
    // visually.  We track it internally.
    return fScrollBarData->fvisibleCount;
  }
  else
  {
    SCROLLINFO scrollinfo;
    scrollinfo.cbSize = sizeof(SCROLLINFO);
    scrollinfo.fMask  = SIF_PAGE;
    GetScrollInfo( this->handle(),
                   SB_CTL,
                   &scrollinfo );
    return scrollinfo.nPage;
  }
#else
  SBCDATA sbcdata;
  WNDPARAMS wndparams;

  wndparams.pszText = 0;
  wndparams.pPresParams = 0;
  wndparams.cbCtlData = sizeof(SBCDATA);
  wndparams.pCtlData = &sbcdata;
  windowData(&wndparams);

  return sbcdata.cVisible;
#endif
}


/*------------------------------------------------------------------------------
| IScrollBar::scrollBoxPosition                                                |
|                                                                              |
| Return the position of the scroll box (the lower boundary).                  |
------------------------------------------------------------------------------*/
unsigned long  IScrollBar :: scrollBoxPosition ( ) const
{
#ifdef IC_WIN
  if (IPlatform::isWin32S())
    return (unsigned long) GetScrollPos( handle(), SB_CTL);
  else
    return (unsigned long) handle().sendEvent(SBM_QUERYPOS, 0, 0);
#else
  return (unsigned long) handle().sendEvent(SBM_QUERYPOS, 0, 0);
#endif
}


/*------------------------------------------------------------------------------
| IScrollBar::pageScrollIncrement                                              |
|                                                                              |
| Return the page scrolling increment (the default value is the visible page   |
| size minus the minimum scrolling increment).                                 |
------------------------------------------------------------------------------*/
unsigned long  IScrollBar :: pageScrollIncrement ( ) const
{
  unsigned long ulScroll = ulClPageScrollIncrement;
  if (ulScroll == 0)
  {                                    //No user-specified value
     long lDelta = visibleCount() - minScrollIncrement();
     ulScroll = (lDelta > 0) ? lDelta : minScrollIncrement();
                                      //Use the default if valid
  }
  return ulScroll;
}


/*------------------------------------------------------------------------------
| IScrollBar::calcMinimumSize                                                  |
|                                                                              |
| Return the size of a "typical" scroll bar.                                   |
------------------------------------------------------------------------------*/
ISize  IScrollBar :: calcMinimumSize ( ) const
{
  Boolean bVertical = isVertical();
  unsigned long ulLength = 2 * systemScrollButtonLength(bVertical) +
                             3 * systemScrollBoxLength(bVertical);
  unsigned long ulWidth = systemScrollBarWidth(bVertical);

  ISize sizMin = bVertical ? ISize(ulWidth, ulLength) :
                             ISize(ulLength, ulWidth);
  sizMin += ISize(2 * IQUERYSYSVALUE( SV_CXBORDER ),
                  2 * IQUERYSYSVALUE( SV_CYBORDER ));
  return sizMin;
}

/*------------------------------------------------------------------------------
| WIN32 Color Support                                                          |
|                                                                              |
| Windows has no API equivalent for Presentation Parameters.  The following    |
| scrollBar color functions are NOPed for Windows.  The query functions return |
| an appropriate default IColor object.                                        |
------------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------
| IScrollBar::foregroundColor                                                  |
|                                                                              |
| Returns the foreground color of the scrollbar.                               |
------------------------------------------------------------------------------*/
IColor IScrollBar::foregroundColor () const
{
#ifdef IC_PM
  return (IWindow::color(PP_FOREGROUNDCOLOR,
                         IGUIColor(IGUIColor::defaultControl)));
#else
  return IGUIColor( IGUIColor::defaultControl );
#endif //IC_PM
}

/*------------------------------------------------------------------------------
| IScrollBar::hiliteForegroundColor                                            |
|                                                                              |
| Returns the hilite foreground color of the scrollbar.                        |
------------------------------------------------------------------------------*/
IColor IScrollBar::hiliteForegroundColor () const
{
#ifdef IC_PM
  return (IWindow::color(PP_HILITEFOREGROUNDCOLOR,
                         IGUIColor(IGUIColor::buttonMiddle)));
#else
  return IGUIColor( IGUIColor::buttonMiddle );
#endif // IC_PM
}

/*------------------------------------------------------------------------------
| IScrollBar::setMinScrollIncrement                                            |
|                                                                              |
| Sets the minimum scroll increment for the scrollbar.                         |
------------------------------------------------------------------------------*/
IScrollBar& IScrollBar :: setMinScrollIncrement ( unsigned long value )
{
  if (this->minScrollIncrement() != value)
  {
    ulClMinScrollIncrement = value;
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IScrollBar::setPageScrollIncrement                                           |
|                                                                              |
| Sets the page scroll increment for the scrollbar.                            |
------------------------------------------------------------------------------*/
IScrollBar& IScrollBar :: setPageScrollIncrement ( unsigned long value )
{
  if (this->pageScrollIncrement() != value)
  {
    ulClPageScrollIncrement = value;
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IScrollBar::minScrollIncrement                                               |
|                                                                              |
| Returns the minimum page scroll increment for the scrollbar.                 |
------------------------------------------------------------------------------*/
unsigned long  IScrollBar :: minScrollIncrement ( ) const
{
  return ulClMinScrollIncrement;
}

